#!/bin/bash
#
#******************************************************************************
#  Licensed Materials - Property of IBM
#
#  (C) Copyright IBM Corp.  2008  All Rights Reserved
#
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#
# -----------------------------------------------------------------------------
# 
# =========================================================================
# 
# Name: cfgERM.sh
#
# Change Log:
#  Date          developer             change
#  08/1/2008     Bryan Jen             Initial script creation
#  02/13/2009    Bryan Jen             DCR1046: Adds sync mirroring support.
#  02/18/2010    Liming (Michael) Yu   Removed hard coded IP addresses
#  03/05/2010    Liming (Michael) Yu   Updated print and log messages; updated
#                                      error messages and exit codes
#
#******************************************************************************


DRLOG="/opt/tivoli/tiam/log/dr"

SCRIPTNAME="cfgERM"
SMCLI="/usr/bin/SMcli"
datestamp=`date +%m%d%Y%H%M%S`
LOGFILE="${DRLOG}/cfgERM.${datestamp}.out"


usage() {
  echo " cfgERM.sh"
  echo "       -h                 help/usage information"
  echo " "
  echo "    eg.   cfgERM.sh"
  return 0
}


pairSubSystems() {
  log "Enter pairSubSystems()"
  printAndLog "Adding Subsystems to Managed Systems list"
  
  cmd=`${SMCLI} -A ${CTRLA} ${CTRLB}`
  result=`echo $?`
  if [ $result != 0 ]; then
    echo $cmd >> $LOGFILE
    printAndLogErrorExit 6 "Failed to add Primary System to SM Managed Systems List"
  fi
  
  cmd=`${SMCLI} -A ${SSCTRLA} ${SSCTRLB}`
  result=`echo $?`
  if [ $result != 0 ]; then
    echo $cmd >> $LOGFILE
    printAndLogErrorExit 7 "Failed to add Secondary System to SM Managed Systems List"
  fi

  printAndLog "Updating Primary Storage Subsystem name to $NAME"
  cmd=`${SMCLI} ${CTRLA} ${CTRLB} -c "set storagesubsystem userLabel=\"$NAME\";"`
  result=`echo $?`
  if [ $result != 0 ]; then
    echo $cmd >> $LOGFILE
    printAndLogErrorExit 8 "Failed to update Primary Storage Subsystem name"
  fi
 
  printAndLog "Updating Secondary Storage Subsystem name to $SSNAME"
  cmd=`${SMCLI} ${SSCTRLA} ${SSCTRLB} -c "set storagesubsystem userLabel=\"$SSNAME\";"`
  result=`echo $?`
  if [ $result != 0 ]; then
    echo $cmd >> $LOGFILE
    printAndLogErrorExit 9 "Failed to update Secondary Storage Subsystem name"
  fi

  sleep 10
  log "Exit pairSubSystems() with rc=0"
}


activateFeatKey() {
  log "Enter activateFeatKey()"

  FEATKEY="/tmp/iaERM_${ctrlNum}_a.key"
  SSFEATKEY="/tmp/iaERM_${ctrlNum}_b.key"
  
  printAndLog "Activating feature key $FEATKEY on primary"
  cmd=`${SMCLI} ${CTRLA} ${CTRLB} -c "enable storageSubsystem feature file=\"$FEATKEY\";"`
  result=`echo $?`
  if [ $result != 0 ]; then
    echo $cmd >> $LOGFILE
    printAndLogErrorExit 10 "Feature Key not found. Please place/name feature key to location: \"$FEATKEY\""
  fi

  printAndLog "Activating feature key $SSFEATKEY on secondary"
  cmd=`${SMCLI} ${SSCTRLA} ${SSCTRLB} -c "enable storageSubsystem feature file=\"$SSFEATKEY\";"`
  result=`echo $?`
  if [ $result != 0 ]; then
    echo $cmd >> $LOGFILE
    printAndLogErrorExit 11 "Feature Key not found. Please place/name feature key to location: \"$SSFEATKEY\""
  fi

  log "Exit activateFeatKey() with rc=0"
}


activateERM() {
  log "Enter activateERM()"

  printAndLog "Activating ERM on primary"
  cmd=`${SMCLI} ${CTRLA} ${CTRLB} -c "activate storageSubsystem feature=remoteMirror repositoryArray=\"iaarray_85_1\";"`
  result=`echo $?`
  if [ $result != 0 ]; then
    echo $cmd >> $LOGFILE
    printAndLogErrorExit 12 "Failed to activate ERM on Local"
  fi

  printAndLog "Activating ERM on secondary"
  cmd=`${SMCLI} ${SSCTRLA} ${SSCTRLB} -c "activate storageSubsystem feature=remoteMirror repositoryArray=\"iaarray_85_1\";"`
  result=`echo $?`
  if [ $result != 0 ]; then
    echo $cmd >> $LOGFILE
    printAndLogErrorExit 13 "Failed to activate ERM on Remote"
  fi

  log "Exit activateERM() with rc=0"
}


pairLuns() {
  log "Enter pairLuns()"

  for LUNNAME in `${SMCLI} ${CTRLA} ${CTRLB}  -c "show alllogicaldrives;" |grep -i "logical drive name" | grep -vi mirror | cut -d: -f2 | awk '{ print $1 }'`;
  do
    printAndLog "Pairing $LUNNAME to $LUNNAME on $SSNAME"
    cmd=`${SMCLI} ${CTRLA} ${CTRLB} -c "create remoteMirror primary=\"$LUNNAME\" secondary=\"$LUNNAME\" remoteStorageSubsystemName=\"$SSNAME\" autoResync=disabled writeOrder=preserved writeMode=$ERMMODE;"`
    result=`echo $?`
    if [ $result != 0 ]; then
      echo $cmd >> $LOGFILE
      printAndLogErrorExit 14 "Pairing $LUNNAME FAILED!"
    fi
  done

  log "Exit pairLuns with rc=0"
}


pingtest() {
  cmd=`ping $1 -c 1`
  if [[ `echo $cmd |grep -i unreachable` ]]; then
    return -1
  else
    return 0
  fi
}


log() {
  timestamp=`date "+%Y-%m-%d %H:%M:%S"`
  echo "$timestamp [$SCRIPTNAME] $1" >> $LOGFILE
}


printAndLog() {
  timestamp=`date "+%Y-%m-%d %H:%M:%S"`
  echo "$timestamp [$SCRIPTNAME] $1"
  echo "$timestamp [$SCRIPTNAME] $1" >> $LOGFILE
}


printAndLogErrorExit() {
  timestamp=`date "+%Y-%m-%d %H:%M:%S"`
  echo "$timestamp [$SCRIPTNAME] ERROR($1): $2"
  echo "$timestamp [$SCRIPTNAME] ERROR($1): $2" >> $LOGFILE
  echo "$timestamp [$SCRIPTNAME] Exiting script with rc=$1. Please refer to the logfile @ $LOGFILE"
  echo "$timestamp [$SCRIPTNAME] Exiting script with rc=$1" >> $LOGFILE
  exit $1
}


doERM() {
  log "Enter doERM()"

  printAndLog "Now pairing Local Storage: $CTRLA $CTRLB to Remote Storage: $SSCTRLA $SSCTRLB"
  pairSubSystems

  if [[ `${SMCLI} $CTRLA $CTRLB -c "show storagesubsystem summary;" |grep -i mirroring |grep -i 'Enabled/Activated'` ]]; then
    printAndLog "System already enabled and activated for ERM. Proceeding with LUN pairing"
  elif [[ `${SMCLI} $CTRLA $CTRLB -c "show storagesubsystem summary;" |grep -i mirroring |grep -i 'Enabled/Deactivated'` ]]; then
    printAndLog "Remote Mirroring feature has been enabled but not activated. Activating Remote Mirroring feature"
    activateERM
  else
    printAndLog "Remote Mirroring feature has been disabled and deactivated. Activating feature key and Remote Mirroring feature"
    activateFeatKey
    activateERM
  fi

  printAndLog "Continuing with LUN pairing"
  pairLuns

  log "Exit doERM() with rc=0"
}


#======= beginning of the main function =======

ls $DRLOG > /dev/null 2>&1
rc=$?
if [ $rc -ne 0 ]; then
  mkdir $DRLOG
fi

printAndLog "Started configuring ERM"

CTRLA="0.0.0.0"
CTRLB="0.0.0.0"
ERMMODE="none"

while [ "$1" != "" ]
do
  case $1 in
  -h | -? | --help )
    usage
    exit
    ;;
  * )
    printAndLogErrorExit -2 "Invalid input parameter. Please enter cfgERM.sh --help to see the usage"
  esac
done

ERMMODE="asynchronous"

if [ $ERMMODE = "none" ]; then
  printAndLogErrorExit -3 "No ermmode specified. Please enter cfgERM.sh --help to see the usage"
fi

ssystems=(`cat /home/iaadmin/.tiam/application.properties |grep ds4k |cut -d= -f2`)
if [[ ${#ssystems[@]} -lt 2 ]]; then
  printAndLogErrorExit 1 "Invalid or missing data in applications.properties"
fi 

printAndLog "Verifying all luns are cleared for remirroring."
/opt/tivoli/tiam/bin/unCfgERM.sh >> $LOGFILE
rc=$?
if [ $rc -ne 0 ]; then
  printAndLogErrorExit 15 "Unable to reset mirror state on local LUNs"
fi

/usr/bin/ssh root@dr_remote_mc /opt/tivoli/tiam/bin/unCfgERM.sh >> $LOGFILE
rc=$?
if [ $rc -ne 0 ]; then
  printAndLogErrorExit 15 "Unable to reset mirror state on remote LUNs"
fi


i=0
while [ $i -lt ${#ssystems[@]} ]
do
  if [[ ${ssystems[$i]} =~ "disk_ctrl" ]]; then
    CTRLA=${ssystems[$i]}
    ctrlNum=`echo ${ssystems[$i]} |cut -d_ -f3 |awk '{print $1}'`
    letter=`echo ${ssystems[$i]} |cut -d_ -f4 |awk '{print $1}'`
    SSCTRLA="mirror_ctrl_${ctrlNum}_${letter}"
  else
    printAndLogErrorExit 2 "Either an IP address or a hostname that does not start with \"disk_ctrl\" was found in applications.properties. Please correct and retry"
  fi
  if [[ ${ssystems[$(($i+1))]} =~ "disk_ctrl" ]]; then
    CTRLB=${ssystems[$(($i+1))]}
    ctrlNum=`echo ${ssystems[$i+1]} |cut -d_ -f3 |awk '{print $1}'`
    letter=`echo ${ssystems[$i+1]} |cut -d_ -f4 |awk '{print $1}'`
    SSCTRLB="mirror_ctrl_${ctrlNum}_${letter}"
  else
    printAndLogErrorExit 3 "Either an IP address or a hostname that does not start with \"disk_ctrl\" was found in applications.properties. Please correct and retry"
  fi

  NAME=`${SMCLI} ${CTRLA} ${CTRLB} -c "show storagesubsystem summary;" | grep -i "PROFILE FOR STORAGE SUBSYSTEM" | cut -d: -f2 | awk '{ print $1 }'`
  result=`echo $?`
  if [ $result != 0 ]; then
    echo $NAME >> $LOGFILE
    printAndLogErrorExit 4 "SMcli command to obtain NAME failed! Please check the storage subsystem on the primary site"
  fi

  SSNAME=`${SMCLI} ${SSCTRLA} ${SSCTRLB} -c "show storagesubsystem summary;" | grep -i "PROFILE FOR STORAGE SUBSYSTEM" | cut -d: -f2 | awk '{ print $1 }'`
  result=`echo $?`
  if [ $result != 0 ]; then
    echo $SSNAME >> $LOGFILE
    printAndLogErrorExit 5 "SMcli command to obtain SSNAME failed! Please check the storage subsystem on the secondary site"
  fi

  #TODO: check if the NAME and SSNAME are correct for the appliance and ds4k we're on and if they're not then change the NAME and SSNAME
  
  doERM

  i=$(($i+2))  
done

timestamp=`date "+%Y-%m-%d %H:%M:%S"`
echo "$timestamp [$SCRIPTNAME] ERM has been successfully configured"
echo "$timestamp [$SCRIPTNAME] ERM has been successfully configured" >> $LOGFILE
echo "$timestamp [$SCRIPTNAME] Exiting script with rc=0. Please refer to the logfile @ $LOGFILE"
echo "$timestamp [$SCRIPTNAME] Exiting script with rc=0" >> $LOGFILE
exit 0
