#!/bin/bash
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Filename: copyRASlogs.sh
#
# Component: RAS
#
# Description:
#    This script pulls all the logs collected on the Management Console node to the
#    current node.  If the argument [-t usb] is specified, the script will mount a flash
#    drive in the USB port (/dev/sr0 by default, unless overridden by -d), the logs will
#    be copied to the drive, and the USB port will be unmounted.
#
#  Usage:
#      sudo copyRASlogs.sh [-t usb [-d device] [-m mount_point]]
#
# Change history
#   Modifier           Date         CQ Change       Description
#   -------------------------------------------------------------------------
#   Brad Trubey        9/15/2010    TIAMC00014828   initial version
# *************************************************************************************

LOG_DIR=/home/iaadmin/logs
LC_DIR=/home/iaadmin/tiamSystemLogs
DEST_DIR=/home/iaadmin
DEVICE=/dev/sdb1
MOUNT_PT=/media/usb
USB=
rc=0

# Override the values from the command line arguments
while getopts "m:t:d:" optchar ; do
   case $optchar in
   d) DEVICE=$OPTARG
      echo "Set USB device to: ${DEVICE}" ;;
   m) MOUNT_PT=$OPTARG
      echo "Set mount point to: ${MOUNT_PT}" ;;
   t) USB=$OPTARG
      echo "Set target to ${USB}" ;;
   esac
done

##########################################################
# Utility to copy logs from MC to current node
##########################################################
copyFromMC() {
    # If we're already on MC, there's no need to copy files from MC
    if [ `hostname` != "iamconsole1" ]; then
        echo "Copying Log Collection archives from Management Server node to `hostname`"
        # Delete and recreate the destination directory
        rm -rf   $LOG_DIR
        rm -rf   $LC_DIR
        mkdir -p $LOG_DIR
        mkdir -p $LC_DIR

        # Copy all the logs from the MC to the RSM
        scp -r -q iamconsole1:$LOG_DIR $DEST_DIR
        scp -r -q iamconsole1:$LC_DIR  $DEST_DIR

        # Clean up the ownership of the resulting file so that iaadmin can read it
        chown -R iaadmin:iaadmin $LOG_DIR
        chown -R iaadmin:iaadmin $LC_DIR

        echo "Copy complete."
    else
        echo "Skipped copying Log Collection archives on Management Server"
    fi
}

##########################################################
# Utility to mount the USB flash drive
##########################################################
mountFlashDrive() {
    # If we already on RSM, flash drive should be automounted
    if [ `hostname` != "iarsm1" ]; then
        echo "Mounting the USB device ${DEVICE}"
        /opt/tivoli/tiam/bin/mount_usb.py -d ${DEVICE} -m ${MOUNT_PT} -w
        rc=$?
        if [ $rc == 0 ]; then
            echo "Flash drive mounted successfully"
        else
            echo "Unable to mount device ${DEVICE}"
            exit 10
        fi
    else
        echo "Not mounting the USB device ${DEVICE} on SLES RSM node"
    fi
}

##########################################################
# Utility to copy files to the USB flash drive
##########################################################
copyToFlashDrive() {
    echo "Copying the logs to device ${DEVICE} in directory ${MOUNT_PT}"
    mkdir -p ${MOUNT_PT}/home/iaadmin/logs
    mkdir -p ${MOUNT_PT}/home/iaadmin/tiamSystemLogs
    cp -Pr /home/iaadmin/logs/*           ${MOUNT_PT}/home/iaadmin/logs
    cp -Pr /home/iaadmin/tiamSystemLogs/* ${MOUNT_PT}/home/iaadmin/tiamSystemLogs
    chown -R iaadmin:users ${MOUNT_PT}/home/iaadmin
    rc=$?
    if [ $rc == 0 ]; then
        echo "Files copied successfully"
    else
        echo "Unable to copy files to device ${DEVICE}"
        exit 11
    fi
}

##########################################################
# Utility to unmount the USB flash drive
##########################################################
unmountFlashDrive() {
    # If we already on RSM, flash drive doesn't need to be unmounted
    if [ `hostname` != "iarsm1" ]; then
        /opt/tivoli/tiam/bin/umount_usb.py -d ${DEVICE}
        rc=$?
        if [ $rc == 0 ]; then
            echo "Unmounted the flash drive"
        else
            echo "Unable to unmount device ${DEVICE}"
            exit 12
        fi
    fi
}


# If this script is running from any node other than MC, retrieve the logs from MC
copyFromMC

# If "-t usb" was specified on the command line, mount the flash drive and copy the files to it, then unmount
if [ "${USB}" == "usb" ]; then
    mountFlashDrive
    copyToFlashDrive
    unmountFlashDrive
fi

exit 0
