#!/bin/bash
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Filename: dirinstall.sh
#
# Component: RAS
#
# Description:
#    This file contains bash script for installation and initial configuration
#    of IBM Director Server, pre-req components and Director extensions.
#
# Change history
#   Modifier          Date        CQ Change       Description
#   ---------------------------------------------------------------------------
#   Igor Ageyev       03/19/2009  TIAMC00004821   Initial creation
#   Eric Biesterfeld  04/13/2009  TIAMC00005333   Temporary disabling a check for IDVT
#   Linda Benhase     07/19/2009  TIAMC00007393
#     Remove use of rc_status and rc_reset since rc.status cannnot be sourced.
#     It truncates the output to 80 characters.
#   Rob Jose          07/20/2009  TIAMC00007172   updated eSA install script to include latest version
#   Brad Trubey       07/25/2009  TIAMC00007730   Fix permissions on eSA install script so that it can successfully install.
#   Linda Benhase     08/15/2009  TIAMC00008158
#     The nonstop profile seems to be causing unnecessary grief, don't use
#     it anymore.  This means NOT updating the dirserv.rsp file for that option.
#   Del Ence          08/26/2009  TIAMC00008616   Create and remove the linux release file so that install will run
#   Linda Benhase     09/01/2009  TIAMC00008840
#       Remove 8616 - we are keeping the SuSE-release file, but it is
#       by install code now since other components are dependent upon it too.
#   Brad Trubey       03/10/2010  TIAMC00012067   Support for Director 6.2
#   Del Ence          03/18/2010  TIAMC00010730   Set Director image install location
#   Brad Trubey       03/23/2010  TIAMC00012345   Go back to Director 6.1
#   Brad Trubey       04/08/2010  TIAMC00012760   Add support for Director 6.2
#   Del Ence            05/05/2010  TIAMC00012720   Added install of dependency RPM's to support Director 6.2
#   Del Ence            05/17/2010  TIAMC00013691   Check and update bond0 network interface if needed so that Director 6.2 install will run
#   Del Ence            06/08/2010  TIAMC00014044   RSA check and update of install script so that we do not error out
#   Del Ence            06/25/2010  TIAMC00007928   Update to use new ESA package for Director 6.2
#  Linda Benhase        7/13/2010   TIAM00014529    Save and restore data for Director for upgrade.
#  Linda Benhase        7/20/2010   TIAM00014666    Add chmod for ESA install script.
#   Del Ence            08/30/2010  TIAMC00014978   Validate Director vendor providers are installed and re-try installation if they were skipped
#   Del Ence            09/09/2010  TIAMC00015553   Updated to apply 6.2.1 upgrade package
#------------------------------------------------------------------------------

# Director Server package names
DIR_SERVER_INSTALL_TGZ="SysDir6_2_Server_Linux_x86.tar.gz"
DIR_SERVER_INSTALL="dirinstall.server"
UNPACK_DIR=/tmp/dirserver62
TMP_RAS=/tmp/RAS
DIR_INSTALL_ROOT=/opt/ibm/director

UPDATE_DIR=$UNPACK_DIR/packages/update
UPDATE_ZIP="SysDir6_2_1.zip"

# Release RPM name
RELEASE_RPM="sles-release-10-1.x86_64.rpm"

# ESA package names
ESA_INSTALL_TGZ="SysDir6_2_Service_Support_Linux_x86.tar.gz"
ESA_INSTALL_SCRIPT="SysDir6_2_Service_Support_Linux_x86.sh"
ESA_UNPACK_DIR=/tmp/esa

# Install image location
PACKAGE_DIR=RPMS

# Include the common RAS functions
. /opt/tivoli/tiam/bin/ras_functions
. /opt/tivoli/tiam/bin/dir_functions

#------------------------------------------------------------------------------
# Functions
#------------------------------------------------------------------------------

#---
# This function checks if RSA cards are installed
#---
check_rsa()
{
   lsusb -d 04b3:4001 > /dev/null 2>&1 && return 0
   lsusb -d 04b3:4003 > /dev/null 2>&1 && return 0
   return 1
}

#---
# This function checks if the provider rpm is installed if not found then tries to install it
#---
function check_provider_rpm()
{
    RPM=$1
    # Query for the RPM
    rpm -q --quiet ${RPM}
    if [ $? -eq 1 ]; then
       # Check if CIMServer is running
       /etc/init.d/cimserverd status &> /dev/null
       if [ $? -ne 0 ]; then
         ras_log -v "Starting CIM server"
         /etc/init.d/cimserverd start &> /dev/null
         if [ $? -ne 0 ]; then
           ras_log -v "Unable to start CIM server."
         fi
       fi
       # CIM Server is running so install the RPM
       ras_log -v "Install ${RPM}"
       execute_and_log rpm -Uvh ${UNPACK_DIR}/packages/${RPM}*sles10.i386.rpm
       if [ $? -ne 0 ]; then
         ras_log -v "Failed to install ${RPM}."
       fi
    fi
}

#---
# This function installs IBM Director Server
#---
function install_director_server() {

  local rc=0
  local rc1=0
  local rc2=0

  local IFCFG_BOND0=/etc/sysconfig/network/ifcfg-bond0
  local TMP_IFCFG_BOND0=${IFCFG_BOND0}.tmp

  # IBM Director installer needs bond0 to be configured with and IP address or it will error out
  #  So if bond0 it's not set  with and IP address then configure it with the loopback address for the install
  if [[ `ifconfig bond0 | grep 'inet addr' | wc -l` == 0 ]]; then
     sed -e "s/^IPADDR=''/IPADDR='127.0.0.1'/" ${IFCFG_BOND0} > \
     ${TMP_IFCFG_BOND0} && mv ${TMP_IFCFG_BOND0} ${IFCFG_BOND0}

     sed -e "s/^NETMASK=''/NETMASK='255.0.0.0'/" ${IFCFG_BOND0} > \
     ${TMP_IFCFG_BOND0} && mv ${TMP_IFCFG_BOND0} ${IFCFG_BOND0}

     # Take bond0 down
     ifdown bond0
     # Now bring bond0 back up
     ifup bond0
  fi

  ras_log -v "Uncompressing Director Server install package"
  rm -Rf ${UNPACK_DIR}
  mkdir -p ${UNPACK_DIR}
  execute_and_log tar xzvf ${PACKAGE_DIR}/${DIR_SERVER_INSTALL_TGZ} -C ${UNPACK_DIR}
  rc=$?

  if [[ $rc -ne 0 ]]; then
    ras_log -v "Uncompressing Director Server install package failed. Exiting ..."
    return $rc
  fi

  # Look for updated InstallMergeTools.jar in package directory
  if [ -f ${PACKAGE_DIR}/InstallMergeTools.jar ]; then
    # Use InstallMergeTools.jar from package directory
    cp ${PACKAGE_DIR}/InstallMergeTools.jar ${UNPACK_DIR}/bin
  fi

  # Look for upgrade zip file in package directory
  if [ -f ${PACKAGE_DIR}/${UPDATE_ZIP} ]; then
    ras_log -v "Uncompressing Director Server upgrade package"
    rm -Rf ${UPDATE_DIR}
    mkdir -p ${UPDATE_DIR}
    execute_and_log unzip ${PACKAGE_DIR}/${UPDATE_ZIP} -d ${UPDATE_DIR}
    rc=$?
    if [[ $rc -ne 0 ]]; then
      ras_log -v "Uncompressing Director Server upgrade package failed."
      return $rc
    fi
  fi

  # If this is a test machine with RSA cards then update the install script so that we do not  try and install ibmusbasm
  if (check_rsa); then
    sed -i "s/lsusb -d 04b3:4001 > \/dev\/null 2>&1 && return 0/return 1/" ${UNPACK_DIR}/${DIR_SERVER_INSTALL}
  fi

  ras_log -v "Modifying response file"
  DIR_RSP_FILE=${UNPACK_DIR}/dirserv.rsp
  TMP_DIR_RSP_FILE=${DIR_RSP_FILE}.tmp

  # Used default options for installing director without nonstop (which currently breaks 6.2 install)
  ras_log -v "Installing Director Server (this takes a few minutes)"
  execute_and_log ${UNPACK_DIR}/${DIR_SERVER_INSTALL} -r ${DIR_RSP_FILE}
  rc=$?

  if [[ $rc -ne 0 ]]; then
    ras_log -v "Installing Director Server failed. Exiting ..."
    return $rc
  fi

  # Check the third party providers and try to manually install if not found on system
  check_provider_rpm qlogic_cna_providers
  check_provider_rpm serveraid-platformagent
  check_provider_rpm aristos_sw_provider

  # Now remove configuration of bond0 if its configured for the loopback address
  if [[ `ifconfig bond0 | grep 'inet addr:127.0.0.1' | wc -l` == 1 ]]; then
    sed -e "s/^IPADDR='127.0.0.1'/IPADDR=''/" ${IFCFG_BOND0} > \
    ${TMP_IFCFG_BOND0} && mv ${TMP_IFCFG_BOND0} ${IFCFG_BOND0}

    sed -e "s/^NETMASK='255.0.0.0'/NETMASK=''/" ${IFCFG_BOND0} > \
    ${TMP_IFCFG_BOND0} && mv ${TMP_IFCFG_BOND0} ${IFCFG_BOND0}

    # Take bond0 down
    ifdown bond0
    # Now bring bond0 back up
    ifup bond0
  fi

  # Cleanup the unpacked installer if the install succeeded
  rm -Rf ${UNPACK_DIR}

  return 0
}


#---
# This function installs IBM eSA
#---
function install_esa() {

  local rc=0

  ras_log -v "Uncompressing Service and Support Manager install package"
  rm -Rf ${ESA_UNPACK_DIR}
  mkdir -p ${ESA_UNPACK_DIR}
  execute_and_log tar xzvf ${PACKAGE_DIR}/${ESA_INSTALL_TGZ} -C ${ESA_UNPACK_DIR}
  rc=$?

  if [[ $rc -ne 0 ]]; then
    ras_log -v "Uncompressing Service and Support Manager install package failed. Exiting ..."
    return $rc
  fi

  # script that gets extracted is not executable by default.  make it so
  execute_and_log chmod +x ${ESA_UNPACK_DIR}/${ESA_INSTALL_SCRIPT}

  # Start the ESA install.
  ras_log -v "Installing Service and Support Manager (this takes a few minutes)"
  execute_and_log ${ESA_UNPACK_DIR}/${ESA_INSTALL_SCRIPT}  -i silent -D\$LICENSE_ACCEPTED\$=true

  rc=$?

  if [[ $rc -ne 0 ]]; then
    return $rc
  fi

  # Copy config files so that eSA knows how to handle machine type 2231, which is unique for IA
  #ras_log -v "Copying new serviceProviderIBM.xml"
  #execute_and_log cp /opt/tivoli/tiam/etc/serviceProviderIBM.xml /opt/ibm/director/lwi/runtime/USMiMain/eclipse/plugins/com.ibm.usmi.services.updates.acquisition.eccbase_1.0.0/ecc/config
  #/opt/ibm/director/lwi/runtime/director/eclipse/plugins/com.ibm.director.mgr.updates.server_6.2.0/ecc/config
  #rc=$?

  #if [[ $rc -ne 0 ]]; then
  #  return $rc
  #fi

  # Cleanup by removing the install script
  rm -Rf ${ESA_UNPACK_DIR}

  ras_log -v "Service and Support Manager installation completed successfully"
  return 0
}


#---
# This function configures the Director Server's Agent Manager
#---
function configure_agent_manager() {

  local rc=0

  ras_log -v "Configuring Agent Manager"
  execute_and_log /opt/ibm/director/bin/configAgtMgr.sh -user iadir -agtmgrpass iadir -regpass iadir
  rc=$?

  return $rc
}

usage() {
  echo " dirinstall.sh [-OPTION]"
  echo "      OPTIONS:"
  echo "       -d        Install Director and Service and Support Manager"
  echo "       -e        Only install Service and Support Manager (ESA) plugin"
  echo "       -h        help/usage information"
  echo " "
  echo "    eg.   dirinstall.sh -d"
  return 0
}

#------------------------------------------------------------------------------
# Main script
#------------------------------------------------------------------------------
DIR_FLAG=0
ESA_FLAG=0

while getopts "deh" optchar ; do case $optchar in
   d) DIR_FLAG=1
      ;;
   e) ESA_FLAG=1
      ;;
   h) usage
      exit 1
      ;;
   ?) echo "Error: "
      usage
      exit 1
      ;;
   esac
done

# Set default to uninstall everything when NO arguments are passed
if [ $# = 0 ]; then
  DIR_FLAG=1
fi
# Check for invalid arguments
if [[ ($DIR_FLAG -eq 0) && ($ESA_FLAG -eq 0) ]]; then
    echo "Error: "
    usage
    exit 1
fi

# create new log file with the basename of "dirinstall".
create_new_log_file dirinstall

# disable MCP slpd. Director installation complains that this version is broken
# and Director will not install if it detects MCP slpd enabled.
execute_and_log /etc/init.d/slpd stop
execute_and_log chkconfig slpd --del

# At inital install time the Director image will be in the "RPMS" directory,
# but during upgrade it will be in /tmp/director/upgrade
if [ -f RPMS/${DIR_SERVER_INSTALL_TGZ} ]; then
    PACKAGE_DIR=RPMS
elif [ -f /tmp/director/upgrade/${DIR_SERVER_INSTALL_TGZ} ]; then
    PACKAGE_DIR=/tmp/director/upgrade
fi


# Do the Full Director install
if [ $DIR_FLAG -eq 1 ]; then
    # We need to install the libcom_err-32bit package for Director 6.2 if it isn't already installed
    if [[ `rpm -qa | grep libcom_err | grep 32bit | wc -l` == 0 ]]; then
      ras_log -v "Installing package libcom_err32bit."
      rpm -Uvh ${PACKAGE_DIR}/libcom_err*.rpm
    else
      ras_log -v "Package libcom_err32bit is already installed."
    fi

    # We need to install the e2fsprogs-32bit package for Director 6.2 if it isn't already installed
    if [[ `rpm -qa | grep e2fsprogs | grep 32bit | wc -l` == 0 ]]; then
      ras_log -v "Installing package e2fsprogs32bit."
      rpm -Uvh ${PACKAGE_DIR}/e2fsprogs*.rpm
    else
      ras_log -v "Package e2fsprogs32bit is already installed."
    fi


    #  Install the dummy release file
    if [[ `rpm -qa | grep sles-release | wc -l` == 0 ]]; then
      ras_log -v "Installing package sles-release."
      execute_and_log rpm -Uvh ${PACKAGE_DIR}/${RELEASE_RPM}
      if [[ $? -ne 0 ]]; then
        ras_log -v "Release file install failed. Exiting ..."
        exit 1
      fi
    else
      ras_log -v "Package sles-release is already installed."
    fi

    install_director_server
    rc=$?

    if [[ $rc -ne 0 ]]; then
      ras_log -v "Director Server installation failed. Exiting ..."
      exit 1
    fi

    install_esa
    if [[ $? -ne 0 ]]; then
      ras_log -v "ESA installation failed. Exiting ..."
      exit 1
    fi

    configure_agent_manager
    if [[ $? -ne 0 ]]; then
      ras_log -v "Agent Manager configuration failed. Exiting ..."
      exit 1
    fi

    # Remove the dummy release file
    execute_and_log rpm -ev --nodeps sles-release
    if [[ $? -ne 0 ]]; then
      ras_log -v "Release file removal failed."
    fi

elif [ $ESA_FLAG -eq 1 ]; then
    install_esa
    if [[ $? -ne 0 ]]; then
      ras_log -v "ESA installation failed. Exiting ..."
      exit 1
    fi
fi

#ras_log -v "Restoring Director data."
if [ -d $TMP_RAS/esa/config ]; then
    ras_log -v "Restoring Director 6.1 esa data from $TMP_RAS/esa to $DIR_INSTALL_ROOT/data/."
    mkdir -p $DIR_INSTALL_ROOT/data/esa
    cp -rf $TMP_RAS/esa $DIR_INSTALL_ROOT/data
    rc=$?
    rm -rf $TMP_RAS/esa
elif [ -d $TMP_RAS/backup ]; then
    ras_log -v "Restoring Director 6.2 data from $TMP_RAS/backup/."
    restore_Director_data
    rc=$?
else
    # data wasn't stored before uninstall for some reason, set bad rc
    ras_log -v "Director esa save data doesn't exist."
    rc=1
fi

if [[ rc -ne 0 ]]; then
    ras_log -v "Director restore data failed. Customer call home data may need to be reconfigured."
fi


# IBM Director Server gets started during the iaras-ds RPM install
ras_log -v "For complete installation log see ${LOGFILE}"
exit 0
