#!/usr/bin/env python
#/****************************************************************************
# * COMPONENT NAME:  IA Log Manager
# *
# * FILE NAME:       el_sanitize_raw_log.py
# *
# * DESCRIPTION:     Process a raw LAS Audit log, 
# *                  taking care of interleaved events.
# *
# * AUTHOR:          Lucas C. Villa Real
# *                  IBM Almaden Research Center
# *                  lucasvr@us.ibm.com
# *
# *                  Liming (Michael) Yu
# *                  IBM Tivoli
# *                  yul@us.ibm.com
# *
# * IBM CONFIDENTIAL -- (IBM Confidential Restricted when
# * combined with the aggregated modules for this product)
# * OBJECT CODE ONLY SOURCE MATERIALS
# *
# * (C) COPYRIGHT International Business Machines Corp. 2008
# * All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# ****************************************************************************
# */

IAbasepath     = "/opt/tivoli/tiam"
IAbinpath      = IAbasepath + "/bin"
IAlibpath      = IAbasepath + "/lib"
IAlogtracepath = "/tiam_utility/log"

import os
import shutil
import getopt
import tempfile
import traceback
import sys
import time
import commands
import glob
import xml
import xml.dom.minidom
import xml.xpath

if IAlibpath not in sys.path:
	sys.path.append(IAlibpath)

import el_msgs
import iamiddleware.ialogging

#
# Global variables
#
logger = None

#
# x86_64 system call numbers.
# Automatically generated from asm-x86_64/unistd.h
#
syscalls_x86_64 = {
	0: "read",
	1: "write",
	2: "open",
	3: "close",
	4: "stat",
	5: "fstat",
	6: "lstat",
	7: "poll",
	8: "lseek",
	9: "mmap",
	10: "mprotect",
	11: "munmap",
	12: "brk",
	13: "rt_sigaction",
	14: "rt_sigprocmask",
	15: "rt_sigreturn",
	16: "ioctl",
	17: "pread64",
	18: "pwrite64",
	19: "readv",
	20: "writev",
	21: "access",
	22: "pipe",
	23: "select",
	24: "sched_yield",
	25: "mremap",
	26: "msync",
	27: "mincore",
	28: "madvise",
	29: "shmget",
	30: "shmat",
	31: "shmctl",
	32: "dup",
	33: "dup2",
	34: "pause",
	35: "nanosleep",
	36: "getitimer",
	37: "alarm",
	38: "setitimer",
	39: "getpid",
	40: "sendfile",
	41: "socket",
	42: "connect",
	43: "accept",
	44: "sendto",
	45: "recvfrom",
	46: "sendmsg",
	47: "recvmsg",
	48: "shutdown",
	49: "bind",
	50: "listen",
	51: "getsockname",
	52: "getpeername",
	53: "socketpair",
	54: "setsockopt",
	55: "getsockopt",
	56: "clone",
	57: "fork",
	58: "vfork",
	59: "execve",
	60: "exit",
	61: "wait4",
	62: "kill",
	63: "uname",
	64: "semget",
	65: "semop",
	66: "semctl",
	67: "shmdt",
	68: "msgget",
	69: "msgsnd",
	70: "msgrcv",
	71: "msgctl",
	72: "fcntl",
	73: "flock",
	74: "fsync",
	75: "fdatasync",
	76: "truncate",
	77: "ftruncate",
	78: "getdents",
	79: "getcwd",
	80: "chdir",
	81: "fchdir",
	82: "rename",
	83: "mkdir",
	84: "rmdir",
	85: "creat",
	86: "link",
	87: "unlink",
	88: "symlink",
	89: "readlink",
	90: "chmod",
	91: "fchmod",
	92: "chown",
	93: "fchown",
	94: "lchown",
	95: "umask",
	96: "gettimeofday",
	97: "getrlimit",
	98: "getrusage",
	99: "sysinfo",
	100: "times",
	101: "ptrace",
	102: "getuid",
	103: "syslog",
	104: "getgid",
	105: "setuid",
	106: "setgid",
	107: "geteuid",
	108: "getegid",
	109: "setpgid",
	110: "getppid",
	111: "getpgrp",
	112: "setsid",
	113: "setreuid",
	114: "setregid",
	115: "getgroups",
	116: "setgroups",
	117: "setresuid",
	118: "getresuid",
	119: "setresgid",
	120: "getresgid",
	121: "getpgid",
	122: "setfsuid",
	123: "setfsgid",
	124: "getsid",
	125: "capget",
	126: "capset",
	127: "rt_sigpending",
	128: "rt_sigtimedwait",
	129: "rt_sigqueueinfo",
	130: "rt_sigsuspend",
	131: "sigaltstack",
	132: "utime",
	133: "mknod",
	134: "uselib",
	135: "personality",
	136: "ustat",
	137: "statfs",
	138: "fstatfs",
	139: "sysfs",
	140: "getpriority",
	141: "setpriority",
	142: "sched_setparam",
	143: "sched_getparam",
	144: "sched_setscheduler",
	145: "sched_getscheduler",
	146: "sched_get_priority_max",
	147: "sched_get_priority_min",
	148: "sched_rr_get_interval",
	149: "mlock",
	150: "munlock",
	151: "mlockall",
	152: "munlockall",
	153: "vhangup",
	154: "modify_ldt",
	155: "pivot_root",
	156: "_sysctl",
	157: "prctl",
	158: "arch_prctl",
	159: "adjtimex",
	160: "setrlimit",
	161: "chroot",
	162: "sync",
	163: "acct",
	164: "settimeofday",
	165: "mount",
	166: "umount2",
	167: "swapon",
	168: "swapoff",
	169: "reboot",
	170: "sethostname",
	171: "setdomainname",
	172: "iopl",
	173: "ioperm",
	174: "create_module",
	175: "init_module",
	176: "delete_module",
	177: "get_kernel_syms",
	178: "query_module",
	179: "quotactl",
	180: "nfsservctl",
	181: "getpmsg",
	182: "putpmsg",
	183: "afs_syscall",
	184: "tuxcall",
	185: "security",
	186: "gettid",
	187: "readahead",
	188: "setxattr",
	189: "lsetxattr",
	190: "fsetxattr",
	191: "getxattr",
	192: "lgetxattr",
	193: "fgetxattr",
	194: "listxattr",
	195: "llistxattr",
	196: "flistxattr",
	197: "removexattr",
	198: "lremovexattr",
	199: "fremovexattr",
	200: "tkill",
	201: "time",
	202: "futex",
	203: "sched_setaffinity",
	204: "sched_getaffinity",
	205: "set_thread_area",
	206: "io_setup",
	207: "io_destroy",
	208: "io_getevents",
	209: "io_submit",
	210: "io_cancel",
	211: "get_thread_area",
	212: "lookup_dcookie",
	213: "epoll_create",
	214: "epoll_ctl_old",
	215: "epoll_wait_old",
	216: "remap_file_pages",
	217: "getdents64",
	218: "set_tid_address",
	219: "restart_syscall",
	220: "semtimedop",
	221: "fadvise64",
	222: "timer_create",
	223: "timer_settime",
	224: "timer_gettime",
	225: "timer_getoverrun",
	226: "timer_delete",
	227: "clock_settime",
	228: "clock_gettime",
	229: "clock_getres",
	230: "clock_nanosleep",
	231: "exit_group",
	232: "epoll_wait",
	233: "epoll_ctl",
	234: "tgkill",
	235: "utimes",
	236: "vserver",
	237: "mbind",
	238: "set_mempolicy",
	239: "get_mempolicy",
	240: "mq_open",
	241: "mq_unlink",
	242: "mq_timedsend",
	243: "mq_timedreceive",
	244: "mq_notify",
	245: "mq_getsetattr",
	246: "kexec_load",
	247: "waitid",
	248: "add_key",
	249: "request_key",
	250: "keyctl",
	251: "ioprio_set",
	252: "ioprio_get",
	253: "inotify_init",
	254: "inotify_add_watch",
	255: "inotify_rm_watch",
	256: "migrate_pages",
	257: "openat",
	258: "mkdirat",
	259: "mknodat",
	260: "fchownat",
	261: "futimesat",
	262: "newfstatat",
	263: "unlinkat",
	264: "renameat",
	265: "linkat",
	266: "symlinkat",
	267: "readlinkat",
	268: "fchmodat",
	269: "faccessat",
	270: "pselect6",
	271: "ppoll",
	272: "unshare"
}

#
# x86_64 32-bit compatibility system call numbers.
# Automatically generated from asm-x86_64/ia32_unistd.h
#
syscalls_compat32 = {
	0: "restart_syscall",
	1: "exit",
	2: "fork",
	3: "read",
	4: "write",
	5: "open",
	6: "close",
	7: "waitpid",
	8: "creat",
	9: "link",
	10: "unlink",
	11: "execve",
	12: "chdir",
	13: "time",
	14: "mknod",
	15: "chmod",
	16: "lchown",
	17: "break",
	18: "oldstat",
	19: "lseek",
	20: "getpid",
	21: "mount",
	22: "umount",
	23: "setuid",
	24: "getuid",
	25: "stime",
	26: "ptrace",
	27: "alarm",
	28: "oldfstat",
	29: "pause",
	30: "utime",
	31: "stty",
	32: "gtty",
	33: "access",
	34: "nice",
	35: "ftime",
	36: "sync",
	37: "kill",
	38: "rename",
	39: "mkdir",
	40: "rmdir",
	41: "dup",
	42: "pipe",
	43: "times",
	44: "prof",
	45: "brk",
	46: "setgid",
	47: "getgid",
	48: "signal",
	49: "geteuid",
	50: "getegid",
	51: "acct",
	52: "umount2",
	53: "lock",
	54: "ioctl",
	55: "fcntl",
	56: "mpx",
	57: "setpgid",
	58: "ulimit",
	59: "oldolduname",
	60: "umask",
	61: "chroot",
	62: "ustat",
	63: "dup2",
	64: "getppid",
	65: "getpgrp",
	66: "setsid",
	67: "sigaction",
	68: "sgetmask",
	69: "ssetmask",
	70: "setreuid",
	71: "setregid",
	72: "sigsuspend",
	73: "sigpending",
	74: "sethostname",
	75: "setrlimit",
	76: "getrlimit",
	77: "getrusage",
	78: "gettimeofday",
	79: "settimeofday",
	80: "getgroups",
	81: "setgroups",
	82: "select",
	83: "symlink",
	84: "oldlstat",
	85: "readlink",
	86: "uselib",
	87: "swapon",
	88: "reboot",
	89: "readdir",
	90: "mmap",
	91: "munmap",
	92: "truncate",
	93: "ftruncate",
	94: "fchmod",
	95: "fchown",
	96: "getpriority",
	97: "setpriority",
	98: "profil",
	99: "statfs",
	100: "fstatfs",
	101: "ioperm",
	102: "socketcall",
	103: "syslog",
	104: "setitimer",
	105: "getitimer",
	106: "stat",
	107: "lstat",
	108: "fstat",
	109: "olduname",
	110: "iopl",
	111: "vhangup",
	112: "idle",
	113: "vm86old",
	114: "wait4",
	115: "swapoff",
	116: "sysinfo",
	117: "ipc",
	118: "fsync",
	119: "sigreturn",
	120: "clone",
	121: "setdomainname",
	122: "uname",
	123: "modify_ldt",
	124: "adjtimex",
	125: "mprotect",
	126: "sigprocmask",
	127: "create_module",
	128: "init_module",
	129: "delete_module",
	130: "get_kernel_syms",
	131: "quotactl",
	132: "getpgid",
	133: "fchdir",
	134: "bdflush",
	135: "sysfs",
	136: "personality",
	137: "afs_syscall",
	138: "setfsuid",
	139: "setfsgid",
	140: "_llseek",
	141: "getdents",
	142: "_newselect",
	143: "flock",
	144: "msync",
	145: "readv",
	146: "writev",
	147: "getsid",
	148: "fdatasync",
	149: "_sysctl",
	150: "mlock",
	151: "munlock",
	152: "mlockall",
	153: "munlockall",
	154: "sched_setparam",
	155: "sched_getparam",
	156: "sched_setscheduler",
	157: "sched_getscheduler",
	158: "sched_yield",
	159: "sched_get_priority_max",
	160: "sched_get_priority_min",
	161: "sched_rr_get_interval",
	162: "nanosleep",
	163: "mremap",
	164: "setresuid",
	165: "getresuid",
	166: "vm86",
	167: "query_module",
	168: "poll",
	169: "nfsservctl",
	170: "setresgid",
	171: "getresgid",
	172: "prctl",
	173: "rt_sigreturn",
	174: "rt_sigaction",
	175: "rt_sigprocmask",
	176: "rt_sigpending",
	177: "rt_sigtimedwait",
	178: "rt_sigqueueinfo",
	179: "rt_sigsuspend",
	180: "pread",
	181: "pwrite",
	182: "chown",
	183: "getcwd",
	184: "capget",
	185: "capset",
	186: "sigaltstack",
	187: "sendfile",
	188: "getpmsg",
	189: "putpmsg",
	190: "vfork",
	191: "ugetrlimit",
	192: "mmap2",
	193: "truncate64",
	194: "ftruncate64",
	195: "stat64",
	196: "lstat64",
	197: "fstat64",
	198: "lchown32",
	199: "getuid32",
	200: "getgid32",
	201: "geteuid32",
	202: "getegid32",
	203: "setreuid32",
	204: "setregid32",
	205: "getgroups32",
	206: "setgroups32",
	207: "fchown32",
	208: "setresuid32",
	209: "getresuid32",
	210: "setresgid32",
	211: "getresgid32",
	212: "chown32",
	213: "setuid32",
	214: "setgid32",
	215: "setfsuid32",
	216: "setfsgid32",
	217: "pivot_root",
	218: "mincore",
	219: "madvise",
	219: "madvise1",
	220: "getdents64",
	221: "fcntl64",
	222: "tuxcall",
	223: "security",
	224: "gettid",
	225: "readahead",
	226: "setxattr",
	227: "lsetxattr",
	228: "fsetxattr",
	229: "getxattr",
	230: "lgetxattr",
	231: "fgetxattr",
	232: "listxattr",
	233: "llistxattr",
	234: "flistxattr",
	235: "removexattr",
	236: "lremovexattr",
	237: "fremovexattr",
	238: "tkill",
	239: "sendfile64",
	240: "futex",
	241: "sched_setaffinity",
	242: "sched_getaffinity",
	243: "set_thread_area",
	244: "get_thread_area",
	245: "io_setup",
	246: "io_destroy",
	247: "io_getevents",
	248: "io_submit",
	249: "io_cancel",
	252: "exit_group",
	253: "lookup_dcookie",
	254: "sys_epoll_create",
	255: "sys_epoll_ctl",
	256: "sys_epoll_wait",
	257: "remap_file_pages",
	258: "set_tid_address",
	259: "timer_create",
	260: "timer_settime",
	261: "timer_gettime",
	262: "timer_getoverrun",
	263: "timer_delete",
	264: "clock_settime",
	265: "clock_gettime",
	266: "clock_getres",
	267: "clock_nanosleep",
	268: "statfs64",
	269: "fstatfs64",
	270: "tgkill",
	271: "utimes",
	272: "fadvise64_64",
	273: "vserver",
	274: "mbind",
	275: "get_mempolicy",
	276: "set_mempolicy",
	277: "mq_open",
	278: "mq_unlink",
	279: "mq_timedsend",
	280: "mq_timedreceive",
	281: "mq_notify",
	282: "mq_getsetattr",
	283: "kexec",
	284: "waitid",
	286: "add_key",
	287: "request_key",
	288: "keyctl",
	289: "ioprio_set",
	290: "ioprio_get",
	291: "inotify_init",
	292: "inotify_add_watch",
	293: "inotify_rm_watch",
	294: "migrate_pages",
	295: "openat",
	296: "mkdirat",
	297: "mknodat",
	298: "fchownat",
	299: "futimesat",
	300: "fstatat64",
	301: "unlinkat",
	302: "renameat",
	303: "linkat",
	304: "symlinkat",
	305: "readlinkat",
	306: "fchmodat",
	307: "faccessat",
	308: "pselect6",
	309: "ppoll",
	310: "unshare",
}

#
# Expected minimum number of events for each system call.
# Those doesn't count in the PATH events, which are extracted from the
# SYSCALL event's items= parameter.
#
minimum_events_list = {
	#
	# x86_64 system calls
	#
	"read":                   { "vfs": 2, "exit": 2 },
	"write":                  { "vfs": 2, "exit": 2 },
	"open":                   { "vfs": 2, "exit": 2 }, # SYSCALL+CWD, SYSCALL+CWD
	"close":                  { "vfs": 2, "exit": 2 },
	"stat":                   { "vfs": 2, "exit": 2 },
	"fstat":                  { "vfs": 2, "exit": 2 },
	"lstat":                  { "vfs": 2, "exit": 2 },
	"poll":                   { "vfs": 2, "exit": 2 },
	"lseek":                  { "vfs": 2, "exit": 2 },
	"mmap":                   { "vfs": 2, "exit": 2 },
	"mprotect":               { "vfs": 2, "exit": 2 },
	"munmap":                 { "vfs": 2, "exit": 2 },
	"brk":                    { "vfs": 2, "exit": 2 },
	"rt_sigaction":           { "vfs": 2, "exit": 2 },
	"rt_sigprocmask":         { "vfs": 2, "exit": 2 },
	"rt_sigreturn":           { "vfs": 2, "exit": 2 },
	"ioctl":                  { "vfs": 2, "exit": 2 },
	"pread64":                { "vfs": 2, "exit": 2 },
	"pwrite64":               { "vfs": 2, "exit": 2 },
	"readv":                  { "vfs": 2, "exit": 2 },
	"writev":                 { "vfs": 2, "exit": 2 },
	"access":                 { "vfs": 2, "exit": 2 },
	"pipe":                   { "vfs": 2, "exit": 2 },
	"select":                 { "vfs": 2, "exit": 2 },
	"sched_yield":            { "vfs": 2, "exit": 2 },
	"mremap":                 { "vfs": 2, "exit": 2 },
	"msync":                  { "vfs": 2, "exit": 2 },
	"mincore":                { "vfs": 2, "exit": 2 },
	"madvise":                { "vfs": 2, "exit": 2 },
	"shmget":                 { "vfs": 2, "exit": 2 },
	"shmat":                  { "vfs": 2, "exit": 2 },
	"shmctl":                 { "vfs": 2, "exit": 2 },
	"dup":                    { "vfs": 2, "exit": 2 },
	"dup2":                   { "vfs": 2, "exit": 2 },
	"pause":                  { "vfs": 2, "exit": 2 },
	"nanosleep":              { "vfs": 2, "exit": 2 },
	"getitimer":              { "vfs": 2, "exit": 2 },
	"alarm":                  { "vfs": 2, "exit": 2 },
	"setitimer":              { "vfs": 2, "exit": 2 },
	"getpid":                 { "vfs": 2, "exit": 2 },
	"sendfile":               { "vfs": 2, "exit": 2 },
	"socket":                 { "vfs": 2, "exit": 2 },
	"connect":                { "vfs": 2, "exit": 2 },
	"accept":                 { "vfs": 2, "exit": 2 },
	"sendto":                 { "vfs": 2, "exit": 2 },
	"recvfrom":               { "vfs": 2, "exit": 2 },
	"sendmsg":                { "vfs": 2, "exit": 2 },
	"recvmsg":                { "vfs": 2, "exit": 2 },
	"shutdown":               { "vfs": 2, "exit": 2 },
	"bind":                   { "vfs": 2, "exit": 2 },
	"listen":                 { "vfs": 2, "exit": 2 },
	"getsockname":            { "vfs": 2, "exit": 2 },
	"getpeername":            { "vfs": 2, "exit": 2 },
	"socketpair":             { "vfs": 2, "exit": 2 },
	"setsockopt":             { "vfs": 2, "exit": 2 },
	"getsockopt":             { "vfs": 2, "exit": 2 },
	"clone":                  { "vfs": 2, "exit": 2 },
	"fork":                   { "vfs": 2, "exit": 2 },
	"vfork":                  { "vfs": 2, "exit": 2 },
	"execve":                 { "vfs": 2, "exit": 2 },
	"exit":                   { "vfs": 2, "exit": 2 },
	"wait4":                  { "vfs": 2, "exit": 2 },
	"kill":                   { "vfs": 2, "exit": 2 },
	"uname":                  { "vfs": 2, "exit": 2 },
	"semget":                 { "vfs": 2, "exit": 2 },
	"semop":                  { "vfs": 2, "exit": 2 },
	"semctl":                 { "vfs": 2, "exit": 2 },
	"shmdt":                  { "vfs": 2, "exit": 2 },
	"msgget":                 { "vfs": 2, "exit": 2 },
	"msgsnd":                 { "vfs": 2, "exit": 2 },
	"msgrcv":                 { "vfs": 2, "exit": 2 },
	"msgctl":                 { "vfs": 2, "exit": 2 },
	"fcntl":                  { "vfs": 2, "exit": 2 }, # SYSCALL+CWD, SYSCALL+CWD
	"flock":                  { "vfs": 2, "exit": 2 },
	"fsync":                  { "vfs": 2, "exit": 2 },
	"fdatasync":              { "vfs": 2, "exit": 2 },
	"truncate":               { "vfs": 2, "exit": 2 }, # SYSCALL+CWD, SYSCALL+CWD
	"ftruncate":              { "vfs": 2, "exit": 2 }, # SYSCALL+CWD, SYSCALL+CWD
	"getdents":               { "vfs": 2, "exit": 2 },
	"getcwd":                 { "vfs": 2, "exit": 2 },
	"chdir":                  { "vfs": 2, "exit": 2 },
	"fchdir":                 { "vfs": 2, "exit": 2 },
	"rename":                 { "vfs": 4, "exit": 2 }, # 2x(SYSCALL+CWD), SYSCALL+CWD
	"mkdir":                  { "vfs": 2, "exit": 2 },
	"rmdir":                  { "vfs": 2, "exit": 2 },
	"creat":                  { "vfs": 2, "exit": 2 }, # SYSCALL+CWD, SYSCALL+CWD
	"link":                   { "vfs": 2, "exit": 2 },
	"unlink":                 { "vfs": 2, "exit": 2 }, # SYSCALL+CWD, SYSCALL+CWD
	"symlink":                { "vfs": 2, "exit": 2 },
	"readlink":               { "vfs": 2, "exit": 2 },
	"chmod":                  { "vfs": 2, "exit": 2 }, # SYSCALL+CWD, SYSCALL+CWD
	"fchmod":                 { "vfs": 2, "exit": 2 }, # SYSCALL+CWD, SYSCALL+CWD
	"chown":                  { "vfs": 2, "exit": 2 },
	"fchown":                 { "vfs": 2, "exit": 2 },
	"lchown":                 { "vfs": 2, "exit": 2 },
	"umask":                  { "vfs": 2, "exit": 2 },
	"gettimeofday":           { "vfs": 2, "exit": 2 },
	"getrlimit":              { "vfs": 2, "exit": 2 },
	"getrusage":              { "vfs": 2, "exit": 2 },
	"sysinfo":                { "vfs": 2, "exit": 2 },
	"times":                  { "vfs": 2, "exit": 2 },
	"ptrace":                 { "vfs": 2, "exit": 2 },
	"getuid":                 { "vfs": 2, "exit": 2 },
	"syslog":                 { "vfs": 2, "exit": 2 },
	"getgid":                 { "vfs": 2, "exit": 2 },
	"setuid":                 { "vfs": 2, "exit": 2 },
	"setgid":                 { "vfs": 2, "exit": 2 },
	"geteuid":                { "vfs": 2, "exit": 2 },
	"getegid":                { "vfs": 2, "exit": 2 },
	"setpgid":                { "vfs": 2, "exit": 2 },
	"getppid":                { "vfs": 2, "exit": 2 },
	"getpgrp":                { "vfs": 2, "exit": 2 },
	"setsid":                 { "vfs": 2, "exit": 2 },
	"setreuid":               { "vfs": 2, "exit": 2 },
	"setregid":               { "vfs": 2, "exit": 2 },
	"getgroups":              { "vfs": 2, "exit": 2 },
	"setgroups":              { "vfs": 2, "exit": 2 },
	"setresuid":              { "vfs": 2, "exit": 2 },
	"getresuid":              { "vfs": 2, "exit": 2 },
	"setresgid":              { "vfs": 2, "exit": 2 },
	"getresgid":              { "vfs": 2, "exit": 2 },
	"getpgid":                { "vfs": 2, "exit": 2 },
	"setfsuid":               { "vfs": 2, "exit": 2 },
	"setfsgid":               { "vfs": 2, "exit": 2 },
	"getsid":                 { "vfs": 2, "exit": 2 },
	"capget":                 { "vfs": 2, "exit": 2 },
	"capset":                 { "vfs": 2, "exit": 2 },
	"rt_sigpending":          { "vfs": 2, "exit": 2 },
	"rt_sigtimedwait":        { "vfs": 2, "exit": 2 },
	"rt_sigqueueinfo":        { "vfs": 2, "exit": 2 },
	"rt_sigsuspend":          { "vfs": 2, "exit": 2 },
	"sigaltstack":            { "vfs": 2, "exit": 2 },
	"utime":                  { "vfs": 2, "exit": 2 },
	"mknod":                  { "vfs": 2, "exit": 2 },
	"uselib":                 { "vfs": 2, "exit": 2 },
	"personality":            { "vfs": 2, "exit": 2 },
	"ustat":                  { "vfs": 2, "exit": 2 },
	"statfs":                 { "vfs": 2, "exit": 2 },
	"fstatfs":                { "vfs": 2, "exit": 2 },
	"sysfs":                  { "vfs": 2, "exit": 2 },
	"getpriority":            { "vfs": 2, "exit": 2 },
	"setpriority":            { "vfs": 2, "exit": 2 },
	"sched_setparam":         { "vfs": 2, "exit": 2 },
	"sched_getparam":         { "vfs": 2, "exit": 2 },
	"sched_setscheduler":     { "vfs": 2, "exit": 2 },
	"sched_getscheduler":     { "vfs": 2, "exit": 2 },
	"sched_get_priority_max": { "vfs": 2, "exit": 2 },
	"sched_get_priority_min": { "vfs": 2, "exit": 2 },
	"sched_rr_get_interval":  { "vfs": 2, "exit": 2 },
	"mlock":                  { "vfs": 2, "exit": 2 },
	"munlock":                { "vfs": 2, "exit": 2 },
	"mlockall":               { "vfs": 2, "exit": 2 },
	"munlockall":             { "vfs": 2, "exit": 2 },
	"vhangup":                { "vfs": 2, "exit": 2 },
	"modify_ldt":             { "vfs": 2, "exit": 2 },
	"pivot_root":             { "vfs": 2, "exit": 2 },
	"_sysctl":                { "vfs": 2, "exit": 2 },
	"prctl":                  { "vfs": 2, "exit": 2 },
	"arch_prctl":             { "vfs": 2, "exit": 2 },
	"adjtimex":               { "vfs": 0, "exit": 1 }, # SYSCALL
	"setrlimit":              { "vfs": 2, "exit": 2 },
	"chroot":                 { "vfs": 2, "exit": 2 },
	"sync":                   { "vfs": 2, "exit": 2 },
	"acct":                   { "vfs": 2, "exit": 2 },
	"settimeofday":           { "vfs": 0, "exit": 1 },
	"mount":                  { "vfs": 2, "exit": 2 },
	"umount2":                { "vfs": 2, "exit": 2 },
	"swapon":                 { "vfs": 2, "exit": 2 },
	"swapoff":                { "vfs": 2, "exit": 2 },
	"reboot":                 { "vfs": 2, "exit": 2 },
	"sethostname":            { "vfs": 2, "exit": 2 },
	"setdomainname":          { "vfs": 2, "exit": 2 },
	"iopl":                   { "vfs": 2, "exit": 2 },
	"ioperm":                 { "vfs": 2, "exit": 2 },
	"create_module":          { "vfs": 2, "exit": 2 },
	"init_module":            { "vfs": 2, "exit": 2 },
	"delete_module":          { "vfs": 2, "exit": 2 },
	"get_kernel_syms":        { "vfs": 2, "exit": 2 },
	"query_module":           { "vfs": 2, "exit": 2 },
	"quotactl":               { "vfs": 2, "exit": 2 },
	"nfsservctl":             { "vfs": 2, "exit": 2 },
	"getpmsg":                { "vfs": 2, "exit": 2 },
	"putpmsg":                { "vfs": 2, "exit": 2 },
	"afs_syscall":            { "vfs": 2, "exit": 2 },
	"tuxcall":                { "vfs": 2, "exit": 2 },
	"security":               { "vfs": 2, "exit": 2 },
	"gettid":                 { "vfs": 2, "exit": 2 },
	"readahead":              { "vfs": 2, "exit": 2 },
	"setxattr":               { "vfs": 2, "exit": 2 }, # SYSCALL+CWD, SYSCALL+CWD
	"lsetxattr":              { "vfs": 2, "exit": 2 }, # SYSCALL+CWD, SYSCALL+CWD
	"fsetxattr":              { "vfs": 2, "exit": 2 }, # SYSCALL+CWD, SYSCALL+CWD
	"getxattr":               { "vfs": 2, "exit": 2 },
	"lgetxattr":              { "vfs": 2, "exit": 2 },
	"fgetxattr":              { "vfs": 2, "exit": 2 },
	"listxattr":              { "vfs": 2, "exit": 2 },
	"llistxattr":             { "vfs": 2, "exit": 2 },
	"flistxattr":             { "vfs": 2, "exit": 2 },
	"removexattr":            { "vfs": 2, "exit": 2 },
	"lremovexattr":           { "vfs": 2, "exit": 2 },
	"fremovexattr":           { "vfs": 2, "exit": 2 },
	"tkill":                  { "vfs": 2, "exit": 2 },
	"time":                   { "vfs": 2, "exit": 2 },
	"futex":                  { "vfs": 2, "exit": 2 },
	"sched_setaffinity":      { "vfs": 2, "exit": 2 },
	"sched_getaffinity":      { "vfs": 2, "exit": 2 },
	"set_thread_area":        { "vfs": 2, "exit": 2 },
	"io_setup":               { "vfs": 2, "exit": 2 },
	"io_destroy":             { "vfs": 2, "exit": 2 },
	"io_getevents":           { "vfs": 2, "exit": 2 },
	"io_submit":              { "vfs": 2, "exit": 2 },
	"io_cancel":              { "vfs": 2, "exit": 2 },
	"get_thread_area":        { "vfs": 2, "exit": 2 },
	"lookup_dcookie":         { "vfs": 2, "exit": 2 },
	"epoll_create":           { "vfs": 2, "exit": 2 },
	"epoll_ctl_old":          { "vfs": 2, "exit": 2 },
	"epoll_wait_old":         { "vfs": 2, "exit": 2 },
	"remap_file_pages":       { "vfs": 2, "exit": 2 },
	"getdents64":             { "vfs": 2, "exit": 2 },
	"set_tid_address":        { "vfs": 2, "exit": 2 },
	"restart_syscall":        { "vfs": 2, "exit": 2 },
	"semtimedop":             { "vfs": 2, "exit": 2 },
	"fadvise64":              { "vfs": 2, "exit": 2 },
	"timer_create":           { "vfs": 2, "exit": 2 },
	"timer_settime":          { "vfs": 0, "exit": 1 },
	"timer_gettime":          { "vfs": 2, "exit": 2 },
	"timer_getoverrun":       { "vfs": 2, "exit": 2 },
	"timer_delete":           { "vfs": 2, "exit": 2 },
	"clock_settime":          { "vfs": 0, "exit": 1 }, # SYSCALL
	"clock_gettime":          { "vfs": 2, "exit": 2 },
	"clock_getres":           { "vfs": 2, "exit": 2 },
	"clock_nanosleep":        { "vfs": 2, "exit": 2 },
	"exit_group":             { "vfs": 2, "exit": 2 },
	"epoll_wait":             { "vfs": 2, "exit": 2 },
	"epoll_ctl":              { "vfs": 2, "exit": 2 },
	"tgkill":                 { "vfs": 2, "exit": 2 },
	"utimes":                 { "vfs": 2, "exit": 2 },
	"vserver":                { "vfs": 2, "exit": 2 },
	"mbind":                  { "vfs": 2, "exit": 2 },
	"set_mempolicy":          { "vfs": 2, "exit": 2 },
	"get_mempolicy":          { "vfs": 2, "exit": 2 },
	"mq_open":                { "vfs": 2, "exit": 2 },
	"mq_unlink":              { "vfs": 2, "exit": 2 },
	"mq_timedsend":           { "vfs": 2, "exit": 2 },
	"mq_timedreceive":        { "vfs": 2, "exit": 2 },
	"mq_notify":              { "vfs": 2, "exit": 2 },
	"mq_getsetattr":          { "vfs": 2, "exit": 2 },
	"kexec_load":             { "vfs": 2, "exit": 2 },
	"waitid":                 { "vfs": 2, "exit": 2 },
	"add_key":                { "vfs": 2, "exit": 2 },
	"request_key":            { "vfs": 2, "exit": 2 },
	"keyctl":                 { "vfs": 2, "exit": 2 },
	"ioprio_set":             { "vfs": 2, "exit": 2 },
	"ioprio_get":             { "vfs": 2, "exit": 2 },
	"inotify_init":           { "vfs": 2, "exit": 2 },
	"inotify_add_watch":      { "vfs": 2, "exit": 2 },
	"inotify_rm_watch":       { "vfs": 2, "exit": 2 },
	"migrate_pages":          { "vfs": 2, "exit": 2 },
	"openat":                 { "vfs": 2, "exit": 2 }, # 2x(SYSCALL+CWD), SYSCALL+CWD
	"mkdirat":                { "vfs": 2, "exit": 2 },
	"mknodat":                { "vfs": 2, "exit": 2 },
	"fchownat":               { "vfs": 4, "exit": 2 },
	"futimesat":              { "vfs": 2, "exit": 2 },
	"newfstatat":             { "vfs": 2, "exit": 2 },
	"unlinkat":               { "vfs": 4, "exit": 2 }, # 2x(SYSCALL+CWD), SYSCALL+CWD
	"renameat":               { "vfs": 8, "exit": 2 }, # 4x(SYSCALL+CWD), SYSCALL+CWD
	"linkat":                 { "vfs": 2, "exit": 2 },
	"symlinkat":              { "vfs": 2, "exit": 2 },
	"readlinkat":             { "vfs": 2, "exit": 2 },
	"fchmodat":               { "vfs": 4, "exit": 2 }, # 2x(SYSCALL+CWD), SYSCALL+CWD
	"faccessat":              { "vfs": 2, "exit": 2 },
	"pselect6":               { "vfs": 2, "exit": 2 },
	"ppoll":                  { "vfs": 2, "exit": 2 },
	"unshare":                { "vfs": 2, "exit": 2 },
	#
	# compat32 system calls
	#
	"waitpid":                { "vfs": 2, "exit": 2 },
	"break":                  { "vfs": 2, "exit": 2 },
	"oldstat":                { "vfs": 2, "exit": 2 },
	"umount":                 { "vfs": 2, "exit": 2 },
	"stime":                  { "vfs": 2, "exit": 2 },
	"oldfstat":               { "vfs": 2, "exit": 2 },
	"stty":                   { "vfs": 2, "exit": 2 },
	"gtty":                   { "vfs": 2, "exit": 2 },
	"nice":                   { "vfs": 2, "exit": 2 },
	"ftime":                  { "vfs": 2, "exit": 2 },
	"prof":                   { "vfs": 2, "exit": 2 },
	"signal":                 { "vfs": 2, "exit": 2 },
	"lock":                   { "vfs": 2, "exit": 2 },
	"mpx":                    { "vfs": 2, "exit": 2 },
	"ulimit":                 { "vfs": 2, "exit": 2 },
	"oldolduname":            { "vfs": 2, "exit": 2 },
	"sigaction":              { "vfs": 2, "exit": 2 },
	"sgetmask":               { "vfs": 2, "exit": 2 },
	"ssetmask":               { "vfs": 2, "exit": 2 },
	"sigsuspend":             { "vfs": 2, "exit": 2 },
	"sigpending":             { "vfs": 2, "exit": 2 },
	"oldlstat":               { "vfs": 2, "exit": 2 },
	"readdir":                { "vfs": 2, "exit": 2 },
	"profil":                 { "vfs": 2, "exit": 2 },
	"socketcall":             { "vfs": 2, "exit": 2 },
	"olduname":               { "vfs": 2, "exit": 2 },
	"idle":                   { "vfs": 2, "exit": 2 },
	"vm86old":                { "vfs": 2, "exit": 2 },
	"ipc":                    { "vfs": 2, "exit": 2 },
	"sigreturn":              { "vfs": 2, "exit": 2 },
	"sigprocmask":            { "vfs": 2, "exit": 2 },
	"bdflush":                { "vfs": 2, "exit": 2 },
	"_llseek":                { "vfs": 2, "exit": 2 },
	"_newselect":             { "vfs": 2, "exit": 2 },
	"vm86":                   { "vfs": 2, "exit": 2 },
	"pread":                  { "vfs": 2, "exit": 2 },
	"pwrite":                 { "vfs": 2, "exit": 2 },
	"ugetrlimit":             { "vfs": 2, "exit": 2 },
	"mmap2":                  { "vfs": 2, "exit": 2 },
	"truncate64":             { "vfs": 2, "exit": 2 },
	"ftruncate64":            { "vfs": 2, "exit": 2 },
	"stat64":                 { "vfs": 2, "exit": 2 },
	"lstat64":                { "vfs": 2, "exit": 2 },
	"fstat64":                { "vfs": 2, "exit": 2 },
	"lchown32":               { "vfs": 2, "exit": 2 },
	"getuid32":               { "vfs": 2, "exit": 2 },
	"getgid32":               { "vfs": 2, "exit": 2 },
	"geteuid32":              { "vfs": 2, "exit": 2 },
	"getegid32":              { "vfs": 2, "exit": 2 },
	"setreuid32":             { "vfs": 2, "exit": 2 },
	"setregid32":             { "vfs": 2, "exit": 2 },
	"getgroups32":            { "vfs": 2, "exit": 2 },
	"setgroups32":            { "vfs": 2, "exit": 2 },
	"fchown32":               { "vfs": 2, "exit": 2 },
	"setresuid32":            { "vfs": 2, "exit": 2 },
	"getresuid32":            { "vfs": 2, "exit": 2 },
	"setresgid32":            { "vfs": 2, "exit": 2 },
	"getresgid32":            { "vfs": 2, "exit": 2 },
	"chown32":                { "vfs": 2, "exit": 2 },
	"setuid32":               { "vfs": 2, "exit": 2 },
	"setgid32":               { "vfs": 2, "exit": 2 },
	"setfsuid32":             { "vfs": 2, "exit": 2 },
	"setfsgid32":             { "vfs": 2, "exit": 2 },
	"madvise1":               { "vfs": 2, "exit": 2 },
	"fcntl64":                { "vfs": 2, "exit": 2 }, # SYSCALL+CWD, SYSCALL+CWD
	"sendfile64":             { "vfs": 2, "exit": 2 },
	"sys_epoll_create":       { "vfs": 2, "exit": 2 },
	"sys_epoll_ctl":          { "vfs": 2, "exit": 2 },
	"sys_epoll_wait":         { "vfs": 2, "exit": 2 },
	"statfs64":               { "vfs": 2, "exit": 2 },
	"fstatfs64":              { "vfs": 2, "exit": 2 },
	"fadvise64_64":           { "vfs": 2, "exit": 2 },
	"kexec":                  { "vfs": 2, "exit": 2 },
	"fstatat64":              { "vfs": 2, "exit": 2 }
}

MIN_EVENT_LINES_WINDOW=200
#MAX_EVENT_LINE_SEPARATION=10000
MAX_EVENT_LINE_SEPARATION=-1
NUM_LINES_TO_PARSE_BETWEEN_DUMPS=500

def get_minimum_events(arch, nr, ignore_vfs) :
	if nr == 0 :
		return 1
	elif arch == "0" :
		# NFS, encoded as x86_64
		syscall_name = syscalls_x86_64[nr]
	elif arch == "40000003" :
		# 32-bit compatibility mode system call
		syscall_name = syscalls_compat32[nr]
	else :
		# x86_64 system call
		if arch != "c000003e" :
			#messagetuples[9017] = (9017, WARNING, """Unknown arch %s, assuming x86_64""")
			logger.LogMsg(el_msgs.messagetuples[9017], arch)
		syscall_name = syscalls_x86_64[nr]

	if ignore_vfs :
		# We're parsing a file which has been taken from the limbo. This means that it's
		# been colapsed by the filters in AuditEntry::append(), so we just wait until we
		# collect the number of events described in the SYSCALL event plus the number of 
		# items indicated on that same event.
		vfs = 0
	else :
		vfs = minimum_events_list[syscall_name]["vfs"]
	exit = minimum_events_list[syscall_name]["exit"]
	return vfs + exit


def safe_mkdir(dirname) :
	if not os.path.isdir(dirname) :
		os.makedirs(dirname)

	return dirname

def remove_oldest_files(dirname, max_size, max_num) :
	st_size = 0
	num_files = 0
	files_sorted_by_time = []
	files = glob.glob(dirname+"/*")
	for f in files :
		statinfo = os.stat(f)
		filetuple = statinfo.st_ctime, statinfo.st_size, f
		files_sorted_by_time.append(filetuple)
	files_sorted_by_time.sort()
	#Start with the newest file and begin counting sizes and numbers.
	#When either the max_size or max_num is reached, start removing
	for f in reversed(files_sorted_by_time):
		num_files += 1
		st_size += f[1]
		if ((max_size != -1 and st_size > max_size) or (max_num != -1 and num_files > max_num)) :
			try:
				os.unlink( f[2] )
			except OSError, ( errno, strerror ):
				#messagetuples[9019] = (9019, ERROR, """Error %d (%s) removing %s.""")
				logger.LogMsg( el_msgs.messagetuples[9019], ( errno, strerror, f[2] ) )

def copy_file_limited(prevname, newname, maxbytes) :
	statinfo = os.stat(prevname)
	prevsize = statinfo.st_size
	if prevsize <= maxbytes :
		shutil.copy(prevname, newname)
	else :
		srcfile = open(prevname, 'rb')
		destfile = open(newname, 'wb')
		srcfile.seek(maxbytes*-1, 2)
		bytes_read = 1
		while bytes_read != 0 :
			bytes = srcfile.read(65536)
			bytes_read = len(bytes)
			destfile.write(bytes)
		srcfile.close()
		destfile.close()

	

def create_backup(options, oldfile, newfile=None) :
	logger.TraceFunctionEntry()

	if oldfile :
		# Copy the original file to /tmp/<collection>/raw_logs
		destdir = safe_mkdir("/tmp/" + options.getCollMount() + "/raw_logs")
		remove_oldest_files(destdir, 1024*1024*1024, 1000)
		filename = "%s/%s-%s" %(destdir, os.path.basename(oldfile), time.strftime("%Y%b%d-%H:%M:%S",time.gmtime()))
		copy_file_limited(oldfile, filename, 1024*1024*1024)
		#messagetuples[9005] = (9005, INFO, """Creating backup of '%s' to '%s'""")
		logger.LogMsg(el_msgs.messagetuples[9005], (oldfile, filename))
	if newfile :
		# Copy the sanitized file to /tmp/<collection>/sanitized_logs
		destdir = safe_mkdir("/tmp/" + options.getCollMount() + "/sanitized_logs")
		remove_oldest_files(destdir, 1024*1024*1024, 1000)
		filename = "%s/%s-%s" %(destdir, os.path.basename(oldfile), time.strftime("%Y%b%d-%H:%M:%S",time.gmtime()))
		copy_file_limited(newfile, filename, 1024*1024*1024)
		#messagetuples[9005] = (9005, INFO, """Creating backup of '%s' to '%s'""")
		logger.LogMsg(el_msgs.messagetuples[9005], (newfile, filename))

	logger.TraceFunctionExit()


#
# Configure logging values
#
logValues = dict()
logValues.setdefault('logginglevel', '1')      # default to logging only errors
logValues.setdefault('tracinglevel', '0')      # default to tracing off ('3' is tracing on)


def get_xml_document(name):
	return xml.dom.minidom.parse(name)


def get_element_value(xmlDocElement, elementName):
	try:
		result = xmlDocElement.getElementsByTagName(elementName)[0].childNodes[0].data
	except IndexError:
		result = None   
	return result


def read_logging_levels(xmlConfigFile):    
	logging_file_keys = ['logginglevel', 'tracinglevel']
	xmlDoc = get_xml_document(xmlConfigFile)
	xmlElement = xmlDoc.documentElement
	for key in logging_file_keys:
		value = get_element_value(xmlElement, key)
		if value != None:
			logValues[key] = str(value)
	xmlDoc.unlink()


class AuditEntry :
	def __init__(self, from_limbo=False, arch="0", syscall="0", sequence="0") :
		self.from_limbo = from_limbo
		self.arch = arch
		self.syscall = int(syscall)
		self.sequence = sequence
		self.lines = []
		self.MRline_nr = 0
		self.expected_events = get_minimum_events(arch, self.syscall, from_limbo)
		self.expected_path_components = 0
		self.parsed_events = 0

	def __del__(self) :
		del self.lines

	def append(self, line, line_nr) :
		self.MRline_nr=line_nr
		self.parsed_events += 1

		# Do not append the same contents again (VFS+EXIT filters)
		if line in self.lines :
			return

		# Update the VFS SYSCALL line with the EXIT one, which will 
		# include the operation's return status. We also update duplicated 
		# VFS SYSCALL lines which appear when the kernel logs more than one
		# PATH component.
		if "type=SYSCALL" in line :
			num_items = 0
			items_offset = line.find("items=")
			if (items_offset >= 0) :
				num_items = int(line[items_offset:].split()[0].split("=")[1])

			if "success=" in line :
				# EXIT filter. Only one is expected for each syscall.
				for l in reversed(self.lines) :
					if (l.find("type=SYSCALL") >= 0) and not ("success=" in l) :
						# Remove the old SYSCALL entry
						index = self.lines.index(l)
						del self.lines[index]
				self.expected_events += num_items
			else :
				# VFS filter. We always take the latest items= number as it gets incremented
				# on each new PATH event sent to userspace. Many VFS messages can be sent for
				# a given system call.
				self.expected_path_components = num_items

			# Prepend the new entry to the start of the list
			self.lines.insert(0, line)
			return

		self.lines.append(line)

	def hasCollectedExpectedEvents(self) :
		return self.parsed_events >= (self.expected_events + self.expected_path_components)

	def dump(self) :
		for line in self.lines :
			print line,

	def dumpToFile(self, fd) :
		for line in self.lines :
			os.write(fd, line)

	def getSequence(self) :
		return self.sequence


class AuditParser :
	def __init__(self, options) :
		self.options = options
		self.tmpFile = None
		self.fp = None
		self.limbofp = None
		self.firstEntry = True
		self.entries = []

	def start(self) :
		logger.TraceFunctionEntry()
		#messagetuples[9001] = (9001, INFO, """Starting the parser...""")
		logger.LogMsg(el_msgs.messagetuples[9001])

		sanit_tmpdir="/tmp/"+self.options.collmount+"/sanitizer"
		safe_mkdir(sanit_tmpdir)
		#allow at most 1GB of old sanitized raw logs. These
		#*should* be getting deleted upon successful completion
		#in the stop() method of this class, but apparently this
		#sanitizer is failing on occasion and this is not happening.
		remove_oldest_files(sanit_tmpdir, 1024*1024*1024, 10)
		self.tmpFile = tempfile.mkstemp(dir=sanit_tmpdir)
		try :
			self.fp = open(self.options.getAuditFile())
		except IOError, errno :
			#messagetuples[9006] = (9006, ERROR, """'%s' could not be opened""")
			logger.LogMsg(el_msgs.messagetuples[9006], self.options.getAuditFile())
			logger.TraceFunctionExit()
			os.sys.exit(1)
		try :
			self.limbofp = open(self.options.getLimboFile())
		except IOError, errno :
			# That's ok, events are not always guaranteed to show up in the limbo
			self.limbofp = open(self.options.getLimboFile(), "w+r")

		logger.TraceFunctionExit()              

	def stop(self) :
		logger.TraceFunctionEntry()

		self.fp.close()
		if len(self.entries) :
			# A few system calls are likely to be crossing the boundaries of the raw
			# log file. Those will be stored in the limbo, awaitinging to be merged 
			# in the next parsing.
			self.dumpToLimbo()

		self.limbofp.close()
		os.close(self.tmpFile[0])
		shutil.copy(self.tmpFile[1], self.options.getOutputFile())
		os.unlink(self.tmpFile[1])

		logger.TraceFunctionExit()

	def findEntry(self, sequence) :
		index = len(self.entries)
		for entry in reversed(self.entries) :
			index -= 1
			if entry.getSequence() == sequence :
				return index, entry
		return -1, None

	def dumpToLimbo(self) :
		num = len(self.entries)
		for i in range(0,num) :
			# Don't add entries which were already taken from the limbo back into it
			if not self.entries[i].from_limbo :
				self.entries[i].dumpToFile(self.limbofp.fileno())
			else :
				# messagetuples[9020] = (9020, INFO, """Entry for syscall %s with sequence number '%s' taken from the limbo log looks like it will never complete. We've only seen %s of %s expected raw log entries. Abandoning.""")
				logger.LogMsg(el_msgs.messagetuples[9020], (str(self.entries[i].syscall), str(self.entries[i].sequence), str(self.entries[i].parsed_events), str(self.entries[i].expected_events + self.entries[i].expected_path_components)))

	def dumpEntries(self, line_nr, final_dump=0) :
		num = len(self.entries)
		uncompleted_entries = []
		for i in range(0,num) :
			if self.entries[i].hasCollectedExpectedEvents() and (((line_nr - self.entries[i].MRline_nr) >= MIN_EVENT_LINES_WINDOW) or final_dump == 1) :
				self.entries[i].dumpToFile(self.tmpFile[0])
			else :
				if (MAX_EVENT_LINE_SEPARATION != -1) and (line_nr - self.entries[i].MRline_nr >= MAX_EVENT_LINE_SEPARATION) :
					#messagetuples[9018] = (9018, ERROR, """Entry for syscall %s with sequence number '%s' looks like it will never complete. The current line number is %s and the last time we saw anything for this sequence number was on line %s. We've only seen %s of %s expected raw log entries. Abandoning.""")
					logger.LogMsg(el_msgs.messagetuples[9018], (str(self.entries[i].syscall), str(self.entries[i].sequence), str(line_nr), str(self.entries[i].MRline_nr), str(self.entries[i].parsed_events), str(self.entries[i].expected_events + self.entries[i].expected_path_components)))
				else :
					uncompleted_entries.append(self.entries[i])
		for i in reversed(range(0,num)) :
				del self.entries[i]
		del self.entries
		self.entries = uncompleted_entries

	def processLimbo(self) :
		logger.TraceFunctionEntry()

		self.process(self.options.getLimboFile(), self.limbofp, False)
		self.limbofp.close()
		os.unlink(self.options.getLimboFile())
		self.limbofp = open(self.options.getLimboFile(), "w+r")

		#logger.TraceMsg(str(len(self.entries)) + " entries processed")
		logger.TraceFunctionExit()

	def processRawLog(self) :
		logger.TraceFunctionEntry()

		self.firstEntry = True
		self.process(self.options.getAuditFile(), self.fp, True)

		logger.TraceFunctionExit()

	def process(self, filename, fp, dump_entries) :
		logger.TraceFunctionEntry()
		#messagetuples[9002] = (9002, INFO, """Processing '%s'...""")
		logger.LogMsg(el_msgs.messagetuples[9002], filename)
		from_limbo_file = not dump_entries

		line_nr = 0
		for line in fp.xreadlines() :
			line_nr += 1
			if (line_nr % 10000) == 0 :
				os.sys.stdout.write(".");
				os.sys.stdout.flush()
			if len(line) <= 1 :
				continue

			if line[0] == '#' :
				# start/end markers
				entry = AuditEntry(from_limbo_file)
				entry.append(line, line_nr)
				if self.firstEntry :
					self.firstEntry = False
					entry.dumpToFile(self.tmpFile[0])
					del entry
					continue
				self.entries.append(entry)
				continue

			try :
				type = line.split(" ")[0].split("=")[1]
			except :
				#messagetuples[9007] = (9007, ERROR, """Error parsing line %d in '%s'""")
				logger.LogMsg(el_msgs.messagetuples[9007], (line_nr, filename))
				logger.TraceFunctionExit()
				raise
			if type == "USER" :
				entry = AuditEntry(from_limbo_file)
				entry.append(line, line_nr)
				entry.dumpToFile(self.tmpFile[0])
				del entry
			elif type == "SYSCALL" :
				# This can be either a new system call or an EXIT system call
				# following up after a VFS entry. In the latter, we only want
				# to update the VFS information with the operation's return 
				# status.
				sequence = line.split(" ")[1].split(":")[1].split(")")[0]
				index, entry = self.findEntry(sequence)
				if entry :
					# Append new contents and move entry to the tail of
					# the list, as this is now the most fresh entry of our
					# workset.
					entry.append(line, line_nr)
					e = self.entries.pop(index)
					self.entries.append(e)
				else :
					arch = line.split(" ")[2].split("=")[1]
					syscall = line.split(" ")[3].split("=")[1]
					entry = AuditEntry(from_limbo_file, arch, syscall, sequence)
					entry.append(line, line_nr)
					self.entries.append(entry)
			else :
				sequence = line.split(" ")[1].split(":")[1].split(")")[0]
				index, entry = self.findEntry(sequence)
				if entry :
					entry.append(line, line_nr)
				else :
					#messagetuples[9008] = (9008, ERROR, """Unknown sequence '%s' on line number %d in '%s'""")
					logger.LogMsg(el_msgs.messagetuples[9008], (sequence, line_nr, filename))

			self.firstEntry = False

			# We dump entries every time we process NUM_LINES_TO_PARSE_BETWEEN_DUMPS more lines.
			# Just the completed entries get dumped, and the ones that
			# still require more information remain in the list.
			if dump_entries and line_nr % NUM_LINES_TO_PARSE_BETWEEN_DUMPS == 0 :
				self.dumpEntries(line_nr, 0)

		if dump_entries :
			self.dumpEntries(line_nr, 1)

		logger.TraceFunctionExit()


class OptionParser :
	def __init__(self) :
		self.appname = os.sys.argv[0]
		self.auditfile = ""
		self.limbofile = ""
		self.outputfile = ""
		self.collmount = ""
		self.shortopts = "hc:o:"
		self.longopts = [ "help", "collmnt=", "output=" ]

	def getAuditFile(self) :
		return self.auditfile

	def getLimboFile(self) :
		return self.limbofile

	def getCollMount(self) :
		return self.collmount

	def getOutputFile(self) :
		return self.outputfile

	def showUsage(self, retval) :
		print "Usage: %s [options] <audit.log>\n" %self.appname
		print "Available options are:"
		print "   -c, --collmnt=<dir>       collection mount point."
		print "   -o, --output=<file>       save output to <file> instead of original audit log file."
		print "   -h, --help                this help."
		return retval

	def parseCmdLine(self) :
		if not os.sys.argv[1:] :
			return self.showUsage(1)

		try :
			options, arguments = getopt.gnu_getopt(os.sys.argv[1:], self.shortopts, self.longopts)
		except getopt.GetoptError, detail :
			print "%s: %s" %(self.appname, str(detail))
			self.showUsage(1)

		for opt, args in options :
			if opt in [ "-c", "--collmnt" ] :
				self.collmount = args
			elif opt in [ "-o", "--output" ] :
				self.outputfile = args
			elif opt in [ "-h", "--help" ] :
				return self.showUsage(1)

		if len(arguments) != 1 or (not len(self.collmount)) :
			return self.showUsage(1)

		self.auditfile = arguments[0]
		limbodir = os.path.dirname(self.auditfile)
		if not limbodir or len(limbodir) == 0 :
			limbodir = os.getenv("PWD")

		node = os.path.basename(self.auditfile)
		self.limbofile = limbodir + "/" + node + ".limbo"

		if len(self.outputfile) == 0 :
			self.outputfile = self.auditfile

		return 0


def main(options) :
	global logger

	if options.parseCmdLine() != 0 :
		print "Error: could not parse command line"
		return 1

	logSize    = 50 * (1024 ** 2)   # 50MB
	logBackups = 3

	rc, nodeUID = commands.getstatusoutput(IAbinpath + "/mw_print_unique_nodeid")
	if rc != 0:
		raise FatalError("Cannot get unique ID for system, error %d." % rc)

	if os.path.exists(options.getCollMount() + "/utility/config/collection.xml") == False:
		print "Error: %s could not be opened" % (options.getCollMount() + "/utility/config/collection.xml")
		logValues['logginglevel'] = '3'
		logValues['tracinglevel'] = '3'
	else:
		read_logging_levels(options.getCollMount() + "/utility/config/collection.xml")

	logger = iamiddleware.ialogging.IALogger(IAlogtracepath, "el", options.getCollMount(), nodeUID, "sanitizer",
			"IA_EL_Log",   "%(asctime)s %(message)s", logValues['logginglevel'],
			"IA_EL_Trace", "%(asctime)s %(process)d  %(message)s", logValues['tracinglevel'],
			logSize, logBackups, False)

	#messagetuples[9000] = (9000, INFO, """el_sanitize_raw_log.py has started...""")
	logger.LogMsg(el_msgs.messagetuples[9000])
	#messagetuples[9009] = (9009, INFO, """LOGGINGLEVEL=%s""")
	logger.LogMsg(el_msgs.messagetuples[9009], logValues['logginglevel'])
	#messagetuples[9010] = (9010, INFO, """TRACINGLEVEL=%s""")
	logger.LogMsg(el_msgs.messagetuples[9010], logValues['tracinglevel'])

	#messagetuples[9012] = (9012, INFO, """collection_mount=%s""")
	logger.LogMsg(el_msgs.messagetuples[9012], options.getCollMount())

	if options.getOutputFile() != "":
		#messagetuples[9013] = (9013, INFO, """output_file=%s""")
		logger.LogMsg(el_msgs.messagetuples[9013], options.getOutputFile())

	#messagetuples[9014] = (9014, INFO, """raw_log=%s""")
	logger.LogMsg(el_msgs.messagetuples[9014], options.getAuditFile())

	logger.TraceMsg("el_sanitize_raw_log.py started, configuration values loaded" + \
			", command line arguments parsed, logging and tracing initiated" )

	create_backup(options, options.getAuditFile())

	parser = AuditParser(options)
	parser.start()
	parser.processLimbo()
	parser.processRawLog()
	parser.stop()

	#messagetuples[9004] = (9004, INFO, """...el_sanitize_raw_log.py has finished""")
	logger.LogMsg(el_msgs.messagetuples[9004])
	logger.TraceMsg("el_sanitize_raw_log.py finished")


if __name__ == "__main__" :
	options = OptionParser()
	try :
		main(options)
	except :
		print "Error in processing the main function"
		if logger != None:
			#messagetuples[9015] = (9015, ERROR, """Error in processing the main function""")
			logger.LogMsg(el_msgs.messagetuples[9015])

		errordir = safe_mkdir("/tmp/" + options.getCollMount() + "/errors/")
		traceback.print_exc(file=open(errordir + options.getAuditFile(), "a"))
		if logger != None:
			print "See error log '" + (errordir + options.getAuditFile()) + "' for debugging purposes"
			#messagetuples[9016] = (9016, ERROR, """See error log '%s' for debugging purposes""")
			logger.LogMsg(el_msgs.messagetuples[9016], (errordir + options.getAuditFile()))
		raise
		os.sys.exit(1)
