#!/bin/bash
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Description:
#    enforce_rsm_compliance.sh removes userids to meet compliance requirements
# Syntax:
#    enforce_rsm_compliance.sh
#
# Results:
#    Removal of the userids in the "non compliant users" table
#
# Change history
#   Modifier      Date         CQ Change       Description
#   -----------------------------------------------------------------------------------
#   Laura Bell    12/13/2010   TIAMC00011955   KPMG - remove unwanted users on RSM
#   Del Ence      01/10/2011   TIAMC00017624   Make sure 'news' user exist. If deleted then create the user so logrotate can run. 
# *************************************************************************************

rc=0

# Build an array of users that should be deleted to meet compliance 
# requirements 
NON_COMPLIANT_USERS=("at"
"bin"
"daemon"
"games"
"lp"
"mail"
"man"
"uucp"
)

USER_TOTAL=${#NON_COMPLIANT_USERS[@]}

# Source the RAS functions for logging utilities
. /opt/tivoli/tiam/bin/ras_functions
rc_reset
create_new_log_file enforce_rsm_compliance

# ======= FUNCTIONS =============

#  Usage error processing
error() {
   ras_log -v $1
   echo "$0: $1"
   echo "Try '$0 -h' for more information."
   exit 1
}

# Output helpful information for how to use the function and its parameters
help() {
   echo "usage:  $0"
   exit 0
}

remove_non_compliant_users() {  
  local curr_user
  local all_rc
  local del_rc
  del_rc=0
  all_rc=0
  curr_user=""
  
  # Walk through the NON_COMPLIANT_USER table and remove users
  for (( i=0;i<$USER_TOTAL;i++));do
    
    if [ -n ${NON_COMPLIANT_USERS[${i}]} ]
    then
      curr_user=${NON_COMPLIANT_USERS[${i}]}
      ras_log "Checking for the existence of non-compliant user: ${curr_user}"
      
      # get the current name in the "non compliance list" and use the 
      # "id" function to check whether a linux user of that name exists 
      # - if the user exists, a string similar to this is returned, 
      #   where the user in this example is "iaadmin":
      #     uid=182(iaadmin) gid=182(iaadmin) groups=182(iaadmin)
      # - if the user does not exist, a string similar to this is 
      #   returned, where the user in this example is "dummy":
      #     id: dummy: No such user
      # redirect both stdout and stderr to a log for easier processing,
      # then clean up the temporary file
      id ${curr_user} &> "/tmp/${curr_user}.log"
      user_exists=`cat /tmp/${curr_user}.log` 
      ras_log "Results of id command for user ${curr_user} are: '${user_exists}'"
      user_exists=`cat /tmp/${curr_user}.log | grep uid | grep gid | grep groups | wc -l` 
      
      # if the user exists, delete it
      if [[ "${user_exists}" -eq 1 ]] ; then
        ras_log "Deleting non-compliant user: $curr_user"
        
        # swallow the 'no crontab for <user_name> message
        userdel -r ${curr_user} 2> /dev/null
        del_rc=$?
        if (( $del_rc == 0 )); then
           ras_log "Successfully removed non-compliant user: ${curr_user}" 
        else
           all_rc=$del_rc
           ras_log "The user delete command returned an unexpected non-zero return code of: $all_rc"
           ras_log "Unable to remove non-compliant user: ${curr_user}"
        fi
      else # user does not exist, so log a message to that effect
        ras_log "User '${curr_user}' does not exist and no additional processing will be done for this user."
      fi
    fi
    ras_log ""
  done
  return $all_rc
}

add_news_user()
{        
    local user
    local add_rc
    user="news"
    add_rc=0
    id ${user} > /dev/null 2>&1
    if [ $? -eq 1 ]; then
        ras_log -v "Creating user: ${user}"
        useradd -r -d /etc/news -g 13 -u 9 ${user} 2> /dev/null 
        add_rc=$?
        if [ $add_rc -eq 0 ]; then
           ras_log -v "Successfully created user: ${user}" 
        else
           ras_log "The user create command returned an unexpected non-zero return code of: $add_rc"
           ras_log -v "Unable to create user: ${user}"
        fi    
    fi
    return $add_rc
}

# ======= MAIN =============

# Verify script is being called correctly; display usage statement and 
# leave if not called correctly
 while getopts "::" optchar ; do
   case $optchar in
   h) help
      ;;
   *) error "Invalid option"
      ;;
   esac
done

ras_log "enforce_rsm_compliance.sh invoked."

# Get rid of users in the "KPMG unwanted user list"
remove_non_compliant_users
rc=$?
if [ $rc -ne 0 ]; then
    exit $rc
fi

# Make sure the news user exists create it if not found
add_news_user
rc=$?
if [ $rc -ne 0 ]; then  
    exit $rc
fi

exit $rc
