#!/bin/bash
#
#******************************************************************************
#  Licensed Materials - Property of IBM
#
#  (C) Copyright IBM Corp.  2008  All Rights Reserved
#
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#
# -----------------------------------------------------------------------------
# 
# =========================================================================
# 
# Name: ia_activaterecoverysite.sh 
#
# Change Log:
#  Date           developer               change
#  02/22/2010     Liming (Michael) Yu     Initial script creation
#  04/11/2010     Bryan Jen               12782: Change pingtest to use RC for fail/success
#  04/23/2010     Liming (Michael) Yu     13177: Change pingtestCluster
#  05/26/2010     Liming (Michael) Yu     13808: Check for rc from telnet to IMM
#  05/26/2010     Liming (Michael) Yu     13778: Add unsynchronized check
#  06/03/2010     Liming (Michael) Yu     13984: Removed call to ia_cfgermpair
#  06/08/2010     Liming (Michael) Yu     13754: Add DR_SECONDARY check
#  06/08/2010     Liming (Michael) Yu     13876: Verify big hammer
#  07/01/2010     Liming (Michael) Yu     14357: Fixed sleeptime counter
#  01/08/2011	Greg McBride		17256: Moved removing phase flags
#					from activaterecoverysite.  Added RC
#					to  ia_powercontrol failure message.
#  01/12/2011	Greg McbRide		17683- Turn on DR_SECONDARY if health
#					check fails
#  01/20/2011   Greg McBride		17827- Move the creation of the 
#					DR_SECONDARY file to before the exit
#                      			caused by printAndLogExit
#  01/24/2011  Greg McBride		17835- Reboot hanging becasue GPFS
#					was down on one or more nodes
#  01/24/2011  Greg McBride		17820- Fix reboot hang during gentle
#                                       shutdown.  Wait 2 minutes before doing
#					hard reboot.
#  01/27/2011  Greg McBride		17918- Remove mmshutdown because it 
#                  			hangs (GPFS CMVC 875294)
#  02/10/2011  Greg McBride		18117- Change RC check for 
#					ia_powercontrol
#  02/10/2011  Greg McBride 		18122- Message change to reboot MC
#
#******************************************************************************


DRLOG="/opt/tivoli/tiam/log/dr"
MMFSBIN="/usr/lpp/mmfs/bin"
TIAMBIN="/opt/tivoli/tiam/bin"
TIAMETC="/opt/tivoli/tiam/etc"

SCRIPTNAME="ia_activaterecoverysite"
PHASESCRIPTNAME="activateRecoverySite"
SMCLI="/usr/bin/SMcli"
datestamp=`date +%m%d%Y%H%M%S`
LOGFILE="${DRLOG}/ia_activaterecoverysite.${datestamp}.out"


usage() {
  echo " ia_activaterecoverysite.sh -m <switch | remove>"
  echo "              -m                 activate recovery site mode [switch|remove]"
  echo "              -h                 help/usage information"
  echo " "
  echo "    eg.   ia_activaterecoverysite.sh -m switch"
  return 0
}


pingtest() {
  cmd=`ping $1 -c 1`
  result=`echo $?`
  return $result
}


pingtestCluster() {
  pingtest ianode1
  ping=$?
  sleeptime=0
  while [ $ping != 0 ]; do
    if [ $sleeptime -ge 2400 ]; then
      printAndLogErrorExit $1 "Before call to $2, can not ping ianode1 for over 40 minutes. Please correct and retry"
    fi
    printAndLog "Can not ping ianode1... $sleeptime seconds out of maximum of 2400 seconds" 
    sleep 30
    sleeptime=$((sleeptime+30))
    pingtest ianode1
    ping=$?
  done

  for node in `sudo -u iaadmin ssh iaadmin@ianode1 sudo $MMFSBIN/mmgetstate -a |grep ianode |awk '{print $2}'`; do
    pingtest $node
    ping=$?
    sleeptime=0
    while [ $ping != 0 ]; do
      if [ $sleeptime -ge 2400 ]; then
        printAndLogErrorExit $1 "Before call to $2, can not ping $node for over 40 minutes. Please correct and retry"
      fi
      printAndLog "Can not ping $node... $sleeptime seconds out of maximum of 2400 seconds" 
      sleep 30
      sleeptime=$((sleeptime+30))
      pingtest $node
      ping=$?
    done
  done
}

#######################################################################################
#
# 17820- There is a problem with the nodes getting hung when they are being rebooted
# through ia_powercontrol with the default 'gentle' shutdown.  To take care of this
# we will now wait for 3 minutes checking that the reboot process has started for each 
# node.  At the end of 3 minutes, we will call ia_powercontrol with the -N option
# to reboot the nodes that have not started rebooting without the shutdown. 
#
# Build an array with a node in each element. Call ia_powercontrol with the -b
# option to get the boot status. Loop checking each node and when a node is no
# longer 'is booted' remove that node from the array.  At the end of 3 minutes
# call ia_powercontrol for each node left in the array. 
#
#######################################################################################

waitForRebooting() {
  declare -a nodearray
  i=0
  for node in `grep ianode /opt/tivoli/tiam/etc/nodes |cut -d, -f2`; do # Build node array
   nodearray[i]=$node
   i=$((i+1))
  done
  numbernodes=${#nodearray[@]}          # Get number of array elements
  sleeptime=0                           # Initialize sleeptime
  if [ $numbernodes -gt 0 ]; then       # If there are nodes in the array
    while [ $sleeptime -lt 180 ] ; do   # Loop while less than 3 minutes
      sleep 30                          # Sleep for 30 seconda
      sleeptime=$((sleeptime+30))       # Keep track of the time waiting
      for (( i=0;i<$numbernodes;i++)); do       # For each element (node) in the array call ia_powercontrol
        result=$($TIAMBIN/ia_powercontrol -b ${nodearray[$i]} 2>&1) # Check to see if node is still booted status
        rc=`echo $?`
        if [ $rc = -1 ]; then     	# 18117- Hack because ia_powercontrol defaults to an error state for not handling an unknown state from the IMM
          printAndLogErrorExit 3 "Failure to get bootstate from ia_powercontrol for node ${nodearray[i]} RC= $rc $result"
        fi
        if [[ $result == *"is booted"* ]]; then
          printAndLog "Waiting for node ${nodearray[$i]} to start rebooting "
        else
          printAndLog "Node ${nodearray[i]} is now booting"
          unset nodearray[$i]                   # Remove booting node from array
          nodearray=("${nodearray[@]}")         # Rebuild array without removed element
          i=$(($i - 1))                         # Decrement i as we just removed an element from array
        fi
        numbernodes=${#nodearray[@]}            # Make sure we have the correct number of elements so to leave loop
      done
      if [ $numbernodes -eq 0 ]; then           # If all nodes are rebooting
        break                                   # leave the sleeptime loop
      fi
    done
    numbernodes=${#nodearray[@]}                # Make sure of the number of elements in array
########################################################################################
#
# If there are any nodes that did not start rebooting with the soft (little hammer)
# enter a loop for those remaining nodes and call ia_powercontrol to power cycle
# the nodes specifying the -N option (big hammer).
#
#######################################################################################

    for (( i=0;i<$numbernodes;i++)); do         # For any elements (nodes) left in the array
      result=$($TIAMBIN/ia_powercontrol -c ${nodearray[$i]} 2>&1) # Reboot with big hammer
      rc=`echo $?`
      if [ $rc != 0 ]; then
        printAndLogErrorExit 3 "Failure to reboot node ${nodearray[i]} from ia_powercontrol RC= $rc $result"
      else
        printAndLog "Rebooting ${nodearray[i]} with no shutdown"
      fi
    done
  fi
}

printAndLog() {
  timestamp=`date "+%Y-%m-%d %H:%M:%S"`
  echo "$timestamp [$SCRIPTNAME] $1"
  echo "$timestamp [$SCRIPTNAME] $1" >> $LOGFILE
}


printAndLogErrorExit() {
  timestamp=`date "+%Y-%m-%d %H:%M:%S"`
  echo "$timestamp [$SCRIPTNAME] ERROR($1): $2"
  echo "$timestamp [$SCRIPTNAME] ERROR($1): $2" >> $LOGFILE
  echo "$timestamp [$SCRIPTNAME] Exiting script with rc=$1. Please refer to the logfile @ $LOGFILE"
  echo "$timestamp [$SCRIPTNAME] Exiting script with rc=$1" >> $LOGFILE
  exit $1
}


#======= beginning of the main function =======

ls $DRLOG > /dev/null 2>&1
rc=$?
if [ $rc -ne 0 ]; then
  mkdir $DRLOG
fi

ls /opt/tivoli/tiam/etc/DR_SECONDARY > /dev/null 2>&1
result=$?
if [ $result != 0 ]; then
  printAndLogErrorExit -3 "ia_activaterecoverysite.sh can only be runned on the secondary appliance. This is not a secondary appliance"
fi

printAndLog "Starting the activation of the appliance at the recovery site"
echo ""

MODE="none"

while [ "$1" != "" ]
do
  case $1 in
  -m )
    if [ "$2" == "switch" ]; then
      MODE="switch"
      timestamp=`date "+%Y-%m-%d %H:%M:%S"`
      echo "$timestamp [$SCRIPTNAME -m switch]" >> $LOGFILE
    elif [ "$2" == "remove" ]; then
      MODE="remove"
      timestamp=`date "+%Y-%m-%d %H:%M:%S"`
      echo "$timestamp [$SCRIPTNAME -m remove]" >> $LOGFILE
    else
      printAndLogErrorExit -1 "Invalid input parameter for mode"
    fi
    shift;shift
    ;;
  -h | -? | --help )
    usage
    exit
    ;;
  * )
    printAndLogErrorExit -2 "Invalid input parameter. Please enter ia_activaterecoverysite.sh --help to see the usage"
  esac
done

if [ $MODE != "remove" ]; then
  #verify that all logical drives are synchronized (optimal)
  printAndLog "Calling checkMirrorStatus.sh to verify all logical drives are synchronized (optimal)"
  $TIAMBIN/checkMirrorStatus.sh >> $LOGFILE
  retVal=`echo $?`
  if [ $retVal != 0 ]; then 
    printAndLogErrorExit 1 "checkMirrorStatus.sh exited with rc=$retVal"
  elif [[ `cat $LOGFILE |grep -i Synchronizing` ]]; then
    printAndLogErrorExit 2 "Not all logical drives are completely synchronized (optimal), checkMirrorStatus.sh exited with rc=$retVal"
  elif [[ `cat $LOGFILE |grep -i Suspended` ]]; then
    printAndLogErrorExit 2 "Some logical drives are suspended, checkMirrorStatus.sh exited with rc=$retVal"
  elif [[ `cat $LOGFILE |grep -i Unsynchronized` ]]; then
    printAndLogErrorExit 2 "Some logical drives are unsynchronized, checkMirrorStatus.sh exited with rc=$retVal"
  else
    printAndLog "All logical drives are completely synchronized (optimal), checkMirrorStatus.sh exited with rc=0"
  fi
  echo ""
fi

#power up all nodes
for node in `grep ianode /opt/tivoli/tiam/etc/nodes |cut -d, -f2`; do
  result=`$TIAMBIN/ia_powercontrol -b $node`
  rc=`echo $?`
  if [ $rc != 0 ]; then
    printAndLogErrorExit 3 "Failure in communication to IMM on $node RC= $rc"
  fi
  if [ $result~="is booted" ]; then
    #good case
    sleep 5
  else
    printAndLog "$node is not booted"
    echo $result >> $LOGFILE
    printAndLog "Power cycling $node"
    out=`$TIAMBIN/ia_powercontrol -c $node`
    result=`echo $?`
    if [ $result != 0 ]; then
      echo $out >> $LOGFILE
      printAndLogErrorExit 3 "Failed to power cycle $node RC= $result . Please correct and retry"
    fi
    sleep 60
  fi

  pingtest $node
  pings=$?
  if [ $pings != 0 ]; then
    printAndLog "$node is not pingable"
    printAndLog "Powering up $node"
    out=`$TIAMBIN/ia_powercontrol -u $node`
    result=`echo $?`
    if [ $result != 0 ]; then
      echo $out >> $LOGFILE
      printAndLogErrorExit 4 "Failed to power cycle $node RC= $result . Please correct and retry"
    fi
    sleep 60
  fi
done
#make sure all nodes are up before continuing
sleeptime=0
for node in `grep ianode /opt/tivoli/tiam/etc/nodes |cut -d, -f2`; do
  pingtest $node
  ping=$?
  while [ $ping != 0 ]; do
    if [[ $sleeptime -ge 2400 ]]; then
        printAndLogErrorExit 16 "$node has been unresponsive for over 40 minutes. Please correct and retry"
      fi
      printAndLog "Waiting for $node reboot... $sleeptime seconds out of maximum of 2400 seconds"
      sleeptime=$((sleeptime+30))
      if [[ $sleeptime -ge 300 ]]; then
        pingtest $node
        ping=$?
        if [ $ping == 0 ]; then
           printAndLog "$node has successfully booted."
           continue
        fi
      else
        ping=1
      fi
      sleep 30
  done
done
echo ""

if [ $MODE != "remove" ]; then
  pingtest dr_remote_mc
  ping=$?
  if [ $ping != 0 ]; then
    printAndLogErrorExit 6 "Can not ping dr_remote_mc. Please repair the tunnel on the primary appliance by calling ia_cfgermpair -enable on the primary appliance"
  fi
 
  printAndLog "Calling ia_cfgsecondary.sh on dr_remote_mc"
  ssh root@dr_remote_mc "sudo $TIAMBIN/ia_cfgsecondary.sh" >> $LOGFILE
  result=`echo $?`
  if [ $result == 0 ]; then
    printAndLog "ia_cfgsecondary.sh completed successfully with rc=0"
  else
    printAndLogErrorExit 7 "ia_cfgsecondary.sh exited with rc=$result"
  fi
fi
echo ""

pingtestCluster 8 "activateRecoverySite"
printAndLog "Calling activateRecoverySite $MODE on ianode1"
sudo -u iaadmin ssh iaadmin@ianode1 "sudo $TIAMBIN/activateRecoverySite $MODE" >> $LOGFILE
result=`echo $?`
retries=0
while [ $retries -lt 3 ]; do
  if [ $result == 0 ]; then
    printAndLog "activateRecoverySite completed successfully with rc=0"
    break

  elif [ $result == 99 ]; then
    printAndLog "activateRecoverySite exited with rc=99"
    for node in `sudo -u iaadmin ssh iaadmin@ianode1 sudo $MMFSBIN/mmgetstate -a |grep ianode |awk '{print $2}'`; do
      printAndLog "Rebooting $node"
      out=`$TIAMBIN/ia_powercontrol -c $node`
      result=`echo $?`
      if [ $result != 0 ]; then
        echo $out >> $LOGFILE
        printAndLogErrorExit 15 "Failed to reboot $node RC= $result . Please correct and retry"
      fi
      sleep 10
    done

#######################################################################################
#
# 17820- Call waitForRebooting to get arond reboot hang caused by gentle reboot
#
######################################################################################

    waitForRebooting   

    printAndLog "Waiting for nodes to reboot..."
    #sleep 300
    #wait until all nodes are back online
    sleeptime=0
    for node in `grep ianode /opt/tivoli/tiam/etc/nodes |cut -d, -f2`; do
      ping=1
      while [ $ping != 0 ]; do
        if [[ $sleeptime -ge 2400 ]]; then
          printAndLogErrorExit 16 "$node has been unresponsive for over 40 minutes. Please correct and retry"
        fi
        printAndLog "Waiting for $node reboot... $sleeptime seconds out of maximum of 2400 seconds"
        sleeptime=$((sleeptime+30))
        if [[ $sleeptime -ge 300 ]]; then
          pingtest $node
          ping=$?
          if [ $ping == 0 ]; then
             printAndLog "$node has successfully booted."
             continue
          fi
        else
          ping=1
        fi
        sleep 30
      done
    done

    retries=$((retries+1))
    printAndLog "Retrying call to activateRecoverySite $MODE on ianode1 ($retries/3)..."
    sudo -u iaadmin ssh iaadmin@ianode1 sudo $TIAMBIN/activateRecoverySite $MODE >> $LOGFILE
    result=`echo $?`

  else
    printAndLogErrorExit 9 "activateRecoverySite exited with rc=$result"
  fi
done

if [ $retries == 3 ]; then
  if [ $result == 0 ]; then
    printAndLog "activateRecoverySite completed successfully with rc=0"
  elif [ $result == 99 ]; then
    printAndLogErrorExit 99 "activateRecoverySite exited with rc=99. Cannot activate recovery site because GPFS is still active after all retries"
  else
    printAndLogErrorExit 9 "activateRecoverySite exited with rc=$result"
  fi
fi
echo ""

pingtestCluster 10 "ia_recovery_healthcheck.sh (phase1)"
printAndLog "Calling ia_recovery_healthcheck.sh on ianode1"
sudo -u iaadmin ssh iaadmin@ianode1 sudo $TIAMBIN/ia_recovery_healthcheck.sh -p 1 >> $LOGFILE
result=`echo $?`
if [ $result == 0 ]; then
  printAndLog "The health is good, ia_recovery_healthcheck.sh exited with rc=0"
else
  printAndLogErrorExit 11 "The health is bad, ia_recovery_healthcheck.sh exited with rc=$result"
fi
echo ""

pingtestCluster 12 "restoreConfigFiles"
printAndLog "Calling restoreConfigFiles on ianode1"
sudo -u iaadmin ssh iaadmin@ianode1 sudo ${TIAMBIN}/restoreConfigFiles >> $LOGFILE
result=`echo $?`
if [ $result == 0 ] || [ $result == 255 ]; then
  printAndLog "restoreConfigFiles completed successfully with rc=0"
else
  printAndLogErrorExit 13 "restoreConfigFiles exited with rc=$result"
fi
echo ""

printAndLog "Calling restoreMCConfigFiles"
sudo $TIAMBIN/restoreMCConfigFiles >> $LOGFILE 
result=`echo $?`
if [ $result == 0 ]; then
  if [[ `echo $LOGFILE | grep "WARNING: RSM"` ]]; then
    printAndLog "WARNING: iapasswdreset.py did not successfully complete on iarsm1. Please login to iarsm1 as root and run iapasswdreset.py"
    printAndLog "restoreMCConfigFiles completed successfully with rc=0"
  else
    printAndLog "restoreMCConfigFiles completed successfully with rc=0"
  fi
else
  printAndLogErrorExit 14 "restoreMCConfigFiles exited with rc=$result"
fi
echo ""

for node in `sudo -u iaadmin ssh iaadmin@ianode1 sudo $MMFSBIN/mmgetstate -a |grep ianode |awk '{print $2}'`; do
  printAndLog "Rebooting $node"
  out=`$TIAMBIN/ia_powercontrol -c $node`
  result=`echo $?`
  if [ $result != 0 ]; then
    echo $out >> $LOGFILE
    printAndLogErrorExit 15 "Failed to reboot $node RC= $result . Please correct and retry"
  fi
  sleep 20
done

#######################################################################################
#
# 17820- Call waitForRebooting to get around reboot hang caused by gentle reboot
#
######################################################################################

waitForRebooting

printAndLog "Waiting for nodes to reboot..." 
#sleep 300
#wait until all nodes are back online
sleeptime=0
for node in `grep ianode /opt/tivoli/tiam/etc/nodes |cut -d, -f2`; do
  ping=1
  while [ $ping != 0 ]; do
    if [[ $sleeptime -ge 2400 ]]; then
      printAndLogErrorExit 16 "$node has been unresponsive for over 40 minutes. Please correct and retry"
    fi
    printAndLog "Waiting for $node reboot... $sleeptime seconds out of maximum of 2400 seconds" 
    sleeptime=$((sleeptime+30))
    if [[ $sleeptime -ge 300 ]]; then
      pingtest $node
      ping=$?
      if [ $ping == 0 ]; then
         printAndLog "$node has successfully booted"
         continue
      fi
    else
      ping=1
    fi
    sleep 30
  done
done

printAndLog "Waiting for IA Software Stack to Load."
sleep 300
echo ""

pingtestCluster 17 "ia_recovery_healthcheck.sh (phase2)"
printAndLog "Calling ia_recovery_healthcheck.sh on ianode1"
sudo -u iaadmin ssh iaadmin@ianode1 sudo $TIAMBIN/ia_recovery_healthcheck.sh -p 2 >> $LOGFILE
result=`echo $?`
if [ $result == 0 ]; then
  printAndLog "The health is good, ia_recovery_healthcheck.sh exited with rc=0"
else
  printAndLog "Attempt to call healtcheck exited with $result. System not ready. Sleeping for 5 min before retry."
  sleep 300 
  printAndLog "Calling ia_recovery_healthcheck.sh on ianode1"
  sudo -u iaadmin ssh iaadmin@ianode1 sudo $TIAMBIN/ia_recovery_healthcheck.sh -p 2 >> $LOGFILE
  result=`echo $?`
  if [ $result == 0 ]; then
    printAndLog "The health is good, ia_recovery_healthcheck.sh exited with rc=0"
  else
#######################################################################################
#
# 17256-
# If the health ceck failed, turn on the DR_SECONDARY flag so we can rerun
# recovery.  The flag is turned off in restoreMCCConfigfile because GPFS
# checks that flag for not doing processing on a secondary.  HOwever,
# we want GPFS to come up as on a primary.  So, we have to turn it back
# on if there was a nerror in this script.
# 17820 - Do this before the printAndLogErrorExit as this will exit the script
#
#######################################################################################

    touch /opt/tivoli/tiam/etc/DR_SECONDARY
    printAndLogErrorExit 18 "The health is bad, ia_recovery_healthcheck.sh exited with rc=$result"
  fi
fi
echo ""

######################################################################################
#
# 17256- Move removal of phase flags to here so customer can rerun recovery in case
# reboot or healthcheck fails.
#
#####################################################################################
printAndLog "Removing phase flags"
rm /opt/tivoli/tiam/log/dr/activateRecoverySite.phase*

printAndLog "Rebooting the management console. If management console does not reboot, please reboot manually"   #18122
out=`$TIAMBIN/ia_powercontrol -c iamconsole1`
result=`echo $?`
if [ $result != 0 ]; then
  echo $out >> $LOGFILE
  printAndLogErrorExit 19 "Failed to reboot ${HOSTNAME} RC= $result . Please correct and retry"
fi
echo ""

timestamp=`date "+%Y-%m-%d %H:%M:%S"`
echo "$timestamp [$SCRIPTNAME] The appliance has been successfully activated on the recovery site"
echo "$timestamp [$SCRIPTNAME] The appliance has been successfully activated on the recovery site" >> $LOGFILE
echo "$timestamp [$SCRIPTNAME] Exiting script with rc=0. Please refer to the logfile @ $LOGFILE"
echo "$timestamp [$SCRIPTNAME] Exiting script with rc=0" >> $LOGFILE
exit 0
