#!/bin/bash
#
#******************************************************************************
#  Licensed Materials - Property of IBM
#
#  (C) Copyright IBM Corp.  2008  All Rights Reserved
#
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#
# -----------------------------------------------------------------------------
#
# =========================================================================
#
# Name: addHostPorts.sh
#
# Change Log:
#
#   1.0     4/16/09     Bryan Jen     Script creation.
#  04/23/2010    Brad Trubey      TIAMC00013107  move logs to location where they will get collected
#  11/09/2010    Brad Trubey      TIAMC00016579  improve error logging and reporting
#
#******************************************************************************

SMCLI="/usr/bin/SMcli"
mkdir -p /var/opt/tivoli/tiam/log
LOGFILE="/var/opt/tivoli/tiam/log/ia_addhostports.log"

usage() {
  echo " ia_addhostports.sh -a <ipaddress> -b <ipaddress>"
  echo "       -a                 IP Address of controller A"
  echo "       -b                 IP Address of controller B"
  echo "       -h                 help/usage information"
  echo " "
  echo "    eg.  ia_addhostports.sh -a 172.31.0.100 -b 172.31.0.101"
  return 0
}

# ************************************************************************
# ************************************************************************
# Function: fail
# - send the configuration failure message to stdout
# - In (optional): $1: return code to use when exiting script
# ************************************************************************
# ************************************************************************
fail() {
  echo "IA Storage configuration FAILED. View $LOGFILE for more information"     | tee -a $LOGFILE
  if [ -n $1 ]; then
    exit $1
  else
    exit 1
  fi
}

# ************************************************************************
# ************************************************************************
# Function: verifySMCmd
# - this function runs an SMcli API. If the SMcli command fails, the
#   failing SMcli command will be logged
# - The SMcli script engine is not case sensitive. You can enter commands
#   by using uppercase letters, lowercase letters, or mixed-case letters
# - Parameters:
#   In: $1: return code from the SMcli command
#   In: $2: output text file from the SMcli command
#   In: $3: Error message to log if the SMcli command fails
# ************************************************************************
# ************************************************************************
verifySMCmd() {
  # ======================================================================
  # Run the caller's command and save its output to the $1 output variable
  # ======================================================================
  if [[ $1 -ne 0 ]]; then
    # ======================================================================
    # If we are here, the SMcli Command failed.
    # If the caller did not pass in an error message ($3), use a generic
    # error message
    # ======================================================================
    if [[ -z "$3" ]]; then
      errMsg="An SMcli command failure has occurred."
    else
      errMsg=$3
    fi

    # ======================================================================
    # -  Log the error message along with additional problem determination info
    # ======================================================================
    echo "$errMsg" >> $LOGFILE
    echo "The SMcli API return code is: " $1 >> $LOGFILE
    if [ -f "$2" ] && [ -s "$2" ]; then
      echo "The SMcli API results follow this message: " >> $LOGFILE
      cat "$2" >> $LOGFILE
      rm -f "$2" $> /dev/null
    fi

    # ======================================================================
    # - the SMcli docs say that the name of the file to which error
    #   information is saved is excprpt.txt which gets overwritten every
    #   time that an exception occurs. In a typical installation with no
    #   changes in the default installation directory, the location of the
    #   file for UNIX-based operating systems is: /var/opt/SM
    # - so if the SMcli /var/opt/SM/excprpt.txt exception file exists,
    #   append its contents to the log file
    # - Call our exit function
    # ======================================================================
    if [ -f /var/opt/SM/excprpt.txt ]; then
      echo "" >> $LOGFILE
      echo "Detailed SMcli error information follows this message:" >> $LOGFILE
      cat /var/opt/SM/excprpt.txt >> $LOGFILE
    fi

  fail $1

  fi
} # end: verifySMCmd

# +++++++++++++++++++++++++++++++++++++++++++++++++++
# +++++++++++++++++++++++ main ++++++++++++++++++++++
# +++++++++++++++++++++++++++++++++++++++++++++++++++
if [ $# != 4 ]; then
  usage
  exit 0
else
  if [ "$1" = "-a" ]; then
      CTRLA=$2
  else
      usage
      exit 0
  fi

  if [ "$3" = "-b" ]; then
      CTRLB=$4
  else
      usage
      exit 0
  fi
fi


if [ `whoami` == "root" ]; then
   echo "=======Start Script $0 `date`=====" >> $LOGFILE
   # ========================================================================
   # Different versions of the firmware respond differently to some CLI calls
   # so get the first part of the controller's firmware version for use
   # later in the script. The firmware information (before cuts), looks
   # like this, so "cut" at the first colon and cut the last two parts
   # of the version number
   #   Firmware version:      07.60.28.00
   # - getting the firmware for the first controller is adequate to limit
   #   the complexity of the script
   # ========================================================================
   echo "Querying controller firmware level with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"show controller [a] summary;\" | grep -i -m 1 firmware\" " >> $LOGFILE
   controllerFirmware=`$SMCLI $CTRLA $CTRLB -c "show controller [a] summary;" | grep -i -m 1 firmware | cut -d: -f2 | cut -d. -f1-2 | awk '{ print $1 }'`
   if [ "${controllerFirmware}" == "" ]; then
       echo "Unable to determine firmware level" >> $LOGFILE
       fail 1
   fi

   echo "Controller Firmware is at level $controllerFirmware" >> $LOGFILE

   # if we are on firmware level less than 7.60 we need to use the "interfaceType" parameter
   if [[ "$controllerFirmware" < "07.60" ]];then
     hostportType="hostType=13"

   # if we are on firmware level 7.60 or above, we need to use the "hostType" parameter
   else
     hostportType="interfaceType=FC"
   fi

   echo "Adding Hosts to the Host Group" >> $LOGFILE

   count=1;
   for host in `${SMCLI} ${CTRLA} ${CTRLB}  -c "show allhostports;" |grep -i "undefined" |cut -d' ' -f1 |grep -i ":" | sed -e 's/://g'`;
   do
     echo "Adding Host #$count port id $host with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"create hostport identifier=\\\"${host}\\\" userlabel=\\\"port${host}\\\" host=\\\"iagroup\\\" $hostportType;\" " >> $LOGFILE
     txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create hostport identifier=\"${host}\" userlabel=\"port${host}\" host=\"iagroup\" $hostportType;" `
     verifySMCmd "$?" "$cliOutputFile" "Create Host port FAILED"

     count=$(($count+1))

   done

   totalports=$(($count-1))
   echo "TOTAL HOST PORTS ADDED: $totalports" >> $LOGFILE
   echo "=======End Script `date`=====" >> $LOGFILE
   echo "Configuration complete.  Please see $LOGFILE for additional details"     | tee -a $LOGFILE
else
   echo "You must be root (or use sudo) to run this script."
fi

