#!/bin/bash
#
#******************************************************************************
#  Licensed Materials - Property of IBM
#
#  (C) Copyright IBM Corp.  2008  All Rights Reserved
#
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#
# -----------------------------------------------------------------------------
# 
# =========================================================================
# 
# Name: ia_cfgsecondary.sh 
#
# Change Log:
#  Date           developer               change
#  02/22/2010     Liming (Michael) Yu     Initial script creation
#
#  01/24/2011  Greg McBride             17820- Fix reboot hang during gentle
#                                       shutdown.  Wait 2 minutes before doing
#                                       hard reboot.
#  02/10/2011  Greg McBride	18117- Change RC check for ia_powercontrol
#  02/10/2011  Greg McBride	18122- Message change to reboot MC
#
#******************************************************************************


DRLOG="/opt/tivoli/tiam/log/dr"
TIAMBIN="/opt/tivoli/tiam/bin"

SCRIPTNAME="ia_cfgsecondary"
SMCLI="/usr/bin/SMcli"
datestamp=`date +%m%d%Y%H%M%S`
LOGFILE="${DRLOG}/ia_cfgsecondary.${datestamp}.out"


usage() {
  echo " ia_cfgsecondary.sh"
  echo "              -h                 help/usage information"
  echo " "
  echo "    eg.   ia_cfgsecondary.sh"
  return 0
}


pingtest() {
  cmd=`ping $1 -c 1`
  if [[ `echo $cmd |grep -i unreachable` ]]; then
    return -1
  else
    return 0
  fi
}


pingtestCluster() {
  pingtest ianode1
  pings=$?
  if [ $pings != 0 ]; then
    printAndLogErrorExit $1 "Before call to $2, can not ping ianode1."
  fi
  for node in `sudo -u iaadmin ssh iaadmin@ianode1 sudo $MMFSBIN/mmgetstate -a |grep ianode |awk '{print $2}'`; do
    pingtest $node
    ping=$?
    if [ $ping != 0 ]; then
      printAndLogErrorExit $1 "Before call to $2, can not ping $node."
    fi
  done
}

#######################################################################################
#
# 17820- There is a problem with the nodes getting hung when they are being rebooted
# through ia_powercontrol with the default 'gentle' shutdown.  To take care of this
# we will no wait for 2 minutes checking that th reboot process has started for each
# node.  At the end of two minutes, we will call ia_powercontrol with the -N option
# to reboot the nodes without the shutdown.
#
# Build an array with a node in each element. Call ia_powercontrol with the -b
# option to bet the boot status. Loop checking each node and when a node is no
# longer 'is booted' remove taht node from the array.  At the end of 2 minutes
# call ia_powercontrol for each node left in the array.
#
#######################################################################################

waitForRebooting() {
  declare -a nodearray
  i=0
  for node in `grep ianode /opt/tivoli/tiam/etc/nodes |cut -d, -f2`; do # Build node array
   nodearray[i]=$node
   i=$((i+1))
  done
  numbernodes=${#nodearray[@]}          # Get number of array elements
  sleeptime=0                           # Initialize sleeptime
  if [ $numbernodes -gt 0 ]; then       # If there are nodes in the array
    while [ $sleeptime -lt 180 ] ; do   # Loop while less than 3 minutes
      sleep 30                          # Sleep for 30 seconda
      sleeptime=$((sleeptime+30))       # Keep track of the time waiting
      for (( i=0;i<$numbernodes;i++)); do       # For each element (node) in the array call ia_powercontrol
        result=$($TIAMBIN/ia_powercontrol -b ${nodearray[$i]} 2>&1) # Check to see if node is still booted status
        rc=`echo $?`
        if [ $rc = -1 ]; then		# 18177- Hack because ia_powercontrol defaults to an error state for not handling an unknown state from IMM
          printAndLogErrorExit 3 "Failure to get bootstate from ia_powercontrol for node ${nodearray[i]} RC= $rc $result"
        fi
        if [[ $result == *"is booted"* ]]; then
          printAndLog "Waiting for node ${nodearray[$i]} to start rebooting "
        else
          printAndLog "Node ${nodearray[i]} is now booting"
          unset nodearray[$i]                   # Remove booting node from array
          nodearray=("${nodearray[@]}")         # Rebuild array without removed element
          i=$(($i - 1))                         # Decrement i as we just removed an element from array
        fi
        numbernodes=${#nodearray[@]}            # Make sure we have the correct number of elements so to leave loop
      done
      if [ $numbernodes -eq 0 ]; then           # If all nodes are rebooting
        break                                   # leave the sleeptime loop
      fi
    done
    numbernodes=${#nodearray[@]}                # Make sure of the number of elements in array
########################################################################################
#
# If there are any nodes that did not start rebooting with the soft (little hammer)
# enter a loop for those remaining nodes and call ia_powercontrol to power cycle
# the nodes specifying the -N option (big hammer).
#
#######################################################################################

    for (( i=0;i<$numbernodes;i++)); do         # For any elements (nodes) left in the array
      result=$($TIAMBIN/ia_powercontrol -c ${nodearray[$i]} 2>&1) # Reboot with big hammer
      rc=`echo $?`
      if [ $rc != 0 ]; then
        printAndLogErrorExit 3 "Failure to reboot node ${nodearray[i]} from ia_powercontrol RC= $rc $result"
      else
        printAndLog "Rebooting ${nodearray[i]} with no shutdown"
      fi
    done
  fi
}

printAndLog() {
  timestamp=`date "+%Y-%m-%d %H:%M:%S"`
  echo "$timestamp [$SCRIPTNAME] $1"
  echo "$timestamp [$SCRIPTNAME] $1" >> $LOGFILE
}


printAndLogErrorExit() {
  timestamp=`date "+%Y-%m-%d %H:%M:%S"`
  echo "$timestamp [$SCRIPTNAME] ERROR($1): $2"
  echo "$timestamp [$SCRIPTNAME] ERROR($1): $2" >> $LOGFILE
  echo "$timestamp [$SCRIPTNAME] Exiting script with rc=$1. Please refer to the logfile @ $LOGFILE"
  echo "$timestamp [$SCRIPTNAME] Exiting script with rc=$1" >> $LOGFILE
  exit $1
}


#======= beginning of the main function =======

ls $DRLOG > /dev/null 2>&1
rc=$?
if [ $rc -ne 0 ]; then
  mkdir $DRLOG
fi

printAndLog "Starting the secondary configuration of the appliance"

while [ "$1" != "" ]
do
  case $1 in
  -h | -? | --help )
    usage
    exit
    ;;
  * )
    printAndLogErrorExit -1 "Invalid input parameter. Please enter ia_cfgsecondary.sh --help to see the usage"
  esac
done

pingtestCluster 1 "cfgSecondary.sh"
printAndLog "Calling cfgSecondary.sh on ianode1"
sudo -u iaadmin ssh iaadmin@ianode1 sudo $TIAMBIN/cfgSecondary.sh >> $LOGFILE
result=$?
if [ $result == 0 ]; then
  printAndLog "cfgSecondary.sh completed successfully with rc=0"
else
  printAndLogErrorExit 2 "cfgSecondary.sh exited with rc=$result"
fi

#######################################################################################
#
# 17820- Call waitForRebooting to get arond reboot hang caused by gentle reboot
#
######################################################################################

waitForRebooting

#printAndLog "Restarting ISC"
#$TIAMBIN/ia_restart_ui_service.sh >> $LOGFILE

timestamp=`date "+%Y-%m-%d %H:%M:%S"`
echo "$timestamp [$SCRIPTNAME] The appliance has been successfully configured as the secondary appliance"
echo "$timestamp [$SCRIPTNAME] The appliance has been successfully configured as the secondary appliance" >> $LOGFILE
echo "$timestamp [$SCRIPTNAME] Exiting script with rc=0. Please refer to the logfile @ $LOGFILE"

printAndLog "Rebooting the management console. If management console does not reboot, please reboot manually."	# 18122
out=`$TIAMBIN/ia_powercontrol -c iamconsole1`
result=`echo $?`
if [ $result != 0 ]; then
  echo $out >> $LOGFILE
  printAndLogErrorExit 19 "Failed to reboot ${HOSTNAME}. Please correct and retry"
fi


echo ""

exit 0
