#!/bin/bash
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Filename: ia_check_cim.sh
#
# Component: RAS
#
# Description:
#    This file contains bash script for checking to see if
#    the process cimserver (part of IBM Director) has pegged
#    the CPU, and to kill it if it has.
#
# Change history
#   Modifier                        Date         CQ Change      Description
#   --------------------------------------------------------------------------------
#   Robert Jose & Eric Biesterfeld  09/20/2009   TIAMC00007808  Initial creation
#   Brad Trubey                     07/01/2010   TIAMC00014422  Fix the script so that it actually works.
#   Linda Benhase                   12/30/2010   TIAMC00017504  Add test for inactive cimserver too
#------------------------------------------------------------------------------
# include ras_functions
. /opt/tivoli/tiam/bin/ras_functions

COMMAND_CHECKED=cimserver
TIMES_CHECKED=3
SLEEP_TIME=45

#This function checks $TIMES_CHECKED times to see if $COMMAND_CHECKED
#is contained within the top CPU-draining command. If it is, then return 1.
#$TOPCPU is set to the top process. $PID is set to the offending process ID.
#Limitations: This does not check to see if multiple processes with the
#same command are caught separately; this can be added relatively painlessly
#if needed.
function chk_process() {
    #This returns a line with CPU %, process ID, and the command attached
    #to the line sorted by CPU percentage. Discard first line, second line
    #will show process eating most CPU. Grep for cimserver.
    for (( i=$TIMES_CHECKED; i > 0; i-- ))
    do
       #0 returns true, 1 returns false. wait for specified time and try again.
       ps -eo pcpu,pid,args | grep -v CPU | sort -k 1 -r | head -1 | grep $COMMAND_CHECKED
       if [ $? -eq 0 ]; then
           ras_log -v "cimserver is the top CPU user ($i/$TIMES_CHECKED): wait 45 seconds and re-check."
           sleep $SLEEP_TIME
       else
           #Set process ID to 0, return 0, everything's fine.
           ras_log -v "cimserver is not the top CPU user."
           PID=0
           return 0
       fi
    done

    #Get top CPU line
    TOPCPU=`ps -eo pcpu,pid,args | grep -v CPU | sort -k 1 -r | head -1`
    #Make sure we're still finding cimserver
    echo $TOPCPU | grep $COMMAND_CHECKED
    if [ $? -eq 0 ]; then
        #Get the process id to kill by trimming leading whitespace and grabbing field 2
        PID=`echo $TOPCPU | sed -e 's/^[ ]*//' | cut -d ' ' -f 2`
        ras_log -v "cimserver is still the top CPU user, with process $PID"
        return 1
    fi
}

restart_process() {
    ras_log -v "Kill Process: $PID"
    if [ $PID -ne 0 ]; then
        execute_and_log kill -9 $PID
    fi
    ras_log -v "Restarting cimserverd"
    execute_and_log /etc/init.d/cimserverd restart
}

# This function checks to make sure the cimserver is not inactive. When the cimserver is 
# inactive it does not detect errors and the director agent does not notify the director
# server. The result is that the error does not cause a call home and IBM does not 
# dispatch a repair action.
function chk_inactive_process() {
    # The cimserver has a cli, /opt/ibm/icc/cimcli that can be used to determine if the 
    # cimserver is active. cimcli returns 0 if the command succeeded and 1 if it fails.
    ras_log -v "Checking for inactive cimserver."
    /opt/ibm/icc/bin/cimcli niall >> /dev/null
    return $?
}

# create log file
create_new_log_file ia_check_cim

chk_process $TIMES_CHECKED
if [ $? -eq 1 ]; then
    restart_process
fi

chk_inactive_process
if [ $? -eq 1 ]; then
    restart_process
fi

ras_log -v "ia_check_cim.sh complete."

