#!/bin/sh
########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

#/****************************************************************************
# * COMPONENT NAME:  IA Middleware
# *
# * FILE NAME:       ia_controlmiddleware.sh
# *
# * DESCRIPTION:     Controls the IA Middleware.
# *
# * AUTHOR:          Michael A. Richmond
# *                  IBM Almaden Research Center
# *                  mar@almaden.ibm.com
# *
# *                  Mark A. Smith
# *                  IBM Almaden Research Center
# *                  mark1smi@us.ibm.com
# *
# ****************************************************************************
# */
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
#               Change Log
#               ----------
#  19 Sep 2008  Initial version.
#

# Configure shell variables
. `dirname $BASH_SOURCE`/mw_environment.sh

# Read the common error values
. `dirname $BASH_SOURCE`/wlm_controlerrors.sh

# Read the common control functions
. `dirname $BASH_SOURCE`/wlm_basiccontrolcommon.sh

##############################################################################
##  Control IA watchdog startup.
##    If START_WATCHDOG is set to 'y' then the watchdog will be 
##    started on this node during system boot. Otherwise the 
##    watchdog will not be started on this node.
##    Note that if the watchdog is running on any node in the 
##    cluster correct watchdog operation will result in startup
##    of the watchdog on all cluster nodes over time.
#START_WATCHDOG="n"
START_WATCHDOG="y"

# Locations of files and executables required for IA middleware startup
TIAM_ROOT="/opt/tivoli/tiam/bin"

CONTROL_WLM_NAME=mw_controlwlm.sh
CONTROL_WLM_BIN="$TIAM_ROOT/$CONTROL_WLM_NAME"
CONTROL_WATCHDOG_NAME=mw_controlwatchdog.sh
CONTROL_WATCHDOG_BIN="$TIAM_ROOT/$CONTROL_WATCHDOG_NAME"

# Timeout while waiting for components to stop (in seconds). Give it a minute
STOP_TIMEOUT=60

doStart()
{

	check_file_exists $CONTROL_WLM_BIN && check_file_exists $CONTROL_WATCHDOG_BIN

	b_exists=$?
	if [ "$b_exists" != 1 ]; then
		return $ERR_FILE_DOES_NOT_EXIST
	fi
	check_file_executable $CONTROL_WLM_BIN && check_file_executable $CONTROL_WATCHDOG_BIN
	b_is_executable=$?
	if [ "$b_is_executable" != 1 ]; then
		return $ERR_FILE_NOT_EXECUTABLE
	fi

	doStatus
	running_status=$?

	if [ $running_status -ne $ERR_PROCESS_NOT_RUNNING ]; then
		return $running_status
	fi

	${CONTROL_WLM_BIN} -start > /dev/null 2>&1 &

	if [ "${START_WATCHDOG}" = "y" ]; then
               ${CONTROL_WATCHDOG_BIN} -start > /dev/null 2>&1 &
	fi	


	doStatus
	running_status=$?

	if [ $running_status -ne $ERR_PROCESS_IS_RUNNING ]; then
		return $running_status
	fi

	return 0
}

doStatus()
{
        if [ "$START_WATCHDOG" == "y" ]; then
                status=`$CONTROL_WLM_BIN -status && $CONTROL_WATCHDOG_BIN -status`
        else
                status=`$CONTROL_WLM_BIN -status`
        fi

	return $status
}

doStop()
{
        if [ "$START_WATCHDOG" == "y" ]; then
                $CONTROL_WATCHDOG_BIN -stop
        fi

        $CONTROL_WLM_BIN -stop
}


parse_args $@

case ${OPERATION} in
	start)
		doStart
		start_retval=$?
		if [ $start_retval -eq 0 ]; then
			exit 0
		else
			if [ $start_retval -eq $ERR_FILE_DOES_NOT_EXIST ]; then
				echo "start issued but $CONTROL_WLM_BIN or $CONTROL_WATCHDOG_BIN does not exist."
			elif [ $start_retval -eq $ERR_FILE_NOT_EXECUTABLE ]; then
				echo "start issued but $CONTROL_WLM_BIN or $WATCHDOG_BIN is not executable."
			elif [ $start_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
				echo "start issued but $CONTROL_WLM_BIN or $CONTROL_WATCHDOG_BIN is already running"
			elif [ $start_retval -eq $ERR_PROCESS_MULTIPLE_RUNNING ]; then
				echo "start issued but multiple $CONTROL_WLM_BIN or $CONTROL_WATCHDOG_BIN are already running"
			elif [ $start_retval -eq $ERR_PROCESS_NOT_HEALTHY ]; then
				echo "start issued but $CONTROL_WLM_BIN or $CONTROL_WATCHDOG_BIN (which is unhealthy) is already running"
			fi

			exit $start_retval
		fi
		;;
	restart)
		echo "restart not implemented"
		exit -1
		;;
	status)
		doStatus
		status_retval=$?
		if [ $status_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
			exit 0
		else
			exit $status_retval
		fi
		;;
	stop)
		doStop
		stop_retval=$?
		if [ $stop_retval -eq 0 ]; then
			exit 0
		else
			if [ $stop_retval -eq $ERR_PROCESS_NOT_RUNNING ]; then
				echo "stop issued for $CONTROL_WLM_BIN or $CONTROL_WATCHDOG_BIN but it was not running"
			fi
			exit $stop_retval
		fi
		;;
	*)
		printHelp
		;;
esac
