#!/bin/bash
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Description:
#    ia_dir_create_user.sh creates userids for IBM Director defined groups
#
#    The format requires an input of the username and the group to add it to.  It validates
#    that the group is one of the valid groups for Director.  The password will be set to
#    the value provided when .
#
# Syntax:
#    ia_dir_create_user.sh -u user_name -g group_name
#
# Results:
#    Creation of the userid as a member of the Director group specified with password provided
#
# Change history
#   Modifier      Date         CQ Change       Description
#   -----------------------------------------------------------------------------------
#   Brad Trubey   07/09/2009   TIAMC00007027   renamed to conform to IA convention
# *************************************************************************************


#Directory holding binaries
bin_dir="/opt/tivoli/tiam/bin/"

new_user=""
dir_group=""

# Source the RAS functions for logging utilities
. /opt/tivoli/tiam/bin/ras_functions
rc_reset
create_new_log_file dir_users

#  Usage: error "error message"
error() {
   ras_log $1
   echo "$0: $1"
   echo "Try '$0 -h' for more information."
   exit 1
}

#Output helpful information regarding how to use the function and its parameters
help() {
   echo "usage:  $0 -u <user> -g <group>"
   echo "where:"
   echo "   -u Specify the new director user to be added."
   echo "   -g Add the new director user to an existing director group."
   echo "      Valid director groups are: iadiradmin, smmgr, smmon, smuser"
   exit 0
}


ras_log "ia_dir_create_user.sh invoked to create new userid"

# Get the user and group
while getopts ":hg:u:" optchar ; do
   case $optchar in
   g) dir_group=$OPTARG
      ;;
   u) new_user=$OPTARG
      ;;
   h) help
      ;;
   *) error "Invalid option"
      ;;
   esac
done

ras_log "User is ${new_user} Group is ${dir_group}"

# Validate the group as one of the groups allowable
case "$dir_group" in
   "iadiradmin" | "smmgr" | "smmon" | "smuser" )
   ras_log "Valid group found"
      ;;
   "" )
      error "No group provided"
      ;;
   *) error "Invalid group provided"
      ;;
esac

# Ensure that we got a group
if [ "${dir_group}" == "" ] ; then
   error "No group specified"
fi

# Ensure that we got a user
if [ "${new_user}" == "" ] ; then
   error "No user specified"
fi

rc=0

# add the new user
useradd ${new_user}

rc=$?

ras_log "Attempted to create ${new_user} rc is $rc"
if !(( $rc == 0 )); then
   ras_log "Create user failed with return code $rc"
   error "Create user failed"
fi

rc_reset

usermod -A ${dir_group} ${new_user}

rc=$?

ras_log "Attempted to add ${new_user} to group ${dir_group} rc is $rc"
if !(( $rc == 0 )); then
   ras_log "Add user to group failed with return code $rc"
   error "Add user to group failed"
fi

rc=0
#Assign initial password to new user
${bin_dir}ia_dir_change_user_password.sh -u ${new_user}
rc=$?

ras_log "Attempted to set initial user password for user: ${user} rc is $rc"
if !(( $rc == 0 )); then
   ras_log "Set initial user password failed with return code $rc"
   error "Set initial user password failed"
fi


#If we get this far, everything worked
echo "Successfully added user: ${new_user} to group: ${dir_group}"

exit $rc


