#!/bin/bash
#
#******************************************************************************
#  Licensed Materials - Property of IBM
#
#  (C) Copyright IBM Corp.  2008  All Rights Reserved
#
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#
# -----------------------------------------------------------------------------
# 
# =========================================================================
# 
# Name: ia_ermsuspend.sh 
#
# Change Log:
#  Date           developer               change
#  02/03/2010     Liming (Michael) Yu     Initial script creation
#  05/20/2010     Liming (Michael) Yu     13737: added writeConsistency=TRUE
#                                         for async
#
#******************************************************************************


DRLOG="/opt/tivoli/tiam/log/dr"
TIAMBIN="/opt/tivoli/tiam/bin"

SCRIPTNAME="ia_ermsuspend"
SMCLI="/usr/bin/SMcli"
datestamp=`date +%m%d%Y%H%M%S`
LOGFILE="${DRLOG}/ia_ermsuspend.${datestamp}.out"


usage() {
  echo " ia_ermsuspend.sh"
  echo "              -h                 help/usage information"
  echo " "
  echo "    eg.   ia_ermsuspend.sh"
  return 0
}


log() {
  timestamp=`date "+%Y-%m-%d %H:%M:%S"`
  echo "$timestamp [$SCRIPTNAME] $1" >> $LOGFILE
}


printAndLog() {
  timestamp=`date "+%Y-%m-%d %H:%M:%S"`
  echo "$timestamp [$SCRIPTNAME] $1"
  echo "$timestamp [$SCRIPTNAME] $1" >> $LOGFILE
}


printAndLogErrorExit() {
  timestamp=`date "+%Y-%m-%d %H:%M:%S"`
  echo "$timestamp [$SCRIPTNAME] ERROR($1): $2"
  echo "$timestamp [$SCRIPTNAME] ERROR($1): $2" >> $LOGFILE
  echo "$timestamp [$SCRIPTNAME] Exiting script with rc=$1. Please refer to the logfile @ $LOGFILE"
  echo "$timestamp [$SCRIPTNAME] Exiting script with rc=$1" >> $LOGFILE
  exit $1
}


suspendERM() {
  log "Enter suspendERM()"
    
  DSNAME=`${SMCLI} ${CTRLA} ${CTRLB} -c "show storagesubsystem summary;" | grep -i "PROFILE FOR STORAGE SUBSYSTEM" | cut -d: -f2 | awk '{ print $1 }'`
  LUNS=(`${SMCLI} ${CTRLA} ${CTRLB}  -c "show alllogicaldrives;" |grep -i "logical drive name" | grep -v "Mirror" | cut -d: -f2 | awk '{ print $1 }'`)
  for LUNNAME in ${LUNS[@]};
  do
    #we only care about ia or util luns for suspending
    if [[ `echo $LUNNAME | grep -i ia` || `echo $LUNNAME | grep -i util` ]]; then
      cmd=`${SMCLI} ${CTRLA} ${CTRLB} -c "show remoteMirror localLogicalDrive[\"$LUNNAME\"] synchronizationProgress;"`
      retVal=`echo $?`
      #we only care about luns that have a remoteMirror
      if [ $retVal == 0 ]; then
        cmd=`${SMCLI} ${CTRLA} ${CTRLB} -c "show allLogicalDrives;" | grep -A 5 -i "Mirrored pair: $LUNNAME" | grep -i "consistent"`
        retVal=`echo $?`
        if [ $retVal != 0 ]; then
          printAndLog "Remote Mirror Pair $LUNNAME is in an unsupported write mode."
          printAndLog "Setting write mode to Consistent Mirror Group."
          cmd=`${SMCLI} ${CTRLA} ${CTRLB} -c "set remoteMirror localLogicalDrive[\"$LUNNAME\"] writeOrder=preserved writeMode=asynchronous;"`
          retVal=`echo $?`
          if [ $retVal != 0 ]; then
            echo $cmd >> $LOGFILE
            printAndLogErrorExit 11 "Setting remote mirror write mode for $LUNNAME failed!"
          fi
        fi
      fi
    fi
  done
  printAndLog "Now suspending Consistent Mirror Group for $DSNAME"
  cmd=`${SMCLI} ${CTRLA} ${CTRLB} -c "suspend remoteMirror primary [\"$LUNNAME\"] writeConsistency=TRUE;"`
  retVal=`echo $?`
  if [ $retVal != 0 ]; then
     echo $cmd >> $LOGFILE
     printAndLog "Suspending remote mirror for $LUNNAME failed!"
  fi

  log "Exit suspendERM() with rc=0"
}


#======= beginning of the main function =======

ls $DRLOG > /dev/null 2>&1
rc=$?
if [ $rc -ne 0 ]; then
  mkdir $DRLOG
fi

printAndLog "Starting the ERM suspension on all logical drives"

CTRLA=""
CTRLB=""

while [ "$1" != "" ]
do
  case $1 in
  -h | -? | --help )
    usage
    exit
    ;;
  * )
    printAndLogErrorExit -1 "Invalid input parameter. Please enter ia_ermsuspend.sh --help to see the usage"
  esac
done

ls /opt/tivoli/tiam/etc/DR_ENABLED > /dev/null 2>&1
rc=$?
if [ $rc -ne 0 ]; then
  printAndLogErrorExit 0 "Cannot suspend ERM if DR is not enabled"
else
  ls /opt/tivoli/tiam/etc/DR_SECONDARY > /dev/null 2>&1
  rc=$?
  if [ $rc -eq 0 ]; then
    printAndLogErrorExit 0 "Cannot suspend ERM if running on DR secondary"
  fi
fi

#verify that all logical drives are synchronized (optimal)
printAndLog "Calling checkMirrorStatus.sh to verify all logical drives are synchronized (optimal)"
$TIAMBIN/checkMirrorStatus.sh >> $LOGFILE
retVal=`echo $?`
if [ $retVal != 0 ]; then
  printAndLogErrorExit 1 "checkMirrorStatus.sh exited with rc=$retVal"
elif [[ `cat $LOGFILE |grep -i Synchronizing` ]]; then
  echo -n "WARNING: Not all logical drives are completely synchronized (optimal). It is recommended that all are synchronized (optimal) before suspending. Do you want to continue with the suspend [y/n]? "
  read REPLY
  if [[ "$REPLY" != "y" ]]; then
    timestamp=`date "+%Y-%m-%d %H:%M:%S"`
    echo "$timestamp [$SCRIPTNAME] Not all logical drives are completely synchronized (optimal)" >> $LOGFILE
    echo "$timestamp [$SCRIPTNAME] Exiting script with rc=0. Please refer to the logfile @ $LOGFILE"
    echo "$timestamp [$SCRIPTNAME] Exiting script with rc=0" >> $LOGFILE
    exit 0
  else
    log "Not all logical drives are completely synchronized (optimal), yet continuing with the suspend"
  fi
fi

ssystems=(`cat /home/iaadmin/.tiam/application.properties |grep ds4k |cut -d= -f2`)
if [[ ${#ssystems[@]} -lt 2 ]]; then
  printAndLogErrorExit 2 "Invalid or missing data in applications.properties"
fi 

i=0
while [ $i -lt ${#ssystems[@]} ]
do
  if [[ ${ssystems[$i]} =~ "disk_ctrl" ]]; then
    CTRLA=${ssystems[$i]}
  else
    printAndLogErrorExit 3 "Either an IP address or a hostname that does not start with \"disk_ctrl\" was found in applications.properties. Please correct and retry"
  fi

  if [[ ${ssystems[$(($i+1))]} =~ "disk_ctrl" ]]; then
    CTRLB=${ssystems[$(($i+1))]}
  else
    printAndLogErrorExit 4 "Either an IP address or a hostname that does not start with \"disk_ctrl\" was found in applications.properties. Please correct and retry"
  fi

  suspendERM

  i=$(($i+2))  
done
 
printAndLog "NOTE: It should be noted that IA is dependent on the DS4000 for recording the changes that occur while the mirror is suspended. \
If the I/O rate to the IA volumes is high and/or the suspend continues for sufficient time, the DS4000 will consume all of the resources \
used to hold the change information, and the DS4000 will internally remove the copy services relationships (the remote mirroring will be \
removed here). If this happens, a full synchronization will be required. The customer should monitor the DS4000 through the DS4000 \
client and issue the ia_ermresume.sh command prior to the internal remove."

timestamp=`date "+%Y-%m-%d %H:%M:%S"`
echo "$timestamp [$SCRIPTNAME] ERM has been successfully suspended on all logical drives"
echo "$timestamp [$SCRIPTNAME] ERM has been successfully suspended on all logical drives" >> $LOGFILE

/opt/tivoli/tiam/bin/checkMirrorStatus.sh

echo "$timestamp [$SCRIPTNAME] Exiting script with rc=0. Please refer to the logfile @ $LOGFILE"
echo "$timestamp [$SCRIPTNAME] Exiting script with rc=0" >> $LOGFILE
exit 0
