#!/bin/bash
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Description:
#    ia_iss.sh converts a model number and serial number into an IBM Director style
#    XML file: ISS_MTMS.xml.  If the model number and serial number are specified
#    on the command line, those values are used; otherwise this script expects to
#    find the values in the files "/tmp/model_number" and "/tmp/serial_number".
#    If no command line arguments are specified, and the necessary files do not
#    exist, the script fails.
#
#    The expected format of the "/tmp" files is a single line of text, containing
#    the model or serial number.
#
#    The machine type is hardcoded by this script to "2231".
#
#    If the XML file already exists, it will be overwritten.
#
# Syntax:
#    ia_iss.sh [ -m model_num ] [ -s serial_num ]
#
# Results:
#    Creation of the file /opt/ISS/ISS_MTMS.xml
#
# Change history
#   Modifier      Date         CQ Change       Description
#   -----------------------------------------------------------------------------------
#   Brad Trubey   07/09/2009   TIAMC00007027   renamed to conform to IA convention
# *************************************************************************************

DEST_DIR=/opt/ISS
DEST_FILE=${DEST_DIR}/ISS_MTMS.xml
USAGE="Usage: $0 -m model -s serial number"

machine_type="2231"
model_number=""
serial_number=""

# Source the RAS functions for logging utilities
. /opt/tivoli/tiam/bin/ras_functions
rc_reset
create_new_log_file serial_number

ras_log -v "ia_iss.sh invoked to create ${DEST_FILE}"

# Attempt to load the model number from /tmp/model_number
if [ -f "/tmp/model_number" ] ; then
   model_number=`cat /tmp/model_number`
   ras_log "Loaded model number from /tmp/model_number: ${model_number}"
fi

# Attempt to load the serial number from /tmp/serial_number
if [ -f "/tmp/serial_number" ] ; then
   serial_number=`cat /tmp/serial_number`
   ras_log "Loaded serial number from /tmp/serial_number: ${serial_number}"
fi

# Override the values from the command line arguments
while getopts ":hm:s:" optchar ; do
   case $optchar in
   m) model_number=$OPTARG
      ras_log "Loaded model number from command line: ${model_number}" ;;
   s) serial_number=$OPTARG
      ras_log "Loaded serial number from command line: ${serial_number}" ;;
   h) echo $USAGE
      exit 0 ;;
   \?) echo "Error $USAGE"
      exit 1;;
   esac
done

# Ensure that we got a model number from somewhere
if [ "${model_number}" = "" ] ; then
   ras_log -v "Model number never specified."
   exit 1
fi

# Ensure that we got a serial number from somewhere
if [ "${serial_number}" = "" ] ; then
   ras_log -v "Serial number never specified."
   exit 1
fi

execute_and_log mkdir ${DEST_DIR}

echo "Creating ${DEST_FILE}"

echo "<ISS>"                              > ${DEST_FILE}
echo "<machine>"                         >> ${DEST_FILE}
echo "<type>${machine_type}</type>"      >> ${DEST_FILE}
echo "<model>${model_number}</model>"    >> ${DEST_FILE}
echo "<serial>${serial_number}</serial>" >> ${DEST_FILE}
echo "</machine>"                        >> ${DEST_FILE}
echo "</ISS>"                            >> ${DEST_FILE}

ras_log -v "Successfully created ${DEST_FILE}"
