#! /usr/bin/env python
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Change Activity
#
# Name           Date         Change ID     Description
# Brad Trubey    07/12/2010   TIAMC00014566 Initial Version in CCRC
# Brad Trubey    08/18/2010   TIAMC00015085 Need sleep to ensure tgz write completes before download
# Brad Trubey    09/13/2010   TIAMC00014828 Add support for "-n" to do single node collection
# Brad Trubey    10/23/2010   TIAMC00016309 Clean up comments to increase clarity
# Brad Trubey    10/26/2010   TIAMC00016264 Clean up comments to increase clarity
# Brad Trubey    10/29/2010   TIAMC00015805 Move LC variables from MC to RAS
################################################################################

import getopt, sys, os, socket, pickle
from tiam_util import *
from logcollector import *
from logcollector_util import *
from mb_rsm_access import *


def usage():
    exitmsg('usage: ' + sys.argv[0] + ' [-i instructionfile][-f filepatternfile][-a on/off][-n hostname]', 1)


def popen2log( cmd):
    logprint( cmd)
    return os.popen2( cmd)


def procs_on_fs(fs):
    #nothing except bzip2 works on the file
    try:
        info = syscallread("lsof %s | grep -v bzip2" % fs)
    except:
        return False
    if len(info) > 1:
        return True
    return False


def tiam_utility_ok( nodes):
    cmd = "mount | grep tiam_utility"
    for node in nodes:
        fullcmd = "ssh %s %s" % (node, cmd)
        try:
            syscall( fullcmd)
        except:
            logprint( "'/tiam_utility' not mounted on '%s'. Treating all nodes as non-GPFS." %node)
            return False
    return True


# Return two arrays containing the GPFS and non-GPFS nodes where the log collector should run
# If the "-n" flag was passed on the command line, then only that node will be collected from.
def get_log_nodes(target_host):
    if target_host != '':
        if pingable(target_host):
            logprint("Collecting logs from only the requested node %s" %target_host)
            non_gpfs_nodes = [ target_host ]
            gpfs_nodes = []
        else:
            logprint("Unable to ping the requested Log Collection node %s" %target_host)
            non_gpfs_nodes = []
            gpfs_nodes = []
    else:
        #return active nodes with mounted /tiam_utility
        #return empty list if local node has no /tiam_utility
        all_nodes = get_all_nodes()
        active_nodes = get_nodes()
        gpfs_nodes = active_nodes
        config = get_global_config()
        non_gpfs_nodes = [ always_by_name( config["mconsoleip"]) ]

        rsmnode = always_by_name( config["rsmip"])
        if pingable(rsmnode):
            rsm_open(get_global_config()["rsmpw"])
            non_gpfs_nodes.append(rsmnode)

        if len(all_nodes) != len(active_nodes) or not(tiam_utility_ok(active_nodes)):
            non_gpfs_nodes += all_nodes
            gpfs_nodes = []

    return non_gpfs_nodes, gpfs_nodes


def reserve_result( logtemplate, datestamp):
    #create empty result and hold it to prevent
    #a second instance of ia_logcollector from starting
    delname = logtemplate % "*"
    logname = logtemplate % datestamp
    syscall( "rm -f %s" % delname)
    reserved = open( logname, 'w')
    return reserved


def release_result( reserved):
    reserved.close()


def compress_result( logtemplate, datestamp, datadir):
    basedir = datadir + "/.."
    logname = logtemplate % datestamp
    (directory, logsource) = os.path.split( datadir)
    popen2log( 'cd %s; tar czf %s %s' % (basedir, logname, logsource))
    return logname


def calculate_skipped( datadir):
    try:
        sline = syscallreadhide( "wc -l %s/skipped_*.txt | tail -n 1" % datadir, "")[0].strip()
        parts = re.split( "[ \t]+", sline)
        return int( parts[0])
    except:
        return 0


def instance_wait( startmarker, endmarker, logtemplate):
    now = time.time()
    timeint = int( now)
    logfiles = logtemplate % "*"
    logname = ""
    #check if other instance has already written a new start marker
    if get_marker_time( startmarker) > get_marker_time( endmarker):
        logprint( "another instance of %s seems to be running!" % sys.argv[0])
        #wait if there still is an owner
        while procs_on_fs( logfiles):
            time.sleep( 5)
        #return result if not too old (3600 secs)
        if now - get_marker_time( startmarker) < 3600:
            try:
                logname = syscallread( "ls %s" % logfiles)[0]
            except:
                # logfile from other instance already deleted
                logname = ""
            # remove the following line if the second instance should
            # return the result of the first one
            # currently we have to exit because the FileDownloadServlet
            # from the first instance deletes the file before the second
            # instance can download it.
            exitmsg( "Download logs was started several times!", 1)
    return logname


def main():
    tiam_utility = "/tiam_utility"
    tmp = "/tmp"
    starttime = 0

    #write info always to tmp
    infodir = tmp + "/logcollection_info"
    startmarker = infodir + "/begin.txt"
    endmarker   = infodir + "/end.txt"
    runninginfo = infodir + "/info.txt"

    #check for other instance early
    if instance_running( startmarker, endmarker):
        time.sleep( 2)
        msg = "another instance of %s seems to be running!" % sys.argv[0]
        logprint( msg)
        f = open( runninginfo, "w")
        f.write( msg + "\n")
        f.close()
        returnmsg()
        logprint( "logname=" + runninginfo)
        logprint( "duration=%d" % 0)
        logprint( "skipped=%d" % 0)
        errmsg().exit( 1, "CTJIM1028E")
    else:
        #remove old stuff if it is still present
        syscall_try( "rm -rf %s" % infodir)
        syscall_try( "mkdir -p %s" % infodir)

        #write start marker early
        starttime = timestamp( startmarker)

    #get global config from tiam_util
    config = get_global_config()

    if not(started_on_cluster()):
        timestamp( endmarker)
        exitmsg( "%s must be launched from a cluster node." % sys.argv[0], 1)

    #read the command line
    try:
        opts, args = getopt.getopt(sys.argv[1:],
                                   "w:u:i:n:f:a:")
    except getopt.GetoptError:
        timestamp( endmarker)
        usage()

    targethost = ''
    cmdlistname = get_logc_config()["ia_commands"]
    filelistname = get_logc_config()["ia_logfiles"]
    all_files = "-a off"
    for opt, arg in opts:
        if opt in ("-i"):
            cmdlistname = arg
        elif opt in ("-f"):
            filelistname = arg
        elif opt in ("-w"):
            rsmpasswd = arg
        elif opt in ("-u"):
            rsmuser = arg
        elif opt in ("-n"):
            targethost = arg
        elif opt in ("-a"):
            if arg == "on":
                all_files = "-a on"
        else:
            timestamp( endmarker)
            exitmsg("unhandled option", 1)

    datestamp = time.strftime('%Y%m%d%H%M', time.gmtime())
    non_gpfs_nodes, gpfs_nodes = get_log_nodes(targethost)

    if len( gpfs_nodes) > 0 and cluster_ok():
        mountpt = tiam_utility
    else:
        mountpt = tmp

    logdir      = mountpt + "/logcollection"
    dataprefix  = logdir + "/IAlogs_"
    datadir     = dataprefix + datestamp
    logtemplate = dataprefix + '%s.tgz'

    #remove old stuff if it is still present
    syscall_try( "rm -rf " + tiam_utility + "/logcollection/IAlogs_*")
    syscall_try( "rm -rf " + tmp + "/logcollection/IAlogs_*")
    syscall_try( "mkdir -p %s" % datadir)

    #reserve file for result
    reserved = reserve_result( logtemplate, datestamp)

    collectorlist = []
    for node in non_gpfs_nodes:
        logprint("Starting log collection on non-GPFS node '%s'..." % node)
        collectorlist.append( logcollector( node, datestamp, True, cmdlistname, filelistname, all_files))

    for node in gpfs_nodes:
        logprint("Starting log collection on GPFS node '%s'..." % node)
        collectorlist.append( logcollector( node, datestamp, False, cmdlistname, filelistname, all_files))

    #write node info for logcollector_progress.py
    filename = infodir + "/nodes.pickle"
    try:
        f = open( filename, "w")
    except:
        timestamp( endmarker)
        errmsg().exit( 1, "CTJIM1003E", filename)
    pickle.dump( collectorlist, f)
    f.close()

    #wait for results
    for collector in collectorlist:
        logprint("Waiting for results from node '%s'..." % collector.node)
        collector.get_result( datadir)

    rsmnode = always_by_name( config["rsmip"])
    if rsmnode in non_gpfs_nodes:
        rsm_close()

    #syscall( "du -s %s" % datadir) # uncompressed size
    release_result( reserved)
    logname = compress_result( logtemplate, datestamp, datadir)
    skipped = calculate_skipped( datadir)

    #write end marker
    endtime = timestamp( endmarker)
    duration = endtime - starttime

    # The log files has all been moved into "datadir" at this point, and the compression into a single tgz file has begun.
    # Once the compression completes, the original datadir will be removed by mb_unused_delete, which monitors the tgz file
    # until it is no longer being used by any process, then deletes the subdirectory.  Once that subdirectory has been deleted,
    # it is safe to exit the log collector and start copying the tgz file.
    syscall( "mb_unused_delete.py -u %s -d %s" % (logname, datadir))
    returnmsg()

    # The following three lines are parsed by the MC GUI, and MUST NOT be changed.
    logprint( "logname=" + logname)
    logprint( "duration=%d" % duration)
    logprint( "skipped=%d" % skipped)

    # Defect 15085: Do not exit log collector until the datadir has been deleted, which signals that the tgz file is created (see explanation above).
    while 1:
        try:
            time.sleep(5)
            syscall( "ls " + datadir )
        except: break

    logprint("ia_logcollector complete.")

if __name__ == "__main__":
    do_main( main)
