#! /usr/bin/env python
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Change Activity
#
# Name           Date         Change ID      Description
# Brad Trubey    06/10/2009   TIAMC00000000  Initial Version
# Brad Trubey    09/15/2009   TIAMC00009208  scp Log Collection archive to MC node
# Brad Trubey    01/26/2010   TIAMC00009143  Trace Config improvements for 1.2
# Brad Trubey    05/21/2010   TIAMC00013770  ia_service umbrella for 2.1
# Del Ence       07/14/2010   TIAMC00014484  Update for new Firmware locations
# Brad Trubey    08/03/2010   TIAMC00014524  Make "min" the default for log collection
# Brad Trubey    08/31/2010   TIAMC00015108  Add support for Director "unlock" command
# Brad Trubey    09/13/2010   TIAMC00014828  Add support for "-n" getlogs, "-d" destinations, and "-c everything"
# Brad Trubey    09/17/2010   TIAMC00015620  Firmware at correct level should return 0
# Brad Trubey    09/22/2010   TIAMC00015701  Change gethostname calls to getIAhostname
# Brad Trubey    09/22/2010   TIAMC00014565  Add support for Backup/Recovery trace
# Brad Trubey    09/22/2010   TIAMC00015044  Add support for middleware-only log collection
# Del Ence       09/27/2010   TIAMC00015795  Updated SMC ethernet switch firware call
# Del Ence       09/30/2010   TIAMC00015857  Updated for new 3950 firmware filename
# Brad Trubey    10/07/2010   TIAMC00015878  Remove support for Backup/Recovery trace (pulled from 2.1)
# Del Ence       10/22/2010   TIAMC00014447  Added support for Director configuration
# Brad Trubey    10/23/2010   TIAMC00016309  "-r getlogs -n node" not working for MC or RSM nodes
# Brad Trubey    11/08/2010   TIAMC00016579  Allow "su to iaadmin" before running commands
# Brad Trubey    11/11/2010   TIAMC00016612  Replace "-r verify -c cnfs" with "-verify -c network"
# Brad Trubey    12/13/2010   TIAMC00017174  Add support for "-r getlogs -c san"
# Brad Trubey    12/14/2010   TIAMC00017308  Add support for "-r getlogs -c varlog"
# Brad Trubey    12/16/2010   TIAMC00017332  Change "xseries" to "systemx"
# Brad Trubey    12/17/2010   TIAMC00017432  Add support for "-r rsm"
# Linda Benhase  01/25/2011   TIAMC00017845  Fix isBconfig compare
################################################################################

import getopt, sys, os, socket, gettext, locale, pwd
from tiam_util import do_main
from gettext import gettext as _
from ia_util import *

################################################################################
# ia_service.py
#
# This utility can be launched from any node on the IA appliance, and serves as
# a front end for all IA RAS utilities.
# ia_service validates command line switches, and maps the run (-r), command (-c)
# and action (-a) switches to a specific script to execute.  A combination of the
# node (-n) switch and command metadata (contained in service.properties) is used
# to determine the node where the command needs to be routed and executed.
################################################################################

# Globalization and Localization setup.  These lines MUST COME BEFORE the "_()" lookups
locale.setlocale(locale.LC_ALL,'')
gettext.textdomain('com.ibm.ia.ras')


# Translatable usage messages:

#@trans The command "ia_service.py" should NOT be translated.
#@trans The -r runapp values (for example, "-r getlogs") should NOT be translated.
USAGE_GENERAL  = [ _('Usage for ia_service.py:')
                 , _('  ia_service.py -r getlogs       Runs the Log Collection utility by specifying components')
                 , _('  ia_service.py -r trace         Runs the Trace Config utility')
                 , _('  ia_service.py -r verify        Validation tests for IA hardware, software and networking')
                 , _('  ia_service.py -r director      Configures the IBM Director software')
                 , _('  ia_service.py -r rsm           Configures the RSM server')
                 , _('  ia_service.py -r util          Assortment of Service related utilities')
                 , _('  ia_service.py -r systemx       Configures the System X nodes')
                 , _('  ia_service.py -r storage       Configures the disk controller subsystems')
                 , _('  ia_service.py -r san           Configures the Fibre Channel SAN switches')
                 , _('  ia_service.py -r enet          Configures the Ethernet switches')
                 , _('        Add "-h" or "--help" flag to see detailed usage for each command')
                 ]

#@trans The -r runapp values (for example, "-r getlogs") should NOT be translated.
#@trans The one character switch arguments (for example, "-c", "-n", "-i", "-f", "-t") should NOT be translated.
#@trans The -t threshold values ("on" and "off") should NOT be translated.
#@trans Any values in double quotes (for example, "full") should NOT be translated.
#@trans The terms "comp", "node" and "filename" can be translated.
USAGE_GETLOGS  = [ _('Usage for "ia_service.py -r getlogs":')
                 , _('  ia_service.py -r getlogs       Runs the Log Collection utility')
                 , _('        -c comp                  Sets target component')
                 , _('                                   comp can be "full", "min", "varlog", "director", "storage", "gpfs", "gui", "dsa", "hsm", "san", "middleware" or "everything"')
                 , _('                                   Selecting "everything" runs collections for all components on all nodes, and can take a long time')
                 , _('        [-n node]                Run log collector on a specific node, valid for components "min", "full", "dsa" and "hsm"')
                 , _('                                   node can be "iarsm1", "iamconsole1", "ianode#"')
                 , _('        [-d rsm | usb ]          Copies the log collection output to the specified destination.  The default is to copy')
                 , _('                                   the output to the Management Console under the /home/iaadmin directory.')
                 , _('                                   Specifying "rsm" will copy the logs to the RSM Server node under the /home/iaadmin directory.')
                 , _('                                   Specifying "usb" will mount a flash drive in the /dev/sdb1 USB port on the node where ia_service')
                 , _('                                   is invoked, copy the results to it, and unmount the flash drive when finished.')
                 , _('        [-t on | off]            Turns thresholds and timeouts on or off for components "full" and "min"')
                 ]

#@trans The -r runapp values (for example, "-r trace") should NOT be translated.
#@trans The one character switch arguments (for example, "-c", "-l", "-n") should NOT be translated.
#@trans Any values in double quotes (for example, "all") should NOT be translated.
#@trans The terms "comp", "level" and "node" can be translated.
USAGE_TRACE    = [ _('Usage for "ia_service.py -r trace":')
                 , _('  ia_service.py -r trace         Runs the Trace Config utility')
                 , _('        -c comp                  Sets target component')
                 , _('                                   comp can be "all" to apply to all components,')
                 , _('                                   comp can be "list" to see all valid components with current and default level')
                 , _('                                   comp can be "tsm.client", "hsm", "dir.agent", "dir.server", "gpfs", "mcp", "rsm", "sles", "cifs" or "nfs"')
                 , _('        -l level                   Sets trace level')
                 , _('                                   level can be "min", "mid", or "max" to set the trace level to the specified value')
                 , _('                                   level can be "disable" to turn off tracing for the specified component')
                 , _('                                   level can be "current" to re-apply trace level to its current value')
                 , _('                                   level can be "default" to set trace level back to default setting')
                 , _('        [-n node]                Sets trace level on a specific node: valid for components "all", "mcp", "hsm", "nfs", "cifs" or "dir.agent"')
                 , _('                                   node can be "iarsm1", "iamconsole1", "ianode#"')
                 ]

#@trans The -r runapp values (for example, "-r verify") should NOT be translated.
#@trans The -c component values (hardware, software) should NOT be translated.
USAGE_VERIFY   = [ _('Usage for "ia_service.py -r verify":')
                 , _('        -c hardware              Runs the IA Wellness Check utility')
                 , _('        -c software              Runs the IA Setup Check utility')
                 , _('        -c network               Display health status for IA network components')
                 ]

#@trans The -r runapp values (for example, "-r director") should NOT be translated.
#@trans The -c component values (agent, server, smis) should NOT be translated.
#@trans The -a action values (config, start, stop, etc) should NOT be translated.
USAGE_DIRECTOR = [ _('Usage for "ia_service.py -r director":')
                 , _('        -c agent                 IBM Director Agent actions')
                 , _('          -a config              Configures the Director Agent')
                 , _('          -a start|stop|status   Starts, Stops or queries status of the Director Agent')
                 , _('        -c server                IBM Director Server actions')
                 , _('          -a start|stop|status   Starts, Stops or queries status of the Director Server')
                 , _('          -a reset               Reset the IBM Director Server to the original install values')
                 , _('          -a query_errors        Query the current number of critical errors and warnings')
                 , _('          -a inventory           Initiates the default IA inventory collection for Director Server')
                 , _('          -a groups              Create all the Director groups needed for IA')
                 , _('          -a config              Configures the Director Server')
                 , _('          -a unlock              Unlocks an IBM Director GUI account that has been frozen')
                 , _('             [ -u userid ]       Optional user ID (default is "iaadmin"')
                 , _('        -c smis                  IBM Director SMI-S Provide actions')
                 , _('          -a config              Configures the SMI-S Provider')
                 ]


#@trans The -r runapp values (for example, "-r rsm") should NOT be translated.
#@trans The -c component values (server) should NOT be translated.
#@trans The -a action values (start, stop) should NOT be translated.
USAGE_RSM      = [ _('Usage for "ia_service.py -r rsm":')
                 , _('        -c server                IBM Remote Support Manager (RSM) Server actions')
                 , _('          -a start|stop          Starts/Restarts or Stops the RSM Server')
                 ]

#@trans The -r runapp values (for example, "-r util") should NOT be translated.
#@trans The -c component values (ssh, usb_drive, system) should NOT be translated.
#@trans The -a action values (regen_keys, mount, umount, dates, nodes) should NOT be translated.
#@trans The term "node" can be translated.
USAGE_UTIL     = [ _('Usage for "ia_service.py -r util":')
                 , _('        -c ssh                   Runs ssh utilities')
                 , _('           -a regen_keys         Triggers regeneration of the appliance ssh keys')
                 , _('        -c usb_drive             Mounts and unmounts the USB drive on the nodes')
                 , _('           -a mount  -n node     Mounts the USB drive on the specified node')
                 , _('           -a umount -n node     Unmounts the USB drive on the specified node')
                 , _('        -c system                Assorted system utilities')
                 , _('           -a dates              Gets the datestamp of all the appliance nodes')
                 , _('           -a nodes              Regenerates the /opt/tivoli/tiam/etc/nodes file on MC')
                 ]

#@trans The -r runapp values (for example, "-r systemx") should NOT be translated.
#@trans The -c component values (base, imm) should NOT be translated.
#@trans The -a action values (mtm, config) should NOT be translated.
#@trans The terms "node", "model_number", and "serial_number" can be translated.
USAGE_SYSTEMX  = [ _('Usage for "ia_service.py -r systemx":')
                 , _('        -c base                  Utilities for the base System X node')
                 , _('          -a mtm                 Sets the MTM info for the node')
                 , _('            -m model_number      Sets the model number for the node')
                 , _('            -s serial_number     Sets the serial number for the node')
                 , _('        -c imm                   Configure the IMM adapter on the specified node')
                 , _('          -a config              Configures the IMM adapter on the node')
                 , _('        [ -n node ]              Routes the command to the specified node for execution (default is current node)')
                 ]

#@trans The -r runapp values (for example, "-r storage") should NOT be translated.
#@trans The -c component values (1, 2, 3) should NOT be translated.
#@trans The -a action values (firmware, config, format) should NOT be translated.
USAGE_STORAGE  = [ _('Usage for "ia_service.py -r storage":')
                 , _('        -c 1|2|3                 Determines which Storage Controller is affected by this change')
                 , _('           -a config             Configures the Storage Controller')
                 , _('           -a firmware           Finds and applies firmware upgrade to the Storage Controller')
                 , _('           -a addhostports       Adds host ports to the Storage Controller')
                 ]

#@trans The -r runapp values (for example, "-r san") should NOT be translated.
#@trans The -c component values (1, 2) should NOT be translated.
#@trans The -a action values (firmware, config, ip, reset) should NOT be translated.
#@trans The term "version" can be translated.
USAGE_SAN      = [ _('Usage for "ia_service.py -r san":')
                 , _('        -c 1|2                   Determines which SAN switch is affected by this change')
                 , _('           -a ip                 Set the IP address of the SAN switch')
                 , _('           -a reset              Resets the SNMP trap target information for the SAN switch to factory defaults')
                 , _('           -a config             Updates the SNMP trap target information for the SAN switch')
                 , _('           -a firmware           Applies firmware upgrade to the SAN switch')
                 , _('             -f version          Determines version of firmware applied to SAN switch (for example, "-f 6.1.2")')
                 ]

#@trans The -r runapp values (for example, "-r enet") should NOT be translated.
#@trans The -c component values (1, 2) should NOT be translated.
#@trans The -a action values (firmware, config) should NOT be translated.
USAGE_ENET     = [ _('Usage for "ia_service.py -r enet":')
                 , _('        -c 1|2                   Determines which Ethernet switch is affected by this change')
                 , _('           -a config             Configures the Ethernet switch')
                 , _('           -a firmware           Applies firmware upgrade to the Ethernet switch')
                 ]

# Translatable messages without a TMS prefix:
DEVICE_ETHERNET  = _('Ethernet switch')
DEVICE_SAN       = _('Fibre Channel switch')
DEVICE_STORAGE   = _('Storage Controller')
DEVICE_SYSTEMX   = _('System X node')
DEVICE_IMM       = _('IMM adapter')

MSG_DIR_ACTIVE   = _('Active')
MSG_DIR_STARTING = _('Starting')
MSG_DIR_STOPPING = _('Stopping')
MSG_DIR_INACTIVE = _('Inactive')
MSG_DIR_FAILED   = _('Failed')
MSG_DIR_UPDATING = _('Updating')

# Other translatable messages:
MSG_SUCC_GEN     = _('CTJIB1000I Command completed successfully.')
MSG_FAIL_GEN     = _('CTJIB1001E Command failed with return code %d.')
MSG_LC_TXFR      = _('CTJIB1002I Log collection complete. Transferring file to Management Server node.')
MSG_LC_TXFR_DEST = _('CTJIB1003I Copying log collection output to requested destination "%s".')
MSG_LC_RUNNING   = _('CTJIB1004W Another instance of Log Collection is already running.')
MSG_SUCC_LC      = _('CTJIB1005I File transfer complete. Output is on Management Server node in directory: %s')
MSG_SUCC_LC_DEST = _('CTJIB1006I File transfer complete. Output has been copied to requested destination "%s" under directory: %s')
MSG_FAIL_LC_DEST = _('CTJIB1007E Unable to copy ouput to requested destination "%s".  The logs are available on the Management Server node under directory: %s')
MSG_FAIL_LC      = _('CTJIB1008E Log Collection failed with return code %d.')
MSG_FAIL_LOGS    = _('CTJIB1009I Information about the failure can be found in the logs: %s')
MSG_FAIL_XFER    = _('CTJIB1010E File transfer failed with return code %d.')
MSG_SUCC_VFY     = _('CTJIB1011I Verification of IA %s completed successfully.')
MSG_FAIL_VFY     = _('CTJIB1012E Verification of IA failed with return code %d.')
MSG_SUCC_MNT     = _('CTJIB1013I Command to %s USB drive completed successfully.')
MSG_FAIL_MNT     = _('CTJIB1014E Command to %s USB drive failed with return code %d.')
MSG_SUCC_IMM     = _('CTJIB1015I Configuration of IMM adapter on node %s completed successfully.')
MSG_FAIL_IMM     = _('CTJIB1016E Attempt to configure IMM adapter failed with return code %d.')
MSG_SUCC_SAN     = _('CTJIB1017I Fibre Channel switch command "%s" completed successfully.')
MSG_FAIL_SAN     = _('CTJIB1018E Attempt to configure Fibre Channel switch failed with return code %d.')
MSG_SUCC_ENET    = _('CTJIB1019I Ethernet switch command "%s" completed successfully.')
MSG_FAIL_ENET    = _('CTJIB1020E Attempt to configure Ethernet switch failed with return code %d.')
MSG_SUCC_STOR    = _('CTJIB1021I Storage Controller command "%s" completed successfully.')
MSG_FAIL_STOR    = _('CTJIB1022E Attempt to configure Storage Controller failed with return code %d.')
MSG_SUCC_DIR     = _('CTJIB1023I Command to %s IBM Director completed successfully.')
MSG_SUCC_UNLOCK  = _('CTJIB1024I The IBM Director account was successfully unlocked: %s')
MSG_FAIL_DIR     = _('CTJIB1025E Command to %s IBM Director failed with return code %d.')
MSG_FAIL_USB_MOUNT   = _('CTJIB1026E Unable to mount the USB drive.')
MSG_FAIL_USB_COPY    = _('CTJIB1027E Unable to copy files to the USB drive.')
MSG_FAIL_USB_UNMOUNT = _('CTJIB1028E Unable to unmount the USB drive.')

MSG_BAD_TRIPLET  = _('CTJIB1040E No IA scripts match this combination of parameters: %s')
MSG_BAD_ACTION   = _('CTJIB1041E Action is not supported for this combination of parameters: %s')
MSG_BAD_N_SWITCH = _('CTJIB1042E Switch "-n" is not supported for this combination of parameters: %s')
MSG_BAD_N_COMP   = _('CTJIB1043E Switch "-n" is not supported for this component: %s')
MSG_BAD_ROUTE    = _('CTJIB1044E Unable to contact host where command should be executed: %s')
MSG_PARSE_ERROR  = _('CTJIB1045E Parser failure processing ia_service command line.')
MSG_BAD_SCRIPT   = _('CTJIB1046E Unrecognized script name: %s')
MSG_BAD_NODE     = _('CTJIB1047E Unrecognized node name: %s')
MSG_BAD_LEVEL    = _('CTJIB1048E Value for "-l" must be one of "min", "mid", "max", "disable", "current" or "default".')
MSG_MISSING_ARG  = _('CTJIB1049E Switch "%s" must be provided on the command line.')

MSG_WAIT_INV     = _('CTJIB1060I Running Director inventory command. This may take several minutes.')
MSG_WAIT_LC      = _('CTJIB1061I Running "%s" log collection. This may take several minutes.')
MSG_WAIT_LC_ONE  = _('CTJIB1062I Running "%s" log collection on node %s. This may take several minutes.')
MSG_WAIT_LC_ALL  = _('CTJIB1063I Running log collection for every component on every node. This may take over an hour.')
MSG_WAIT_TC      = _('CTJIB1064I Routing trace config command to %s. This may take several minutes.')
MSG_WAIT_SWITCH  = _('CTJIB1065I Connecting to %s to perform configuration. This may take several minutes.')
MSG_WAIT_VFY     = _('CTJIB1066I Running %s verification command. This may take several minutes.')
MSG_WAIT_SSH     = _('CTJIB1067I Regenerating SSH keys. This may take several minutes.')
MSG_SAME_FW      = _('CTJIB1068W The firmware is already at the requested level. No action is being taken.')
MSG_NO_STORAGE   = _('CTJIB1069W No storage controllers were found on this appliance.')
MSG_FAIL_TELNET  = _('CTJIB1070E Unable to telnet to switch. Configuration needs to be performed using a serial cable.')
MSG_FAIL_NO_FW   = _('CTJIB1071E Unable to locate the following firmware files on node %s:')
MSG_RETRY_FW     = _('CTJIB1072I Ensure the files exist before retrying the command.')
MSG_QUERY_PREFIX = _('CTJIB1073I Number of Critical Errors, Number of Warnings:')


# user id authorized to ssh among IA nodes for executing serviceability commands
sshUser              = 'iaadmin'
IaServiceLogFile     = '/var/opt/tivoli/tiam/log/MBTrace.log'
sudoPrefix           = ''
sshPrefix            = ''
suToIaadmin          = False
targetNode = None

################################################################################
# Function: main
# This function is the main method for the ia_service wrapper application.  It
# is called by the management broker's main() wrapper method.
################################################################################
def main():

    # initialize command line switch variables
    usageRequested = False;
    runapp = ''                    # -r (runapp)
    component = '[none]'           # -c (component) [none] is used in the map for values that should not be specified
    action = '[none]'              # -a (action)
    node = ''                      # -n (node)
    trace_level = ''               # -l min|mid|max|disable (trace level)
    log_thresholds = ''            # -t on|off (thresholds)
    switch_firmware = ''           # -f (switch version)
    dir_user = ''                  # -u (Director username)
    dir_group = ''                 # -g (Director groupname)
    mtm_model = ''                 # -m (IA model number)
    mtm_serial = ''                # -s (IA serial number)
    destination = ''               # -d (destination)
    parseAndQuit = False           # -q (parse and quit)  ... test purposes only
    ermSpecified = False           # --erm (use the "B" box ip addresses)
    errorMessage = None
    parseResult = 0

    # log the command line arguments for help debugging ssh/sudo issues
    logAndPrint('command line: ' + str(sys.argv))

    # process the command line switches and assign values to local variables
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'r:c:a:n:l:t:u:d:g:f:m:s:qhv', ['erm', 'help'])
    except getopt.GetoptError, err:
        logAndPrint(str(err), True)
        showUsage()

    for opt, arg in opts:
        if opt in ('-r'):
            runapp = arg
        elif opt in ('-c'):
            component = arg
        elif opt in ('-a'):
            action = arg
        elif opt in ('-n'):
            node = arg
        elif opt in ('-l'):
            trace_level = arg
        elif opt in ('-a'):
            if arg=='on':
                log_thresholds = 'off'
        elif opt in ('-t'):
            log_thresholds = arg
        elif opt in ('-u'):
            dir_user = arg
        elif opt in ('-g'):
            dir_group = arg
        elif opt in ('-f'):
            switch_firmware = arg
        elif opt in ('-m'):
            mtm_model = arg
        elif opt in ('-s'):
            mtm_serial = arg
        elif opt in ('-d'):
            destination = arg
        elif opt in ('-v'):
            setVerboseOutput(True)
        elif opt in ('-h') or opt in ('--help'):
            usageRequested = True;
        elif opt in ('-q'):
            parseAndQuit = True;
        elif opt in ('--erm'):
            ermSpecified = True;
        else:
            showUsage()

    # if '-h' was entered on the comand line, the user will either see the general purpose usage
    # message, or if '-r' was also entered, they will see usage specific to the app matching '-r'
    if usageRequested==True:
        showUsage(runapp);                    # ia_service exits from showUsage call

    # Parse the service.properties file into a couple of maps, each of which has the
    # Runapp/Component/Action flags as the key.  Then use the RCA triplet to retrieve
    # the script and node values.
    # - The "script map" maps the R/C/A values to the underlying script to run
    # - The "node map" maps the R/C/A vaules to the node(s) to which the script should be routed for execution.
    rcaScriptMap, rcaNodeMap = readServicePropertiesFile('/opt/tivoli/tiam/etc/service.properties')
    searchKey = runapp + '/' + component + '/' + action
    runScript = rcaScriptMap.get(searchKey)
    runOnNode = rcaNodeMap.get(searchKey)        # this will return None for an invalid R/C/A combination

    # construct a string used for some parser error messages
    triplet = '-r '+runapp
    if component != '[none]':
        triplet = triplet + ' -c '+component
    if action != '[none]':
        triplet = triplet + ' -a '+action
    triplet = '"'+triplet+'"'


    # Did the RCA map to a known script?
    if runScript == None:
        errorMessage = MSG_BAD_TRIPLET % triplet
        parseResult = parseResult+1
    else:
        logAndPrint('That combination of switches ('+searchKey+') maps to the script '+runScript+'; runOnNode=['+runOnNode+']')

        # Determine whether or not this command needs to be prefixed with sudo by searching for the "s" flag in the service.properties metadata
        if (runOnNode.find('s') != -1):
            global sudoPrefix
            sudoPrefix = 'sudo '

        # Determine whether this command is running as root.  If so, we must su to iaadmin
        global suToIaadmin
        if pwd.getpwuid(os.getuid())[0] == 'root':
            logAndPrint('ia_service invoked as root: adding su prefix to change to iaadmin ')
            suToIaadmin = True

        # Find the appropriate node on which to execute this command.  See the findRoutingNode
        # function header for a description of the algorithm used to find the target node.
        global targetNode
        targetNode = findRoutingNode(runapp, component, node, runOnNode)
        if targetNode == None:
            errorMessage = MSG_BAD_N_SWITCH % (triplet)
            parseResult=parseResult+1

        # If the script needs to run on a different node, prepend an ssh command to route it
        global sshPrefix
        if (targetNode!=None and targetNode!=getIAhostname()):
            if ping(targetNode):
                sshPrefix = 'ssh '+sshUser+'@'+targetNode
            else:
                errorMessage = MSG_BAD_ROUTE % targetNode
                parseResult=parseResult+1
        else:
            sshPrefix=''

    # if '-q' was specified on the command line, exit quietly now ... this is an undocumented unit test flag
    if parseAndQuit==True:
        errorExit(parseResult)

    # if the parsing failed, exit with an error message and usage instructions
    if parseResult > 0:
        if errorMessage==None:
            errorMessge = MSG_PARSE_ERROR
        logAndPrint(errorMessage, True)
        showUsage(runapp)                    # ia_service exits from showUsage call

    #########################################################################################################
    # Generic command line mapping and routing determination is complete.  Now we validate arguments
    # specific to the individual command, and if that succeeds, we route and execute the command.
    #########################################################################################################

    if runScript == '/opt/tivoli/tiam/bin/ia_trace.py':
        result = runTraceCommand(runScript, component, trace_level, node)

    elif runScript == '/opt/tivoli/tiam/bin/ia_logcollector.py':
        result = runCollectCommand(runScript, component, log_thresholds, node, destination)

    elif runScript == '[CollectEverything]':
        result = runCollectAllCommands(destination)

    elif (runScript == '/opt/tivoli/tiam/bin/ia_run_guilogs.sh'
       or runScript == '/opt/tivoli/tiam/bin/ia_run_grablogs.sh'
       or runScript == '/opt/tivoli/tiam/bin/ia_run_storage.sh'
       or runScript == '/opt/tivoli/tiam/bin/ia_run_hsmcollect.sh'
       or runScript == '/opt/tivoli/tiam/bin/ia_run_san.sh'
       or runScript == '/opt/tivoli/tiam/bin/ia_run_gpfssnap.sh'
       or runScript == '/opt/tivoli/tiam/bin/ia_run_middleware.sh'
       or runScript == '/opt/tivoli/tiam/bin/ia_run_dsa.sh'):
        result = runGetLogsScript(runScript, component, destination)

    elif (runScript == '/opt/ibm/director/bin/testMOStatus.sh'):
        result = runWithOutputScript(runScript, MSG_QUERY_PREFIX)

    elif (runScript == '/opt/tivoli/tiam/bin/ia_getalldates.sh'):
        result = runWithOutputScript(runScript)

    elif (runScript == '/opt/tivoli/tiam/bin/network_health.sh'):
        result = runVerifyCommand(runScript, component, ermSpecified)

    elif (runScript == '/opt/tivoli/tiam/bin/verify_wellness'
       or runScript == '/opt/tivoli/tiam/bin/run_setupcheck.sh') :
        result = runVerifyCommand(runScript, component, ermSpecified)

    # todo: Defect 14140 opened for getting correct return codes
    elif (runScript == '/opt/tivoli/tiam/bin/mount_usb.py'
       or runScript == '/opt/tivoli/tiam/bin/umount_usb.py'):
        result = runUsbCommand(runScript, action)

    elif runScript == '/opt/tivoli/tiam/bin/snmp_config/ia_imm_config.sh':
        result = runImmCommand(runScript, runapp, node, ermSpecified)

    elif (runScript == '/opt/tivoli/tiam/bin/ia_dir_agent.sh'
       or runScript == '/opt/tivoli/tiam/bin/ia_dir_server.sh'
       or runScript == '/opt/tivoli/tiam/bin/setupDirGroups.sh'
       or runScript == '/opt/tivoli/tiam/bin/ia_dirconfig.sh'
       or runScript == '/opt/tivoli/tiam/bin/ia_setup_storage.sh'
       or runScript == '/opt/tivoli/tiam/bin/ia_configure_common_agent.sh'):
        result = runDirectorScript(runScript, component, action)

    elif runScript == '/opt/tivoli/tiam/bin/ia_collect_inventory.sh':
        result = runInventoryScript(runScript, action)

    elif (runScript == '/opt/tivoli/tiam/bin/ia_force_ssh_key_regen'):
        result = runGeneralScript(runScript, component, '')

    elif (runScript == '/opt/tivoli/tiam/bin/ia_update_nodes_file.py'):
        result = runGeneralScript(runScript, component)

    elif runScript == '/opt/tivoli/tiam/bin/ia_iss.sh':
        assertArgument(runapp, '-m', mtm_model)
        assertArgument(runapp, '-s', mtm_serial)
        result = runGeneralScript(runScript, component, ' -m '+mtm_model+' -s '+mtm_serial)

    elif (runScript == '/opt/tivoli/tiam/bin/ia_firmware_storagesubsystem.sh'
       or runScript == '/opt/tivoli/tiam/bin/ia_addhostports.sh'
       or runScript == '/opt/tivoli/tiam/bin/ia_cfg_storagesubsystem.sh'
       or runScript == '/opt/tivoli/tiam/bin/ia_mes_storagesubsystem.sh'):
        result = runStorageCommand(runScript, component, action, ermSpecified)

    elif (runScript == '/opt/tivoli/tiam/bin/snmp_config/ia_telnet_san_fw.py'
       or runScript == '/opt/tivoli/tiam/bin/snmp_config/ia_telnet_san_ip.py'
       or runScript == '/opt/tivoli/tiam/bin/snmp_config/ia_telnet_san_snmp_reset.py'
       or runScript == '/opt/tivoli/tiam/bin/snmp_config/ia_telnet_san_snmp.py'):
        result = runSanCommand(runScript, component, action, switch_firmware, ermSpecified)

    elif (runScript == '/opt/tivoli/tiam/bin/snmp_config/ia_telnet_smc_fw.py'
       or runScript == '/opt/tivoli/tiam/bin/snmp_config/ia_telnet_smc_config.py'):
        result = runEnetCommand(runScript, component, action, ermSpecified)

    elif runScript == '[UnlockDirector]':
        result = runUnlockDirectorCommand(action, dir_user)

    elif runScript == '[RsmCommand]':
        result = runRsmCommand(action)

    else:
        logAndPrint(MSG_BAD_SCRIPT % (runScript), True)
        showUsage(runapp)                    # ia_service exits from showUsage call

    return result

################################################################################
# runWithOutputScript called when the request to run a command should allow
# the command's output to be echo'd to the screen.  No return code processing
# happens here.
################################################################################
def runWithOutputScript (runScript, preOutput=''):
    command = sudoPrefix+runScript
    result = runCommand(command, sshPrefix, suToIaadmin, True, preOutput)

    # this command has no return code messages.  It already echo'd the datestamps to the screen

    return result

################################################################################
# runVerifyCommand called when '-r verify' is processed
################################################################################
def runVerifyCommand (runScript, component, erm):

    toScreen = False
    if component == 'hardware':
        if erm or isBconfig():
            secondary  = ' -s y'
            additions  = ' -a y'
        else:
            secondary  = ' -s n'
            additions  = ' -a y'

        command = sudoPrefix+runScript+secondary+additions
    elif component == 'software':
        command = sudoPrefix+runScript
    elif component == 'network':
        command = sudoPrefix+runScript
        toScreen = True

    logAndPrint(MSG_WAIT_VFY % (component), True)
    result = runCommand(command, sshPrefix, suToIaadmin, toScreen)

    # process results
    if result==0:
        logAndPrint(MSG_SUCC_VFY % (component), True)
    else:
        logAndPrint(MSG_FAIL_VFY % (result), True)
        logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)

    return result

################################################################################
# runUsbCommand called when '-r util -c usb_drive' is processed
################################################################################
def runUsbCommand (runScript, action):

    if action == 'mount':
        args = ' -d /dev/sdb1 -w'
    else:
        args = ' -d /dev/sdb1'

    command = sudoPrefix+runScript+args
    result = runCommand(command, sshPrefix, suToIaadmin)

    # process results
    if result==0:
        logAndPrint(MSG_SUCC_MNT % (action), True)
    else:
        logAndPrint(MSG_FAIL_MNT % (action, result), True)
        logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)

    return result

################################################################################
# runImmCommand called when '-r systemx -c imm' is processed.  The IMM commands
# always get routed to the MC node for execution, but the IMM that gets
# configured is identified by the "-n" switch.  So, for example, the command
#    ia_service.py -r systemx -c imm -a config -n ianode2
# gets routed to the MC node.  From the MC node, the IMM adapter on ianode2 gets
# configured over telnet using the IMM's IP address (172.31.0.102).
################################################################################
def runImmCommand (runScript, runapp, node, erm):
    # Convert the '-n node' into the IP Address of that node's IMM card
    ipAddressA, ipAddressB = readIpAddressPropertiesFile('/opt/tivoli/tiam/etc/ip_addresses.properties')

    # If the "-n" switch is not supplied, it is assumed that the desired target is the IMM
    # card on the localhost, so that IP address is identified and added to the commanfd line.
    if node=='':
        node=getIAhostname()

    if erm or isBconfig():
        immAddress = ipAddressB.get(node+'_imm')
    else:
        immAddress = ipAddressA.get(node+'_imm')

    if immAddress == None:
        logAndPrint(MSG_BAD_NODE % (node), True)
        showUsage(runapp)                    # ia_service exits from showUsage call

    command = sudoPrefix+runScript+' '+immAddress
    logAndPrint(MSG_WAIT_SWITCH % (DEVICE_IMM), True)
    result = runCommand(command, sshPrefix, suToIaadmin)

    # process results
    if result==0:
        logAndPrint(MSG_SUCC_IMM % (node), True)
    else:
        logAndPrint(MSG_FAIL_IMM % (result), True)
        logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)

    return result

################################################################################
# runDirectorScript called when a Director script is called.
################################################################################
def runDirectorScript (runScript, component, action):
    command = sudoPrefix+runScript
    if (action != '[none]' and action != ''):
        command = command + ' ' + action

    result = runCommand(command, sshPrefix, suToIaadmin)

    if action=='status':
        if component=='server':
            if result==0:
                logAndPrint(MSG_DIR_ACTIVE, True)
            elif result==1:
                logAndPrint(MSG_DIR_STARTING, True)       # inexplicably, Director Agent and Server use different numbers for STARTING and INACTIVE
            elif result==2:
                logAndPrint(MSG_DIR_STOPPING, True)
            elif result==3:
                logAndPrint(MSG_DIR_INACTIVE, True)
            elif result==4:
                logAndPrint(MSG_DIR_FAILED, True)
            elif result==6:
                logAndPrint(MSG_DIR_UPDATING, True)
        elif component=='agent':
            if result==0:
                logAndPrint(MSG_DIR_ACTIVE, True)
            elif result==2:
                logAndPrint(MSG_DIR_INACTIVE, True)
            elif result==4:
                logAndPrint(MSG_DIR_STARTING, True)
    else:
        if result==0:
            logAndPrint(MSG_SUCC_DIR % (action), True)
        else:
            logAndPrint(MSG_FAIL_DIR % (action, result), True)
            logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)

    return result

################################################################################
# runInventoryScript called when the Director inventory script is called.
# default for this script is to collect "all" inventory, but script can be
# augmented if necessary to support additional arguments.
################################################################################
def runInventoryScript (runScript, action):

    logAndPrint(MSG_WAIT_INV, True)
    command = sudoPrefix+runScript+' -a'
    result = runCommand(command, sshPrefix, suToIaadmin)

    if result==0:
        logAndPrint(MSG_SUCC_DIR % (action), True)
    else:
        logAndPrint(MSG_FAIL_DIR % (action, result), True)
        logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)

    return result

################################################################################
# runGeneralScript called when any script is triggered which takes
# zero or one arguments on the command line.
################################################################################
def runGeneralScript (runScript, comp, args=''):
    command = sudoPrefix+runScript+args

    if comp=='ssh':
        logAndPrint(MSG_WAIT_SSH, True)

    result = runCommand(command, sshPrefix, suToIaadmin)

    # process results
    if result==0:
        logAndPrint(MSG_SUCC_GEN, True)
    else:
        logAndPrint(MSG_FAIL_GEN % (result), True)
        logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)

    return result

################################################################################
# runEnetCommand called when '-r enet' is processed
################################################################################
def runEnetCommand (runScript, component, action, erm):
    ipAddressA, ipAddressB = readIpAddressPropertiesFile('/opt/tivoli/tiam/etc/ip_addresses.properties')

    if erm or isBconfig():
        hostname   = 'enet' + component
        switchAddr = ipAddressB.get(hostname)
        mcAddr     = ipAddressB.get('iamconsole1')
        rFlag      = ' -r'
    else:
        hostname   = 'enet' + component
        switchAddr = ipAddressA.get(hostname)
        mcAddr     = ipAddressA.get('iamconsole1')
        rFlag      = ''

    if action == 'config':
        command = sudoPrefix+runScript+' -n '+switchAddr+' -s '+component+' -t '+mcAddr+rFlag
    elif action == 'firmware':
        command = sudoPrefix+runScript+' '+switchAddr+' '+mcAddr

    logAndPrint(MSG_WAIT_SWITCH % (DEVICE_ETHERNET), True)
    result = runCommand(command, sshPrefix, suToIaadmin)

    # process results
    if result==0:
        logAndPrint(MSG_SUCC_ENET % (action), True)
    elif result==12:
        logAndPrint(MSG_SAME_FW, True)
        result = 0                          # don't return an error result if the firmware is already at the correct level
    elif result==13:
        logAndPrint(MSG_FAIL_TELNET, True)
    elif result==15:
        # Filename would need to change if ia_telnet_smc_fw.py ever changes how it constructs the firmware filename.
        logAndPrint(MSG_FAIL_NO_FW % (targetNode), True)
        logAndPrint('   /home/iaadmin/firmware/smc/SMC8126_50R_opcode_V*.bix', True)
        logAndPrint(MSG_RETRY_FW, True)
    else:
        logAndPrint(MSG_FAIL_ENET % (result), True)
        logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)

    return result

################################################################################
# runSanCommand called when '-r san' is processed
################################################################################
def runSanCommand (runScript, component, action, fwVersion, erm):
    ipAddressA, ipAddressB = readIpAddressPropertiesFile('/opt/tivoli/tiam/etc/ip_addresses.properties')

    if erm or isBconfig():
        hostname   = 'san' + component
        switchAddr = ipAddressB.get(hostname)
        mcAddr     = ipAddressB.get('iamconsole1')
        configFlag = ' remote'
    else:
        hostname   = 'san' + component
        switchAddr = ipAddressA.get(hostname)
        mcAddr     = ipAddressA.get('iamconsole1')
        configFlag = ' primary'

    if action == 'reset':
        command = sudoPrefix+runScript+' '+switchAddr
    elif action == 'config':
        command = sudoPrefix+runScript+' '+switchAddr+configFlag
    elif action == 'ip':
        command = sudoPrefix+runScript+' -n '+switchAddr
    elif action == 'firmware':
        # -f switch is mandatory
        assertArgument('san', '-f', fwVersion)
        command = sudoPrefix+runScript+' '+switchAddr+' '+mcAddr+' '+fwVersion

    logAndPrint(MSG_WAIT_SWITCH % (DEVICE_SAN), True)
    result = runCommand(command, sshPrefix, suToIaadmin)

    # process results
    if result==0:
        logAndPrint(MSG_SUCC_SAN % (action), True)
    elif result==13:
        logAndPrint(MSG_FAIL_TELNET, True)
    elif result==15:
        # Filename would need to change if ia_telnet_san_fw.py ever changes how it constructs the firmware filename.
        logAndPrint(MSG_FAIL_NO_FW % (targetNode), True)
        logAndPrint('   /home/iaadmin/firmware/brocade/v'+fwVersion+'.tar.gz', True)
        logAndPrint(MSG_RETRY_FW, True)
    else:
        logAndPrint(MSG_FAIL_SAN % (result), True)
        logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)

    return result


################################################################################
# runStorageCommand called when '-r storage' is processed
################################################################################
def runStorageCommand (runScript, component, action, erm):
    ipAddressA, ipAddressB = readIpAddressPropertiesFile('/opt/tivoli/tiam/etc/ip_addresses.properties')

    if erm or isBconfig():
        hostname1 = 'disk_ctrl_' + component + '_a'
        hostname2 = 'disk_ctrl_' + component + '_b'
        ipAddr1 = ipAddressB.get(hostname1)
        ipAddr2 = ipAddressB.get(hostname2)
    else:
        hostname1 = 'disk_ctrl_' + component + '_a'
        hostname2 = 'disk_ctrl_' + component + '_b'
        ipAddr1 = ipAddressA.get(hostname1)
        ipAddr2 = ipAddressA.get(hostname2)


    command = sudoPrefix+runScript+' -a '+ipAddr1+' -b '+ipAddr2
    logAndPrint(MSG_WAIT_SWITCH % (DEVICE_STORAGE), True)
    result = runCommand(command, sshPrefix, suToIaadmin)

    # process results
    if result==0:
        logAndPrint(MSG_SUCC_STOR % (action), True)
    elif result==12:
        logAndPrint(MSG_SAME_FW, True)
        result = 0                          # don't return an error result if the firmware is already at the correct level
    elif result==39:
        # Filename would need to change if ia_storage_common.sh ever changes how it constructs the firmware filename.
        logAndPrint(MSG_FAIL_NO_FW % (targetNode), True)
        logAndPrint('   /home/iaadmin/firmware/DS3950/RC*.dlp', True)
        logAndPrint('   /home/iaadmin/firmware/DS3950/N*.dlp', True)
        logAndPrint('   /home/iaadmin/firmware/DS3950/esm*.esm', True)
        logAndPrint(MSG_RETRY_FW, True)
    elif result==42:
        # Filename would need to change if ia_storage_common.sh ever changes how it constructs the firmware filename.
        logAndPrint(MSG_FAIL_NO_FW % (targetNode), True)
        logAndPrint('   /home/iaadmin/firmware/DS4200/FW*.dlp', True)
        logAndPrint('   /home/iaadmin/firmware/DS4200/N*.dlp', True)
        logAndPrint('   /home/iaadmin/firmware/DS4200/esm*.esm', True)
        logAndPrint(MSG_RETRY_FW, True)
    else:
        logAndPrint(MSG_FAIL_STOR % (result), True)
        logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)

    return result

################################################################################
# runUnlockDirector called when '-r director -c server -a unlock' is processed
################################################################################
def runUnlockDirectorCommand (action, userid='iaadmin'):

    # userid might be empty string ... set to default
    if userid=='':
        userid = 'iaadmin'

    command = sudoPrefix+'smcli chuser -m unlock '+userid
    result = runCommand(command, sshPrefix, suToIaadmin)

    # process results
    if result==0:
        logAndPrint(MSG_SUCC_UNLOCK % (userid), True)
    else:
        logAndPrint(MSG_FAIL_DIR % (action, result), True)
        logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)

    return result


################################################################################
# runRsmCommand called when '-r rsm -c server -a start|stop' is processed
################################################################################
def runRsmCommand (action):

    if action=='start':
        command = sudoPrefix+'rsm-start'
    elif action=='stop':
        command = sudoPrefix+'rsm-stop'
    else:
        logAndPrint(MSG_BAD_ACTION % action, True)
        showUsage('rsm')

    result = runCommand(command, sshPrefix, suToIaadmin)

    # process results
    if result==0:
        logAndPrint(MSG_SUCC_GEN, True)
    else:
        logAndPrint(MSG_FAIL_GEN % result, True)
        logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)

    return result

################################################################################
# runTraceCommand called when '/opt/tivoli/tiam/etc/bin/ia_trace.py' is triggered.
#
# runTraceCommand is different than many of the other ia_service scripts.  Like
# ia_logcollector, it always gets routed to a GPFS node to be run.
################################################################################
def runTraceCommand (runScript, component, level, node):
    # -c switch is mandatory
    assertArgument('trace', '-c', component)
    switches = ' -c ' + component

    # -n switch is optional for some components, and unsupported for others.
    # Without it, the trace level will be set on every node running the component
    if (node != ''):
        if (component=='hsm'
         or component=='dir.agent'
         or component=='nfs'
         or component=='mcp'
         or component=='cifs'
         or component=='br'
         or component=='all'):
            switches = switches + ' -n '+ node
        else:
            logAndPrint(MSG_BAD_N_COMP % component, True)
            showUsage('trace')

    # -l switch is mandatory, unless asking for a list ('-c list')
    if (component != 'list'):
        assertArgument('trace', '-l', level)
        if (level=='min' or level=='mid' or level=='max' or level=='disable' or level=='current' or level=='default'):
            switches = switches + ' -l '+ level
            # Only warn the user this may take time to complete if this is not a "-c list" trace config request.
            logAndPrint(MSG_WAIT_TC % (targetNode), True)
        else:
            logAndPrint(MSG_BAD_LEVEL, True)
            showUsage('trace')

    # -v switch is optional (English only and undocumented)
    if getVerboseOutput()==True:
        switches = switches + ' -v '

    command = sudoPrefix+runScript+switches

    # Since ia_trace is globalized, allow its stdout to get echo'd to the display
    logAndPrint('Routing command: '+command)
    result = runCommand(command, sshPrefix, suToIaadmin, True)

    # ia_trace already produces translated messages, so no messages are needed here other than log location after failures.
    if result!=0:
        logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)

    return result

################################################################################
# runCollectCommand called when '/opt/tivoli/tiam/etc/bin/ia_logcollector.py' is triggered.
################################################################################
def runCollectCommand (runScript, component, thresholds, node, destination):
    # -a switch is optional.  Without it, the default thresholds and timeouts will be in effect (-a off)
    if thresholds == 'off':
        switches = ' -a on'
    else:
        switches = ' -a off'

    if component == 'full':
        switches = switches + ' -i /opt/tivoli/tiam/etc/ia_commands.txt'
        switches = switches + ' -f /opt/tivoli/tiam/etc/ia_logfiles.txt'
    elif component == 'varlog':
        switches = switches + ' -i /opt/tivoli/tiam/etc/ia_commands_varlog.txt'
        switches = switches + ' -f /opt/tivoli/tiam/etc/ia_logfiles_varlog.txt'
    else:
        switches = switches + ' -i /opt/tivoli/tiam/etc/ia_commands_min.txt'
        switches = switches + ' -f /opt/tivoli/tiam/etc/ia_logfiles_min.txt'

    if node != '':
        switches = switches + ' -n ' + node
        logAndPrint(MSG_WAIT_LC_ONE % (component, node), True)
    else:
        logAndPrint(MSG_WAIT_LC % (component), True)

    global sshPrefix
    command = sudoPrefix+runScript + switches
    result = runLogCollectionCommand(command, sshPrefix, suToIaadmin)

    # process results
    if result==0:
        logAndPrint (MSG_LC_TXFR, True)
        # Once log collection completes, the collection needs to get transferred from the GPFS
        # node to the MC node.  The transfer is performed by the pushLogCollectionToMC script.
        command = '/opt/tivoli/tiam/bin/pushLogCollectionToMC.sh'
        result = runCommand(command, sshPrefix, suToIaadmin)
        if result==0:
            outputDir = '/home/iaadmin/tiamSystemLogs'
            if destination != '':
                copyToDest(destination)
            else:
                logAndPrint(MSG_SUCC_LC % (outputDir), True)
        else:
            logAndPrint(MSG_FAIL_XFER % (result), True)
            logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)
    elif result==42:
        # runLogCollectionCommand returns 42 when it detects that another instance of the log
        # collector is running.  That results in a different message to the console than a "real"
        # failure.  42 is an arbitrary value (selected from The Hitchhiker's Guide) that gets set
        # in ia_util.py:runLogCollectionCommand after it parses the stdout produced by ia_logcollector.py.
        logAndPrint(MSG_LC_RUNNING, True)
        result = 0                                      # don't return an error result if log collector is already running
    else:
        logAndPrint(MSG_FAIL_LC % (result), True)
        logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)

    return result

################################################################################
# runGetLogsScript called when the request to run any of the component specific
# log collectors is made.
################################################################################
def runGetLogsScript (runScript, component, destination):
    if targetNode=='':
        logAndPrint(MSG_WAIT_LC % (component), True)
    else:
        logAndPrint(MSG_WAIT_LC_ONE % (component, targetNode), True)

    command = sudoPrefix+runScript
    result = runCommand(command, sshPrefix, suToIaadmin)

    # process results
    if result==0:
        if destination != '':
            copyToDest(destination)
        else:
            logAndPrint(MSG_SUCC_LC % ('/home/iaadmin/logs/'+targetNode+'/'), True)
    elif result==14:
        logAndPrint(MSG_NO_STORAGE, True)
        result = 0                    # don't return an error code for a system with no controllers
    else:
        logAndPrint(MSG_FAIL_LC % (result), True)
        logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)

    return result

################################################################################
# runCollectAllCommands called when the user has foolishly requested collecting
# absolutely everything.
################################################################################
def runCollectAllCommands (destination):
    logAndPrint(MSG_WAIT_LC_ALL, True)

    # Create an array of all IA components supported by log collection
    components = ('director', 'gui', 'storage', 'gpfs', 'middleware', 'san', 'hsm', 'dsa', 'full')

    # Launch "ia_service.py -r getlogs" for each component
    for component in components:
        if component=='hsm':
            # For HSM, run ia_service separately for each node in the system running HSM (ie. not RSM)
            nodes = ('iamconsole1', 'ianode1', 'ianode2', 'ianode3')
            for node in nodes:
                if ping(node):
                    logAndPrint(MSG_WAIT_LC_ONE % (component, node), True)
                    command = 'ia_service.py -r getlogs -c '+component+' -n '+node
                    result=runCommand(command)
                    if result==0:
                        logAndPrint(MSG_SUCC_LC % ('/home/iaadmin/logs/'+node+'/'), True)
                    else:
                        logAndPrint(MSG_FAIL_LC % (result), True)

        elif component=='dsa':
            # For DSA, run ia_service separately for each node in the system
            nodes = ('iamconsole1', 'iarsm1', 'ianode1', 'ianode2', 'ianode3')
            for node in nodes:
                if ping(node):
                    command = 'ia_service.py -r getlogs -c '+component+' -n '+node
                    logAndPrint(MSG_WAIT_LC_ONE % (component, node), True)
                    result=runCommand(command)
                    if result==0:
                        logAndPrint(MSG_SUCC_LC % ('/home/iaadmin/logs/'+node+'/'), True)
                    else:
                        logAndPrint(MSG_FAIL_LC % (result), True)

        else:
            # All other components run on one node only
            command = 'ia_service.py -r getlogs -c '+component
            if component=='full':
                command = command + ' -t off'
                outputDir = '/home/iaadmin/tiamSystemLogs'
            else:
                outputDir = '/home/iaadmin/logs/'
            logAndPrint(MSG_WAIT_LC % (component), True)
            result=runCommand(command)
            if result==0:
                logAndPrint(MSG_SUCC_LC % (outputDir), True)
            elif result==14:
                logAndPrint(MSG_NO_STORAGE, True)
            else:
                logAndPrint(MSG_FAIL_LC % (result), True)

    # Log collection is complete.  Now copy the results to the desired destination
    if destination != '':
        copyToDest(destination)
    else:
        logAndPrint(MSG_SUCC_LC % ('/home/iaadmin'), True)


################################################################################
# copyToDest called when the user has asked to copy the log collector output
# somewhere in addition to the MC.  This method also displays the final success/
# failure message for the operation.
# Supported targets:
#    rsm: Copy the output to the same directory on the RSM node
#    usb: Mount the USB drive on this system, copy the files to it, then unmount
################################################################################
def copyToDest(targetName):
    logAndPrint(MSG_LC_TXFR_DEST % (targetName), True)
    result = 0

    if targetName == 'rsm':
        command = 'ssh iarsm1 sudo /opt/tivoli/tiam/bin/copyRASlogs.sh'
        logAndPrint('copying files with command: '+command)
        result = runCommand(command)
    elif targetName == 'usb':
        # If we aren't running from the MC, we need to copy the logs locally so that
        # we can push them to the USB flash drive.
        command = 'sudo /opt/tivoli/tiam/bin/copyRASlogs.sh -t usb'
        logAndPrint('copying files with command: '+command)
        result = runCommand(command)
    else:
        logAndPrint('unrecognized destination will be ignored: '+targetName)

    if result==0:
        logAndPrint(MSG_SUCC_LC_DEST % (targetName, '/home/iaadmin'), True)
    else:
        if result==10:
            logAndPrint(MSG_FAIL_USB_MOUNT, True)
        elif result==11:
            logAndPrint(MSG_FAIL_USB_COPY, True)
        elif result==12:
            logAndPrint(MSG_FAIL_USB_UNMOUNT, True)

        logAndPrint(MSG_FAIL_LC_DEST % (targetName, '/home/iaadmin'), True)
        logAndPrint(MSG_FAIL_LOGS % (IaServiceLogFile), True)


################################################################################
# Function: showUsage
# If no argument is passed in, this function dumps the command line syntax for
# ia_service to standard output, then exits with RC=1.
#
# If a runapp value is passed in, only the usage specific to that '-r' value
# gets displayed, then the program exits with RC=1.
################################################################################
def showUsage(runapp=''):
    logAndPrint('', True)

    if runapp=='getlogs':
        for s in USAGE_GETLOGS:
            logAndPrint(s, True)
    elif runapp=='trace':
        for s in USAGE_TRACE:
            logAndPrint(s, True)
    elif runapp=='verify':
        for s in USAGE_VERIFY:
            logAndPrint(s, True)
    elif runapp=='director':
        for s in USAGE_DIRECTOR:
            logAndPrint(s, True)
    elif runapp=='rsm':
        for s in USAGE_RSM:
            logAndPrint(s, True)
    elif runapp=='util':
        for s in USAGE_UTIL:
            logAndPrint(s, True)
    elif runapp=='systemx':
        for s in USAGE_SYSTEMX:
            logAndPrint(s, True)
    elif runapp=='storage':
        for s in USAGE_STORAGE:
            logAndPrint(s, True)
    elif runapp=='san':
        for s in USAGE_SAN:
            logAndPrint(s, True)
    elif runapp=='enet':
        for s in USAGE_ENET:
            logAndPrint(s, True)
    else:
        for s in USAGE_GENERAL:
            logAndPrint(s, True)

    logAndPrint('', True)
    errorExit(1)

################################################################################
# assertArgument confirms that a required command line switch was provided.  If
# not, it triggers a usage message for the RunApp and exits.
################################################################################
def assertArgument (runapp, switch, value):
    if value == '':
        logAndPrint(MSG_MISSING_ARG % (switch), True)
        showUsage(runapp)                    # ia_service exits from showUsage call

################################################################################
# isBconfig examines this node's IP address, to determine whether it is being
# run on an "A box", a "B box", or somewhere else (eg. a manufacturing PC).
################################################################################
def isBconfig ():
    ip = socket.gethostbyaddr(socket.gethostname())[-1][0]
    firstDot = ip.find('.')
    secondDot = ip.find('.', firstDot+1)
    octet = ip[firstDot+1:secondDot]
    if octet == '30':
        return True
    else:
        return False

# Call the Management Broker's wrapper for main()
if __name__ == '__main__':
    do_main(main, True)

