#!/bin/bash
#
#******************************************************************************
#  Licensed Materials - Property of IBM
#
#  (C) Copyright IBM Corp.  2008  All Rights Reserved
#
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#
# -----------------------------------------------------------------------------
#
# =========================================================================
#
# Name: ia_storage_common.sh
#
# Change Log:
#  Date          developer           change
#  08/1/2008     Bryan Jen           Initial script creation
#  03/31/2009    Bryan Jen           DCR 1048: Add Host Groups to config script
#  04/06/2009    Bryan Jen           Changed LUN Mapping code to only map for iastorage1.
#                                    other storage subsystems will need to be mapped after
#                                    Node Install
#  02/07/2010    Robert Guthrie      MES addtions
#  02/23/2010    Laura Bell          2 TB Drive support
#  03/18/2010    Laura Bell          TIAMC00012279: Drive capacity not queried/set
#                                    correctly for 1 TB drives
#  03/19/2010    Bryan Jen           Add support for R1.2 DR Secondary IP Addresses
#  03/30/2010    Laura Bell          TIAMC00012507: failure when configuring expansion
#                                    drawer 3 (enclosure 2)
#  04/06/2010    Laura Bell          TIAMC00012684: storage config failed when
#                                    configuring iadata_85_1 in expansion rack
#  04/23/2010    Brad Trubey         TIAM13107: add support for firmware update
#  04/29/2010    Brad Trubey         TIAM13290: fix enclosure query for new version of SMclient
#  05/03/2010    Brad Trubey         TIAM13392: fix enclosure query for new version of SMclient
#  06/13/2010    Robert Guthrie      TIAM14109: When initializing the 1st 8 pack in the
#                                    controller and MES then remove LUN mapping.
#  06/24/2010    Brad Trubey         TIAM13770: Must return non-zero for failure conditions
#  07/02/2010    Del Ehce            TIAM14484: Detect controller and update firmware for DS4200 and 3950
#  07/22/2010    Robert Guthrie      The size string for 2 TB is different on DS3950 vs DS4200
#  09/30/2010    Del Ence            TIAM15857 Updated for new firmware file format used for DS3950
#  10/27/2010    Brad Trubey         TIAMC00015805  append to the log instead of overwriting
#  11/09/2010    Brad Trubey         TIAMC00016579  improve error logging and reporting
#  12/15/2010    Laura Bell          TIAMC00011955  kpmg compliance (reword smcli doc comments)
#  02/18/2011    Del Ence           TIAMC00018206  Added resetController function; Reset controler after firmware update
#******************************************************************************
SMCLI="/usr/bin/SMcli"
FIRMWARE_PATH="/home/iaadmin/firmware"
PATTERN_FIRMWARE="FW*.dlp"
PATTERN_NVSRAM="N*.dlp"
PATTERN_ESM="esm*.esm"
VERSION_CMD=""
RMLUNMAP=0
CONTROLLER_TYPE=0

# ************************************************************************
# ************************************************************************
# Function: usage
# ************************************************************************
# ************************************************************************
usage() {
  echo " ia_storage_common.sh -a <ipaddress> -b <ipaddress> -l <logfile> [-n|-m|-f]"
  echo "       -a                 IP Address of controller A"
  echo "       -b                 IP Address of controller B"
  echo "       -l                 logfile name"
  echo "       -f                 update the firmware for this DS4200"
  echo "       -n                 completely initialize this DS4200"
  echo "       -m                 only initialize new 8 packs"
  echo "       -h                 help/usage information"
  echo " "
  echo "    MES examples: Initialize only new hard drives: "
  echo "       ia_storage_common.sh -a 172.31.0.100  -b 172.31.0.101  -l /tmp/logfile.txt -m"
  echo "       ia_storage_common.sh -a disk_ctrl_1_a -b disk_ctrl_1_b -l /tmp/logfile.txt -m"
  echo ""
  echo "    Manufacturing examples: Initialize/format ALL hard drives: "
  echo "       ia_storage_common.sh -a 172.31.0.100  -b 172.31.0.101  -l /tmp/logfile.txt -n"
  echo "       ia_storage_common.sh -a disk_ctrl_1_a -b disk_ctrl_1_b -l /tmp/logfile.txt -n"
  echo "       ia_storage_common.sh -a 172.31.0.100  -b 172.31.0.101  -l /tmp/logfile.txt -n"

  return 0
}

# ************************************************************************
# ************************************************************************
# Function: fail
# - send the configuration failure message to stdout
# - In (optional): $1: return code to use when exiting script
# ************************************************************************
# ************************************************************************
fail() {
  echo "IA Storage configuration FAILED. View $LOGFILE for more information"
  if [ -n "$1" ]; then
    exit $1
  else
    exit 1
  fi
}

# ************************************************************************
# ************************************************************************
# Function: verifySMCmd
# - this function runs an SMcli API. If the SMcli command fails, the
#   failing SMcli command will be logged
# - The SMcli script engine is not case sensitive. You can enter commands
#   by using uppercase letters, lowercase letters, or mixed-case letters
# - Parameters:
#   In: $1: return code from the SMcli command
#   In: $2: output text file from the SMcli command
#   In: $3: Error message to log if the SMcli command fails
# ************************************************************************
# ************************************************************************
verifySMCmd() {
  # ======================================================================
  # Run the caller's command and save its output to the $1 output variable
  # ======================================================================
  # if this is a debug call and we have an output file, display it
  if [ -n "$DEBUG" ] && [ -f "$2" ] && [ -s "$2" ]; then
    echo "debug: verifySMCmd: output from SMcli call is:"
    cat "$2"
    echo "$3"
    echo ""
  fi

  if [[ $1 -ne 0 ]]; then
    # ======================================================================
    # If we are here, the SMcli Command failed.
    # If the caller did not pass in an error message ($3), use a generic
    # error message
    # ======================================================================
    if [[ -z "$3" ]]; then
      errMsg="An SMcli command failure has occurred."
    else
      errMsg=$3
    fi

    # ======================================================================
    # -  Log the error message along with additional problem determination info
    # ======================================================================
    echo "$errMsg" >> $LOGFILE
    echo "The SMcli API return code is: " $1 >> $LOGFILE
    if [ -f "$2" ] && [ -s "$2" ]; then
      echo "The SMcli API results follow this message: " >> $LOGFILE
      cat "$2" >> $LOGFILE
      rm -f "$2" $> /dev/null
    fi

    # ======================================================================
    # ======================================================================
    # - SMcli writes error info to excprpt.txt and it gets overwritten 
    #   every time an exception occurs. For a typical UNIX-based OS 
    #   install using default install dir, this file lives here: 
    #      /var/opt/SM
    # - so if the SMcli /var/opt/SM/excprpt.txt exception file exists,
    #   append its contents to the log file
    # - Call our exit function
    # ======================================================================
    if [ -f /var/opt/SM/excprpt.txt ]; then
      echo "" >> $LOGFILE
      echo "Detailed SMcli error information follows this message:" >> $LOGFILE
      cat /var/opt/SM/excprpt.txt >> $LOGFILE
    fi

  # if this is a debug call, skip calling the fail function
  if [ -n "$DEBUG" ]; then
    echo "debug: verifySMCmd: skipping 'fail' function to continue with debug"
    read -p "debug: press enter to continue" INPUT
  else
    fail $1
  fi # end: debug

  fi
}

# ************************************************************************
# ************************************************************************
# Function: min
# - this function returns the smaller of two numbers
#   In: first number to use in comparison
#   In: second number to use in comparison
#   Out: smallest of 2 numbers
# - Note: when you return a value from a script using the echo statement,
#   everything that is echo'd in the script is assigned to the caller's
#   "return" variable, so be sure to just echo the number back
# ************************************************************************
# ************************************************************************
min() {
  if  [ $1 -lt $2 ]; then
    echo $1
  else
    echo $2
  fi
}

# ************************************************************************
# ************************************************************************
# Function: max
# - this function returns the larger of two numbers
#   In: first number to use in comparison
#   In: second number to use in comparison
#   Out: largest of 2 numbers
# ************************************************************************
# ************************************************************************
max() {
  if  [ $1 -gt $2 ]; then
    echo $1
  else
    echo $2
  fi
}

# ************************************************************************
# ************************************************************************
# Function: createHostGrp
# ************************************************************************
# ************************************************************************
createHostGrp() {
  # ========================================================================
  # Different versions of the firmware respond differently to some CLI calls
  # so get the first part of the controller's firmware version for use
  # later in the script. The firmware information (before cuts), looks
  # like this, so we cut at the first colon and cut the last two parts
  # of the version number
  #   Firmware version:      07.60.28.00
  # - getting the firmware for the first controller is adequate to limit
  #   the complexity of the script
  # ========================================================================
  echo "Querying controller firmware level with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"show controller [a] summary;\" | grep -i -m 1 firmware\" -o \"$cliOutputFile\"" >> $LOGFILE
  controllerFirmware=`$SMCLI $CTRLA $CTRLB -c "show controller [a] summary;" | grep -i -m 1 firmware | cut -d: -f2 | cut -d. -f1-2 | awk '{ print $1 }'`
  echo "Controller Firmware is at level $controllerFirmware" >> $LOGFILE

  # if we are on firmware level less than 7.60 we need to use the "interfaceType" parameter
  if [[ "$controllerFirmware" < "07.60" ]];then
    hostportType="interfaceType=FC"
  # if we are on firmware level 7.60 or above, we need to use the "hostType" parameter
  else
    hostportType="hostType=13"
  fi

  # ======================================================================
  # - A host is the machine/server attached to the storage subsystem and 
  #   it accesses the storage subsystem logical drives through its HBA
  #   HBA host ports. 
  #   - Define logical drive-to-LUN mappings to an individual computer/host. 
  #   - Assign the computer/host to a host group that shares access to >= 1 
  #     logical drives. 
  #   - A host group is a logical entity that names a collection of hosts 
  #     that share access to the same logical drives and gets defined 
  #     only if two or more hosts share access to the same logical drives.
  # - call our SMcli command processor with the command and the error message
  #   to be logged if an error occurs
  # ======================================================================
  echo "Creating host userlabel with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"create host userlabel=\\\"iagroup\\\" $hostportType;\" -o \"$cliOutputFile\"" >> $LOGFILE
  echo "" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create host userlabel=\"iagroup\" $hostportType;" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "Create Host Group FAILED"

}

# ************************************************************************
# ************************************************************************
# Function: getEnclIDs
# - Out: EnclIDs array
# - Out: numofEncl
# - Note that the DS4K controller firmware automatically assigns enclosure
#   IDs at power up. In an MES scenario (where we will be adding enclosures),
#   the DS4K controller firmware will automatically assign a new enclosure id
#   when a new enclosure is added. It should assign the enclosure ID sequentially.
# ************************************************************************
# ************************************************************************
getEnclIDs() {

  # ========================================================================
  #   Use 'show storagesubsystem' command without any parms to display/list
  #   the entire storage subsystem profile.
  #
  #   Sample command and results:
  #   /usr/bin/SMcli 172.31.0.100 172.31.0.101 -c "show storagesubsystem;" | grep -m 1 -i "Number of drive slots discovered:"
  #      Number of drive slots discovered:  16
  # - With the DS3950 controller we can no longer look for "Number of drive enclosures:"
  #     or "Number of diskDrive enclosures:"
  # ========================================================================
  echo "Getting number of drive enclosures with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"show storagesubsystem;\" " >> $LOGFILE
  numofDriveSlots=`${SMCLI} ${CTRLA} ${CTRLB} -c "show storagesubsystem;" | grep -m 1 -i "Number of drive slots discovered:" | awk '{ print $6 }'`
  numofEncl=$(( $numofDriveSlots- 16 ))
  numofEncl=$(( $numofEncl / 16 ))
  echo "The show storagesubsystem results show that there are $numofEncl drive enclosures " >> $LOGFILE

  # ========================================================================
  # - initialize the enclosure id array: Enclosure ID (85, 0, 1, 2, 3, etc)
  # - the 0 element is set to 85 to indicate that it is the Disk
  #   Controller and not an expansion drawer
  # - initialize the "starting drive" index to 0
  # - Note: having 0 enclosures is valid because we may just have a disk
  #   controller drawer with no additional enclosures
  # ========================================================================
  EnclIDs[0]=85

  i=0;
  while [ $i -lt $numofEncl ]
  do
    EnclIDs[(( $i + 1 ))]=$i
    i=$(( $i + 1 ))
  done
}

# ************************************************************************
# ************************************************************************
# Function: getTotalDrives
# - Out: totalDrvCount
# ************************************************************************
# ************************************************************************
getTotalDrives() {
  # ======================================================================
  # Total number of drives in controller + all expansion drawers
  # Example:
  #   iamconsole1:~ # /usr/bin/SMcli 172.31.0.101 172.31.0.102 -c "show storagesubsystem profile;" |grep -m 1 -i "Number of drives:"
  #      Number of drives:        64
  # - the 'grep -m 1' will return the first instance of "Number of drives:"
  # - call our SMcli command processor with the 'show storagesubsystem
  #   profile' command
  # - Since our SMcli results have been saved in the "result" variable and
  #   it contains a large number of strings, use the "eval" command
  #   and escape chars to make sure our grep command gives us the results
  #   we expect
  # ========================================================================
  echo "Getting the total number of drives using output from command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"show storagesubsystem profile;\"" >> $LOGFILE
  totalDrvCount=`${SMCLI} ${CTRLA} ${CTRLB} -c "show storagesubsystem profile;" |grep -m 1 -i "Number of drives:" |awk '{ print $4 }'`
  echo "The show storagesubsystem profile results show that there are $totalDrvCount drives" >> $LOGFILE

  if [ -n "$DEBUG" ]; then
    echo "debug: getTotalDrives: totalDrvCount is $totalDrvCount"
  fi # end: debug
}

# ************************************************************************
# ************************************************************************
# Function: resetEnclIDs
# - if the first unassigned drive is something other than 1, we need to
#   rebuild our EnclID array
# - based on the enclosure of the 1st unassigned drive number, build
#   an EnclID array that only contains the enclosure IDs with drives
#   that need to be formatted
# - In: indexEncl
# - In: numofEncl
# - In: EnclIDs (array)
# - Out: EnclIDs (array)
# ************************************************************************
# ************************************************************************
resetEnclIDs() {
  # ========================================================================
  # - initialize the "starting enclosure" index to be 0
  # - create a temporary enclosure array and copy only part of the
  #   original enclosure array into it. For example, suppose that the
  #   contents of our original EnclIDs array contains these elements: 85 0 1 2
  #   and the first drive that we want to format is in enclosure 1
  #   Our "reset" enclosure array should contain these elements: 1 2
  # - copy elements from the original EnclIDs array. Start with:
  #   -  our temporary array's 0th element
  #      and
  #   - the enclosure ID enclosure where the new drive was found
  # ========================================================================
  i=0;
  while [ $(( $i + $indexEncl )) -le $numofEncl ]
  do
    newEnclIDs[(( $i ))]=${EnclIDs[(( $i + $indexEncl ))]}
    i=$(( $i + 1 ))
  done

  # ========================================================================
  # - now that we have saved the enclosure IDs that we are interested in
  #   into a temporary array, zero out our original enclosure array
  # ========================================================================
  i=0;
  while [ $i -le $numofEncl ]
  do
    EnclIDs[(( $i ))]=
    i=$(( $i + 1 ))
  done

  # ========================================================================
  # - and copy the temporary enclosure array back into our 'global'
  #   enclosure array
  # ========================================================================
  i=0;
  while [ $(( $i + $indexEncl )) -le $numofEncl ]
  do
    EnclIDs[(( $i ))]=${newEnclIDs[(( $i ))]}
    i=$(( $i + 1 ))
  done

  if [ -n "$DEBUG" ]; then
    echo "debug: contents of encl IDs is: ${EnclIDs[*]}"
    read -p "debug: press enter to continue" INPUT
  fi # end: debug
}

# ************************************************************************
# ************************************************************************
# Function: getDriveNumber
# - In:  driveSize: size of drive (e.g. 1 or 2 TB) to search for
# - In:  drive "Mode" to search for (Assigned/Unassigned)
# - Out: foundDrive
# - Out: foundEncl
# - Out: foundEnclDrive
# - Out: foundDrvIndex
# ************************************************************************
# ************************************************************************
getDriveNumber() {

  # ========================================================================
  # - Examples (where the output from 'show drive' has been redirected to
  #   a file called 'show_unassigned_drive_info.txt' and the file was
  #   tweaked to contain a drive with a mode of "Unassigned" and an
  #   associated array of "None"
  #   1) For a 1 TB drive:
  #      grep -A 4 Unassigned show_unassigned_drive_info.txt
  #      Output:
  #          Mode:                    Unassigned
  #          Raw capacity:            931.513 GB
  #          Usable capacity:         931.013 GB
  #          World-wide identifier:   20:00:00:a0:b8:4f:8d:65:00:00:00:00:00:00:00:00
  #          Associated array:        None
  #   2) For a 2 TB drive:
  #      Output:
  #          Mode:                    Unassigned
  #          Raw capacity:            1.819 TB
  #          Usable capacity:         1.819 TB
  #          World-wide identifier:   20:00:00:a0:b8:6b:23:fa:00:00:00:00:00:00:00:00
  #          Associated array:        None
  #   2) grep -A 4 Unassigned show_unnassigned_drive_85_1_info.txt | grep None
  #      Output:
  #          Associated array:        None
  # ========================================================================

  # ========================================================================
  # - loop through the disk controller and enclosures until we find the
  #   first drive with the required attributes (size of 1 TB, size of 2
  #   TB plus Assigned/Unassigned, etc)
  # ========================================================================
  driveSize=${1}
  modeType=${2}

  currDrvIndex=1
  foundDrive="false"
  foundEncl=-1
  foundEnclDrive=-1
  foundDrvIndex=-1

  if [ -n "$DEBUG" ]; then
    echo ""
    echo "debug: getDriveNumber: drive size is $driveSize"
    echo "debug: getDriveNumber: mode type is $modeType"
  fi # end: debug


  for encl in ${EnclIDs[@]}
  do
    # =====================================================================
    # - initialize the drive counter for this disk controller/enclosure
    # - loop through the enclosure to see if we find any unassigned drives
    # - the enclosure might only be half full, so check the drive index
    #   against the total drives each time through the loop
    # =====================================================================
    j=1
    while [ $j -lt 16 ] && [ $foundDrive = "false" ] && [ $currDrvIndex -lt $totalDrvCount ]
    do
      # =================================================================
      # Since we call the "get Drive Number" function several times, and since
      # it takes about 8 seconds to process each SMcli API "get drive
      # information" request, we can speed things up by just calling the "get
      # drive info" API one time for each drive and redirecting the output to
      # a file. After that, each subsequent query for the same drive will
      # get the information out of the saved "drive info" text file, which
      # will save the user a lot of wait time
      #
      # - check to see if the information for this drive has already
      #   been queried/saved
      #   - if we already have the drive information saved to a text file,
      #     grep the file
      #   - if we do not yet have the drive info saved to a file, query
      #     the file info, save it to a file
      # - call our SMcli command processor with the 'show drive' command
      # =================================================================
      if [ ! -f /tmp/drive_${encl}_${j}.txt ]; then
        results=`${SMCLI} ${CTRLA} ${CTRLB} -c "show Drive[$encl,$j];" > /tmp/drive_${encl}_${j}.txt`
      fi

      # =================================================================
      # - query for "Mode" and grab the 4 lines that follow "Mode"
      #   so we can check the size of the drive and its "associated
      #   array" value
      # =================================================================
      grepResults=`grep -A 4 "Mode:" /tmp/drive_${encl}_${j}.txt `

      if [ -n "$DEBUG" ]; then
        echo "debug: getDriveNumber: grep for Mode string results are:"
        echo "$grepResults"
      fi # end: debug

      # =================================================================
      # we should always have results, but check just in case
      # =================================================================
      if [ -n "$grepResults" ]; then
        # ================================================================
        # ================================================================
        # get the actual drive size, mode, and assoc'd arry info for this drive
        # ================================================================
        tmpDrive=`echo "$grepResults" | grep "$driveSize" | grep "Raw"`
        tmpMode=`echo "$grepResults" | grep "$modeType"`
        tmpAssocArray=`echo "$grepResults" | grep "Associated array" | awk '{print $3}' | grep None`

        if [ -n "$DEBUG" ]; then
          echo "debug: getDriveNumber: tmpDrive grep for drive size results: $tmpDrive"
          echo "debug: getDriveNumber: tmpMode grep for mode results: $tmpMode"
          echo "debug: getDriveNumber: tmpAssocArray grep for None: $tmpAssocArray"
          echo ""
        fi # end: debug

        # ================================================================
        # if we found the requested drive size and mode
        # and
        # the requested mode is "Unassigned" and its "Associated Array" value is "None"
        # - we have found the unassigned drive we are looking for
        # - set our "found drive" output variable for the caller
        # ================================================================
        if [ -n "$tmpDrive"  ] && [ -n "$tmpMode" ] && [ "$modeType" = "Unassigned" ] && [ -n "$tmpAssocArray" ]; then
            foundDrive="true"

        # ================================================================
        # otherwise, if we found the requested drive size and mode, we
        # do not have to check the value of the associated array and
        # can just set the found flag to true
        # ================================================================
        elif [ -n "$tmpDrive" ] && [ -n "$tmpMode" ] && [ "$modeType" = "Assigned" ];  then
          foundDrive="true"
        fi

      fi

      # ================================================================
      # - if we found the drive we are looking for, set the rest of the
      #   output values our caller is expecting
      # ================================================================
      if [ "$foundDrive" = "true" ]; then
        foundEncl=$encl
        foundEnclDrive=$j
        foundDrvIndex=$currDrvIndex
      fi


      # ================================================================
      # - increment the drive counter for this enclosure by 8 (to go to
      #   the first drive in the next RAID array)
      # - increment the total number of drives that we have processed
      # ================================================================
      j=$(( $j + 8 ))
      currDrvIndex=$(( $currDrvIndex + 8 ))
    done # end: inner while loop
  done   # end: outer for loop

}

# ************************************************************************
# ************************************************************************
# Function: cleanupDriveInfoFiles
# - this function cleans up the temporary (/tmp/drive_${encl}_${j}.txt)
#   drive information files that we created in the get-Drive-Number
#   function above
# ************************************************************************
# ************************************************************************
cleanupDriveInfoFiles() {
  for encl in ${EnclIDs[@]}
  do
    # =====================================================================
    # - initialize the drive counter for this disk controller/enclosure
    # - loop through the enclosure
    # =====================================================================
    j=1
    while [ $j -lt 16 ]
    do
      # =================================================================
      # - check to see if the information for this drive exists
      # - remove it if it exists
      # =================================================================
      if [ -f /tmp/drive_${encl}_${j}.txt ]; then
        rm /tmp/drive_${encl}_${j}.txt
      fi

      # ================================================================
      # - increment the drive counter for this enclosure by 8 (to go to
      #   the first drive in the next RAID array)
      # ================================================================
      j=$(( $j + 8 ))
    done # end: inner while loop
  done   # end: outer for loop

}

# ************************************************************************
# ************************************************************************
# Function: get2TBFlags
# - In: TwoTBAssignedDriveIndex
# - In: TwoTBUnassignedDriveIndex
# - Out: TwoTBStartFormatIndex
# - Out: is2TBHotSpare
# ************************************************************************
# ************************************************************************
get2TBFlags() {
  # =====================================================================
  # - we will NOT set the "2 TB hot spare" flag when:
  #   - formatting all drives and there are no 2 TB drives
  #   - formatting some drives and a 2 TB drive has already been assigned
  #     to an array.
  # - we WILL set the "2 TB hot spare" flag when:
  #   - formatting ALL drives
  #     and
  #     some 2 TB drives exist
  #     and
  #     the 2 TB drives start in something other than drive 0 (1st drive
  #     of disk controller a) or drive 48 (1st drive of enclosure 3)
  #   - formatting SOME drives and
  #     - 2 TB drives exist
  #       and
  #     - a 2 TB drive has not yet been assigned to an array
  # =====================================================================
  TwoTBStartFormatIndex=-1
  is2TBHotSpare="false"

  # ================================================================
  # check to see whether we have any 2 TB drives
  # ================================================================
  if [ $TwoTBAssignedDriveIndex -ne -1 ] || [ $TwoTBUnassignedDriveIndex -ne -1 ]; then
    # ================================================================
    # we have some 2 TB drives.
    # ================================================================
    if [ "$CONFIG_NEW" -eq 1 ]; then
      # ================================================================
      # if we are formatting all the drives, set the drive number of the
      # first 2 TB drive to be formatted
      # ================================================================
      if [ $TwoTBAssignedDriveIndex -ne -1 ] && [ $TwoTBUnassignedDriveIndex -ne -1 ]; then
        TwoTBStartFormatIndex=`min "$TwoTBAssignedDriveIndex" "$TwoTBUnassignedDriveIndex"`
      elif [ $TwoTBAssignedDriveIndex -ne -1 ]; then
        TwoTBStartFormatIndex=$TwoTBAssignedDriveIndex
      else
        TwoTBStartFormatIndex=$TwoTBUnassignedDriveIndex
      fi

    elif [ "$CONFIG_MES" -eq 1 ]; then
      # ================================================================
      # we do have some 2 TB drives
      # - if we are only formatting the new drives, use the 1st unassigned
      #   2 TB drive
      # ================================================================
      TwoTBStartFormatIndex=`max "$TwoTBAssignedDriveIndex" "$TwoTBUnassignedDriveIndex"`
    fi

    # ================================================================
    # now figure out whether this "start formatting here" drive requires
    # a hot spare (outside of the normal drive 1/drive 49 hot spare)
    # ================================================================
    if [ $TwoTBStartFormatIndex -ne 1 ] && [ $TwoTBStartFormatIndex -ne 49 ]; then
      if [ $TwoTBAssignedDriveIndex -ge 1 ]; then
        is2TBHotSpare="false"
      else
        is2TBHotSpare="true"
      fi
    fi
  fi # end: if any 2 TB drives
}

# ************************************************************************
# ************************************************************************
# Function: getDriveCapacity
# - In:  Enclosure number
# - In:  Drive number
# - Out: driveCapacity
# ************************************************************************
# ************************************************************************
getDriveCapacity() {
  # =====================================================================
  # - query the capacity of the specified drive
  # - if the capacity includes "GB", this must be a 1 TB drive, e.g.
  #          Raw capacity:            931.513 GB
  #   otherwise it must be a 2 TB drive, for example:
  #          Raw capacity:            1.819 TB
  # =====================================================================
  echo "Getting the drive information for drive $1,$2 with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"show Drive[$1,$2];\"" >> $LOGFILE
  rawCapacity=`${SMCLI} ${CTRLA} ${CTRLB} -c "show Drive[$1,$2];" | grep "Raw capacity:"`
  capacity=`echo $rawCapacity | grep "931.513 GB"`
  echo "" >> $LOGFILE

  if [ -n "$capacity" ] ; then
    driveCapacity=1
  else
    driveCapacity=2
  fi
  echo "Drive $1,$2 is a $driveCapacity TB drive" >> $LOGFILE
  echo "The actual SMcli capacity for Drive $1,$2 is: $rawCapacity" >> $LOGFILE
}

# ************************************************************************
# ************************************************************************
# Function: cfgSubsystem
# - set the storage subsystem name on the disk controller and its mirror
#   if applicable)
# - set the storage subsystem's media scan rate
# - set the storage subsystem's cache block size
# - set the storage subsystem's cache flush (start and stop) size
# ************************************************************************
# ************************************************************************
cfgSubsystem() {
  # ========================================================================
  # - if this is the 1st controller or the 1st controller's mirror, set
  #   - the storage subsystem name to iastorage1 or iastorage1b
  #   - the rack id number
  #   - the Logical Unit Number
  #
  # LUN information from SMclient docs:
  #   In the context of the CLI commands and the script commands, a
  #   logical unit number (LUN) is a unique value that identifies the
  #   logical drives in a storage subsystem. The hosts identify the
  #   logical drives that they want to access using the LUN values. When
  #   you create a logical drive, the firmware assigns the LUN values, or
  #   you can assign LUN values when you enable the Storage Partitioning
  #   premium feature. A logical drive can have only one LUN and can be
  #   mapped to only one host or host group. Each host has unique
  #   addressing capability. That is, when more than one host accesses a
  #   storage subsystem, each host might use the same LUN to access
  #   different logical drives. The LUNs might be the same, but the
  #   logical drives are different. If you are mapping to a host group,
  #   the LUN that you specify must be available on every host in the host
  #   group.
  # - call our SMcli-processor function, which will process invalid
  #   return codes
  # =====================================================================
  if [ $CTRLA = "172.31.0.100" -a $CTRLB = "172.31.0.101" ]; then
    echo "Setting Subsystem name to iastorage1 with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set storagesubsystem userLabel=\\\"iastorage1\\\";\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set storagesubsystem userLabel=\"iastorage1\";" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "Setting Subsystem name FAILED"

  elif [ $CTRLA = "172.31.0.200" -a $CTRLB = "172.31.0.201" ] || [ $CTRLA = "172.30.0.100" -a $CTRLB = "172.30.0.101" ] ; then
    echo "Setting Subsystem name to iastorage1b with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set storagesubsystem userLabel=\\\"iastorage1b\\\";\" -o \"$cliOutputFile"\" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set storagesubsystem userLabel=\"iastorage1b\";" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "Setting Subsystem name FAILED"

  # ========================================================================
  # - if this is the 2nd controller or the 2nd controller's mirror, set
  #   - the storage subsystem name to iastorage2 or iastorage2b
  #   - the rack id number
  #   - the Logical Unit Number
  # ========================================================================
  elif [ $CTRLA = "172.31.0.102" -a $CTRLB = "172.31.0.103" ]; then
    echo "Setting Subsystem name to iastorage2 with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set storagesubsystem userLabel=\\\"iastorage2\\\";\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set storagesubsystem userLabel=\"iastorage2\";" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "Setting Subsystem name FAILED"

  elif [ $CTRLA = "172.31.0.202" -a $CTRLB = "172.31.0.203" ] || [ $CTRLA = "172.30.0.102" -a $CTRLB = "172.30.0.103" ] ; then
    echo "Setting Subsystem name to iastorage2b with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set storagesubsystem userLabel=\\\"iastorage2b\\\";\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set storagesubsystem userLabel=\"iastorage2b\";" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "Setting Subsystem name FAILED"

  # ========================================================================
  # - if this is the 3rd controller or the 3rd controller's mirror, set
  #   - the storage subsystem name to iastorage3 or iastorage3b
  #   - the rack id number
  #   - the Logical Unit Number
  # ========================================================================
  elif [ $CTRLA = "172.31.0.104" -a $CTRLB = "172.31.0.105" ]; then
    echo "Setting Subsystem name to iastorage3 with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set storagesubsystem userLabel=\\\"iastorage2b\\\";\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set storagesubsystem userLabel=\"iastorage3\";" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "Setting Subsystem name FAILED" >> $LOGFILE

  elif [ $CTRLA = "172.31.0.204" -a $CTRLB = "172.31.0.205" ] || [ $CTRLA = "172.30.0.104" -a $CTRLB = "172.30.0.105" ] ; then
    echo "Setting Subsystem name to iastorage3b with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set storagesubsystem userLabel=\\\"iastorage3b\\\";\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set storagesubsystem userLabel=\"iastorage3b\";" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "Setting Subsystem name FAILED"

  # ========================================================================
  # - if this is not disk controller 1/2/3 (or mirror), error out
  # ========================================================================
  else
    echo "INVALID IP ADDRESS"
    fail;
  fi

  # ========================================================================
  #  Use 'set storageSubsystem' to defines how often to run a media
  #  on a storage subsystem. For example:
  #    set storageSubsystem mediaScanRate=(disabled | 1-30)
  #  - mediaScanRate defines the number of days over which the scan runs.
  #    - use "disabled" to turn off the media scan
  #    or
  #    - user 1 to 30 days: 1 day is the fastest scan rate/30 is slowest
  # ========================================================================
  echo "Setting Subsystem media scan rate to 30 with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set storagesubsystem mediaScanRate=30;\" -o \"$cliOutputFile\"" >> $LOGFILE
  txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set storagesubsystem mediaScanRate=30;" -o "$cliOutputFile"`
  verifySMCmd "$?" "$cliOutputFile" "Setting Subsystem media scan rate FAILED"

  # ========================================================================
  #  - cache block size is the size of the data block the controller uses 
  #    transfer data in and out of the cache.
  #  - Set the cache block size to 4KB or 16KB, which applies to the entire
  #    storage subsystem and all its logical drives. For redundant controller
  #    configs, cache block size applies to all logical drives owned by 
  #    both controllers.
  # ========================================================================
  echo "Setting cache block size to 16 with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set storagesubsystem cacheBlockSize=16;\" -o \"$cliOutputFile\"" >> $LOGFILE
  txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set storagesubsystem cacheBlockSize=16;" -o "$cliOutputFile"`
  verifySMCmd "$?" "$cliOutputFile" "Setting cache block size FAILED"

  # ========================================================================
  #  - The controller writes cache data (flushes the cache) to the disk 
  #    drives when the unwritten data in the cache reaches a predefined 
  #    ("cache flush start" percentage) level.
  #  - The controller also flushes the cache when data has been in the 
  #    cache for a preset amount of time. The controller writes data until 
  #    the data in the cache drops to a "stop percentage" level.
  #  - valid cache flush start values: 0 to 100 (default value is 80).
  #  - cacheFlushStart must be >= cacheFlushStop. For example, if 
  #    cacheFlushStart is set to 80, set cacheFlushStop between 0 and 80.
  # ========================================================================
  echo "Setting cache flush start to 80 with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set storagesubsystem cacheFlushStart=80;\" -o \"$cliOutputFile\"" >> $LOGFILE
  txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set storagesubsystem cacheFlushStart=80;" -o "$cliOutputFile"`
  verifySMCmd "$?" "$cliOutputFile" "Setting cache flush start FAILED"

  # ========================================================================
  # - cache flush stop percentage stops a cache flush in progress
  # - valid values: 0 to 100; must be less than cacheFlushStart 
  #  Note: This is NOT the percent of unwritten cache data - it is the 
  #  percent of unused area in the cache.
  # ========================================================================
  echo "Setting cache flush stop to 80 with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set storagesubsystem cacheFlushStop=80;\" -o \"$cliOutputFile\"" >> $LOGFILE
  txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set storagesubsystem cacheFlushStop=80;" -o "$cliOutputFile"`
  verifySMCmd "$?" "$cliOutputFile" "Setting cache flush stop FAILED"
  echo "" >> $LOGFILE

}

# ************************************************************************
# ************************************************************************
# Function: cfgLogicalDrive
# - this function sets the properties of the logical drive
# - In: parameter 1 ($1): name of the logical drive
# - Out: lunNum
# ************************************************************************
# ************************************************************************
cfgLogicalDrive() {

  echo "Setting Logical Drive $1 cacheFlushModifier=10 with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set logicaldrive[\\\"$1\\\"] cacheFlushModifier=10;\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set logicaldrive[\"$1\"] cacheFlushModifier=10;" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "SMcli FAILED"

  echo "Setting Logical Drive $1 cacheWithoutBatteryEnabled=false with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set logicaldrive[\\\"$1\\\"] cacheWithoutBatteryEnabled=false;\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set logicaldrive[\"$1\"] cacheWithoutBatteryEnabled=false;" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "SMcli FAILED"

  # ========================================================================
  # TBD: test to see whether to use "mirrorEnabled" or "mirrorCacheEnabled"
  # Feb 3, 2010 Per lkb ST w/Thomas Phelan (DS4000 development): "OK, so maybe
  # I have it backwards. I have 10.60 on my system and it uses
  # mirrorCacheEnabled.  I would stick with that parameter and assume if you
  # get a syntax error it is a defect."
  # ========================================================================
  echo "Setting Logical Drive $1 mirrorEnabled=true with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set logicaldrive[\\\"$1\\\"] mirrorEnabled=true;\"-o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set logicaldrive[\"$1\"] mirrorEnabled=true;" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "SMcli FAILED"

  echo "Setting Logical Drive $1 readCacheEnabled=true with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set logicaldrive[\\\"$1\\\"] readCacheEnabled=true;\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set logicaldrive[\"$1\"] readCacheEnabled=true;" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "SMcli FAILED"

  echo "Setting Logical Drive $1 writeCacheEnabled=true with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set logicaldrive[\\\"$1\\\"] writeCacheEnabled=true;\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set logicaldrive[\"$1\"] writeCacheEnabled=true;" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "SMcli FAILED"

  echo "Setting Logical Drive $1 mediaScanEnabled=true with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set logicaldrive[\\\"$1\\\"] mediaScanEnabled=true;\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set logicaldrive[\"$1\"] mediaScanEnabled=true;" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "SMcli FAILED"

  echo "Setting Logical Drive $1 redundancyCheckEnabled=false with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set logicaldrive[\\\"$1\\\"] redundancyCheckEnabled=false;\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set logicaldrive[\"$1\"] redundancyCheckEnabled=false;" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "SMcli FAILED"

  echo "Setting Logical Drive $1 readAheadMultiplier=1 with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set logicaldrive[\\\"$1\\\"] readAheadMultiplier=1;\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set logicaldrive[\"$1\"] readAheadMultiplier=1;" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "SMcli FAILED"

  echo "Setting Logical Drive $1 modificationPriority=high with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set logicaldrive[\\\"$1\\\"] modificationPriority=high;\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set logicaldrive[\"$1\"] modificationPriority=high;" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "SMcli FAILED"

  echo "Setting Logical Drive $1 preReadRedundancyCheck=false with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set logicaldrive[\\\"$1\\\"] preReadRedundancyCheck=false;\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set logicaldrive[\"$1\"] preReadRedundancyCheck=false;" -o "$cliOutputFile"`
    verifySMCmd "$?" "$cliOutputFile" "SMcli FAILED"

  if [ "$CONFIG_NEW" -eq 1 ]; then
    if [ $1 = "iarecovery" ]; then
      echo "Setting LUN-to-Host mapping to IA Host Group with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set logicaldrive[\\\"$1\\\"] logicalUnitNumber=99 hostgroup=defaultGroup;\" -o \"$cliOutputFile\"" >> $LOGFILE
      txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set logicaldrive[\"$1\"] logicalUnitNumber=99 hostgroup=defaultGroup;" -o "$cliOutputFile"`
      verifySMCmd "$?" "$cliOutputFile" "SMcli FAILED"
    else
      if [ $RACKID -eq 1 ]; then
        echo "Setting LUN-to-Host mapping to IA Host Group with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set logicaldrive[\\\"$1\\\"] logicalUnitNumber=$lunNum hostgroup=defaultGroup;\" -o \"$cliOutputFile\"" >> $LOGFILE
        txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set logicaldrive[\"$1\"] logicalUnitNumber=$lunNum hostgroup=defaultGroup;" -o "$cliOutputFile"`
        verifySMCmd "$?" "$cliOutputFile" "LUN Mapping FAILED"
      else
        echo "INFO: Rack ID: $RACKID: The LUN mapping for this rack is being skipped at this time. This is expected behavior."  >> $LOGFILE
      fi
    fi
  else
    echo "INFO: Rack ID: $RACKID: The LUN mapping for this rack is being skipped at this time. This is expected behavior."  >> $LOGFILE
  fi

  echo "" >> $LOGFILE

  lunNum=$(( $lunNum + 1 ))
}

# ************************************************************************
# ************************************************************************
# Function: cfgArrayUtil
# ************************************************************************
# ************************************************************************
cfgArrayUtil() {
  # ==============================================================
  # - tell SMclient that drive #8 is a hot spare
  # ==============================================================
  echo "Creating Hot Spare at [85,8] with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set drive [85,8] hotSpare=TRUE;\" -o \"$cliOutputFile\"" >> $LOGFILE
  txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set drive [85,8] hotSpare=TRUE;" -o "$cliOutputFile"`
  verifySMCmd "$?" "$cliOutputFile" "Create Hot Spare FAILED"

  # ==============================================================
  # - Now that we have allocated a hot spare:
  # - Get the drive capacity for the first drive in this array
  #   - For 1 TB drives, SM Client GUI shows a total of 4.546 TB of space
  #     in this array
  #   - For 2 TB drives, there should be a total of 9.092 TB of space
  #     in this array (2 * 4.456 TB)
  # - In: Enclosure ID
  # - In: Drive/slot for drive in enclosure
  # - Output: driveCapacity
  # ==============================================================
  getDriveCapacity 85 1

  # ==============================================================
  # - Same command here as in the cfgArrayWSpare function (below):
  # - Create a new 1 TB logical drive (utilfs) while simultaneously
  #   creating a new RAID 6 array (iaarray_85_1) to which we assign the
  #   disk drives (drives 1-7)
  # - The capacity parameter defines the size of the logical drive. You do
  #   not need to assign the entire capacity of the disk drives to the
  #   logical drive. Later, you can assign any unused space to another
  #   logical drive.
  #
  # Create logical drive calls:
  # - What causes the "long running formatting" action to start?
  #   - there are several long running operations that you cannot
  #     stop. They are performed in the background and do not prevent
  #     other commands from running. 
  #   - long running operatings include setting the capacity, drives, 
  #     raid level and segment size of a logical drive.
  #
  #   These long-running operations are for "set logicalDrive" calls, 
  #   and it is safe to assume that they are also long running operations
  #   when you specify the parms during "create logicalDrive".
  # ==============================================================
  if [ -n "$DEBUG" ]; then
    echo "debug: cfgArrayUtil: preparing to create RAID array iaaray_85_1"
    read -p "press enter to continue" INPUT
  fi # end: debug

  echo "Creating RAID-6 Array iaarray_85_1  with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"create array drives=( 85,1 85,2 85,3 85,4 85,5 85,6 85,7 ) raidLevel=6 userLabel=\"\"iaarray_85_1\"\";\" -o \"$cliOutputFile\" " >> $LOGFILE
  txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create array drives=( 85,1 85,2 85,3 85,4 85,5 85,6 85,7 ) raidLevel=6 userLabel=\"iaarray_85_1\";" -o "$cliOutputFile" `
  verifySMCmd "$?" "$cliOutputFile" "Creation of RAID array iaarray_85_1 FAILED"

  if [ -n "$DEBUG" ]; then
    echo "debug: cfgArrayUtil: RAID array iaaray_85_1 created"
    echo "debug: cfgArrayUtil: Preparing to create logical drive utilfs"
    read -p "press enter to continue" INPUT
  fi # end: debug

  # TBD: remove the following two lines when unit test verifies the updated create logical drive call works
  # echo "Creating Logical Drive utilfs with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"create logicalDrive drives=( 85,1 85,2 85,3 85,4 85,5 85,6 85,7 ) raidLevel=6 userLabel=\\\"utilfs\\\" volumeGroupUserLabel=\\\"iaarray_85_1\\\" capacity=1 TB owner=a segmentSize=128 dssPreAllocate=true;\" -o \"$cliOutputFile\"" >> $LOGFILE
  # txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create logicalDrive drives=( 85,1 85,2 85,3 85,4 85,5 85,6 85,7 ) raidLevel=6 userLabel=\"utilfs\" volumeGroupUserLabel=\"iaarray_85_1\" capacity=1 TB owner=a segmentSize=128 dssPreAllocate=true;" -o "$cliOutputFile" `
  echo "Creating Logical Drive utilfs with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"create logicalDrive array=\\\"iaarray_85_1\\\" userLabel=\\\"utilfs\\\" capacity=1 TB owner=a segmentSize=128 dssPreAllocate=true;\" -o \"$cliOutputFile\"" >> $LOGFILE
  txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create logicalDrive array=\"iaarray_85_1\" userLabel=\"utilfs\" capacity=1 TB owner=a segmentSize=128 dssPreAllocate=true;" -o "$cliOutputFile" `
  verifySMCmd "$?" "$cliOutputFile" "Creation of utilfs logical drive FAILED"

  if [ -n "$DEBUG" ]; then
    echo "debug: cfgArrayUtil: logical drive utilfs created"
    echo "debug: cfgArrayUtil: preparing to configure the logical drive"
    read -p "press enter to continue" INPUT
  fi # end: debug

  cfgLogicalDrive "utilfs"

  # ==============================================================
  # - Available space in this (iaarray_85_1) array:
  #   - For 1 TB drives, there is currently 3.546 TB of space in this array
  #   - For 2 TB drives, there is currently 8.092 TB of space in this array
  #
  # - Create the data file system, but leave enough space unallocated for:
  #   - The GPFS bootstrap logical drive (LUN)
  #   - Enhanced Remote Mirror (ERM) space (1 GB minimum unallocated)
  #     i.e. Mirror Repository logical drives (LUNs)
  #
  # Create the data file system:
  #   - For 1 TB drives, create a 3.5 TB logical drive for data
  #   - For 2 TB drives, create an  8 TB logical drive for data
  # ==============================================================
  if [ $driveCapacity -eq 1 ]; then
    logicalDriveCapacity="3.5 TB"
  else
    logicalDriveCapacity="8.0 TB"
  fi

  if [ -n "$DEBUG" ]; then
    echo "debug: cfgArrayUtil: Preparing to create logical drive iadata_85_1"
    read -p "press enter to continue" INPUT
  fi # end: debug

  echo "Creating Logical Drive iadata_85_1 with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"create logicalDrive array=\\\"iaarray_85_1\\\" userLabel=\\\"iadata_85_1\\\" capacity=$logicalDriveCapacity owner=a segmentSize=128 dssPreAllocate=true;\" -o \"$cliOutputFile\"" >> $LOGFILE
  # keep a copy of (commented out) SMcli command for future reference
  # txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create logicalDrive array=\"iaarray_85_1\" userLabel=\"iadata_85_1\" capacity=3.5 TB owner=a segmentSize=128 dssPreAllocate=true;"`
  txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create logicalDrive array=\"iaarray_85_1\" userLabel=\"iadata_85_1\" capacity=$logicalDriveCapacity owner=a segmentSize=128 dssPreAllocate=true;" -o "$cliOutputFile"`
  verifySMCmd "$?" "$cliOutputFile" "Logical Drive Create FAILED"

  if [ -n "$DEBUG" ]; then
    echo "debug: cfgArrayUtil: logical drive iadata_85_1 created"
    echo "debug: cfgArrayUtil: preparing to configure the logical drive"
    read -p "press enter to continue" INPUT
  fi # end: debug

  cfgLogicalDrive "iadata_85_1"

  # ==============================================================
  # - Create a new (30 MB) logical drive (iarecovery) in the existing
  #   array (iaarray_85_1). Per Brian Jen: the iarecovery lun is a 30MB
  #   lun that is used to bootstrap GPFS on DR
  #
  # - Available space in this (iaarray_85_1) array:
  #   - For 1 TB drives, there is .046 TB/46 GB of space left in this array
  #   - For 2 TB drives, there is .092 TB/92 GB of space left in this array
  #
  # - The unallocated can be used for:
  #     - 1 GB: Enhanced Remote Mirroring (ERM)
  #     - when DR gets "activated", Mirror Repository disks get
  #       created in the first array (example from Bryan Jen shows these
  #       disks to be approx 132 MB in size)
  # ==============================================================
  if [ -n "$DEBUG" ]; then
    echo "debug: cfgArrayUtil: Preparing to create logical drive iarecovery"
    read -p "press enter to continue" INPUT
  fi # end: debug

  echo "Creating IA Emergency Boot Drive with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"create logicalDrive array=\\\"iaarray_85_1\\\" userLabel=\\\"iarecovery\\\" capacity=30 MB owner=a segmentSize=128 dssPreAllocate=true;\" -o \"$cliOutputFile\"" >> $LOGFILE
  txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create logicalDrive array=\"iaarray_85_1\" userLabel=\"iarecovery\" capacity=30 MB owner=a segmentSize=128 dssPreAllocate=true;" -o "$cliOutputFile"`
  verifySMCmd "$?" "$cliOutputFile" "Logical Drive Create FAILED"

  if [ -n "$DEBUG" ]; then
    echo "debug: cfgArrayUtil: finished creating logical drive iarecovery"
    echo "debug: cfgArrayUtil: preparing to configure the logical drive"
    read -p "press enter to continue" INPUT
  fi # end: debug

  cfgLogicalDrive "iarecovery"

  if [ -n "$DEBUG" ]; then
    echo "debug: cfgArrayUtil: logical drive iarecovery configured"
    read -p "press enter to continue" INPUT
  fi # end: debug

}

# ************************************************************************
# ************************************************************************
# Function: cfgArray
# - In: parameter 1 ($1): enclosure number
# - In: parameter 2 ($2): the half of the enclosure (1 or 2) that will
#       contain the hot spare
# ************************************************************************
# ************************************************************************
cfgArray() {

  # ==============================================================
  # - if $2 is 1 (first half of enclosure), create a new RAID 6 array
  #   using drives 1-8
  #
  # Create logical drive calls:
  #   - there are several long running operations that you cannot
  #     stop. They are performed in the background and do not prevent
  #     other commands from running. 
  #   - long running operatings include setting the capacity, drives, 
  #     raid level and segment size of a logical drive.
  #
  #   These long-running operations are for "set logicalDrive" calls, 
  #   and it is safe to assume that they are also long running operations
  #   when you specify the parms during "create logicalDrive".
  # ==============================================================
  if [ $2 -eq "1" ]; then
    # ==============================================================
    # - if parm 2 ($2) says to create the array in the first half of the
    #   enclosure, create a new RAID 6 array using drives 1-8
    # - set the array owner to "a"
    # ==============================================================
    drivesList="$1,1 $1,2 $1,3 $1,4 $1,5 $1,6 $1,7 $1,8"
    arrayOwner="a"
  else
    # ==============================================================
    # - if parm 2 ($2) says to create the array in the (second half of the
    #   enclosure), create a new RAID 6 array using drives 9-16
    # - set the array owner to "b"
    # ==============================================================
    drivesList="$1,9 $1,10 $1,11 $1,12 $1,13 $1,14 $1,15 $1,16"
    arrayOwner="b"
  fi

  if [ -n "$DEBUG" ]; then
    echo "debug: cfgArray: Preparing to create RAID array iaarray_$1_$2"
    read -p "press enter to continue" INPUT
  fi # end: debug

  echo "Creating RAID-6 Array iaarray_$1_$2 with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"create array drives=( $drivesList ) raidLevel=6 userLabel=\\\"iaarray_$1_$2\\\";\" -o \"$cliOutputFile\" " >> $LOGFILE
  txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create array drives=( $drivesList ) raidLevel=6 userLabel=\"iaarray_$1_$2\";" -o "$cliOutputFile" `
  verifySMCmd "$?" "$cliOutputFile" "Creation of RAID array iadata_$1_$2 FAILED"


  # keep a copy of (commented out) SMcli command for future reference
  # txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create logicalDrive drives=( $1,1 $1,2 $1,3 $1,4 $1,5 $1,6 $1,7 $1,8 ) raidLevel=6 userLabel=\"iadata_$1_1\" volumeGroupUserLabel=\"iaarray_$1_1\" owner=a segmentSize=128 dssPreAllocate=true;"`
  # echo "Creating Logical Drive iadata_$1_$2 with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"create logicalDrive drives=( \\\"$drivesList\\\" ) raidLevel=6 userLabel=\\\"iadata_$1_$2\\\" volumeGroupUserLabel=\\\"iaarray_$1_$2\\\" owner=\\\"$arrayOwner\\\" segmentSize=128 dssPreAllocate=true;\" -o \"$cliOutputFile\"" >> $LOGFILE
  # txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create logicalDrive drives=( \"$drivesList\" ) raidLevel=6 userLabel=\"iadata_$1_$2\" volumeGroupUserLabel=\"iaarray_$1_$2\" owner=\"$arrayOwner\" segmentSize=128 dssPreAllocate=true;" -o "$cliOutputFile"`
  if [ -n "$DEBUG" ]; then
    echo "debug: cfgArray: RAID array iaarray_$1_$2 created"
    echo "debug: cfgArray: Preparing to create logical drive iadata_$1_$2"
    read -p "press enter to continue" INPUT
  fi # end: debug

  echo "Creating Logical Drive iadata_$1_$2 with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"create logicalDrive array=\\\"iaarray_$1_$2\\\"  userLabel=\\\"iadata_$1_$2\\\" owner=$arrayOwner segmentSize=128 dssPreAllocate=true;\" -o \"$cliOutputFile\"" >> $LOGFILE
  txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create logicalDrive array=\"iaarray_$1_$2\" userLabel=\"iadata_$1_$2\" owner=$arrayOwner segmentSize=128 dssPreAllocate=true;" -o "$cliOutputFile"`
  verifySMCmd "$?" "$cliOutputFile" "Creation of logical drive iadata_$1_$2 FAILED"

  if [ -n "$DEBUG" ]; then
    echo "debug: cfgArray: finished creating logical drive iadata_$1_$2"
    echo "debug: cfgArray: preparing to configure the logical drive"
    read -p "press enter to continue" INPUT
  fi # end: debug

  cfgLogicalDrive "iadata_$1_$2"

}

# ************************************************************************
# ************************************************************************
# Function: cfgArrayWSpare
# - In: parameter 1 ($1): enclosure number
# - In: parameter 2 ($2): the half of the enclosure (1 or 2) that will
#       contain the hot spare
# ************************************************************************
# ************************************************************************
cfgArrayWSpare() {
  # ==============================================================
  # - set the values we will use when creating the array
  #   - the slot where we start building the RAID array is going
  #     to be "1" if we are in the 1st half of the enclosure and "9"
  #     if we are in the 2nd half of the enclosure
  #   - the hot spare drive will be the last drive in the RAID array
  #     (either 8 or 16)
  #   - the list of drives will be 1-7 for the 1st half of the enclosure
  #     or 9-16 for the 2nd half of the enclosure
  # ==============================================================
  if [ $2 -eq 1 ]; then
    slotIndex=1
    hotSpareDrive=8
    drivesList="$1,1 $1,2 $1,3 $1,4 $1,5 $1,6 $1,7"
    arrayOwner="a"
  else
    slotIndex=9
    hotSpareDrive=16
    drivesList="$1,9 $1,10 $1,11 $1,12 $1,13 $1,14 $1,15"
    arrayOwner="b"
  fi

  echo "Creating Hot Spare at [$1,$hotSpareDrive] with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"set drive [$1,$hotSpareDrive] hotSpare=TRUE;\" -o \"$cliOutputFile\"" >> $LOGFILE
  txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "set drive [$1,$hotSpareDrive] hotSpare=TRUE;" -o "$cliOutputFile"`
  verifySMCmd "$?" "${txtOut}" "Hot Spare Create FAILED"

  # ==============================================================
  # - Get the drive capacity for the first disk in the RAID array
  #   - Output: driveCapacity
  # ==============================================================
  getDriveCapacity $1 $slotIndex

  # ==============================================================
  # - if this is the 1st half of a disk controller (enclosure ID is 85),
  #   and slotIndex is 1, calculate how much space to allocate for this
  #   logical drive, (since we must leave at least 1 GB of unallocated
  #   space for ERM (Enhanced Remote Mirroring)
  # - otherwise, this is either the second half of a disk controller or an
  #   expansion drawer and we will let the SMCli allocate all the space in
  #   the array for the logical drive
  # ==============================================================
  logicalDriveCapacity=
  if [ $1 -eq 85 ] && [ $slotIndex -eq 1 ]; then
    # if we are in the first half of the controller
    if [ $driveCapacity -eq 1 ]; then
      logicalDriveCapacity="4.5 TB"
    else
      logicalDriveCapacity="9.09 TB"
    fi
  fi

  # ==============================================================
  # set the name of the RAID array name and logical drive name
  # based on the half of the enclosure that the drive is in
  # ==============================================================
  arrayName="iaarray_$1_$2"
  dataName="iadata_$1_$2"

  # ==============================================================
  # - Available space in this array:
  #   - For 1 TB drives, SM Client GUI shows a total of 4.546 TB of space
  #     in this array, so allocate 4.5 TB to the logical drive
  #   - For 2 TB drives, there should be a total of 9.092 TB of space
  #     in this array (2 * 4.456 TB), so allocate 9 TB to the logical
  #     drive
  #
  # - the SMcli "show drive" command returns the capacity of a single
  #   drive.
  #   Question: should we use the "show drive" command to get the
  #     capacity of drives 1? and if greater than.... say 1 TB, allocate
  #     9 TB for the RAID array, otherwise, allocate 4.5 TB?
  #
  # Create logical drive calls:
  #   - there are several long running operations that you cannot
  #     stop. They are performed in the background and do not prevent
  #     other commands from running. 
  #   - long running operatings include setting the capacity, drives, 
  #     raid level and segment size of a logical drive.
  #
  #   These long-running operations are for "set logicalDrive" calls, 
  #   and it is safe to assume that they are also long running operations
  #   when you specify the parms during "create logicalDrive".
  # ==============================================================

  # ==============================================================
  # - we set the RAID array name, logical drive name, array owner, and
  #   logical drive capacity values above. Now we can use a single version
  #   of the "create logical drives" command using substitution variables
  # - note that the value for logicalDriveCapacity was set to an empty
  #   string (above) unless we are processing the 1st slot of a drive
  #   controller
  # - if we are in the disk controller AND we are in the first half of it
  #   we need to leave part of the capacity of the logical drive for ERM
  #   We set the logical drive capacity (above) for this case
  # ==============================================================
  if [ -n "$DEBUG" ]; then
    echo "debug: cfgArrayWSpare: Preparing to create RAID array iaarray_$1_$2"
    read -p "press enter to continue" INPUT
  fi # end: debug

  echo "Creating RAID-6 Array iaarray_$1_$2 with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"create array drives=( $drivesList ) raidLevel=6 userLabel=\\\"iaarray_$1_$2\\\";\" -o \"$cliOutputFile\" " >> $LOGFILE
  txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create array drives=( $drivesList ) raidLevel=6 userLabel=\"iaarray_$1_$2\";" -o "$cliOutputFile" `
  verifySMCmd "$?" "$cliOutputFile" "Creation of RAID array iaaray_$1_$2 FAILED"

  # keep a copy of (commented out) SMcli command for future reference
  # txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create logicalDrive drives=( $1,1 $1,2 $1,3 $1,4 $1,5 $1,6 $1,7 ) raidLevel=6 userLabel=\"iadata_$1_1\" volumeGroupUserLabel=\"iaarray_$1_1\" owner=a capacity=4.5 TB segmentSize=128 dssPreAllocate=true;"`
  # txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create logicalDrive drives=( $1,1 $1,2 $1,3 $1,4 $1,5 $1,6 $1,7 ) raidLevel=6 userLabel=\"iadata_$1_1\" volumeGroupUserLabel=\"iaarray_$1_1\" owner=a segmentSize=128 dssPreAllocate=true;"`
  if [ -n "$DEBUG" ]; then
    echo "debug: cfgArrayWSpare: RAID array iaarray_$1_$2 created"
    echo "debug: cfgArrayWSpare: Preparing to create logical drive iadata_$1_$2"
    read -p "press enter to continue" INPUT
  fi # end: debug

  if [ $1 -eq 85 ] && [ $slotIndex -eq 1 ]; then
    # if we are in the first half of one of the IS3 controllers (85,1), we
    # need to specify a capacity value since at least 1 GB must be
    # reserved for use by ERM
  echo "Creating Logical Drive $dataName with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"create logicalDrive array=\\\"iaarray_$1_$2\\\" userLabel=\\\"iadata_$1_$2\\\" owner=$arrayOwner segmentSize=128 dssPreAllocate=true capacity=$logicalDriveCapacity;\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create logicalDrive array=\"iaarray_$1_$2\" userLabel=\"iadata_$1_$2\" owner=$arrayOwner segmentSize=128 dssPreAllocate=true capacity=$logicalDriveCapacity;" -o "$cliOutputFile"`
  else
    # otherwise, we are in a drawer where SMcli can use all the available space
    echo "Creating Logical Drive $dataName with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"create logicalDrive array=\\\"iaarray_$1_$2\\\" userLabel=\\\"iadata_$1_$2\\\" owner=$arrayOwner segmentSize=128 dssPreAllocate=true;\" -o \"$cliOutputFile\"" >> $LOGFILE
    txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "create logicalDrive array=\"iaarray_$1_$2\" userLabel=\"iadata_$1_$2\" owner=$arrayOwner segmentSize=128 dssPreAllocate=true;" -o "$cliOutputFile"`
  fi

  verifySMCmd "$?" "$cliOutputFile" "Logical Drive Create FAILED"

  if [ -n "$DEBUG" ]; then
    echo "debug: cfgArrayWSpare: finished creating logical drive iadata_$1_$2"
    echo "debug: cfgArrayWSpare: preparing to configure the logical drive"
    read -p "press enter to continue" INPUT
  fi # end: debug

  cfgLogicalDrive "iadata_$1_$2"


}

# ************************************************************************
# ************************************************************************
# Function: setControllerVars
# - Out: CRTLA
# - Out: CRTLB
# - Out: RACKID
# - Out: lunNum
# ************************************************************************
# ************************************************************************
setControllerVars() {
  # ========================================================================
  # - if the disk controller's hostname was passed in (e.g. disk_ctrl_1_a,
  #   disk_ctrl_1_b, mirror_ctrl_1_a, mirror_ctrl_1_b, etc), get its
  #    numeric IP from the hosts file and reassign it to our CTRLA and
  #    CTRLB variables
  # ========================================================================
  if [[ $CTRLA =~ "disk_ctrl" ]] || [[ $CTRLA =~ "mirror_ctrl" ]]; then
    CTRLA=`cat /etc/hosts | grep $CTRLA |awk '{print $1}'`
  fi

  if [[ $CTRLB =~ "disk_ctrl" ]] || [[ $CTRLB =~ "mirror_ctrl" ]]; then
    CTRLB=`cat /etc/hosts | grep $CTRLB |awk '{print $1}'`
  fi

  # ========================================================================
  # - Set the rack id and LUN vars
  # ========================================================================
  if [[ $CTRLA = *0.100 && $CTRLB = *0.101 ]] || [[ $CTRLA = *0.200 && $CTRLB = *0.201 ]]; then
    RACKID=1
    lunNum=0
  elif [[ $CTRLA = *0.102 && $CTRLB = *0.103 ]] || [[ $CTRLA = *0.202 && $CTRLB = *0.203 ]]; then
    RACKID=2
    lunNum=33
  elif [[ $CTRLA = *0.104 && $CTRLB = *0.105 ]] || [[ $CTRLA = *0.204 && $CTRLB = *0.205 ]]; then
    RACKID=3
    lunNum=66
  else
    echo "INVALID IP ADDRESS"
    fail
  fi

  echo -n "CTRLA=" >> $LOGFILE
  echo $CTRLA >> $LOGFILE
  echo -n "CTRLB=" >> $LOGFILE
  echo $CTRLB >> $LOGFILE
  echo -n "LOGFILE=" >> $LOGFILE
  echo $LOGFILE >> $LOGFILE
  echo -n "CONFIG_FW=" >> $LOGFILE
  echo $CONFIG_FW >> $LOGFILE
  echo -n "CONFIG_NEW=" >> $LOGFILE
  echo $CONFIG_NEW >> $LOGFILE
  echo -n "CONFIG_MES=" >> $LOGFILE
  echo $CONFIG_MES >> $LOGFILE

  cliOutputFile="/tmp/ia_storage_out.txt"

  # ========================================================================
  # - Ping controller A and controller B (swallow the error message by
  #   redirecting stderr to /dev/null)
  # - If both pings succeeded, continue, otherwise log an error message
  #   and call our exit routine
  # ========================================================================
  pingAResults=`ping $CTRLA -c 1 2> /dev/null`
  pingARC=$?
  pingBResults=`ping $CTRLB -c 1 2> /dev/null`
  pingBRC=$?
  if [ $pingARC -ne 0 ] || [ $pingBRC -ne 0 ]; then
    echo "Unable to ping one of the disk controllers: $CTRLA or $CTRLB" | tee -a $LOGFILE
    echo "Ping results for $CTRLA are: $pingAResults" | tee -a $LOGFILE
    echo "Ping results for $CTRLB are: $pingBResults" | tee -a $LOGFILE
    fail
  fi

  # move any existing SMcli exception report files out of the way
  # of this particular "run" of the storage configuration script
  if [ -f /var/opt/SM/excprpt.txt ]; then
    echo "Moving existing SMcli error report file to some version of /var/opt/SM/excprpt.txt.prev before running $0" >> $LOGFILE
    mv --force --backup=numbered /var/opt/SM/excprpt.txt /var/opt/SM/excprpt.txt.prev >> $LOGFILE
  fi

  # determine the controller type DS4200 or DS3950
  # DS4200 -->  CONTROLLER_TYPE=1
  # DS3950 -->  CONTROLLER_TYPE=2
  # First get the Product ID
  controllerProductId=`$SMCLI $CTRLA $CTRLB -c "show controller [a] summary;" | grep -i -m 1 "Product ID" | cut -d: -f2 | awk '{ print $1$2 }' | sed 's/\.//g'`
  echo "Controller Product ID is $controllerProductId"  >> $LOGFILE
  # Both DS4200 and DS3950 have the same Product ID
  if [[ "$controllerProductId" == "1814FAStT" ]]; then
    # Now get the revision to determine if it's 4200 or 3950
    controllerRevision=`$SMCLI $CTRLA $CTRLB -c "show controller [a] summary;" | grep -i -m 1 "Revision" | cut -d: -f2 | awk '{ print $1 }' | sed 's/\.//g'`
    echo "Controller Revision is $controllerRevision"   >> $LOGFILE
    # DS4200 controllers have Revision of "0916"
    # DS3950 controllers have Revision of "1065"
    if [[ "$controllerRevision" == "0916" ]]; then
      CONTROLLER_TYPE=1
    elif [[ "$controllerRevision" == "1065" ]]; then
      CONTROLLER_TYPE=2
    else
      echo "Error: Unable to determine controller as DS4200 or DS3950."  | tee -a $LOGFILE
      fail;
    fi
  else
    echo "Error: Unsupported controller."  | tee -a $LOGFILE
    fail;
  fi

  echo -n "CONTROLLER_TYPE=" >> $LOGFILE
  echo $CONTROLLER_TYPE >> $LOGFILE

}

# ************************************************************************
# ************************************************************************
# Function: findFirmwareImages
# - Search the local filesystem to ensure that the firmware images can
#   be found in the directory $FIRMWARE_PATH
# - Return code 15 indicates that one or more firmwares file could not be
#   found, and will trigger an error message from ia_service.py
# ************************************************************************
# ************************************************************************
findFirmwareImages() {
   # Set different ERR_CODE for each differnt controller
   # This is needed so that ia_service.py can display the correct
   # path location for the firmware files based on the controller type.
   # ERR_CODE=42 for DS4200 controller
   # ERR_CODE=39 for DS3950 controller
   # Set the firmware path based on the controller_type
   # Also set pattern used to find firmware images and command used to extract version from file name
   if [ $CONTROLLER_TYPE -eq 1 ]; then
     FIRMWARE_PATH=$FIRMWARE_PATH/DS4200
     PATTERN_FIRMWARE="FW_*.dlp"
     VERSION_CMD="ls ${FIRMWARE_PATH}/${PATTERN_FIRMWARE} | cut -d/ -f6 | cut -d_ -f3 | cut -d. -f1"
     ERR_CODE=42
   elif [ $CONTROLLER_TYPE -eq 2 ]; then
     FIRMWARE_PATH=$FIRMWARE_PATH/DS3950
     PATTERN_FIRMWARE="RC_*.dlp"
     VERSION_CMD="ls ${FIRMWARE_PATH}/${PATTERN_FIRMWARE} | cut -d/ -f6 | cut -d_ -f2 | cut -d. -f1"
     ERR_CODE=39
   fi

   if [ ! -x ${FIRMWARE_PATH} ]; then
      echo "Error: Firmware directory ${FIRMWARE_PATH} does not exist."      | tee -a $LOGFILE
      fail $ERR_CODE;
   fi

   # Ensure that exactly one file matching N*.dlp exists in the FIRMWARE_PATH
   fwcount=`ls ${FIRMWARE_PATH}/${PATTERN_NVSRAM} 2>/dev/null | wc -l`
   if [ "$fwcount" -lt 1 ]; then
      echo "Error: No NVSRAM firmware found in ${FIRMWARE_PATH}"                        | tee -a $LOGFILE
      fail $ERR_CODE;
   elif [ "$fwcount" -gt 1 ]; then
      echo "Error: More than one NVSRAM firmware file found in ${FIRMWARE_PATH}"        | tee -a $LOGFILE
      fail $ERR_CODE;
   else
      echo "Found NVSRAM firmware file `ls ${FIRMWARE_PATH}/${PATTERN_NVSRAM}`"   >> $LOGFILE
   fi

   # Ensure that exactly one file matching FW*.dlp exists in the FIRMWARE_PATH
   fwcount=`ls ${FIRMWARE_PATH}/${PATTERN_FIRMWARE} 2>/dev/null | wc -l`
   if [ "$fwcount" -lt 1 ]; then
      echo "Error: No controller firmware found in ${FIRMWARE_PATH}"         | tee -a $LOGFILE
      fail $ERR_CODE;
   elif [ "$fwcount" -gt 1 ]; then
      echo "Error: More than one controller firmware file found in ${FIRMWARE_PATH}"    | tee -a $LOGFILE
      fail $ERR_CODE;
   else
      echo "Found controller firmware file `ls ${FIRMWARE_PATH}/${PATTERN_FIRMWARE}`"   >> $LOGFILE
   fi

   # Ensure that exactly one file matching esm*.esm exists in the FIRMWARE_PATH
   fwcount=`ls ${FIRMWARE_PATH}/${PATTERN_ESM} 2>/dev/null | wc -l`
   if [ "$fwcount" -lt 1 ]; then
      echo "Error: No ESM firmware found in ${FIRMWARE_PATH}"                           | tee -a $LOGFILE
      fail $ERR_CODE;
   elif [ "$fwcount" -gt 1 ]; then
      echo "Error: More than one ESM firmware file found in ${FIRMWARE_PATH}"           | tee -a $LOGFILE
      fail $ERR_CODE;
   else
      echo "Found ESM firmware file `ls ${FIRMWARE_PATH}/${PATTERN_ESM}`"   >> $LOGFILE
   fi

   echo "All 3 firmware files have been located."                                       | tee -a $LOGFILE
}

# ************************************************************************
# ************************************************************************
# Function: checkFirmwareVersion
# - Search the local filesystem to ensure that the target firmware images
#   are a different version (either older or newer) than the version
#   being installed.
#
# File format looks like this: FW_DS4200_07603800.dlp
# Controller query returns this format: 07.60.38.00
# ************************************************************************
# ************************************************************************
checkFirmwareVersion() {
  controllerFirmware=`$SMCLI $CTRLA $CTRLB -c "show controller [a] summary;" | grep -i -m 1 firmware | cut -d: -f2 | awk '{ print $1 }'`
  controllerFirmwareTrimmed=`echo $controllerFirmware | sed 's/\.//g'`
  echo "Controller Firmware is at level $controllerFirmware"   >> $LOGFILE

  proposedFirmware=`eval $VERSION_CMD`
  echo "Version on disk is $proposedFirmware"                  >> $LOGFILE

  if [ "${controllerFirmwareTrimmed}x" == "${proposedFirmware}x" ]; then
      # Return code 12 indicates that firmware is already at the requested level, and will trigger a warning message from ia_service.py
      echo "Controller is already at the proposed firmware level ($controllerFirmware), and will not be upgraded."   | tee -a $LOGFILE
      fail 12;
  else
      echo "Controller firmware is being changed from ${controllerFirmware} to version ${proposedFirmware}"   | tee -a $LOGFILE
  fi
}


# ************************************************************************
# ************************************************************************
# Function: upgradeFirmware
# - Apply all 3 firmware upgrades to the controller.
# ************************************************************************
# ************************************************************************
upgradeFirmware() {
echo "output file is $cliOutputFile"

   firmwareFilename=`ls ${FIRMWARE_PATH}/${PATTERN_NVSRAM}`
   echo "Configuring NVSRAM firmware for controllers $CTRLA and $CTRLB from file ${firmwareFilename} ..."   | tee -a $LOGFILE
   ${SMCLI} $CTRLA $CTRLB -c "download storageSubsystem NVSRAM file=\"${firmwareFilename}\";" -o "$cliOutputFile"
   verifySMCmd "$?" "$cliOutputFile" "Configure controller FAILED"

   firmwareFilename=`ls ${FIRMWARE_PATH}/${PATTERN_FIRMWARE}`
   echo "Configuring base firmware for controllers $CTRLA and $CTRLB from file ${firmwareFilename} ..."   | tee -a $LOGFILE
   ${SMCLI} $CTRLA $CTRLB -c "download storageSubsystem firmware file=\"${firmwareFilename}\";" -o "$cliOutputFile"
   verifySMCmd "$?" "$cliOutputFile" "Configure NVSRAM controller FAILED"

   firmwareFilename=`ls ${FIRMWARE_PATH}/${PATTERN_ESM}`
   echo "Configuring ESM firmware for controllers $CTRLA and $CTRLB from file ${firmwareFilename} ..."   | tee -a $LOGFILE
   ${SMCLI} $CTRLA $CTRLB -c "download allEnclosures firmware file=\"${firmwareFilename}\";" -o "$cliOutputFile"
   verifySMCmd "$?" "$cliOutputFile" "Configure ESM controller FAILED"

   return $?
}

# ************************************************************************
# ************************************************************************
# Function: resetController
# - Reset the controller
# ************************************************************************
# ************************************************************************
resetController() {
   echo "Reset controller [A] - $CTRLA"   | tee -a $LOGFILE
   ${SMCLI} $CTRLA $CTRLB -c "reset controller [A];" -o "$cliOutputFile"
   verifySMCmd "$?" "$cliOutputFile" "Controller [A] reset FAILED"
   
   echo "Reset controller [B] - $CTRLB"   | tee -a $LOGFILE
   ${SMCLI} $CTRLA $CTRLB -c "reset controller [B];" -o "$cliOutputFile"
   verifySMCmd "$?" "$cliOutputFile" "Controller [B] reset FAILED"

   return $?
}

# ========================================================================
# - input RACKID
# - Add ds4kNprimary=disk_ctrl_N_a to /home/iaadmin/.tiam/application.properties
# - Add ds4kNsecondary=disk_ctrl_N_b to /home/iaadmin/.tiam/application.properties
# ========================================================================
updateApplicationProperties() {
   TEMP1="ds4k${1}primary=disk_ctrl_${1}_a"
   TEMP2="ds4k${1}secondary=disk_ctrl_${1}_b"
   TEMP3=`grep $TEMP1 /home/iaadmin/.tiam/application.properties`
   if [ -z "$TEMP3" ]; then
      echo $TEMP1 >> /home/iaadmin/.tiam/application.properties
   fi
   TEMP4=`grep $TEMP2 /home/iaadmin/.tiam/application.properties`
   if [ -z "$TEMP4" ]; then
      echo $TEMP2 >> /home/iaadmin/.tiam/application.properties
   fi
}

# ************************************************************************
# ************************************************************************
# Function: remove_lun_mapping
# - input CTRLA
# - input CTRLB
# - remove LUN mapping when initializing the 1st 8 pack in the controller and MES
# During manufacturing (not MES) the LUN mapping is removed after this
# script has been run. For MES we don't necessarily want to remove all LUN
# mapping. However the LUN mapping must be removed at some point otherwise
# there will be mapping defined for LUN 31. When we are initializing the 1st
# 8 pack in the controller is no preexisting mapping that
# needs to be maintained.
# ************************************************************************
# ************************************************************************
remove_lun_mapping() {
  # ========================================================================
  # Remove the LUN mapping from all of the logical drives.
  # ========================================================================
  echo "Removing all LUN mapping with command: ${SMCLI} ${CTRLA} ${CTRLB} -c \"remove allLogicalDrives lunMapping;\" -o \"$cliOutputFile\"" >> $LOGFILE
  txtOut=`${SMCLI} ${CTRLA} ${CTRLB} -c "remove allLogicalDrives lunMapping;" -o "$cliOutputFile"`
  verifySMCmd "$?" "$cliOutputFile" "Remove all LUN mapping FAILED"
  echo "" >> $LOGFILE
}

# ************************************************************************
# ************************************************************************
# "main"
#  start of ia_storage_common.sh script processing
# ************************************************************************
# ************************************************************************

# ==============================================================
# Use the bash getopts command to process command line arguments
# Notes:
# - start the getopts options string with a colon to suppress the getopts
#   error message when an invalid option is passed into the script
# - require arguments for option a and b (CTRLA/CTRLB hostname or IP)
# - require an argument for the logfile name
# - require one of "-f", "-n" or "-m", but do not allow more than one
# - allow the "d" (debug) parameter for development purposes
# ==============================================================
while getopts ":a:b:l:mnfdh" optchar ; do case $optchar in
   a) CTRLA=$OPTARG
      ;;
   b) CTRLB=$OPTARG
      ;;
   l) LOGFILE=$OPTARG
      ;;
   f) CONFIG_NEW=0
      CONFIG_MES=0
      CONFIG_FW=1
      ;;
   m) CONFIG_NEW=0
      CONFIG_MES=1
      CONFIG_FW=0
      ;;
   n) CONFIG_NEW=1
      CONFIG_MES=0
      CONFIG_FW=0
      ;;
   d) DEBUG="true"
      ;;
   h) usage
      exit 1
      ;;
   ?) echo "Error: "
      usage
      exit 1
      ;;
   esac
done

if [ -n "$DEBUG" ]; then
  echo "debug: after getopts: CONFIG_FW is $CONFIG_FW; CONFIG_NEW is $CONFIG_NEW; CONFIG_MES is $CONFIG_MES"
fi # end: debug

if [ $# = 0 ]; then
  usage
  exit 0
fi

echo "=======Start Script $0 `date`=====" >> $LOGFILE

# ========================================================================
# Set variables used later in script
# - Out: CRTLA
# - Out: CRTLB
# - Out: RACKID
# - Out: lunNum
# ========================================================================
setControllerVars

# ========================================================================
# - query the number of enclosures that this disk controller is
#   responsible for and create an array with their enclosure numbers
# - set the values for variables used in our for loop below:
#   - Out: EnclIDs array
# ========================================================================
getEnclIDs

if [ -n "$DEBUG" ]; then
  echo "debug: contents of encl IDs is: ${EnclIDs[*]}"
fi # end: debug

# =====================================================================
# If "-f" was specified, upgrade the firmware for the specified
# controllers.  If we got to this point, the controllers have both
# been pinged and responded to SMcli commands, so we should be good
# to go if the firmware images can be found.
# =====================================================================
if [ "$CONFIG_FW" -eq 1 ]; then
   echo "Configuring firmware for controllers"
   findFirmwareImages
   checkFirmwareVersion
   upgradeFirmware
   rc=$?

   if [[ $rc -eq 0 ]]; then
      echo "Controller firmware updates completed successfully."   | tee -a $LOGFILE
      # TIAMC00018206 Firmware update was successful now reset the controller 
      resetController
      rc=$?
      if [[ $rc -eq 0 ]]; then
          echo "Controller reset completed successfully."          | tee -a $LOGFILE
      else
          echo "Controller reset failed."                          | tee -a $LOGFILE
      fi      
   else
      echo "Controller firmware updates failed."                   | tee -a $LOGFILE
   fi

   exit $rc
fi

# ========================================================================
# - get the total number of drives in the controller + all expansion
#   drawers for this controller
# - set the values for variables used in our for loop below:
#   - Out: totalDrvCount
# ========================================================================
getTotalDrives
if [ $totalDrvCount -lt 8 ]; then
   echo "ERROR: less than minimum required amount of drives" >> $LOGFILE
   fail
fi

# =====================================================================
# - This might be an intermixed setup so we have to figure out the drive
#   and enclosure we will work with when we start configuring the storage
#   - find the first 1 unassigned TB drive
#   - find the first 2 assigned TB drive
#   - find the first unassigned 2 TB drive
# =====================================================================

# =====================================================================
# - initialize our 1 TB "unassigned" vars
#   If we are in a 1 TB MES scenario, we will probably have 1 TB
#   unassigned drives
# - When we call "getDriveNumber", the output will include:
#   Out: foundDrive
#   Out: foundEncl
#   Out: foundEnclDrive
#   Out: foundDrvIndex
# =====================================================================
OneTBUnassignedEncl=-1
OneTBUnassignedEnclDrive=-1
OneTBUnassignedDriveIndex=-1

# Note: the SMcli "show drive" results for 1 TB drives shows this capacity:
#    Raw capacity:            931.513 GB
getDriveNumber "931.513 GB" "Unassigned"
if [ "$foundDrive" = "true" ]; then
  OneTBUnassignedEncl="$foundEncl"
  OneTBUnassignedEnclDrive="$foundEnclDrive"
  OneTBUnassignedDriveIndex="$foundDrvIndex"
fi

if [ -n "$DEBUG" ]; then
  echo "debug: OneTBUnassignedEncl       is $OneTBUnassignedEncl      "
  echo "debug: OneTBUnassignedEnclDrive  is $OneTBUnassignedEnclDrive "
  echo "debug: OneTBUnassignedDriveIndex is $OneTBUnassignedDriveIndex"
  read -p "debug: press enter to continue" INPUT
fi # end: debug

# =====================================================================
# - initialize our 2 TB "assigned" vars
# =====================================================================
TwoTBAssignedEncl=-1
TwoTBAssignedEnclDrive=-1
TwoTBAssignedDriveIndex=-1

# Note: the SMcli "show drive" results for 2 TB drives shows this capacity:
#    Raw capacity:            1.819 TB for DS4200
#    Raw capacity:            1,863.017 GB for DS3950

if [ $CONTROLLER_TYPE -eq 1 ]; then
  getDriveNumber "1.819 TB" "Assigned"
elif [ $CONTROLLER_TYPE -eq 2 ]; then
  getDriveNumber "1,863.017 GB" "Assigned"
fi

if [ "$foundDrive" = "true" ]; then
  TwoTBAssignedEncl="$foundEncl"
  TwoTBAssignedEnclDrive="$foundEnclDrive"
  TwoTBAssignedDriveIndex="$foundDrvIndex"
fi

if [ -n "$DEBUG" ]; then
  echo "debug: TwoTBAssignedEncl       is $TwoTBAssignedEncl      "
  echo "debug: TwoTBAssignedEnclDrive  is $TwoTBAssignedEnclDrive "
  echo "debug: TwoTBAssignedDriveIndex is $TwoTBAssignedDriveIndex"
  read -p "debug: press enter to continue" INPUT
fi # end: debug

# =====================================================================
# - initialize our 2 TB "unassigned" vars
# =====================================================================
TwoTBUnassignedEncl=-1
TwoTBUnassignedEnclDrive=-1
TwoTBUnassignedDriveIndex=-1

# Note: the SMcli "show drive" results for 2 TB drives shows this capacity:
#    Raw capacity:            1.819 TB for DS4200
#    Raw capacity:            1,863.017 GB for DS3950

if [ $CONTROLLER_TYPE -eq 1 ]; then
  getDriveNumber "1.819 TB" "Unassigned"
elif [ $CONTROLLER_TYPE -eq 2 ]; then
  getDriveNumber "1,863.017 GB" "Unassigned"
fi

if [ "$foundDrive" = "true" ]; then
  TwoTBUnassignedEncl="$foundEncl"
  TwoTBUnassignedEnclDrive="$foundEnclDrive"
  TwoTBUnassignedDriveIndex="$foundDrvIndex"
fi

if [ -n "$DEBUG" ]; then
  echo "debug: TwoTBUnassignedEncl       is $TwoTBUnassignedEncl      "
  echo "debug: TwoTBUnassignedEnclDrive  is $TwoTBUnassignedEnclDrive "
  echo "debug: TwoTBUnassignedDriveIndex is $TwoTBUnassignedDriveIndex"
  read -p "debug: press enter to continue" INPUT
fi # end: debug

# =====================================================================
# - clean up temporary drive info text files
# =====================================================================
cleanupDriveInfoFiles

# =====================================================================
# - figure out which (assigned/unassigned) 2 TB drive we will actually
#   use to start processing
# - figure out whether we will have to add a hot spare (the 2 TB
#   drive we are processing is also unassigned)
# - Out: TwoTBStartFormatIndex
# - Out: is2TBHotSpare
# =====================================================================
get2TBFlags

# =====================================================================
# Figure out what drive we will start formatting
# - if we are formatting all the drives, start at 1 (so that the "mod"
#   function will work as expected in our last loop)
# =====================================================================
if [ "$CONFIG_NEW" -eq 1 ]; then
  cfgDrvIndex=1
  drvEncl=85
  indexEncl=0
else
  # =====================================================================
  # if we are here, we must be formatting only new drives
  # - if the 2 TB drive value has been set, we start formatting there
  # =====================================================================
  if [ $TwoTBStartFormatIndex -ge 1 ]; then
    cfgDrvIndex=$TwoTBStartFormatIndex
  else
    # ===================================================================
    # - otherwise, start formatting at the 1st unassigned 1 TB drive
    # ===================================================================
    cfgDrvIndex=$OneTBUnassignedDriveIndex
  fi

  # ===================================================================
  # - get the index of the enclosure where our starting drive lives
  # - get the number of the enclosure (85, 0, 1, etc) where the "starting"
  #   drive lives
  # ===================================================================
  indexEncl=$(( $cfgDrvIndex / 16 ))
  drvEncl=${EnclIDs[(( $indexEncl ))]}
fi

# =====================================================================
# - if the first unassigned drive is something other than 1, we need to
#   rebuild our EnclID array
# - based on the enclosure of the 1st unassigned drive number, build
#   an EnclID array that only contains the enclosure IDs with drives
#   that need to be formatted
# - Input to resetEnclIDs: indexEncl
# - Input to resetEnclIDs: numofEncl
# - Output from resetEnclIDs: EnclIDs
# =====================================================================
if [ $cfgDrvIndex -gt 1 ]; then
  resetEnclIDs
fi

# ========================================================================
#   Note: Even with an MES request, we may be setting up a new disk
#   controller (new subsystem) and the first free drive will be set to 1
# - if the entire subsystem is being formatted, configure everything
# - otherwise this information has already been set - do not modify it
# ========================================================================
if [ "$CONFIG_NEW" -eq 1 ] || [ "$cfgDrvIndex" -eq 1 ]; then
  # =====================================================================
  # - set the storage subsystem name on the disk controller (or its
  #   mirror if applicable)
  # - set the storage subsystem's media scan rate
  # - set the storage subsystem's cache block size
  # - set the storage subsystem's cache flush (start and stop) size
  # =====================================================================
  cfgSubsystem

  # ========================================================================
  # - create a host group.
  # - A host is the machine/server attached to the storage subsystem and 
  #   it accesses the storage subsystem logical drives through its HBA
  #   HBA host ports. 
  # ========================================================================
  createHostGrp

  # ========================================================================
  # - input RACKID
  # - Add ds4kNprimary=disk_ctrl_N_a to /home/iaadmin/.tiam/application.properties
  # - Add ds4kNsecondary=disk_ctrl_N_b to /home/iaadmin/.tiam/application.properties
  # ========================================================================
  if [ "$CONFIG_MES" -eq 1 ]; then
    updateApplicationProperties $RACKID
    RMLUNMAP=1
  fi
fi

# ========================================================================
# Notes:
# - in a (UNIX scripting) array (in this case "EnclIDs"), if you use * or
#   @ in place of the index, the value will be all elements, separated by
#   spaces
# - the for loop has a range that starts with the first array element in
#   the EnclIDs array and ends with the last array element in the EnclIDs
#   array
# - loop through all the enclosures (disk controller and drawers) in this
#   rack
# ========================================================================
# ========================================================================
# - initialize the slot number before entering the loops
# ========================================================================
for encl in ${EnclIDs[@]}
do

  if [ -n "$DEBUG" ]; then
    echo "Outer while loop: total drive count: $totalDrvCount; Enclosure: $encl; current slot: $slotNum"
    echo ""
  fi # end: debug

  # for slotNum in 1 9
  slotNum=$(( ${cfgDrvIndex} % 16 ))
  while [ $slotNum -lt 10 ] && [ $cfgDrvIndex -lt $totalDrvCount ] # i.e. 1 or 9
  do

    # ========================================================================
    # - cfg drv index gets incremented at the bottom of the loop, so
    #   recalculate the slot number before going into the inner loop
    # - mod/remainder results of 1 mean that this is the 1st half of the
    #   drawer ()
    #   mod/remainder results of 9 mean that this is the 2nd half of the
    #   drawer
    # ========================================================================
    if [ -n "$DEBUG" ]; then
      echo "Inner while loop: current enclosure: $encl; current slot: $slotNum; current cfgDrvIndex: $cfgDrvIndex"
      echo ""
      read -p "debug: press enter to continue" INPUT
    fi # end: debug

    if [ $cfgDrvIndex -lt $totalDrvCount ]; then
      echo "Configuring enclosure $encl at drive $slotNum" >> $LOGFILE
      echo "Current drive index: ${cfgDrvIndex}" >> $LOGFILE

      # ========================================================================
      # - Enclosure IDs are in this order: 85, 0, 1, 2, etc (through 7)
      #   - array element 0: 85 (Disk controller)
      #   - array element 1:  0 (Expansion drawer 1)
      #   - array element 2:  1 (Expansion drawer 2)
      #   - array element 3:  2 (Expansion drawer 3)
      #   - etc
      # - if this is the 0th element (contents == 85) in the
      #   enclosure/array, we are configuring 1 of 3 possible Disk
      #   Controllers
      # ========================================================================

      if [ $encl -eq 85 ]; then

        if [ $slotNum -eq 1 ];then
          # ==============================================================
          # - if we are working in the 1st half of the drawer
          #   and
          #   we are working with the disk controller 1 in rack 1, we
          #   need to configure it as a RAID 6 array with:
          #   - a hot spare in drive 8
          #   - the 1 TB util file system (utilfs, to be used as GPFS file
          #     system /tiam_utility)
          #   - a 3.5 TB data file system (iadata_85_1)
          #   - a 30 MB emergency boot drive (iarecovery) used to bootstrap
          #     GPFS on Disaster Recover (DR)
          #
          # - Note that cfg-array-util checks the capacity of the drive
          #   being processed so it can configure RAID array w/correct size
          #
          # - After running cfgArrayUtil, available space left in this
          #   (iaarray_85_1) array is:
          #   - For 1 TB drives, there is .046 TB/46 GB of space left in this array
          #   - For 2 TB drives, there is .092 TB/92 GB of space left in this array
          #
          #   - unallocated space will be used for:
          #     - 1 GB: Enhanced Remote Mirroring (ERM)
          #     - when DR gets "activated", Mirror Repository disks get
          #       created in the first array (example from Bryan Jen shows
          #       each of the Mirror Repository disks to be approx 132 MB in size)
          # ==============================================================
          if [ $RACKID -eq 1 ]; then
         cfgArrayUtil

          # ==============================================================
          # - if we are working in the 1st half of the drawer
          #   and
          #   we are working with disk controller 2 or 3 in rack 2, it needs
          #   to be configured with a hot spare (and reserved/unassigned)
          #   space for ERM support
          # - Note that cfg-array-w/spare checks the capacity of the drive
          #   being processed so it can configure RAID array w/correct size
          # ==============================================================
          else
         cfgArrayWSpare $encl 1
          fi

        else # slot is 9/2nd half of array
          # ==============================================================
          # - if the very first, not configured new 2 TB drive lives in the
          #   second half of the first disk controller, (i.e. cfgDrvIndex is
          #   9) we need to give it a hot spare
          # ==============================================================
          if [ $cfgDrvIndex -eq $TwoTBStartFormatIndex ] && [ $is2TBHotSpare = "true" ]; then
             cfgArrayWSpare $encl 2

          # ==============================================================
          # - otherwise, just configure the RAID array
          # ==============================================================
          else [ $cfgDrvIndex -lt $totalDrvCount ]
            cfgArray $encl 2
          fi
        fi

      # ========================================================================
      # - if we got here, we are working on a non-disk-controller drawer
      # ========================================================================
      elif [ $encl -eq 2 ]; then
        # ======================================================================
        # - if this is the 3rd expansion drawer (enclosure == 2, drives
        #   numbers 49-64) in the enclosure/array, it ALWAYS needs a hot
        #   spare in the first half of its enclosure
        # ======================================================================
        if [ $slotNum -eq 1 ]; then
       cfgArrayWSpare $encl 1

        else
          # ==============================================================
          # - if the very first, not configured new 2 TB drive lives in the
          #   second half of the 3rd expansion drawer (i.e. cfgDrvIndex is
          #   57), we need to give this half of the enclosure a hot spare
          # ==============================================================
          if [ $cfgDrvIndex -eq $TwoTBStartFormatIndex ] && [ $is2TBHotSpare = "true" ]; then
             cfgArrayWSpare $encl 2

          else [ $cfgDrvIndex -lt $totalDrvCount ]
             cfgArray $encl 2
          fi
        fi

      # ========================================================================
      # - if we got here, we are working on a non-disk-controller drawer
      #   and this is something other than the 3rd drawer in the
      #   enclosure/array. Normally, no hot spare is needed
      # ========================================================================
      else # not encl 85 or 2
        if [ $slotNum -eq 1 ];then
          # ==============================================================
          # - if the very first, not configured new 2 TB drive lives in the
          #   first half of this drawer, we need to give it a hot spare
          # ==============================================================
          if [ $cfgDrvIndex -eq $TwoTBStartFormatIndex ] && [ $is2TBHotSpare = "true" ]; then
            cfgArrayWSpare $encl 1
          else
            cfgArray $encl 1
          fi

        else # slot is 9/2nd half of array
          # ==============================================================
          # - similarly, if the very first, not configured new 2 TB
          #   drive lives in the second half of this drawer, we need to
          #   give it a hot spare
          # ==============================================================
          if [ $cfgDrvIndex -eq $TwoTBStartFormatIndex ] && [ $is2TBHotSpare = "true" ]; then
            cfgArrayWSpare $encl 2
          else
            cfgArray $encl 2
          fi
        fi
      fi     # end: start of 'if/else [ $encl -eq 85 ];'
    fi       # end: if [ $cfgDrvIndex -lt $totalDrvCount ]

  # ==============================================================
  # - this half of the enclosure has been configured so move to
  #   the next 8 pack
  # ==============================================================
  cfgDrvIndex=$(( ${cfgDrvIndex} + 8 ))
  slotNum=$(( ${slotNum} + 8 ))

  done  # while $slotNum in 1 or 9

done    # for encl in ${EnclIDs[@]}

# ========================================================================
# - input CTRLA
# - input CTRLB
# - remove LUN mapping when initializing the 1st 8 pack in the controller and MES
# ========================================================================
if [ "$RMLUNMAP" -eq 1 ]; then
  remove_lun_mapping
fi

echo "Configuration is complete. Logical Drives should be formatting now."
echo "For more information, refer to logfile $LOGFILE"

echo "=======End Script `date`=====" >> $LOGFILE
