#! /usr/bin/env python
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Change Activity
#
# Name           Date         Change ID      Description
# Brad Trubey    06/10/2009   TIAMC00000000  Initial Version
# Brad Trubey    07/02/2009   TIAMC00006665  Add -p support for sending commands to only one cluster node
# Brad Trubey    07/21/2009   TIAMC00007641  Skip component commands when setting level to default if the
#                                            default level is the same as the current level
# Brad Trubey    08/03/2009   TIAMC00007828  Add support for skipping trace config on reboot.
# Brad Trubey    08/10/2009   TIAMC00007840  Prevent stack dump when trace_current cannot be found.
# Brad Trubey    08/21/2009   TIAMC00008476  Move properties files to local filesystem
# Brad Trubey    09/04/2009   TIAMC00008933  Update trace_current regardless of runCommand results
# Brad Trubey    01/26/2010   TIAMC00009143  Trace Config improvements for 1.2
# Brad Trubey    05/21/2010   TIAMC00013770  ia_service umbrella for 2.1
# Brad Trubey    09/22/2010   TIAMC00015701  Change gethostname calls to getIAhostname
################################################################################

import getopt, sys, os, socket, gettext, locale
from tiam_util import do_main
from gettext import gettext as _
from ia_util import *

################################################################################
# ia_trace.py
#
# This is the IA trace configuration utility.  It serves two roles:
#
# First, it determines what components need to be configured based on the
# specified  command line, then identifies all nodes which are running that
# component.  It uses ssh to launch an instance of the trace config utility on
# each of those nodes.  This is known as its "command routing" mode, and is
# triggered if "-m execute" is NOT passed on the command line.
#
# Second, it determines what commands need to be run for the specified
# component, runs those commands, and captures the output.  This is known as
# its "command execution" mode, and is triggered if "-m execute" is passed on
# the command line.
################################################################################

# Globalization and Localization setup.  These lines MUST COME BEFORE the "_()" lookups
locale.setlocale(locale.LC_ALL,'')
gettext.textdomain('com.ibm.ia.ras')

#@trans The relative order of the %s and %d parameters must not be changed by the translation!
MSG_TC_SUMMARY      = _('CTJIB1050I Configuring tracing for component %s on hostname %s')
MSG_TC_FAIL_SUMMARY = _('CTJIB1051E Failed to run configure tracing for component %s on hostname %s')
MSG_TC_SUCC_COMP    = _('CTJIB1052I Successfully set trace level for component %s to level %s on node %s')
MSG_TC_FAIL_COMP    = _('CTJIB1053E Failed to set trace level for component %s to level %s on node %s (return code %d)')
MSG_TC_SKIP_DEFAULT = _('CTJIB1054W Skipped component %s, which is already at the default level %s')
MSG_TC_SKIP_NODE    = _('CTJIB1055W Skipped component %s, because trace was configured on another node')
MSG_TC_SUCC_GEN     = _('CTJIB1056I Trace configuration ran successfully.')
MSG_TC_FAIL_GEN     = _('CTJIB1057E Trace configuration failed for %d component(s) on one or more nodes.')
MSG_TC_FAIL_APPEND  = _('CTJIB1058E Unable to append to file %s')
MSG_TC_COMP_HEADER  = _("Component name must be one of the following:")


# go ahead and load the default and current values
defaultMap = readPropertiesFile("/opt/tivoli/tiam/etc/trace_default.properties")
currentMap = readPropertiesFile("/opt/tivoli/tiam/etc/trace_current.properties")
componentMap = readPropertiesFile("/opt/tivoli/tiam/etc/trace_compnodes.properties")

################################################################################
# Function: main
# This function is the main method for the ia_trace application.  It
# is called by the management broker's main() wrapper method.
################################################################################
def main():
    level=""
    component=""
    oneTargetNode=""
    mode="routing"
    primary=False
    result = 0

    # log the command line arguments for help debugging ssh/sudo issues
    logAndPrint('command line: ' + str(sys.argv))

    # read the command line switches
    try:
        opts, args = getopt.getopt(sys.argv[1:], "c:l:m:n:ph?v")
    except getopt.GetoptError, err:
        logAndPrint(str(err), True)
        showUsage()

    runapp = ""
    for opt, arg in opts:
        if opt in ("-c"):
            component = arg
        elif opt in ("-l"):
            level = arg
        elif opt in ("-m"):
            mode = arg
        elif opt in ("-n"):
            oneTargetNode = arg
        elif opt in ("-p"):
            primary = True
        elif opt in ("-v"):
            setVerboseOutput(True)
        else:
            showUsage()

    # if user asked for component list, dump it and exit.  Note that the component
    # list also dumps the current setting, as well as the default setting.
    if component=="list":
        listComponentStatus()
        errorExit(0, '')

    # ensure that the first message in the trace log is this overview
    if (mode=="routing"):
        logAndPrint("***** Running ia_trace.py in command routing mode on "+getIAhostname()+" *****")
    elif (mode=="execute"):
        logAndPrint("***** Running ia_trace.py in command execution mode on "+getIAhostname()+" *****")
    else:
        showUsage("Unrecognized mode "+mode)

    if oneTargetNode!="":
        nodeFound = False
        nodes = getAllNodes()
        logAndPrint('Searching node list for '+oneTargetNode+' in list '+str(nodes))
        for node in nodes:
            if oneTargetNode==node:
                nodeFound = True
        if nodeFound==False:
            errorExit(1, 'Node '+oneTargetNode+' is not a valid appliance hostname.')


    if component=="":
        showUsage("Component (-c) must be specified on command line.")

    if level=="":
        showUsage("Level (-l) must be specified on command line.")
    elif (level=="min" or level=="mid" or level=="max" or level=="disable" or level=="current" or level=="default"):
        logAndPrint("Got a valid level argument: "+level)
    else:
        showUsage("Unrecognized level "+level)

    # If the mode is "routing", determine which nodes should receive the command and use ssh to send it to them.
    if (mode=="routing"):
        # route the command to the proper nodes
        result = routeCommand(component, level, oneTargetNode)
        logAndPrint('Trace configuration router returns '+str(result))

    # If the mode is "execute", load the commands dictated by the "component" and "level" flag and execute them.
    elif (mode=="execute"):
        # run the command to configure tracing
        result = executeCommand(component, level, primary)
        logAndPrint('Trace configuration on node '+getIAhostname()+' returns '+str(result))

    return result

################################################################################
# Function: routeCommand
# Verify that this command is running on a GPFS cluster node, then identify
# a list of nodes where the specified component is installed.  Route the trace
# config command to each of those nodes, with the "execute" mode set.
#
# Note that a level of "default" or "current" cannot be evaluated by the command
# router because no such mapping exists for the component "all".  The conversion
# to the real level (min/mid/max/disable) does not happen until command
# execution on the node.
#
# Another design point is that some trace commands (I'm looking at you, GPFS)
# can only be executed on one node in the cluster, and not all.  Unfortunately,
# if the user has specified a component of "all" on the command line, then the
# command router must forward the command to the node for execution, and yet
# provide the node with a way of determining whether it is the "primary" GPFS
# node, and therefore responsible for executing the trace config command for
# components (GPFS) which require that it only be run once.  To achieve this
# goal, the command router adds a flag "-p" to the command line for exactly
# one of the GPFS cluster nodes.
#
# The Trace Config tool also allows specifying a specific node for which to
# change the trace level of a component.  For example, you could choose to
# set the MCP trace level to mid on ianode2, without affecting the MCP trace
# level on ianode1 or iamconsole1.  This is achieved by specifying the "-n"
# switch with a hostname, which results in the variable "oneNode" getting
# passed to this method.
#
# This function returns a string consisting of the results of every attempt
# to set the trace level of a component on each node.
################################################################################
def routeCommand(component, level, oneNode):
    # verify that the routing is occuring on a GPFS cluster node
    if confirmClusterNode()==False:
        message= 'Program must be launched from a cluster node'
        errorExit(1,message)

    firstClusterNode = True
    suffix = ''
    allNodesSummary = ''
    result = 0

    # Pass along the verbose flag if it was specified
    if getVerboseOutput()==True:
        verbose = ' -v '
    else:
        verbose = ''

    # identify all the nodes which contain the specified component
    targetNodes = getTargetNodes(component)
    logAndPrint("Retrieved node list for component "+component+": "+str(targetNodes))
    for hostname in targetNodes:
        if (oneNode == '') or (oneNode == hostname):
            # Add the '-p' switch to the command line of the first GPFS cluster node only
            if hostname.startswith('ianode') and firstClusterNode==True:
                suffix = ' -p '
                firstClusterNode = False
            else:
                suffix = ''

            command = ' "sudo /opt/tivoli/tiam/bin/ia_trace.py -m execute -c '+component+' -l '+level+verbose+suffix+'"'
            try:
                logAndPrint(MSG_TC_SUMMARY % (component, hostname), True)
                result += runCommand(command, 'ssh '+hostname)
            except:
                print formatExceptionInfo()
                logAndPrint(MSG_TC_FAIL_SUMMARY % (component, hostname), True)
                result = 1

            # retrieve tc_summary.log file from node
            nodeResults = readSummaryFile(hostname)
            allNodesSummary += nodeResults
        else:
            logAndPrint("Not routing trace command to node '"+hostname+"' because it does not match '"+oneNode+"'")

    # print the summary for all nodes.
    logAndPrint(allNodesSummary, True)
    if result==0:
        logAndPrint(MSG_TC_SUCC_GEN, True)
    else:
        logAndPrint(MSG_TC_FAIL_GEN % (result), True)

    return result

################################################################################
# Function: executeCommand
# Open the file matching the trace level, search through it for the component,
# load the corresponding commands, and execute them.
################################################################################
def executeCommand(component, level, primary):
    combinedResult = 0

    # Start by deleting any summary file from a previous run
    summaryFile = '/tmp/tc_summary.log'
    try:
        os.remove(summaryFile);
    except OSError:
        logAndPrint('File '+summaryFile+' does not exist ... skipping delete.')
        pass

    # retrieve a list of components that should run on this host
    targetComponents = getComponentList(component)
    logAndPrint("Converted component "+component+" into list "+str(targetComponents))

    # For each targeted component, create a command line, identify the target nodes, and execute the command
    for xComponent in targetComponents:
        # changeLevel is true if the "set level" command should be run; false if it should be skipped
        changeLevel = True
        # summaryMsg is the string that will get written into tc_summary.log and ends up in stdout
        summaryMsg = ''

        # intercept any requests to set the level to current, and load level from trace_current.properties
        if level=='current':
            xLevel = currentMap.get(xComponent)
            logAndPrint("For component "+xComponent+", converted level "+level+" into value "+xLevel)

        # intercept any requests to set the level to default, and load level from trace_default.properties
        elif level=='default':
            xLevel = defaultMap.get(xComponent)
            logAndPrint("For component "+xComponent+", converted level "+level+" into value "+xLevel)
            # Compare the default level to level in trace_current.properties.  Skip execution if they are the same
            currentLevel = currentMap.get(xComponent)
            if xLevel == currentLevel:
                changeLevel = False
                summaryMsg = MSG_TC_SKIP_DEFAULT % (xComponent, xLevel)

        # nothing special about this level ... just pass it along to the command line as is
        else:
            xLevel = level

        # if the target node type is the special "ianode_one" type (for restricting GPFS command execution
        # to a single cluster node) and this node is not the primary node, then do not send a command to
        # change the trace level.  However, the trace_current file still needs to be updated.
        hostname = getIAhostname()
        targetNodes = componentMap[xComponent]
        if hostname[0:5] in targetNodes:
            if ('ianode_one' in targetNodes) and (primary==False):
                changeLevel = False
                summaryMsg = MSG_TC_SKIP_NODE % (xComponent)
                updateTraceCurrentFile(xComponent, xLevel)

        # if this component still needs to have its trace level changed, do it here
        if changeLevel == True:
            logAndPrint('Setting the trace level for '+xComponent+' to '+xLevel+'.')
            compMap = readPropertiesFile("/opt/tivoli/tiam/etc/trace_"+xLevel+".properties")
            command = compMap.get(xComponent)
            logAndPrint("Configuring '"+xComponent+"' with command: "+command)
            result = runLocalCommand(command)
            if result==0:
                summaryMsg = MSG_TC_SUCC_COMP % (xComponent, xLevel, hostname)
                # trace_current.properties on this node only gets updated in TC call succeeded
                updateTraceCurrentFile(xComponent, xLevel)
            else:
                summaryMsg = MSG_TC_FAIL_COMP % (xComponent, xLevel, hostname, result)
                combinedResult += 1     # combined result is the count of failed calls

        # Update the Summary File with the action taken and result
        try:
            f = open(summaryFile, "at")
            logAndPrint(summaryMsg, True)
            f.write(summaryMsg+'\n')
            f.close()
        except IOError:
            logAndPrint(MSG_TC_FAIL_APPEND % (summaryFile), True)
            combinedResult += 1     # combined result is the count of failed calls
            pass

    # return a count of failed calls.  0 means everything worked.
    return combinedResult

################################################################################
# Function: listComponentStatus
# This function displays the current component status for every component on
# every node in the appliance, based on the status tracked in the file
# trace_current.properties retrieved from each node.
################################################################################
def listComponentStatus():

    compSummaryLines = dict()
    nodes = []
    nodes = getAllNodes()
    compnames = getAllComponents()
    logAndPrint(MSG_TC_COMP_HEADER, True)

    # Initialize an output string for each component with the component name and default value
    for compname in compnames:
        default = defaultMap.get(compname)
        if (str(default) == 'None'):
            default = 'unknown'

        compView = "   " + compname
        if compname!='all':
            # add an extra tab to short component names to help alignment
            if len(compname) <= 4:
                compView = compView + "\t"
            compView += "\t(default="+str(default)
        compSummaryLines[compname] = compView

    # Retrieve the current trace level for each component on each node, and add it to the output string
    for node in nodes:
        nodeLevelMap = dict()        # initialize map to clear previous entries
        if (ping(node) == True):
            try:
                # Retrieve the trace_current file from the node
                command = 'scp -B '+node+':/opt/tivoli/tiam/etc/trace_current.properties /tmp'
                runLocalCommand(command)

                # load the current trace levels for that node into a map
                nodeLevelMap = readPropertiesFile("/tmp/trace_current.properties")
            except IOError:
                logAndPrint('   Unable to retrieve status for components on node '+node)
                pass
        else:
            logAndPrint('   Unable to contact node '+node)

        # Now the trace_current file has been retrieved from a node and loaded into a map.  Go through
        # the map and for each component, append the current trace level to the output string.
        for compname in compnames:
            if compname!='all':
                visible = compSummaryLines[compname]
                if isComponentOnNode(compname, node):
                    level = nodeLevelMap.get(compname)
                    if str(level)=='None':
                        visible += ", "+node+"=unknown"
                    else:
                        visible += ", "+node+"="+level
                    compSummaryLines[compname] = visible

    # Now display the untranslated output strings
    for compname in compnames:
        if compname=='all':
            logAndPrint(compSummaryLines[compname], True)         # no closing paren on 'all'
        else:
            logAndPrint(compSummaryLines[compname]+")", True)


################################################################################
# Function: isComponentOnNode
# This function determines whether the specified component is installed on
# the specified hostname.
################################################################################
def isComponentOnNode(component, hostname):
    compnodes = componentMap.get(component)
    targetNodeTypes = compnodes.split(",")
    targetNodes = convertNodesToHostnames(targetNodeTypes)
    for node in targetNodes:
        if node==hostname:
            return True

    return False

################################################################################
# Function: getTargetNodes
# Used by the command router to retrieve a list of hostnames that are running
# the component by querying metadata from trace_compnodes.properties
################################################################################
def getTargetNodes(comp):
    logAndPrint('Locating all nodes containing component "'+comp+'"')
    compMap = readPropertiesFile('/opt/tivoli/tiam/etc/trace_compnodes.properties')
    value = compMap.get(comp)
    # Now split the comma separated string into individual nodes
    if (value==None):
        message = 'Unrecognized component received: '+comp
        errorExit(1,message)

    targetNodeTypes = value.split(",")
    targetNodes = convertNodesToHostnames(targetNodeTypes)
    return targetNodes


################################################################################
# Function: showUsage
# This function dumps the command line syntax for the program to standard
# output, then exits with RC=1.
#
# Note that from a globalization standpoint, these messages should never get
# displayed and are not translated.  IA only documents the usage of ia_service,
# which validates all input before calling ia_trace, so ia_trace usage messages
# will not get displayed.
################################################################################
def showUsage(message):
    if message != '':
        logAndPrint(message, True)
        logAndPrint('', True)
    logAndPrint('usage:', True)
    logAndPrint('  '+sys.argv[0]+'                  Runs the Trace Config utility                              ', True)
    logAndPrint('usage:', True)
    logAndPrint('        -c <arg>       Sets target component                                                  ', True)
    logAndPrint('                         <arg> can be component name, "all" to apply to all component,        ', True)
    logAndPrint('                         or "list" to see all valid components with current and default level ', True)
    logAndPrint('        -l <arg>       Sets trace level                                                       ', True)
    logAndPrint('                         <arg> can be "min", "mid", "max", "disable", "current", "default"    ', True)
    logAndPrint('                         min, mid, or max sets the trace level to the specified value         ', True)
    logAndPrint('                         "disable" turns off tracing for the specified component              ', True)
    logAndPrint('                         "current" re-applies trace level to its current value                ', True)
    logAndPrint('                         "default" sets trace level back to default setting                   ', True)
    logAndPrint('        [-n <arg>]     Sets trace level on a specific node                                    ', True)
    logAndPrint('                         <arg> can be "iarsm1", "iamconsole1", "ianodeN"                      ', True)
    logAndPrint('        [-v]           Enables verbose output                                                 ', True)
    errorExit(1)


# Call the Management Broker's wrapper for main()
if __name__ == "__main__":
    do_main(main, True)
