#! /usr/bin/env python
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Change Activity
#
# Name           Date         Change ID      Description
# Brad Trubey    06/10/2009   TIAMC00000000  Initial Version
# Brad Trubey    07/02/2009   TIAMC00006665  Add -p support for sending commands to only one cluster node
# Brad Trubey    08/10/2009   TIAMC00007840  Prevent stack dump when trace_current cannot be found.
# Brad Trubey    08/21/2009   TIAMC00008476  Move properties files to local filesystem
# Linda Benhase  09/16/2009   TIAMC00008348  Handle when ssh keys do not match on the node by not failing.
# Linda Benhase  10/8/2009    TIAMC00009889  If the hostname is not iarsm or ianode then it MUST be iamconsole.
#                                            Customer may change MC hostname so we can't test for iamconsole.
# Brad Trubey    01/26/2010   TIAMC00009143  Trace Config improvements for 1.2
# Brad Trubey    05/21/2010   TIAMC00013770  ia_service umbrella for 2.1
# Brad Trubey    09/03/2010   TIAMC00015108  Add support for "director unlock"
# Brad Trubey    09/22/2010   TIAMC00015701  Change gethostname calls to getIAhostname
# Brad Trubey    11/08/2010   TIAMC00016579  Allow "su to iaadmin" before running commands
# Brad Trubey    12/16/2010   TIAMC00017332  Change "xseries" to "systemx"
################################################################################

import getopt, sys, os, socket, gettext, locale
from tiam_util import *
from gettext import gettext as _
from socket import gethostname

################################################################################
# ia_util.py
#
# This is a collection of methods used by the ia_service and ia_trace utilities.
# Note that some of these functions are now in wider use within IA, and should
# therefore be treated as public APIs.  DO NOT CHANGE the interfaces without a
# search of the both the GUI and main codebase to ensure that a change will not
# impact other code!
################################################################################

# controlled by the "-v" switch on the command line
verboseOutput = False;

# Globalization and Localization setup.  These lines MUST COME BEFORE the "_()" lookups
locale.setlocale(locale.LC_ALL,'')
gettext.textdomain('com.ibm.ia.ras')

MSG_ERR_PING_NODE = _('CTJIB1070E Unable to contact node %s')

################################################################################
# Function: findRoutingNode
# Identify the appropriate node to route this command to, using the following
# algorithm:
#   1. If the component is 'imm', this command will be run from the MC node.
#      The imm scripts use telnet to connect to the IMM attached to the System X
#      node specified by '-n', but we want the command to run from MC, not
#      re-routed to the node itself.
#   2. If the runapp is 'trace', or 'getlogs' combined with 'min' or 'full',
#      this command will be run from a GPFS node.
#      The trace utility will take care of routing the command using the
#      node specified by '-n', but ia_service always routes these commands
#      to GPFS first.
#   3. If "-n" is valid for the command and is specified (passed via the variable
#      commandLineNode), the command is routed to that node.
#   4. If "-n" is not specified, and the localhost is a member of the RunsOnNode
#      list, then the command is run locally.  The variable runsOnNode contains
#      some combination of the letters "m" (for Management Console), "r" (for
#      RSM node, and "g" (for GPFS nodes).
#   5. If none of the above conditions are true, the command gets routed to one
#      of the nodes indicated by the RunsOnNode value.  If the RunsOnNode value
#      identifies multiple nodes, one of the matching nodes is chosen in a
#      non-deterministic manner.
#
################################################################################
def findRoutingNode(runapp, component, commandLineNode, runsOnNode):

    # Test 1: if component is 'imm' this command will be run from MC
    if component == 'imm':
        return 'iamconsole1'

    # Test 2: if runapp is 'trace' this command will be run from GPFS
    if ((runapp == 'trace') or
        (runapp == 'getlogs' and component == 'min') or
        (runapp == 'getlogs' and component == 'full')):
        nodes = ('ianode1', 'ianode2', 'ianode3')
        return searchForNode(nodes)

    # Test 3: if '-n' tells use where to run the command, go with that
    if commandLineNode != '':
        if runsOnNode.find('n') == -1:
            logAndPrint('Unsupported option "-n" specified for command')
            return None
        return commandLineNode

    # Test 4: if localhost matches one of the RunsOnNode on options, go with that
    mynode = getIAhostname()

    if (runsOnNode.find('m') != -1) and (mynode == 'iamconsole1'):
        return mynode

    if (runsOnNode.find('r') != -1) and (mynode == 'iarsm1'):
        return mynode

    if (runsOnNode.find('g') != -1) and (mynode.startswith('ianode')):
        return mynode

    # Test 5: This command needs to be routed to a node where it can be run
    if runsOnNode.find('m') != -1:
        return 'iamconsole1'

    if runsOnNode.find('r') != -1:
        return 'iarsm1'

    if runsOnNode.find('g') != -1:
        nodes = ('ianode1', 'ianode2', 'ianode3')
        return searchForNode(nodes)

    # if none of those things work, return None so the parser fails
    return None

################################################################################
# Function: searchForNode takes an array of hostnames and returns the first one
# that can be pinged, or an empty string if none can be reached.
#
# Public API
################################################################################
def searchForNode(nodes):
    for node in nodes:
        logAndPrint('Pinging node '+node)
        rc = syscall_nolog('ping -c 1 '+node+' > /dev/null')
        if rc==0:
            logAndPrint('Located active node '+node)
            return node
    return None

################################################################################
# Function: ping returns True or False depending on whether the node responds to
# a ping.
################################################################################
def ping(node):
    logAndPrint('Pinging node '+node)
    rc = syscall_nolog('ping -c 1 '+node+' > /dev/null 2> /dev/null')        # redirect both stderr and stdout
    if rc==0:
        logAndPrint('Node is reachable: '+node)
        return True
    return False

################################################################################
# Function: confirmClusterNode
# Confirm that this utility is being run from a GPFS node
#
# Public API
################################################################################
def confirmClusterNode():
    hostname = os.uname()[1]
    if (hostname.startswith('ianode')):
        logAndPrint('Confirmed that this node is a GPFS cluster node: '+hostname)
        return True
    else:
        return False

################################################################################
# Function: errorExit echo a message to the console, then exit with the return
# code.
#
# Note that for globalization, the string passed in to this method should
# already have been converted to the translated message by a call to "_()".
################################################################################
def errorExit(retval, message=''):
    if message!='':
        logAndPrint(message, True)
    sys.exit(retval)

################################################################################
# Function: get and set functions which track the setting of the "-v" switch on
# the command line.  This setting is used by logAndPrint to determine whether to
# produce normal or verbose output to the console.
################################################################################
def setVerboseOutput(val=False):
    global verboseOutput
    verboseOutput = val

def getVerboseOutput():
    global verboseOutput
    return verboseOutput

################################################################################
# Function: alwaysByName
# Convert a node type into a hostname
################################################################################
def alwaysByName(node):
    try:
        int(node[0])
        name = socket.gethostbyaddr(node)[0]
        return name
    except:
        return node

################################################################################
# Function: runLocalCommand
# Uses ssh the execute the specified command, with the output echo'd to the
# console, indented for readability.
################################################################################
def runLocalCommand(command):

    logAndPrint('Running full command: '+command)
    output, result = syscallread2(command, 0, command)

    # <todo> redirect this to a log file?  -- removed True
    for line in output:
        line = line.rstrip()
        logAndPrint('  '+line)

    logAndPrint('Call completed with return code ' + str(result))

    return result

################################################################################
# Function: runLogCollectionCommand
# Uses ssh the execute the specified command, without dumping command output to
# the console (unless -v is specified).  After completion, the output is scanned
# for a string indicating that log collection is already running, which triggers
# a special return code to ia_service.py.
################################################################################
def runLogCollectionCommand(command, sshTarget, suToIaadmin=False):
    if sshTarget == '':
        fullcommand = command
    else:
        fullcommand = sshTarget + ' ' + command

    if suToIaadmin == True:
        logAndPrint('Adding su wrapper to command: '+fullcommand)
        fullcommand = 'su iaadmin -c "' + fullcommand + '"'

    logAndPrint('Running full command: '+fullcommand)
    output, result = syscallread2(fullcommand, 0, fullcommand)

    logAndPrint('Call completed with return code ' + str(result))
    for line in output:
        line = line.rstrip()
        if (line.find('seems to be running') != -1):
            result = 42
        if (verboseOutput == True):
            logAndPrint(line, True)

    return result

################################################################################
# Function: runCommand
# Uses ssh the execute the specified command, with the output echo'd to the
# console.
#
# If flag suToIaadmin is set, then wrap the entire command in an su call.
#
# If flag toScreen is set, then echo command output to stdout in addition to logging it.
#
# If outputPrefix string is provided, write the string to the logs before the command output.
#
# Public API
################################################################################
def runCommand(command, sshTarget='', suToIaadmin=False, toScreen=False, outputPrefix=''):

    if sshTarget == '':
        fullcommand = command
    else:
        fullcommand = sshTarget + ' ' + command

    if suToIaadmin == True:
        logAndPrint('Adding su wrapper to command: '+fullcommand)
        fullcommand = 'su iaadmin -c "' + fullcommand + '"'

    logAndPrint('Running full command: '+fullcommand)
    output, result = syscallread2(fullcommand, 0, fullcommand)

    # If command output is being echo'd to the screen, allow caller to pass a
    # prefix message to be prepended to the output.
    if ((toScreen==True) and (outputPrefix != '')):
        logAndPrint(outputPrefix, True)

    # This output from the command gets logged, but not echoed to the screen
    for line in output:
        line = line.rstrip()
        logAndPrint(line, toScreen)

    logAndPrint('Call completed with return code ' + str(result))

    return result

################################################################################
# Function: formatExceptionInfo
# Convert an exception into something that can be printed and easily understood.
################################################################################
def formatExceptionInfo():
    classname, exception, callstack = sys.exc_info()

    try:
        excName = classname.__name__
        excArgs = exception.__dict__['args']
    except KeyError:
        excArgs = '<no args>'
    excStacktrace = traceback.format_tb(callstack)
    return (excName, excArgs, excStacktrace)


################################################################################
# Function: logAndPrint
# Log a message depending on the current trace level, and write it to the
# console conditionally depending on a config variable which can be overridden
# by a method argument.
################################################################################
def logAndPrint(message, always=False):
    if tracelevel > TRACE_OFF:
        syslog(message)
    if (verboseOutput == True) or (always==True):
        print message

################################################################################
# Function: getAllNodes
# Retrieve a list of all nodes in the appliance.
# Note: This function only works when run on a GPFS node: under the covers it
# calls "mmgetstate -a", which will not work from an MC or RSM node.
################################################################################
def getAllNodes():
    config = get_global_config()
    hostnames = []
    activeNodes = get_nodes()
    for gpfsnode in activeNodes:
        hostnames.append(gpfsnode)
    hostnames.append(alwaysByName(config['mconsoleip']))
    hostnames.append(alwaysByName(config['rsmip']))

    return hostnames

################################################################################
# Function: getAllComponents
# Retrieve a list of all components supported by the trace config utility.
################################################################################
def getAllComponents():
    compMap = readPropertiesFile('/opt/tivoli/tiam/etc/trace_compnodes.properties')

    filteredComps = compMap.keys()
    filteredComps.sort()
    logAndPrint('Found these components: '+ str(filteredComps))
    return filteredComps

################################################################################
# Function: getComponentList
# Retrieve a list of all components supported by the trace config utility on
# the current host.  If a specific component is passed on the command line,
# this is a straightforward map lookup in the comp->node map.  If the value
# on the command line is "all", then we must examine each component to decide
# if it should be run on this node.
################################################################################
def getComponentList(component):
    compMap = readPropertiesFile('/opt/tivoli/tiam/etc/trace_compnodes.properties')

    hostname = getIAhostname()
    # intercept any requests targeted to component 'all', and convert into a list of components.
    if component=='all':
        # Filter based on the target node
        logAndPrint('Locating all components running on node '+hostname)
        filtMap = dict()
        allComps = compMap.keys()

        for xComponent in allComps:
            # the full list of components will contain 'all', but we want to filter it out
            if xComponent=='all':
                continue
            targetNodes = compMap[xComponent]
            if hostname[0:5] in targetNodes:
                filtMap[xComponent] = targetNodes
        filteredComps = filtMap.keys()
        filteredComps.sort()

    # Only one component specified ... determine whether it should be restricted
    else:
        targetNodes = compMap[component]
        if hostname[0:5] in targetNodes:
            filteredComps = [component]

    logAndPrint('Found these components running on '+hostname+': '+ str(filteredComps))
    return filteredComps

################################################################################
# Function: convertNodesToHostnames
# Convert an array of node types into a list of corresponding hostnames
# For example, convert "iamconsole,ianode" into ["iamconsole1", "ianode1", "ianode2"]
#
# Note: This function only works when run on a GPFS node: under the covers it
# calls "mmgetstate -a", which will not work from an MC or RSM node.
################################################################################
def convertNodesToHostnames(nodes):
    hostnames = []
    config = get_global_config()
    for node in nodes:
        if (node.startswith('iamconsole')):
            hostnames.append(alwaysByName(config['mconsoleip']))
        elif (node.startswith('iarsm')):
            hostnames.append(alwaysByName(config['rsmip']))
        elif (node.startswith('ianode')):
            activeNodes = get_nodes()
            for gpfsnode in activeNodes:
                hostnames.append(gpfsnode)
        else:
            logAndPrint('Skipping unrecognized node type '+node)
    logAndPrint('Converted "'+node+'" into hostname(s): '+str(hostnames))
    return hostnames

################################################################################
# Function: readSummaryFile
# Retrieve a file containing the summary of trace configuration for the
# specified node, and return it as a string.
################################################################################
def readSummaryFile(node):
    srcSummaryFile = '/tmp/tc_summary.log'
    dstSummaryFile = '/tmp/tc_summary.log.'+node

    try:
        os.remove(dstSummaryFile);
    except OSError:
        logAndPrint('File '+dstSummaryFile+' does not exist ... skipping delete.')
        pass

    summary = ''
    if (ping(node) == True):
        try:
            # Retrieve the summary file from the node
            command = 'scp -B '+node+':'+srcSummaryFile+' '+dstSummaryFile
            runLocalCommand(command)
            logAndPrint('Reading the summary file '+srcSummaryFile+' from hostname '+node)
            try:
                f = open( dstSummaryFile, "r")
                for line in f:
                    sline = line.strip() + '\n'
                    summary += sline
                f.close()
            except IOError:
                logAndPrint('Error: Unable to read file '+srcSummaryFile+' from '+node)
                pass
        except IOError:
            pass
    else:
        summary += MSG_ERR_PING_NODE % (node)

    try:
        os.remove(dstSummaryFile);
    except OSError:
        logAndPrint('File '+dstSummaryFile+' does not exist ... skipping delete.')
        pass

    return summary

################################################################################
# Function: updateTraceCurrentFile
# Updates the file which tracks the current trace level for components:
#    /opt/tivoli/tiam/etc/trace_current.properties
# Note that each node now tracks its own status for components, so the same
# component on multiple nodes might have a different trace level on each node.
# This is now reflected in the output of the "-c list" call, which displays
# the current status for a component for every node where it is installed.
################################################################################
def updateTraceCurrentFile(component, newLevel):
    filename = '/opt/tivoli/tiam/etc/trace_current.properties'
    currentMap = readPropertiesFile(filename)

    oldLevel = currentMap.get(component)          # using ".get()" because it returns None if the element doesn't exist, rather than failing
    currentMap[component] = newLevel
    logAndPrint('Updating "'+filename+'" on hostname '+getIAhostname()+' to change '+component+' from '+str(oldLevel)+' to '+newLevel)
    try:
        f = open(filename, "w")
        allComps = currentMap.keys()
        for compname in allComps:
            line = compname + '=' + currentMap[compname]
            f.write(line+'\n')
        f.close()
    except IOError:
        pass

################################################################################
# Function: readPropertiesFile
# Process the Trace Config Utility properties files into a map of component
# names to target nodes or commands.  A typical line in trace_compnodes.properties
# might look like this:
#     tsm.client=ianode,iamconsole
# A typical line in trace_min.properties might look like this:
#     tsm.client=/opt/tivoli/tiam/bin/setTSMTrace.sh min
################################################################################
def readPropertiesFile(filename):
    logAndPrint('Reading the properties file "'+filename+'" on hostname "'+getIAhostname()+'"')
    compMap = dict()
    try:
        f = open( filename, 'r')
        for line in f:
            sline = line.strip()
            if (len(sline) > 1) and (sline[0] != '#'):
                eq_sign = [ sline.find(c) for c in ':= ' ] + [ len(sline) ]
                found = min( [ pos for pos in eq_sign if pos != -1 ] )
                # component type is to the left of the equals sign
                name = sline[:found].rstrip()
                # CSV of node types is to the right of the equals sign
                value = sline[found:].lstrip(':= ').rstrip()
                compMap[name]= value
        f.close()
    except IOError:
        logAndPrint('Error: Unable to read properties file "'+filename+'" on hostname "'+getIAhostname()+'"')
        pass

    #logAndPrint('Returning map: '+str(compMap))
    return compMap


################################################################################
# Function: readIpAddressPropertiesFile
# Process the ia_service utility properties file into 2 maps of component name
# to its "A" or "B" IP Address.  The lines in the properties file are each parsed
# into one entry in each output map.  They must be in this form:
#    pwrctl-iamconsole1 = 172.31.3.102, 172.30.3.102
#
# That line would result in this entry in ipAddressMapA:
#    ipAddressesA[iarsm1] = 172.31.3.1
# That line would result in this entry in ipAddressMapB:
#    ipAddressesB[iarsm1] = 172.30.3.1
#
# Note on terminology.  IA uses the following terms interchangeably:
#    "A" == "local"
#    "B" == "remote" == "mirror"
# The terms "primary" and "secondary" are dynamic, and refer to which appliance
# is currently ingesting files.  Coming out of manufacturing, "primary" is the
# same as "A", and "secondary" is the same as "B".  But if something triggers
# disaster recovery or fallback in the field, then the "primary" appliance can
# be switched to "B", and "secondary" changes to "A".
#
# This function returns two maps:
#    ipAddressesA - map of IA component name to its "A" IP Address
#    ipAddressesB - map of IA component name to its "B" IP Address
#
################################################################################
def readIpAddressPropertiesFile(filename):
    logAndPrint('Reading the properties file "'+filename+'" on hostname "'+getIAhostname()+'"')
    ipAddressesA = dict()
    ipAddressesB = dict()
    try:
        f = open( filename, 'r')
        for line in f:
            sline = line.strip()
            if (len(sline) > 1) and (sline[0] != '#'):
                # Divide the line into two strings based on the equals sign
                equalsIdx     = sline.find('=');
                component     = sline[0:equalsIdx].strip()
                rightOfEquals = sline[equalsIdx+1:].strip()

                # R/C/A entry is to the left of the equals sign.  Split it
                # into its component parts.
                comma     = rightOfEquals.find(',')
                aAddress  = rightOfEquals[0:comma].strip()
                bAddress  = rightOfEquals[comma+1:].strip()

                #logAndPrint('Adding IP Addresses: ['+component+'] = ['+aAddress+'] and ['+bAddress+']')
                ipAddressesA[component]= aAddress
                ipAddressesB[component]= bAddress
        f.close()
    except IOError:
        logAndPrint('Error: Unable to read properties file "'+filename+'" on hostname "'+getIAhostname()+'"')
        pass

    # logAndPrint('Returning map: '+str(rcaScriptMap))
    return ipAddressesA, ipAddressesB


################################################################################
# Function: readServicePropertiesFile
# Process the ia_service utility properties file into 2 maps of run/comp/action
# entries.  Here are some sample lines in service.properties with an explanation
# of how they are handled by this function:
#
# This line:
#    director/server/[start,stop,status,reset] = /opt/tivoli/tiam/bin/ia_dir_server.sh[m]
# Results in these map keys:
#      director/server/start
#      director/server/stop
#      director/server/status
#      director/server/reset
#
# This line:
#    collect/gui/[none] = /opt/tivoli/tiam/bin/zipguilogs.sh[m]
# Results in these map keys:
#      collect/gui/none
#
# This line:
#    san/[1,2]/config = /opt/tivoli/tiam/bin/ia_telnet_san_snmp.py[m]
# Results in these map keys:
#      san/a1/config
#      san/a2/config
#      san/b1/config
#      san/b2/config
#
# This line:
#    systemx/base/mtm = ia_iss.sh[mrg]
# Results in these map keys:
#      systemx/base/mtm
#
# This function returns two maps:
#    rcaScriptMap - map key is "Runapp/Component/Action"; map value is fully
#                   qualified path to the script to run
#    rcaNodeMap   - map key is "Runapp/Component/Action"; map value is string
#                   containing "m", "r", "g" and "n", based on which nodes this
#                   command can be run on (m/r/g), and whether it supports the
#                   "-n" command line switch (n).
#
################################################################################
def readServicePropertiesFile(filename):
    logAndPrint('Reading the properties file "'+filename+'" on hostname "'+getIAhostname()+'"')
    rcaScriptMap = dict()
    rcaNodeMap = dict()
    try:
        f = open( filename, 'r')
        for line in f:
            sline = line.strip()
            if (len(sline) > 1) and (sline[0] != '#'):
                # Divide the line into two strings based on the equals sign
                equalsIdx = sline.find('=');
                leftOfEquals  = sline[0:equalsIdx]
                rightOfEquals = sline[equalsIdx+1:]

                # R/C/A entry is to the left of the equals sign.  Split it
                # into its component parts.
                firstSlash   = leftOfEquals.find('/')
                secondSlash  = leftOfEquals.find('/', firstSlash+1)
                runapp       = leftOfEquals[0:firstSlash]
                component    = leftOfEquals[firstSlash+1:secondSlash].rstrip()
                action       = leftOfEquals[secondSlash+1:].rstrip()
                #logAndPrint('R='+runapp+'/C='+component+'/A='+action)

                # Convert the component(s) into a list
                components = splitCommasIntoList(component)

                # Convert the component(s) into a list
                actions = splitCommasIntoList(action)

                # create an array to hold all the permutations of RCA values
                rcaValues = []
                for comp in components:
                    for act in actions:
                        rcaValues.append(runapp+'/'+comp+'/'+act)

                # parse out the scriptName and runOnNode, which are to the right of the equals sign
                leftBracketIdx = rightOfEquals.rfind('[')          # find the LAST occurence of the '[]'
                rightBracketIdx = rightOfEquals.rfind(']')
                scriptName = rightOfEquals[0:leftBracketIdx].strip()
                runsOnNode = rightOfEquals[leftBracketIdx+1:rightBracketIdx]

                # add map entries for every permutation of the RCA that was discovered
                for rcaValue in rcaValues:
                    #logAndPrint('Adding script entry: ['+rcaValue+'] = ['+scriptName+']')
                    rcaScriptMap[rcaValue]= scriptName
                    #logAndPrint('Adding node entry: '+rcaValue+' = '+runsOnNode, True)
                    rcaNodeMap[rcaValue]= runsOnNode
        f.close()
    except IOError:
        logAndPrint('Error: Unable to read properties file "'+filename+'" on hostname "'+getIAhostname()+'"')
        pass

    # logAndPrint('Returning map: '+str(rcaScriptMap))
    return rcaScriptMap, rcaNodeMap

################################################################################
# splitCommasIntoList takes a string with comma separated entries and returns
# a list of all of the entries.  For example, the string "a1,a2,b1,b2" would
# return this list:
#       values[0] = a1
#       values[1] = a2
#       values[2] = b1
#       values[3] = b2
################################################################################
def splitCommasIntoList(value):
    # If the string has multiple comma separated values, split them into a list
    values = []
    comma = value.find(',')
    if comma == -1:                     # only one action
        values.append(value)
    else:
        while comma != -1:
            comma = value.find(',')
            if comma==-1:
                values.append(value)
            else:
                values.append(value[0:comma])
            value = value[comma+1:]                        # go to the next entry

    return values


################################################################################
# getIAhostname returns the hostname of the localhost, but automatically maps
# the Management Console hostname (which might be changed) to the string
# 'iamconsole1'.  This conversion is triggered by matching the reported IP
# address of the node to '172.31.3.2'.
################################################################################
def getIAhostname():
    hn = socket.gethostname()
    fqdn,alias,ips = socket.gethostbyaddr(hn)
    if (ips[0] == '172.31.3.2' or ips[0] == '172.30.3.2'):
        logAndPrint('Converting hostname '+hn+' to iamconsole1')
        hn = 'iamconsole1'
    return hn


