#! /usr/bin/env python
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Indexing Controller
# *
# * FILE NAME:       inxc_feedIndexBroker.py
# *
# * DESCRIPTION:     Prepares Solr documents for insert/update/delete
#                    via processing by the indexbroker from a list of files
#                    1)called via the doc mgr to feed Solr documents for
#                      inserts/updates to the indexbroker. The list of
#                      files is created in the doc mgr and contains all
#                      files being ingested.
#                    2)called via the ingestion coordinator to feed Solr
#                      documents for deletes to the indexbroker. The list
#                      is created via the LAS logging of deletes. 
# *
# * AUTHOR:          K.W.Brannon
# *                  IBM Almaden Research Center
# *                  kbrannon@us.ibm.com
# *
# *
# ****************************************************************************
# */

import os, sys
import stat
import getopt
import socket
import md5
import grp
import cStringIO
from struct import *
from datetime import *
from traceback       import extract_stack, format_exc 
from ssm_defines  import *
from ssm_common    import *
from ssm_config_common import *
from inxc_logmsgs import *

_socket = None
_fileobj = None
INXC_HOST = "localhost"
INXC_OP_DELETE = "delete"
INXC_OP_UPDATE = "update"
INXC_MAXFNBYTES = 4
INXC_DTSTRING = "<data_type>string</data_type>"
INXC_N_EXTR = str("<field><name>%s</name>%s<value>%s</value></field>"
                %("ExtractContent",INXC_DTSTRING,"N"))
INXC_Y_EXTR = str("<field><name>%s</name>%s<value>%s</value></field>"
                %("ExtractContent",INXC_DTSTRING,"Y"))
INXC_4_LVL = str("<field><name>%s</name>%s<value>%s</value></field>"
                %(SSMMF_FIELD_NAME['SSMMF_INDEXLEVEL'],INXC_DTSTRING,"4"))
INXC_5_LVL = str("<field><name>%s</name>%s<value>%s</value></field>"
                %(SSMMF_FIELD_NAME['SSMMF_INDEXLEVEL'],INXC_DTSTRING,"5"))
INXC_6_LVL = str("<field><name>%s</name>%s<value>%s</value></field>"
                %(SSMMF_FIELD_NAME['SSMMF_INDEXLEVEL'],INXC_DTSTRING,"6"))
#Static messages sent by inxc to IB during IB protocol
INXC_IB_MSG_FILE_INFO = 2
INXC_IB_MSG_END_OF_FILELIST = 5
#Return codes indicating success in IB protocol
INXC_IB_MSG_CONN_REQ_SUCCESS = 1
INXC_IB_MSG_RCVD_FILE_INFO_SUCCESS = 3
INXC_IB_MSG_INDEXING_SUCCESS = 4
INXC_IB_MSG_COMMIT_SUCCESS = 6
#IB Error return codes indicating problem with post to Solr
INXC_IB_ERR_CONNECT_TOSOLR = 101
INXC_IB_ERR_TIKAEXCEPTION = 137
INXC_IB_ERR_OTHERTIKAEXCEPTION = 139
INXC_IB_ERR_SOLR_OUTOFMEMORY = 171
INXC_IB_ERR_SOLR_LOCKOBTAIN = 190
INXC_IB_ERR_TIKA_FILENOTFOUND = 191
INXC_EA_ERR_TIKAPARSE = INXC_IB_ERR_TIKAEXCEPTION
INXC_EA_ERR_TIKAOTHER = INXC_IB_ERR_OTHERTIKAEXCEPTION
INXC_EA_ERR_TIKAFILENOTFOUND = INXC_IB_ERR_OTHERTIKAEXCEPTION
INXC_EA_ERR_MAXMBFILESIZE = 140
#error list for adds [err number, retry, retryasmetadataonly, EAtoSet,restartsolr,  maxretrycount]
#retry means during this txn
#maxretrycount means across txn's (i.e. re-ingest), then drops to metadataonly
INXC_IB_ERR_LIST = [ 
    [INXC_IB_ERR_CONNECT_TOSOLR,    True,  False, None, False, 0], 
    [INXC_IB_ERR_SOLR_OUTOFMEMORY,  False, False, None, True, 0 ], 
    [INXC_IB_ERR_SOLR_LOCKOBTAIN,   False, False, None, True, 0], 
    [INXC_IB_ERR_TIKAEXCEPTION,     True,  True, INXC_EA_ERR_TIKAPARSE, False, 0],
    [INXC_IB_ERR_OTHERTIKAEXCEPTION,True,  False, INXC_EA_ERR_TIKAOTHER, False, 2],
    [INXC_IB_ERR_TIKA_FILENOTFOUND, True,  True, INXC_EA_ERR_TIKAFILENOTFOUND, False, 0]]
INXC_EA_ERR_RETRYCOUNT_START = 100
INXC_EA_ERR_RETRYCOUNT_MAX = 109
#IB Error return codes indicating problem with commit on Solr
INXC_IB_ERR_SOLRCOMMIT_GENERAL = 192
INXC_IB_ERR_SOLRCOMMIT_IO = 193
INXC_IB_ERR_SOLRCOMMIT_SOCKET = 194
#error list for commit [err number, fail-list, restartsolr]
INXC_IB_ERR_SOLRCOMMIT_LIST = [ 
    [INXC_IB_ERR_SOLRCOMMIT_GENERAL, True,  False], 
    #[INXC_IB_ERR_SOLRCOMMIT_IO,      True,  True ],
    [INXC_IB_ERR_SOLRCOMMIT_IO,      True,  False ],
    [INXC_IB_ERR_SOLRCOMMIT_SOCKET,  True,  False ]]

INXC_IB_RC_SUCCESS      = 0
INXC_IB_RC_PROTOCOL_ERR = 1
INXC_RETRY_RECORD_MAX = 2
INXC_RETRY_INGEST_MAX = 3
INXC_RETRY_DELETE_MAX = 3
INXC_RC_SUCCESS = 0
INXC_RC_FATAL = 1


def usage(logger,errmsg):
    umsg = str("\nusage: %s -m collectionMntPt -f filelist [--doDeletes][--doUpdates]\n"
               %(sys.argv[0]))
    if (logger != None):
        logger.LogMsg(SSM_LOGMSG_USAGE, (errmsg,umsg))
        logger.TraceMsg("Usage Error msg: %s" %(errmsg))
        logger.TraceMsg("Usage: %s" %(umsg))
    else:
        sys.stdout.write("Error: " +errmsg)
        sys.stdout.write(umsg)
    sys.exit(1)
   
def connect(port):
    global _socket
    global _fileobj
    global logger
    if _socket == None:
        logger.LogMsg(INXC_LOGMSG_INFO,str("No socket found for indexbroker host:%s port:%s"
                                           %(INXC_HOST,str(port))))
        try:
            _socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            _socket.connect((INXC_HOST,int(port)))
            _fileobj = _socket.makefile('r',0)
            msg = conn_receive_int()
            #May want to consider a retry if initial connection fails
            if msg == INXC_IB_MSG_CONN_REQ_SUCCESS:
                logger.LogMsg(INXC_LOGMSG_INFO,str("Connect success indexbroker host: %s port %s"
                                                   %(INXC_HOST,str(port))))
                return True
            else:
                lmsg = str("Error connecting to indexbroker host: %s port %s"
                       %(INXC_HOST,str(port)))
                logger.TraceMsg(lmsg)
                logger.LogMsg(INXC_LOGMSG_CONNECT, (INXC_HOST,str(port)))
                return False
        except:
            lmsg = str("Error connecting to indexbroker host: %s port %s"
                       %(INXC_HOST,str(port)))
            logger.TraceMsg(lmsg)
            logger.LogMsg(INXC_LOGMSG_CONNECT, (INXC_HOST,str(port)))
            return False 
    else:
        logger.LogMsg(INXC_LOGMSG_INFO,str("Already connected to indexbroker host: %s port %s"
                                           %(INXC_HOST,str(port))))
        return True

def try_connect(port):
    global logger
    logger.LogMsg(INXC_LOGMSG_INFO,str("Check connection to indexbroker host: %s port: %s"
                                       %(INXC_HOST,str(port))))
    ret = connect(port)
    if ret == True:
        logger.LogMsg(INXC_LOGMSG_INFO,str("Connection to indexbroker host:%s port:%s succeeded"
                        %(INXC_HOST,str(port))))
    if ret == False:
        logger.LogMsg(INXC_LOGMSG_INFO,str("Failed connection to indexbroker host %s port %s"
                                           %(INXC_HOST,str(port))))
    return ret    

def conn_send(msg):
    global _socket
    global _fileobj
    global logger
    logger.LogMsg(INXC_LOGMSG_INFO,str("Sending message: %s" %(msg)))
    try:
        _fileobj.write(msg)
        logger.LogMsg(INXC_LOGMSG_INFO,str("Sent message"))
        return True
    except:
        logger.TraceMsg("Exception sending message: %s" %(msg))
        logger.LogMsg(INXC_LOGMSG_SEND, msg)
        return False

def conn_send_short(n):
    global _socket
    global _fileobj
    global logger
    logger.LogMsg(INXC_LOGMSG_INFO,str("Sending as short: %d" %(n)))
    try:
        _fileobj.write(pack("!h",n))
        logger.LogMsg(INXC_LOGMSG_INFO,str("Sent packed short: %d" %(n)))
        return True
    except:
        logger.TraceMsg("Exception sending short: %d" %(n))
        logger.LogMsg(INXC_LOGMSG_SEND, str(n))
        return False
        
def conn_send_int(n):
    global _socket
    global _fileobj
    global logger
    logger.LogMsg(INXC_LOGMSG_INFO,str("Sending as int: %d" %(n)))
    try:
        _fileobj.write(pack("!i",n))
        logger.LogMsg(INXC_LOGMSG_INFO,str("Sent packed int: %d" %(n)))
        return True
    except:
        logger.TraceMsg("Exception sending int: %d" %(n))
        logger.LogMsg(INXC_LOGMSG_SEND, str(n))
        return False

def conn_receive_int():
    global _socket
    global _fileobj
    global logger
    logger.LogMsg(INXC_LOGMSG_INFO,str("Receiving  int"))
    try:
        buff = _fileobj.readline(2)
        ret = unpack('!h',buff)[0]
        logger.LogMsg(INXC_LOGMSG_INFO,str("Received  unpacked int: %d" %(ret)))
        return ret
    except:
        logger.LogMsg(INXC_LOGMSG_RCV, "exception on receive")
        return None
    
def signal_solr_restart():
    global logger
    global mountpt
    rc = ssm_set_solrNotHealthy_signal(logger, mountpt)

def get_EAdict(prefix, pathname):
    global logger
    logger.LogMsg(INXC_LOGMSG_INFO,str("form dictionary of EAs with prefix: %s" %(prefix)))
    cmd = str('/usr/bin/getfattr -d --match="%s*" %s' %(prefix, 
                                                        re.escape(pathname)))
    rc,output = ssm_run_cmdStdout(logger,cmd)
    if (rc == 0):
        EAdict = {}
        numlines = output.count('\n')
        if numlines == 0:
            #no matching EAs at all
            pass
        else:
            EAdict = {}
            n = numlines - 1
            outlines = output.splitlines()
            #read from bottom up in case newline in filename
            while n > 0:
                line = outlines[n].strip()
                if line.startswith(prefix) == True:
                    l = line.split(prefix,1)[1]
                    nv = l.split("=",1)
                    name = str("'%s'" %(nv[0]))
                    #drop the enclosing quotes returned by getfattr
                    v = nv[1]
                    vlen = len(v)
                    if v[0] == '"' and v[vlen-1] == '"':
                        value = v[1:vlen-1]
                    else:
                        value = v
                    EAdict[name] = value
                n = n - 1
    else:
        EAdict = None
    logger.LogMsg(INXC_LOGMSG_INFO,str("dictionary of EAs:  %s" %(str(EAdict))))
    return EAdict

def get_next_record(operation):
    global fl
    global len4seek
    global ft
    global logger
    if (operation == INXC_OP_DELETE):
        maxrecord = fl.read(INXC_MAXFNBYTES + 1)
        if maxrecord == "":
            record = maxrecord
            logger.LogMsg(INXC_LOGMSG_INFO,str("delete record blank: %s " %(str(record))))
        else:
            logger.LogMsg(INXC_LOGMSG_INFO,str("processingDeletes record reading maxrecordlen: %s"
                                               %(str(maxrecord))))
            r = maxrecord.split('!',2)
            bytes = int(r[0])
            rest = fl.read(bytes - INXC_MAXFNBYTES + len(r[0]))
            fn = r[1] + rest
            logger.LogMsg(INXC_LOGMSG_INFO,str("filename to be deleted: %s " %(fn)))
            rest = fl.readline()
            rr = rest.split('\t',3)
            docid = rr[1].strip()
            corenum = rr[2].strip()
            record = [fn, docid, corenum]
            logger.LogMsg(INXC_LOGMSG_INFO,str("delete record: %s" %(str(record))))
    elif (operation == INXC_OP_UPDATE):
        maxrecord = fl.read(INXC_MAXFNBYTES + 1)
        if maxrecord == "":
            record = maxrecord
            logger.LogMsg(INXC_LOGMSG_INFO,str("update record blank: %s" %(str(record))))
        else:
            logger.LogMsg(INXC_LOGMSG_INFO,str("update record reading maxrecordlen: %s "
                                               %(str(maxrecord))))
            try:
                r = maxrecord.split('!',2)
                bytes = int(r[0])
                rest = fl.read(bytes - INXC_MAXFNBYTES + len(r[0]))
                fn = r[1] + rest
                logger.LogMsg(INXC_LOGMSG_INFO,str("filename for insert/update: %s" %(fn)))
                rest = fl.readline()
                rr = rest.split('\t',4)
                docid = rr[1].strip()
                inodenum = rr[2].strip()
                genid = rr[3].strip()
                record = [fn, docid, inodenum, genid]
                logger.LogMsg(INXC_LOGMSG_INFO,str("insert/update record: %s" %(str(record))))
            except:
                logger.LogMsg(INXC_LOGMSG_SYS_ERR, ("bad record in filelist"))
                record = [None, None, None, None]
    else:
        logger.LogMsg(INXC_LOGMSG_SYS_ERR,("bad operation value"))
        logger.TraceMsg("bad operation, only %s and %s supported"
                        %(INXC_OP_DELETE,INXC_OP_UPDATE))
        sys.exit(1)
    return record

def get_delete_xml(pathname, docID, coreID):
    global logger
    logger.LogMsg(INXC_LOGMSG_INFO,str("Forming delete xml for %s" %(pathname)))
    connCore = str("/solr/%s%s" %(SSM_INDEX_CORE_NAME,coreID))
    pth = pathname
    d = datetime.utcnow()
    current_time = str("%04i-%02i-%02iT%02i:%02i:%02iZ"
                       %(d.year,d.month,d.day,d.hour,
                         d.minute,d.second))
    ib_msg = cStringIO.StringIO()
    ib_msg.write("<meta>")
    ib_msg.write("<field><name>%s</name>%s<value>%s</value></field>"
              %("InsOrDel",INXC_DTSTRING,"Del"))
    ib_msg.write("<field><name>%s</name>%s<value>%s</value></field>"
              %("connectionCore", INXC_DTSTRING, connCore))
    #write out the bytes of the filename
    i = 0
    while i < len(pth):
        ib_msg.write(pth[i])
        i = i + 1
    msg = str("</value></field>")
    ib_msg.write(msg)
    ib_msg.write("<field><name>%s</name>%s<value>%s</value></field>"
              %(SSMMF_FIELD_NAME['SSMMF_DOCID'], INXC_DTSTRING, docID))
    ib_msg.write("<field><name>%s</name>%s<value>%s</value></field>"
              %(SSMMF_FIELD_NAME['SSMMF_INDEXTIME'], INXC_DTSTRING, current_time))
    ib_msg.write("</meta>")
    meta = ib_msg.getvalue()
    logger.LogMsg(INXC_LOGMSG_INFO,str("Delete xml for %s is %s" %(pathname, meta)))
    ib_msg.close()
    return meta

def get_update_xml(mountpt, record, statout, indexlevel,
                   current_coreID, startofindexing_forSolr):
    global logger
    global mfdotxml, parsed_mfdotxml
    EA_INerror2set = None
    coreID = current_coreID
    indexlevel_used = indexlevel
    pathname = record[0]
    docID = record[1]
    inode = record[2]
    genid = record[3]
    logger.LogMsg(INXC_LOGMSG_INFO,str("Forming insert/update xml for %s" %(pathname)))
    #Form a dictionary of all the archive EAs
    archiveEA = get_EAdict(SSM_EA_ARCHIVE_PREFIX, pathname)
    logger.LogMsg(INXC_LOGMSG_INFO,str("Archive EA list: %s" %(str(archiveEA))))
    if len(archiveEA) > 0:
        try:
            #if coreID EA exists, need to use that for update
            k = str("'%s'" %(SSMMF_EA_NAME['SSMMF_COREID']))
            coreID = str(archiveEA[k])
            logger.LogMsg(INXC_LOGMSG_INFO,str("This is an update on %s" %(pathname)))
        except KeyError:
            #no coreID EA exists, must be a create
            logger.LogMsg(INXC_LOGMSG_INFO,str("This is a create on %s" %(pathname)))
            pass
        try:
            #if INerror EA exists, need to track that for retrys
            k = str("'%s'" %(SSMMF_EA_NAME['SSMMF_INDEXERROR']))
            existing_EA_INerror = int(archiveEA[k])
        except KeyError:
            #no INerror EA exists, must not be a retry
            logger.LogMsg(INXC_LOGMSG_INFO,str("This is not a retry on %s" %(pathname)))
            existing_EA_INerror = 0
            pass
    else:
        #no archive EAs at all
        logger.TraceMsg("No archive EAs, returning None for insert_xml for %s"
                        %pathname)
        meta = None
        return meta
    
    connCore = str("/solr/%s%s" %(SSM_INDEX_CORE_NAME,coreID))
    filesizebytes = statout[stat.ST_SIZE]
    filesize = str(filesizebytes)
    owner = str(statout[stat.ST_UID])
    group = str(statout[stat.ST_GID])
    #Get values to be indexed from the fuse metafile
    pth = pathname
    c = pth.split("/",4)
    fmf = str("/meta/%s/%s/%s/%s" %(c[1],c[2],"meta",c[4]))
    #parsed_fmf = ssm_parse_xmlfile(logger, fmf)
    parsed_fmf = ssm_parse_fusemetafile(logger, fmf)
    md5field = ssm_get_valueFromXmlfile(logger,SSMMF_FIELD_NAME['SSMMF_MD5'],
                                          parsed_fmf, fmf)
    srvclass = ssm_get_valueFromXmlfile(logger,
                                          SSMMF_FIELD_NAME['SSMMF_SRVCLASS'],
                                          parsed_fmf, fmf)
    retained = ssm_get_valueFromXmlfile(logger,
                                          SSMMF_FIELD_NAME['SSMMF_RETAINED'],
                                          parsed_fmf, fmf)
    evntdur  = ssm_get_valueFromXmlfile(logger,
                                          SSMMF_FIELD_NAME['SSMMF_EVENTDUR'],
                                          parsed_fmf, fmf)
    holds    = ssm_get_valueFromXmlfile(logger,
                                          SSMMF_FIELD_NAME['SSMMF_CURHOLDS'],
                                          parsed_fmf, fmf)
    minret   = ssm_get_valueFromXmlfile(logger,
                                          SSMMF_FIELD_NAME['SSMMF_MINRET'],
                                          parsed_fmf, fmf)
    value = {
        'SSMMF_MD5':          md5field,
        'SSMMF_MINRET':       minret,
        'SSMMF_EVENTDUR':     evntdur,
        'SSMMF_SRVCLASS':     srvclass,
        'SSMMF_RETAINED':     retained,
        'SSMMF_CURHOLDS':     holds,
        'SSMMF_OWNER':        owner,
        'SSMMF_GROUP':        group,
        'SSMMF_FILESIZE':     filesize,
        'SSMMF_INODE':        inode,
        'SSMMF_GENID':        genid,
        'SSMMF_DOCID':        docID,
        'SSMMF_COREID':       coreID,
        'SSMMF_INDEXLEVEL':   indexlevel,
        'SSMMF_INDEXTIME':    startofindexing_forSolr,
        'SSMMF_EPOCH':        epoch,
        'SSMMF_PATHNAME':     pth
    }
    #write out the xml to be sent to the index broker based on indexlevel
    ib_msg = cStringIO.StringIO()
    ib_msg.write("<meta>")
    ib_msg.write("<field><name>%s</name>%s<value>%s</value></field>"
              %("InsOrDel",INXC_DTSTRING,"Ins"))
    if SSM_INDEX_LEVEL[indexlevel] == 'index_and_store_metadata_only':
        ib_msg.write(INXC_N_EXTR)
    elif SSM_INDEX_LEVEL[indexlevel] == 'index_and_store_metadata_index_content'\
           or \
           SSM_INDEX_LEVEL[indexlevel] == 'index_and_store_metadata_plus_content':
        #Check filesize(is in bytes), large files get indexed at metadata only
        if maxMBfilesize != None and filesizebytes >  maxMBfilesize*1024*1024:
            logger.LogMsg(INXC_LOGMSG_FILESIZE,(pathname, filesize, str(maxMBfilesize)))
            logger.TraceMsg("filesize limit for metadata+content exceeded for file: %s filesize: %s limit(MB): %s" %(pathname, filesize, str(maxMBfilesize)))
            ib_msg.write(INXC_N_EXTR)
            EA_INerror2set  = INXC_EA_ERR_MAXMBFILESIZE 
            indexlevel_used = SSM_INDEX_LEVEL_METADATAONLY
            value['SSMMF_INDEXLEVEL'] = indexlevel_used
        else:
            ib_msg.write(INXC_Y_EXTR)
    else:
        pmsg = "error - bad indexlevel: %s " %(str(indexlevel))
        logger.LogMsg(INXC_LOGMSG_SYS_ERR,(pmsg))
        logger.TraceMsg(pmsg)
        return None
    msg = str("<field><name>%s</name>%s<value>%s</value></field>"
              %("connectionCore", INXC_DTSTRING, connCore))
    ib_msg.write(msg)
    msg = str("<field><name>%s</name>%s<value>" %("filename", INXC_DTSTRING))
    ib_msg.write(msg)
    #write out the bytes of the filename
    i = 0
    while i < len(pth):
        ib_msg.write(pth[i])
        i = i + 1
    msg = str("</value></field>")
    ib_msg.write(msg)
    for name in SSMMF_FIELD_NAME:
        if name == 'SSMMF_PATHNAME':
            msg = str('<field><name>%s</name>%s<value>' 
                      %(SSMMF_FIELD_NAME[name], INXC_DTSTRING))
            ib_msg.write(msg)
            i = 0
            while i < len(pth):
                ib_msg.write(pth[i])
                i = i + 1
            msg = str("</value></field>")
            ib_msg.write(msg)
        else:
            if value[name] != None:
                if SSMMF_FIELD_TYPE[name] == SSMMF_TYPE_TDATE:
                    val = value[name] +"Z"
                    val = str(val)
                elif SSMMF_FIELD_TYPE[name] == SSMMF_TYPE_INT:
                    val = str(value[name])
                else:
                    val = value[name]
                msg = str('<field><name>%s</name>%s<value>%s</value></field>' 
                           %(SSMMF_FIELD_NAME[name], INXC_DTSTRING, val))
                ib_msg.write(msg)
    #get user defined fields
    nameAndtypelist = ssm_get_mfuserfields(logger, parsed_mfdotxml, mfdotxml)
    i = 0
    while i < len(nameAndtypelist):
        name = nameAndtypelist[i][0]
        type = nameAndtypelist[i][1]
        value = ssm_get_valueFromXmlfile(logger,name,parsed_fmf,fmf)
        if value != None:
            entry = str('<field><name>%s</name>%s<value>' %(name, INXC_DTSTRING))
            ib_msg.write(entry)
            if type == "string":
                m = 0
                while m < len(value):
                    ib_msg.write(value[m])
                    m = m + 1
                msg = str("</value></field>")
                ib_msg.write(msg)
            else:
                if type == "datetime":
                    value = value +"Z"
                    value = str(value)
                elif type == "integer":
                    value = str(value)
                elif type == "double":
                    value = str(value)
                elif type == "boolean":
                    value = str(value)
                entry = str('%s</value></field>' %value)
                ib_msg.write(entry)
        i = i + 1
    ib_msg.write("</meta>")
    meta = ib_msg.getvalue()
    ib_msg.close()
    return meta,coreID,indexlevel_used,EA_INerror2set,existing_EA_INerror

def set_minerACL(pathname, minerGroups, default, mask):
    global logger
    MASK = mask
    if default == False:
        logger.LogMsg(INXC_LOGMSG_INFO,str("set_minerACL on %s for groups: %s\n"
                                           %(pathname, str(minerGroups))))
        cmd = str('%s/mmgetacl %s' %(SSM_MMFSBIN, re.escape(pathname)))
    else:
        logger.LogMsg(INXC_LOGMSG_INFO,str("set_minerACL default ACL on %s for groups:%s\n"
                                           %(pathname, str(minerGroups))))
        cmd = str('%s/mmgetacl -d %s' %(SSM_MMFSBIN, re.escape(pathname)))
    rc, stdout = ssm_run_cmdStdout(logger, cmd)
    if rc == 0:
        originalACL = stdout
        logger.LogMsg(INXC_LOGMSG_INFO,str("Original ACL on %s: %s\n"
                                           %(pathname, str(originalACL))))
        match_pred = "mask::"
        res = re.search(match_pred, originalACL)
        if res != None:
            #mask already present
            #check that all archive miner groups are present
            logger.LogMsg(INXC_LOGMSG_INFO,str("found %s in original ACL" %match_pred))
            newACL = originalACL
            allfound = True
            for g in minerGroups:
                match_pred = str("group:%s:" %g[0])
                res = re.search(match_pred, originalACL)
                match_pred = str("group:%s:" %g[1])
                res1 = re.search(match_pred, originalACL)
                if res != None or res1 != None:  
                    pass
                else:
                    allfound = False
                    newACL = str("%sgroup:%s:r-x-\n" %(newACL, g[0]))
        else:
            logger.LogMsg(INXC_LOGMSG_INFO,str("did not find %s in original ACL" %match_pred))
            allfound = False
            if default == False:
                newACL = originalACL
            else:
                #for default ACL may need to create it
                logger.LogMsg(INXC_LOGMSG_INFO,str("check if need to create default ACL for %s"
                                                   %pathname))
                match_pred = "user::"
                res = re.search(match_pred, originalACL)
                if res != None:
                    logger.LogMsg(INXC_LOGMSG_INFO,str("found %s in original ACL" %match_pred))
                    newACL = originalACL
                else:
                    logger.LogMsg(INXC_LOGMSG_INFO,str("did not find %s in original ACL"
                                                       %match_pred))
                    newACL = originalACL
                    newACL = str("%suser::rwxc\n" %newACL)
                    newACL = str("%sgroup::----\n" %newACL)
                    newACL = str("%sother::----\n" %newACL)
            newACL = str("%s%s\n" %(newACL, MASK))
            for group in minerGroups:
                newACL = str("%sgroup:%s:r-x-\n" %(newACL, group[0]))
            logger.LogMsg(INXC_LOGMSG_INFO,str("New ACL on %s: %s\n" %(pathname, str(newACL))))
        if allfound == True:
            logger.LogMsg(INXC_LOGMSG_INFO,str("Original ACL on %s already has all miner groups"
                                               %pathname))
            return True
        else:
            #update with the new ACL
            logger.LogMsg(INXC_LOGMSG_INFO,str("new ACL for %s: %s\n"
                                               %(pathname, str(newACL))))
            if default == False:
                cmd = str('%s/mmputacl %s'
                          %(SSM_MMFSBIN, re.escape(pathname)))
            else:
                cmd = str('%s/mmputacl -d %s'
                          %(SSM_MMFSBIN, re.escape(pathname)))
            rc, stdout = ssm_run_cmdStdout_stdin(logger, cmd, newACL)
            if rc == 0:
                return True
            else:
                return False
    else:
        return False
    
    
def set_minerACLs(mountpt, pathname, minerGroups):
    global logger
    logger.LogMsg(INXC_LOGMSG_INFO,str("set_minerACLs - add archive miner ACL on %s, groups: %s"
                                       %(pathname, str(minerGroups))))  
    if minerGroups == None:
        #nothing to do
        pass
    else:
        #first set ACL on the file itself
        mask = str("mask::r---")
        result = set_minerACL(pathname, minerGroups, False, mask)
        if result == True:
            #now loop thru the parent directories
            logger.LogMsg(INXC_LOGMSG_INFO,str("Set miner ACL on parent directories of %s"
                                               %pathname))
            dir = os.path.dirname(pathname)
            while dir != mountpt:
                logger.LogMsg(INXC_LOGMSG_INFO,str("Set miner ACLs for %s" %dir))
                mask = str("mask::r-x-")
                result = set_minerACL(dir, minerGroups, False, mask)
                if result == True:
                    #DO NOT set default ACL
                    #logger.TraceMsg("Set default ACL for %s" %dir)
                    #result = set_minerACL(dir, minerGroups, True)
                    if result == True:
                        pass
                        #logger.TraceMsg("All ACLs set for %s" %dir)
                    #else:
                        #logger.TraceMsg("Problem setting default ACL for %s"
                                        #%dir)
                        #return 1
                else:
                    lmsg = str("Problem setting ACL for %s" %dir)
                    logger.TraceMsg(lmsg)
                    logger.LogMsg(INXC_LOGMSG_SYS_ERR,lmsg)
                    return 1
                dir = os.path.dirname(dir)
        else:
            lmsg = str("Problem setting ACL for %s" %pathname)
            logger.TraceMsg(lmsg)
            logger.LogMsg(INXC_LOGMSG_SYS_ERR,lmsg)
            return 1
    return 0

def process_record(port, operation, record):
    global logger
    global file_retrylist
    global EAlist
    skip = False
    retry = False
    if operation  == INXC_OP_DELETE:
        pathname = record[0]
        docIDforDelete =  record[1].strip()
        corenum = record[2].strip()
        logger.LogMsg(INXC_LOGMSG_INFO, str("check if should skip path: %s docID: %s core: %s"
                                            %(pathname,docIDforDelete,corenum)))
        if docIDforDelete == "00000000000000000000000000000000" and corenum == "0":
            #skip this record - it is for the audit_health_dummy_file unlink"
            logger.LogMsg(INXC_LOGMSG_INFO,str("skipping audit health check record: %s " %(str(record))))
            skip = True
        else:
            logger.LogMsg(INXC_LOGMSG_INFO,str("delete processing log record: %s " %(str(record))))
            coreIDforDelete = corenum
            logger.LogMsg(INXC_LOGMSG_INFO,str("Processing %s, get delete xml, docID:%s coreID:%s"
                                               %(pathname,docIDforDelete,coreIDforDelete)))
            meta = get_delete_xml(pathname, docIDforDelete, coreIDforDelete)
                
    elif operation == INXC_OP_UPDATE:
        logger.LogMsg(INXC_LOGMSG_INFO,str("insert/update processing log record: %s" %(str(record))))
        pathname = record[0]
        docID = record[1]
        inode = record[2]
        genid = record[3]
        if pathname != None and docID != None and inode != None and genid != None:
            if os.path.isfile(pathname):
                logger.LogMsg(INXC_LOGMSG_INFO,str("Processing %s for insert/update" %(pathname))) 
                statout = os.stat(pathname)
                if (inode != str(statout[stat.ST_INO])):
                    lmsg = str("inodes do not match: %s %s"
                               %(inode,str(statout[stat.ST_INO])))
                    logger.TraceMsg("Skipping file %s, %s" %(pathname, lmsg))
                    logger.LogMsg(INXC_LOGMSG_SKIP,(pathname, lmsg))
                    #Maybe this could be a retry instead
                    skip = True
                coreID = coreID4insert
                d = datetime.utcnow()
                startofindexing_asEA = re.escape(str("%04i-%02i-%02i %02i:%02i:%02i"
                                           %(d.year,d.month,d.day,d.hour,
                                             d.minute,d.second))) 
                startofindexing_forSolr = str("%04i-%02i-%02iT%02i:%02i:%02i"
                                              %(d.year,d.month,d.day,d.hour,
                                                d.minute,d.second))
                logger.LogMsg(INXC_LOGMSG_INFO,str("get insert xml"))
                meta,coreIDused,indexlevel_used,EA_INerror2set,existing_EA_INerror = get_update_xml(mountpt, record, statout, indexlevel, coreID, startofindexing_forSolr)
                if (meta != None):
                    pass
                else:
                    logger.TraceMsg("Retry this record, no update xml, %s "
                                    %str(record))
                    retry_record = True
                    skip = True
            else:
                lmsg = str("file not found ")
                logger.TraceMsg("Skipping file %s, %s" %(pathname, lmsg))
                logger.LogMsg(INXC_LOGMSG_SKIP,(pathname,lmsg))
                skip = True
        else:
            lmsg = str("bad record: %s %s %s %s "
                       %(str(pathname),str(docID),str(inode),str(genid)))
            logger.TraceMsg("Skipping file: %s, %s" %(pathname, lmsg))
            logger.LogMsg(INXC_LOGMSG_SKIP,(str(pathname),lmsg))
            skip = True
    else:
        logger.LogMsg(INXC_LOGMSG_SYS_ERR,("bad operation value"))
        logger.TraceMsg("bad operation, only %s and %s supported"
                        %(INXC_OP_DELETE,INXC_OP_UPDATE))
        rc =  INXC_RC_FATAL
        return rc
    rc = INXC_RC_SUCCESS
    if skip == False:
        rc = INXC_RC_SUCCESS
        if operation == INXC_OP_UPDATE:
            errnum = existing_EA_INerror
            if errnum >= INXC_EA_ERR_RETRYCOUNT_START and errnum <= INXC_EA_ERR_RETRYCOUNT_MAX:
                err2set = errnum
            else:
                err2set = 0 
            logger.LogMsg(INXC_LOGMSG_INFO,str("Before run IB protocol on %s, set EAs so will be re-ingested in case of error: %s %s"
                                               %(pathname, docID, coreIDused)))
            path =  pathname
            cmd = str('%s%s --filename=%s --documentid=%s --coreid=%s --indexerror=%s --indexlevel=0 --indexepoch=0'
                       %(SSM_IA_INSTALL_BIN, "/mw_set_indexing_info",
                       re.escape(path), docID, coreIDused, str(err2set)))
            crc = ssm_run_cmd(logger, cmd)
            if crc != 0:
                lmsg = str("Error setting coreid EA before run IB protocol")
                logger.TraceMsg(lmsg)
                logger.LogMsg(INXC_LOGMSG_SYS_ERR, lmsg)
        logger.LogMsg(INXC_LOGMSG_INFO,str("Processing %s, invoke indexbroker protocol" %pathname))
        retmsg = run_indexbroker_protocol(port, pathname, meta)
        if retmsg == INXC_IB_RC_SUCCESS:
            logger.TraceMsg("Index operation successful for %s "
                            %(pathname))
            rc = INXC_RC_SUCCESS
            #save EAs for inserts/updates to set after commit succeeds
            if operation == INXC_OP_UPDATE:
                logger.LogMsg(INXC_LOGMSG_INFO,str("Save EAs, since update success "))
                corenum = coreIDused
                corenum = int(corenum)
                if existing_EA_INerror >= INXC_EA_ERR_RETRYCOUNT_START and existing_EA_INerror  <= INXC_EA_ERR_RETRYCOUNT_MAX:
                    EA_INerror2set = 0
                EA = [pathname, docID, indexlevel_used, str(corenum), epoch,
                      startofindexing_asEA, EA_INerror2set, existing_EA_INerror]
                if EAlist == None or len(EAlist) == 0:
                    EAlist = [EA]
                else:
                    EAlist.append(EA)
                logger.TraceMsg("For %s, saved to EAlist: %s" %(pathname,str(EA)))
        else:
            lmsg = str("Err code, %s from indexbroker for %s, check for retry"
                           %(str(retmsg), pathname))
            logger.LogMsg(INXC_LOGMSG_SYS_ERR, lmsg)
            logger.TraceMsg(lmsg)
            rc = INXC_RC_FATAL
            #Determine remaining processing based on error code from IB
            found_error = False
            #err list [number, retry, retryasmetadataonly, EAtoSet, restartsolr, maxretrycount]
            for err in INXC_IB_ERR_LIST:
                if retmsg == err[0]:
                    found_error = True
                    if err[1] == True:
                        #save this entry to the retry list
                        if operation == INXC_OP_UPDATE:
                            logger.TraceMsg("Get EAs for retry list to set after commit file:%s existing_EA_INerror:%s err2set:%s" %(pathname, str(existing_EA_INerror),str(err2set)))
                            corenum = coreIDused
                            corenum = int(corenum)
                            maxretrycount = INXC_EA_ERR_RETRYCOUNT_START + err[5]
                            if err[2] == True or err2set > maxretrycount:
                                #modify the meta msg to index metadata only
                                logger.TraceMsg("err[2]:%s  maxretrycount:%s err2set:%s" %(str(err[2]),str(maxretrycount),str(err2set)))
                                logger.TraceMsg("Modify meta to index metadata only for %s"
                                                 %pathname)
                                retrymeta = change_to_mdonly(meta)
                                EA = [pathname, docID,
                                      SSM_INDEX_LEVEL_METADATAONLY,
                                      str(corenum), epoch, startofindexing_asEA,
                                      err[3], None]
                            else:
                                #retry at current indexlevel
                                retrymeta = meta
                                EA = [pathname, docID, indexlevel_used,
                                      str(corenum), epoch, startofindexing_asEA,
                                      None, existing_EA_INerror]
                        else:
                            #for deletes save the pathname for convenience
                            EA = [pathname]
                            retrymeta = meta
                        if retrymeta != None:
                            rt = [retrymeta, EA]
                            if file_retrylist == None or len(file_retrylist) == 0:
                                file_retrylist = [rt]
                            else:
                                file_retrylist.append(rt)
                            rc = INXC_RC_SUCCESS
                        else:
                            lmsg = str("Could not change meta to metadata only, %s"
                                       %meta)
                            logger.TraceMsg(lmsg)
                            logger.LogMsg(INXC_LOGMSG_SYS_ERR, lmsg)
                            rc = INXC_RC_FATAL
                    else:
                        #check if should signal to stop solr as well as terminate
                        lmsg = str("Fatal error code, %s from indexbroker for %s"
                                       %(str(retmsg), pathname))
                        logger.LogMsg(INXC_LOGMSG_INDEXING, lmsg)
                        logger.TraceMsg(lmsg)
                        rc = INXC_RC_FATAL
                        if err[4] == True:
                            signal_solr_restart()
            if found_error == False:
                #unknown error code
                lmsg = str("Unknown error code, %s from indexbroker for %s"
                               %(str(retmsg), pathname))
                logger.LogMsg(INXC_LOGMSG_SYS_ERR, lmsg)
                logger.TraceMsg(lmsg)
                rc = INXC_RC_FATAL
    logger.LogMsg(INXC_LOGMSG_INFO,str("inxc process record returning %s for record %s "
                                       %(str(rc),str(record))))
    return rc
    

def run_indexbroker_protocol(port, pathname, meta):
    global logger
    logger.TraceMsg("Run indexbroker protocol on %s" %pathname)
    c = try_connect(port)
    if c == True:
        logger.LogMsg(INXC_LOGMSG_INFO,str("Processing %s, connect OK to host:%s, port:%s "
                                           %(pathname,INXC_HOST,str(port))))
    else:
        #fails the entire list
        logger.TraceMsg("Processing %s, connect error, ib host:%s, port: %s"
                        %(pathname,INXC_HOST,str(port)))
        logger.LogMsg(INXC_LOGMSG_CONNECT,(INXC_HOST,port))
        return INXC_IB_RC_PROTOCOL_ERR
    init = INXC_IB_MSG_FILE_INFO
    logger.LogMsg(INXC_LOGMSG_INFO,str("Processing %s, send init message: %s "
                                       %(pathname, str(init))))
    ret = conn_send_short(init)
    if ret == True:
        pass
    else:
        logger.TraceMsg("Processing %s, exception in send" %pathname)
        return INXC_IB_RC_PROTOCOL_ERR
    meta4ib = meta
    meta_len = len(meta4ib)
    logger.LogMsg(INXC_LOGMSG_INFO,str("Processing %s, send meta data message len: %s"
                                       %(pathname,str(meta_len))))
    ret = conn_send_int(meta_len)
    if ret == True:
        pass
    else:
        logger.TraceMsg("Processing %s, exception in send" %pathname)
        return INXC_IB_RC_PROTOCOL_ERR
    logger.TraceMsg("Processing %s, send meta data message: %s" %(pathname, str(meta4ib)))
    ret = conn_send(meta4ib)
    if ret == True:
        pass
    else:
        logger.TraceMsg("Processing %s, exception in send" %pathname)
        return INXC_IB_RC_PROTOCOL_ERR
    logger.TraceMsg("Processing %s, receive file info result from indexbroker"
                    %pathname)
    retmsg = conn_receive_int()
    logger.LogMsg(INXC_LOGMSG_INFO,str("Processing %s, received fileinfo result: %s "
                                       %(pathname,str(retmsg))))
    if retmsg == INXC_IB_MSG_RCVD_FILE_INFO_SUCCESS:
        logger.LogMsg(INXC_LOGMSG_INFO,str("Processing %s, rcvd_file_info OK returned msg: %s"
                                           %(pathname,str(retmsg))))
    elif retmsg == None:
        logger.TraceMsg("Processing %s, exception in receive" %pathname)
        return INXC_IB_RC_PROTOCOL_ERR
    else:
        #return the indexing error provided by the indexbroker
        logger.TraceMsg("Processing %s, error from indexbroker: %s"
                        %(pathname, str(retmsg)))
        return retmsg
    logger.LogMsg(INXC_LOGMSG_INFO,str("Processing %s, receive indexing result "
                    %pathname))
    retmsg = conn_receive_int()
    logger.LogMsg(INXC_LOGMSG_INFO,str("Processing %s, received indexing result: %s "
                                       %(pathname,str(retmsg))))
    if retmsg == INXC_IB_MSG_INDEXING_SUCCESS:
        logger.TraceMsg("Processing %s, indexing successful returned msg: %s "
                        %(pathname,str(retmsg)))
        return INXC_IB_RC_SUCCESS
    elif retmsg == None:
        logger.TraceMsg("Processing %s, exception in receive" %pathname)
        return INXC_IB_RC_PROTOCOL_ERR
    else:
        #return the indexing error provided by the indexbroker
        logger.TraceMsg("Processing %s, error from indexbroker: %s"
                        %(pathname, str(retmsg)))
        return retmsg
    
def change_to_mdonly(meta):
    n = meta.find(INXC_Y_EXTR)
    if n != -1:
        m = n + len(INXC_Y_EXTR)
        newmeta = meta[:n] +INXC_N_EXTR +meta[m:]
        l5 = meta.find(INXC_5_LVL)
        l6 = meta.find(INXC_6_LVL)
        if l5 != -1: 
            ll = l5 + len(INXC_5_LVL)
            newnewmeta = newmeta[:l5] +INXC_4_LVL +newmeta[ll:] 
        elif l6 != -1: 
            ll = l6 + len(INXC_6_LVL)
            newnewmeta = newmeta[:l6] +INXC_4_LVL +newmeta[ll:] 
        else:
            pass
    else:
        newnewmeta = None
    return newnewmeta

def set_EAs_ACLs(EA):
    global logger
    global minerGroups
    logger.TraceMsg("set archive miner ACLs and EAs for file %s"
                    %(EA[0]))
    rc = set_minerACLs(mountpt, EA[0], minerGroups)
    if rc == 0:
        logger.LogMsg(INXC_LOGMSG_INFO,str("setting indexing success in EAs for file %s EAs: %s"
                                           %(EA[0],str(EA))))
        if EA[6] == None:
            cmd = str('%s%s --filename=%s --documentid=%s --indexlevel=%s --coreid=%s --indexepoch=%s --indextime=%s'
                      %(SSM_IA_INSTALL_BIN,"/mw_set_indexing_info",
                        re.escape(EA[0]),EA[1],
                        str(EA[2]),EA[3],EA[4],EA[5]))
        else:
            cmd = str('%s%s --filename=%s --documentid=%s --indexlevel=%s --coreid=%s --indexepoch=%s --indextime=%s --indexerror=%s'
                      %(SSM_IA_INSTALL_BIN,"/mw_set_indexing_info",
                        re.escape(EA[0]),EA[1],
                        str(EA[2]),EA[3],EA[4],EA[5],str(EA[6])))
        logger.TraceMsg("Set EAs for %s via cmd: %s" %(EA[0],cmd))
        rc1,stdout = ssm_run_cmdStdout(logger, cmd)
    if rc != 0 or rc1 != 0:
        #Set EAs so will get re-ingested
        logger.TraceMsg("Error setting index success EAs for %s"
                        %(EA[0]))
        errnum = 0
        cmd = str('%s%s --filename=%s --documentid=%s --indexlevel=0 --coreid=%s --indexepoch=0 --indextime=%s'
                  %(SSM_IA_INSTALL_BIN,"/mw_set_indexing_info",
                    re.escape(EA[0]),EA[1],
                    EA[3],EA[5]))
        logger.TraceMsg("Set EAs indicating indexing error for %s"
                        %(EA[0],cmd))
        rc = ssm_run_cmd(logger, cmd)
        if rc != 0:
            logger.TraceMsg("Error setting EAs for indexing error")

def terminate_ib_protocol(setEAs):
    global EAlist
    logger.TraceMsg("Terminate indexbroker protocol")
    end_msg = INXC_IB_MSG_END_OF_FILELIST
    logger.LogMsg(INXC_LOGMSG_INFO,str("About to send end of filelist message:%s" %(str(end_msg))))
    ret = conn_send_short(end_msg)
    logger.LogMsg(INXC_LOGMSG_INFO,str("Send of end of filelist message success: %s" %(str(ret))))
    rc = INXC_RC_FATAL
    if ret == True:
        #now get the result of the Solr commit
        if len(EAlist) > 0:
            logger.LogMsg(INXC_LOGMSG_INFO,str("Do a receive to get commit result from indexbroker"))
            commit_result = conn_receive_int()
            logger.LogMsg(INXC_LOGMSG_INFO,str("Received commit result: %s " %str(commit_result)))
            if setEAs == True:
                if commit_result == INXC_IB_MSG_COMMIT_SUCCESS:
                    #Commit worked, set the EAs, ACLs for this group of files
                    logger.TraceMsg("Commit success for this filelist, now set EAs")
                    for EA in EAlist:
                        ret = set_EAs_ACLs(EA)
                        rc = INXC_RC_SUCCESS
                else:
                    logger.TraceMsg("Commit failure for this filelist, do error processing")
                    rc = INXC_RC_FATAL
                    found_error = False
                    #err list [number, fail-entire-list, restartsolr]
                    for err in INXC_IB_ERR_SOLRCOMMIT_LIST:
                        if commit_result == err[0]:
                            found_error = True
                            if err[1] == True:
                                #Fail the entire list
                                if err[2] == True:
                                    signal_solr_restart()
                    if found_error == False:
                        #unknown error code
                        lmsg = str("Unknown commit error code: %s from indexbroker"
                                   %str(retmsg))
                        logger.LogMsg(INXC_LOGMSG_SYS_ERR, lmsg)
                        logger.TraceMsg(lmsg)
            else:
                logger.TraceMsg("Terminated indexbroker protocol, prior errors so did not set EAs")
                rc = INXC_RC_SUCCESS
        else:
            #Index broker not expecting a commit request if no files had success
            rc = INXC_RC_SUCCESS
    else:
        lmsg = "Error on send of END OF FILELIST"
        logger.TraceMsg(lmsg)
        logger.LogMsg(INXC_LOGMSG_SYS_ERR, lmsg)
        rc = INXC_RC_FATAL
    logger.TraceMsg("closing file handles, socket")
    if _fileobj != None:
        _fileobj.close()
    if _socket != None:
        _socket.shutdown(socket.SHUT_RDWR)
    return rc

def process_retry_entry(rt):
    global logger
    global file_retrylist
    global EAlist
    meta = rt[0]
    xattr = rt[1]
    pathname = xattr[0]
    docID = xattr[1]
    indexlevel = xattr[2]
    corenum = xattr[3]
    epoch = xattr[4]
    startofindexing_asEA = xattr[5]
    previous_errmsg = xattr[6]
    existing_EA_INerror = xattr[7]
    rc = INXC_RC_FATAL
    logger.TraceMsg("Processing retry of %s " %(pathname))
    errnum = existing_EA_INerror
    retmsg = run_indexbroker_protocol(port, pathname, meta)
    if retmsg == INXC_IB_RC_SUCCESS:
        logger.TraceMsg("Index operation successful for %s"
                        %(pathname))
        #Save EAs for inserts/updates to set after commit succeeds
        if operation == INXC_OP_UPDATE:
            if existing_EA_INerror  >= INXC_EA_ERR_RETRYCOUNT_START and existing_EA_INerror   <= INXC_EA_ERR_RETRYCOUNT_MAX:
                previous_errmsg = 0
            logger.TraceMsg("Save EAs, set after commit for insert/update")
            EA = [pathname, docID, indexlevel, str(corenum), epoch,
                  startofindexing_asEA, previous_errmsg, existing_EA_INerror]
            if EAlist == None or len(EAlist) == 0:
                EAlist = [EA]
            else:
                EAlist.append(EA)
            logger.TraceMsg("Saved to EAlist: %s" %(str(EA)))
            rc = INXC_RC_SUCCESS
    else:
        lmsg = str("Err code, %s from indexbroker for %s during retry, check for another retry"
                   %(str(retmsg), pathname))
        logger.LogMsg(INXC_LOGMSG_SYS_ERR, lmsg)
        logger.TraceMsg(lmsg)
        rc = INXC_RC_FATAL
        #Determine remaining processing based on error code from IB
        found_error = False
        #err list [number, retry, retryasmetadataonly, EAtoSet, restartsolr, maxretrycount]
        for err in INXC_IB_ERR_LIST:
            if retmsg == err[0]:
                found_error = True
                if err[1] == True and err[5] == 0:
                    #save this entry to the retry list
                    if operation == INXC_OP_UPDATE:
                        logger.TraceMsg("Get EAs for retry list to set after commit ")
                        if err[2] == True:
                            #modify the meta msg to index metadata only
                            logger.TraceMsg("Modify meta to index metadata only for %s"
                                            %pathname)
                            retrymeta = change_to_mdonly(meta)
                            EA = [pathname, docID,
                                  SSM_INDEX_LEVEL_METADATAONLY,
                                  str(corenum), epoch,
                                  startofindexing_asEA, err[3], existing_EA_INerror]
                        else:
                            #retry at current indexlevel
                            retrymeta = meta
                            EA = [pathname, docID, indexlevel,
                                  str(corenum), epoch,
                                  startofindexing_asEA, None, existing_EA_INerror]
                        rc = INXC_RC_SUCCESS
                    else:
                        #for deletes save the pathname for convenience
                        EA = [pathname]
                        retrymeta = meta
                        rt = [retrymeta, EA]
                        if file_retrylist == None or len(file_retrylist) ==0:
                            file_retrylist = [rt]
                        else:
                            file_retrylist.append(rt)
                        rc = INXC_RC_SUCCESS
                elif err[1] == True and err[5] > 0:
                    #Don't retry again in this txn, but increment retry count via INerror
                    if errnum == None or errnum == 0:
                        newerr = INXC_EA_ERR_RETRYCOUNT_START
                    else:
                        newerr = errnum + 1
                    cmd = str('%s%s --filename=%s --documentid=%s --coreid=%s --indexerror=%s --indexlevel=0 --indexepoch=0'
                              %(SSM_IA_INSTALL_BIN, "/mw_set_indexing_info",
                                re.escape(pathname), docID, corenum, newerr))
                    crc = ssm_run_cmd(logger, cmd)
                    if crc != 0:
                        lmsg = str("Error setting EAs for %s incrementing retry count EA" %pathname)
                        logger.TraceMsg(lmsg)
                        logger.LogMsg(INXC_LOGMSG_SYS_ERR, lmsg)            
                    rc = INXC_RC_SUCCESS
                else:
                    #check if should signal to stop solr as well as terminate
                    lmsg = str("Fatal error code, %s from indexbroker for %s"
                               %(retmsg, pathname))
                    logger.LogMsg(INXC_LOGMSG_INDEXING, lmsg)
                    logger.TraceMsg(lmsg)
                    rc = INXC_RC_FATAL
                    if err[4] == True:
                        signal_solr_restart()
                    rc = INXC_RC_FATAL
        if found_error == False:
            #unknown error code
            lmsg = str("Unknown error code, %s from indexbroker for %s"
                       %(str(retmsg), pathname))
            logger.LogMsg(INXC_LOGMSG_SYS_ERR, lmsg)
            logger.TraceMsg(lmsg)
            rc = INXC_RC_FATAL
    return rc
             
    
try:
    #Must have three arguments, the collection mount point, filelist and operation
    mountpt = None
    filelist = None
    operation = None
    logger_subcomponent = None
    try:
        opts, args = getopt.getopt(sys.argv[1:],"m:f:h",['doUpdates','doDeletes'])
    except getopt.GetoptError, err:
        logger = None
        usage(logger,str(err))

    for opt, arg in opts:
        if opt in ("-m"):
            mountpt = arg.rstrip('/')
        elif opt in ("-f"):
            filelist = arg
        elif opt in ("-h"):
            msg = "Help on command"
            logger = None
            usage(logger,msg)
        elif opt in ("--doDeletes"):
            logger_subcomponent = SSM_LOG_INXC_DELETE
            operation = INXC_OP_DELETE
        elif opt in ("--doUpdates"):
            logger_subcomponent = SSM_LOG_INXC_INGEST
            operation = INXC_OP_UPDATE
        else:
            msg = "unhandled option"
            logger = None
            usage(logger,msg)
        
    #check that mountpt is a directory
    if mountpt == None or not os.path.isdir( mountpt):
        msg = "collectionMountPoint must be a fully qualified directory"
        logger = None
        usage(logger, msg)

    if logger_subcomponent == None:
        logger = None
        msg = "must specify operation"
        usage(logger, msg)

    #parse the config files once for this group of files        
    cdotxml = str("%s%s" %(mountpt, SSM_CDOTXML_RMNT))
    logger = None
    parsed_cdotxml = ssm_nl_parse_xmlfile(cdotxml)
    #get the logger
    if parsed_cdotxml != None:
        loglevel = ssm_nl_get_loglevel(parsed_cdotxml, cdotxml)
        tracelevel = ssm_nl_get_tracelevel(parsed_cdotxml, cdotxml)
        logger = ssm_nl_get_logger(logger_subcomponent, mountpt, 
                                   loglevel, tracelevel)
        if (logger != None):
            pass
        else:
            msg = str("Error in %s, cannot get logger" %(sys.argv[0]))
            sys.stdout.write(msg)
            sys.exit(1)
    else:
        msg = str("Error in %s, parsing collection.xml\n" %(sys.argv[0]))
        sys.stdout.write(msg)

    #check that filelist is a file
    if not os.path.isfile(filelist):
        msg = "filelist for indexing controller to process must exist: " +filelist
        usage(logger, msg)

    logger.LogMsg(INXC_LOGMSG_INFO,str("Get epoch,port,indexlevel from %s" %(cdotxml)))
    epoch = ssm_get_solrValueFromCdotxml(logger, SSM_SOLR_EL_EPOCHNUM,
                                            parsed_cdotxml, cdotxml)
    port = ssm_get_solrValueFromCdotxml(logger, SSM_SOLR_EL_INDEXBROKERPORT,
                                           parsed_cdotxml, cdotxml)
    indexlevel = ssm_get_valueFromXmlfile(logger, SSM_C_EL_INDEXLEVEL,
                                            parsed_cdotxml, cdotxml)
    if (epoch == None or port == None or indexlevel == None):
        logger.LogMsg(SSM_LOGMSG_PARSE, (cdotxml))
        logger.TraceMsg("Error in values from %s" %(cdotxml))
        logger.TraceMsg("epoch:%s port:%s indexlevel:%s"
                        %(epoch, str(port), str(indexlevel)))
        sys.exit(1)
    maxMBfilesize = ssm_get_solrValueFromCdotxml(logger,
                                                 SSM_SOLR_EL_MAXMBFILESIZE,
                                                 parsed_cdotxml, cdotxml)
    if maxMBfilesize != None:
        maxMBfilesize = int(maxMBfilesize)

    try:
        logger.TraceMsg("Opening filelist:%s for inxc processing, operation:%s " %(filelist,operation))
        fl = open(filelist, "r")
        len4seek = 0
    except:
        logger.LogMsg(INXC_LOGMSG_FILEOPEN, (filelist))
        logger.TraceMsg("Open failed for filelist:%s" %(filelist))
        sys.exit(1)
    
    mfdotxml = str("%s%s" %(mountpt,SSM_MFDOTXML_RMNT))
    parsed_mfdotxml = ssm_parse_xmlfile(logger,mfdotxml)

    logger.TraceMsg("%s called for mntpt: %s, with operation: %s, for filelist: %s"
                    %(sys.argv[0],mountpt,operation, filelist))

    if operation == INXC_OP_UPDATE:
        if SSM_INDEX_LEVEL[indexlevel] != 'no_index':
            logger.LogMsg(INXC_LOGMSG_INFO,str("getting jettyip and jettyport from %s" %(cdotxml)))
            jettyname = ssm_get_valueFromXmlfile(logger, SSM_SOLR_EL_VIRTUAL_JETTYIP,
                                                 parsed_cdotxml, cdotxml)
            if (jettyname != None):
                jettyip = ssm_get_ipnumberFromName(logger, jettyname)
                if (jettyip != None):
                    jettyport = ssm_get_valueFromXmlfile(logger, SSM_SOLR_EL_JETTYPORT,
                                                         parsed_cdotxml, cdotxml)
                    if (jettyport == None):
                        logger.LogMsg(SSM_LOGMSG_PARSE_EL, (cdotxml, SSM_SOLR_EL_JETTYPORT))
                        sys.exit(1)
                else:
                    msg = str("Cannot get ipnumber for ipname: %s" %(str(jettyname)))
                    logger.LogMsg(SSM_LOGMSG_SYS_ERR, (msg))
                    sys.exit(1)
            else:
                logger.Logmsg(SSM_LOGMSG_PARSE_EL, (cdotxml,SSM_SOLR_EL_VIRTUAL_JETTYIP))
                logger.TraceMsg("got jettyip:%s jettyport:%s for collection %s"
                                %(str(jettyip), str(jettyport), str(mountpt)))
            minerList = ssm_get_archiveminersFrom_Collxml(logger, parsed_cdotxml,
                                                            cdotxml)
            minerGroups = []
            #mmputacl displays group names when available - so check grpname and gid
            for mg in minerList:
                try:
                    gid = int(mg)
                    try:
                        mg_alt = grp.getgrgid(gid)[0]
                        minerGroups.append([mg,mg_alt])
                    except:
                        mg_alt = mg
                        minerGroups.append([mg,mg_alt])
                except:
                    try:
                        mg_alt = str(grp.getgrnam(mg)[2])
                        minerGroups.append([mg,mg_alt])
                    except:
                        msg = str("Found invalid miner group: %s" %mg)
                        logger.LogMsg(SSM_LOGMSG_SYS_ERR, (msg))
                        sys.exit(1)
            if len(minerGroups) == 0:
                 minerGroups = None
            logger.TraceMsg("Using archive miner groups: %s" %(str(minerGroups)))
            currentCore = str(ssm_get_currentCoreid(logger, mountpt))
            logger.LogMsg(INXC_LOGMSG_INFO,str("currentCoreid: %s" %(currentCore)))
            nd = ssm_get_numDocs(logger, mountpt, currentCore, jettyip, jettyport)
            if nd == None:
                nd = 0
            logger.LogMsg(INXC_LOGMSG_INFO,str("numDocs in current core found as: %s" %(str(nd))))
            maxdocspercore = ssm_get_solrValueFromCdotxml(logger,
                                  SSM_SOLR_EL_MAXDOCSPERCORE, parsed_cdotxml, cdotxml)
            if int(nd) > int(maxdocspercore):
                logger.LogMsg(INXC_LOGMSG_INFO,str("start new core processing"))
                createnewcore = True
                starting = False
                rc = ssm_update_solrConfig(logger, mountpt, indexlevel, jettyip,
                                           jettyport, createnewcore, starting )
                if rc == SSM_UPDATE_LOCKED:
                    #no need to wait for the lock, get a new core the next time
                    logger.LogMsg(INXC_LOGMSG_INFO,str("new core creation already in progress "))
                    coreID4insert = currentCore
                elif rc == SSM_UPDATE_SUCCESS:
                    logger.LogMsg(INXC_LOGMSG_INFO,str("new core created"))
                    #a new core was created
                    coreID4insert = ssm_get_currentCoreid(logger, mountpt)
                elif rc == SSM_UPDATE_ERROR:
                    msg = str("Error getting update lock for collection %s use current core"
                              %(mountpt))
                    logger.LogMsg(INXC_LOGMSG_INFO,msg)
                    coreID4insert = currentCore
                else:
                    msg = str("Error doing update of solr config for  collection %s used current core"
                              %(mountpt))
                    logger.TraceMsg(msg)
                    coreID4insert = currentCore
            else:
                coreID4insert = currentCore
            logger.TraceMsg("Current core for creates set to %s" %(str(coreID4insert)))
        else:
            #should not get called to ingest documents when indexlevel = 3
            msg = str('inxc_feedIndexBroker.py -m %s --doUpdates called for indexlevel %s' 
                       %(mountpt, indexlevel))
            logger.TraceMsg(msg)
            logger.LogMsg(INXC_LOGMSG_SYS_ERR, msg)
            sys.exit(1)
       
    fretry = 0
    EAlist = []                 #EAs to set for list of ingested files after commit
    file_retrylist = None
    logger.TraceMsg("Process each record in filelist %s for %s"
                    %(filelist,operation))
    record = get_next_record(operation)
    while record:
        logger.LogMsg(INXC_LOGMSG_INFO,str("in while record loop, operation: %s  " %(operation)))
        rc = process_record(port, operation, record)
        logger.LogMsg(INXC_LOGMSG_INFO,str("inxc process_record returned %s for record %s"
                                           %(str(rc),str(record))))
        if rc == INXC_RC_SUCCESS:
            record = get_next_record(operation)
            logger.LogMsg(INXC_LOGMSG_INFO,str("Get_next_record returned: %s for filelist %s"
                                               %(str(record),filelist)))
        elif rc == INXC_RC_FATAL:
            lmsg = str("Indexbroker returned fatal indexing rc %s from record %s"
                       %(str(rc),str(record)))
            logger.LogMsg(INXC_LOGMSG_INDEXING, lmsg)
            logger.TraceMsg(lmsg)
            setEAs = False
            rc = terminate_ib_protocol(setEAs)
            sys.exit(1)
        else:
            lmsg = str("unknown return code %s, from inxc process_record"
                       %(str(rc)))
            logger.LogMsg(INXC_LOGMSG_SYS_ERR, lmsg)
            logger.TraceMsg(lmsg)
            setEAs = False
            rc = terminate_ib_protocol(setEAs)
            sys.exit(1)
    logger.TraceMsg("exited processing of file records, operation: %s  "
                    %(operation))

    #process the file retry list
    logger.TraceMsg("Process each record in file retry list for %s for %s"
                    %(filelist,operation))
    if operation == INXC_OP_DELETE:
        fretry_max = INXC_RETRY_DELETE_MAX
    elif operation == INXC_OP_UPDATE:
        fretry_max = INXC_RETRY_INGEST_MAX
    while fretry < fretry_max:
        if file_retrylist != None:
            pf_retrylist = file_retrylist
            file_retrylist = None
            for rt in pf_retrylist:
                logger.TraceMsg("Process file retry record list, entry %s"
                                %(str(rt)))
                rc = process_retry_entry(rt)
                if rc == INXC_RC_SUCCESS:
                    pass
                elif rc == INXC_RC_FATAL:
                    lmsg = str("Indexbroker returned fatal indexing rc from retry record %s"
                               %(str(rt)))
                    logger.LogMsg(INXC_LOGMSG_INDEXING, lmsg)
                    logger.TraceMsg(lmsg)
                    setEAs = False
                    rc = terminate_ib_protocol(setEAs)
                    sys.exit(1)
                else:
                    lmsg = str("unknown return code %s, from retry record %s"
                               %(str(rc),str(rt)))
                    logger.LogMsg(INXC_LOGMSG_SYS_ERR, lmsg)
                    logger.TraceMsg(lmsg)
                    setEAs = False
                    rc = terminate_ib_protocol(setEAs)
                    sys.exit(1)
        fretry = fretry + 1
    logger.TraceMsg("exited processing of file retries, operation: %s  "
                    %(operation))

    setEAs = True
    rc = terminate_ib_protocol(setEAs)
    if rc == INXC_RC_SUCCESS:
        sys.exit(0)
    else:
        sys.exit(1)

except SystemExit: 
    pass
except: 
    pmsg = "inxc_feedIndexBroker.py has encountered a serious error: "
    pmsg += format_exc()
    if logger != None:
        logger.LogMsg(INXC_LOGMSG_SYS_ERR, pmsg)
        logger.TraceMsg(pmsg)
    else:
        sys.stdout.write(pmsg)
    sys.exit(1)
