#! /usr/bin/env python
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Change Activity
#
# Name           Date         Change ID     Description
# Brad Trubey    07/12/2010   TIAMC00014566 Initial Version in CCRC
# Brad Trubey    10/29/2010   TIAMC00015805 Move LC variables from MC to RAS
################################################################################
# logcollector.py
"""
objects to represent nodecollector tasks
"""

from tiam_util import *
from logcollector_util import *
from socket import gethostname


class logcollector:

    def __init__( self, node, datestamp, compress, cmdlistname, filelistname, all_files):
        self.node = node
        self.datestamp = datestamp
        self.compress = compress
        self.use_gpfs = not( compress)
        self.cmdlistname = cmdlistname
        self.filelistname = filelistname
        self.all_files = all_files
        self.timeout = get_logc_config()["logc_timeout"]
        if all_files ==  "-a on":
            self.timeout = get_logc_config()["logc_long_timeout"]
        self.running = False
        self.progresspos = 0
        self.progressmax = 0
        self.progresstime = int( time.time())
        self.progressaction = "not started."

        # There are some commands which can only be run on a single GPFS node.  Since we are guaranteed that
        # ia_logcollector.py is always running on a GPFS node, we can set a flag when calling nodecollector.py
        # on only that node to indicate that it should execute the "single node" commands.  The flag "-m yes"
        # (aka the "master" switch) indicates to nodecollector.py that it should execute commands that are prefixed
        # by the string "[SingleNodeOnly]".  Other instances of nodecollector will ignore those commands.
        if gethostname() == self.node:
            master_switch = '-m on'
            print '************* Setting master switch ON ****************'
        else:
            master_switch = '-m off'
            print '************* Setting master switch OFF ****** '+gethostname()+' not equal '+self.node+' **********'

        if compress:
            self.logdir = "/tmp/logcollection_node"
            self.scp_collect(master_switch)
        else:
            self.logdir = "/tiam_utility/logcollection_node"
            self.gpfs_collect(master_switch)


    def scp_collect( self, master_switch):
        try:
            cmd = "nodecollector.py -l %s -h %s -c on -i %s -f %s %s %s"
            cmd = cmd % (self.logdir, self.node, self.cmdlistname, self.filelistname, self.all_files, master_switch)
            syscall_node( self.node, cmd)
            self.running = True
        except:
            traceback.print_exc(file=sys.stdout)
            nologtxt = "could not collect logs from %s" % self.node
            logprint( nologtxt)
            self.running = False


    def gpfs_collect( self, master_switch):

        cmd = "nodecollector.py -l %s -h %s -i %s -f %s %s %s"
        cmd = cmd % (self.logdir, self.node, self.cmdlistname, self.filelistname, self.all_files, master_switch)
        print 'RUNNING COMMAND: %s' % (cmd)
        syscall_node( self.node, cmd)
        self.running = True


    def extract_progress( self, line):
        parts = re.split( "\|", line.strip())
        try:
            self.progresspos    = int( parts[0])
            self.progressmax    = int( parts[1])
            self.progresstime   = int( parts[2])
            self.progressaction = parts[3]
        except:
            logprint( "problem in extract_progress!")
        return ( self.progresspos, self.progressmax, self.progresstime, self.progressaction)


    def progress( self):
        retval = ( self.progresspos, self.progressmax, self.progresstime, self.progressaction)
        if not( self.running):
            return retval

        if self.use_gpfs:
            datadir = "/tiam_utility/logcollection_node/%s_info/" % self.node
            progressfile = datadir + "progress.txt"
            return self.extract_progress( display_progress( progressfile))
        else:
            try:
                cmd = "ssh %s nodecollector_progress.py -l %s -h %s" % ( self.node, self.logdir, self.node)
                output, result = syscallread2( cmd, 0, "")
                logprint( output[ 1].strip())
                return self.extract_progress( output[ 1].strip())
            except:
                logprint( "problem in progress calculation!")
                return retval
        return retval


    def ready( self):
        if not( self.running):
            return True

        if self.use_gpfs:
            datadir = "/tiam_utility/logcollection_node/%s_info/" % self.node
            startmarker = datadir + "begin.txt"
            endmarker   = datadir + "end.txt"
            progressfile = datadir + "progress.txt"

            if get_marker_time( startmarker) <= get_marker_time( endmarker):
                return True
            else:
                display_progress( progressfile)
                return False
        else:
            try:
                syscall_node( self.node, "nodecollector_progress.py -l %s -h %s" % ( self.logdir, self.node))
            except:
                return False
        return True


    # wait loops for logc_timeout seconds, checking every 10 seconds to see if log collection has completed on the node.
    # Once this function returns, the log collector begins copying the logs for assembly.
    def wait( self):
        timeout = self.timeout
        for i in range( 0, timeout/10):
            if self.ready():
                return
            else:
                time.sleep(10)
        logprint( "timeout of %d seconds reached in logcollector" % timeout)
        #todo: maybe we should kill the process here if it is still there
        return


    def get_result( self, datadir):
        self.wait()
        logprint("logcollector wait has completed for node "+self.node)
        if self.use_gpfs:
            syscall( "mv %s/%s %s" % (self.logdir, self.node, datadir))
            try:
                syscall( "mv %s/skipped_%s_* %s" % (self.logdir, self.node, datadir))
            except:
                pass
            return
        else:
            try:
                logprint( "Copying log collection results from node '%s'..." % self.node)

                #check sizeinfo before copying
                sizeinfo = datadir + "/sizeinfo.txt"
                syscall_try( "rm -f " + sizeinfo)
                syscall_try( "scp %s:%s/%s_info/sizeinfo.txt %s" % (self.node, self.logdir, self.node, datadir))
                if os.path.isfile( sizeinfo):
                    f = open( sizeinfo, "r")
                    size = int( f.readline().strip())
                    f.close()
                    if 1.1 * size >= free_space( datadir):
                        logprint( "out of space on '%s'" % datadir)
                        raise Exception, "out of space"

                #now do the copying
                syscall( "scp %s:%s/*.tgz %s" % (self.node, self.logdir, datadir))
                try:
                    syscall( "scp %s:%s/skipped_*.txt %s" % (self.node, self.logdir, datadir))
                except:
                    pass
                logprint( 'Copy complete.')
            except:
                nologtxt = "Could not collect logs from node '%s'." % self.node
                logprint( nologtxt)
                syscall( 'echo "%s" > %s/%s.txt' % (nologtxt, datadir, self.node))

            # source can be deleted
            try:
                syscall_node( self.node, "rm %s/*.tgz" % self.logdir)
            except:
                logprint( "Cleanup of logs on node '%s' failed" % self.node)
