#!/bin/sh
########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Workload Manager
# *
# * FILE NAME:       mw_controlwatchdog.sh
# *
# * DESCRIPTION:     Controls the Watchdog component of
# *                  the IA Middleware.
# *
# * AUTHOR:          Michael A. Richmond
# *                  IBM Almaden Research Center
# *                  mar@almaden.ibm.com
# *
# ****************************************************************************
# */
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
#               Change Log
#               ----------
#  19 Sep 2008  Initial version.
#
#  09 Oct 2008  Updated to include GPFS node ID in the report.
#
#  03 Mar 2009  Updated to set ALLOW_REBOOT flag to value that
#               the IA appliance will ship with.
#

# Configure shell variables
. `dirname $BASH_SOURCE`/mw_environment.sh

# Read the common error values
. `dirname $BASH_SOURCE`/wlm_controlerrors.sh

# Read the common control functions
. `dirname $BASH_SOURCE`/wlm_basiccontrolcommon.sh

##############################################################################
##  Control reboot authority for wlm_watchdog.
##    If ALLOW_REBOOT is not set to 'y' then the watchdog will be 
##    prevented from affecting the power state of nodes the watchdog
##    is monitoring. If the watchdog is prevented from affecting
##    the power state of nodes then the watchdog will simply
##    print a message indicating that a node should be power cycled.
#ALLOW_REBOOT="n"
#   allow watchdog to reboot nodes
ALLOW_REBOOT="y"

# Locations of files and executables required for WLM Watchdog
WATCHDOG_NAME=ia_watchdog
WATCHDOG_BIN=/opt/tivoli/tiam/bin/$WATCHDOG_NAME

# Timeout while waiting for watchdog to stop (in seconds). Give it a minute
WATCHDOG_STOP_TIMEOUT=60

doStart()
{
	check_file_exists $WATCHDOG_BIN
	b_exists=$?
	if [ "$b_exists" != 1 ]; then
		return $ERR_FILE_DOES_NOT_EXIST
	fi
	check_file_executable $WATCHDOG_BIN
	b_is_executable=$?
	if [ "$b_is_executable" != 1 ]; then
		return $ERR_FILE_NOT_EXECUTABLE
	fi

	doStatus
	running_status=$?

	if [ $running_status -ne $ERR_PROCESS_NOT_RUNNING ]; then
		return $running_status
	fi

	if [ "${ALLOW_REBOOT}" = "y" ]; then
		REBOOT_FLAG=""
	else
		REBOOT_FLAG="--disallowreboot"
	fi	

        #LOGFILE="/tmp/wd_"`hostname`"-"`date "+%Y-%m-%d-%H:%M:%S"`".log"
        LOGFILE="/dev/null"
	nohup ${WATCHDOG_BIN} ${REBOOT_FLAG} > ${LOGFILE} 2>&1 &

	doStatus
	running_status=$?

	if [ $running_status -ne $ERR_PROCESS_IS_RUNNING ]; then
		return $running_status
	fi

	return 0
}

doStatus()
{
	num_running=0

	running_pid_list=`/bin/ps -C \`basename $WATCHDOG_BIN\` -f | tail +2 | sed 's/ -d / /g' | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`

	for watchdog_pid in $running_pid_list
	do
		num_running=`expr $num_running + 1`
	done


	if [ $num_running -eq 0 ]; then
		return $ERR_PROCESS_NOT_RUNNING
	elif [ $num_running -eq 1 ]; then
		return $ERR_PROCESS_IS_RUNNING
	elif [ $num_running -gt 1 ]; then
		return $ERR_PROCESS_MULTIPLE_RUNNING
	fi
}

doStop()
{
	doStatus
	running_status=$?

	if [ $running_status -eq $ERR_PROCESS_NOT_RUNNING ]; then
		return $running_status
	fi

	running_pid_list=`/bin/ps -C \`basename $WATCHDOG_BIN\` -f | tail +2 | sed 's/ -d / /g' | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`

	for watchdog_pid in $running_pid_list
	do
		kill_process $watchdog_pid SIGKILL
	done

	waited_seconds=0
	timeout_seconds=$WATCHDOG_STOP_TIMEOUT
	wait_interval=1
	while [ $waited_seconds -lt $timeout_seconds ]
	do
		doStatus
		running_status=$?

		if [ $running_status -eq $ERR_PROCESS_NOT_RUNNING ]; then
			return 0
		fi

		sleep $wait_interval
		waited_seconds=`expr $waited_seconds + $wait_interval`
	done

	return $ERR_PROCESS_FAILED_TO_DIE
}


parse_args $@

case ${OPERATION} in
	start)
		doStart
		start_retval=$?
		if [ $start_retval -eq 0 ]; then
			exit 0
		else
			if [ $start_retval -eq $ERR_FILE_DOES_NOT_EXIST ]; then
				echo "start issued but $WATCHDOG_BIN does not exist."
			elif [ $start_retval -eq $ERR_FILE_NOT_EXECUTABLE ]; then
				echo "start issued but $WATCHDOG_BIN is not executable."
			elif [ $start_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
				echo "start issued but $WATCHDOG_BIN is already running"
			elif [ $start_retval -eq $ERR_PROCESS_MULTIPLE_RUNNING ]; then
				echo "start issued but multiple $WATCHDOG_BIN are already running"
			elif [ $start_retval -eq $ERR_PROCESS_NOT_HEALTHY ]; then
				echo "start issued but $WATCHDOG_BIN (which is unhealthy) is already running"
			fi

			exit $start_retval
		fi
		;;
	restart)
		echo "restart not implemented"
		exit -1
		;;
	status)
		doStatus
		status_retval=$?
		if [ $status_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
			exit 0
		else
			exit $status_retval
		fi
		;;
	stop)
		doStop
		stop_retval=$?
		if [ $stop_retval -eq 0 ]; then
			exit 0
		else
			if [ $stop_retval -eq $ERR_PROCESS_NOT_RUNNING ]; then
				echo "stop issued for $WATCHDOG_BIN but it was not running"
			fi
			exit $stop_retval
		fi
		;;
	*)
		printHelp
		;;
esac
