#!/bin/sh
########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Workload Manager
# *
# * FILE NAME:       mw_controlwlm.sh
# *
# * DESCRIPTION:     Controls the Workload Manager 
# *                  component of the IA Middleware.
# *
# * AUTHOR:          Michael A. Richmond
# *                  IBM Almaden Research Center
# *                  mar@almaden.ibm.com
# *
# ****************************************************************************
# */
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
#               Change Log
#               ----------
#  14 Nov 2008  Initial version.
#

# Configure shell variables
. `dirname $BASH_SOURCE`/mw_environment.sh

# Read the common error values
. `dirname $BASH_SOURCE`/wlm_controlerrors.sh

# Read the common control functions
. `dirname $BASH_SOURCE`/wlm_basiccontrolcommon.sh

##############################################################################
##  Enable/Disable load balancing.
#LOAD_BALANCE="n"
LOAD_BALANCE="y"
#
##############################################################################
## Daemonize the manage_collections process.
#DAEMONIZE="n"
DAEMONIZE="y"

AUDITD_NAME=auditd

# Locations of files and executables required for WLM Watchdog
MANAGE_COLLECTIONS_NAME=manage_collections
MANAGE_COLLECTIONS_BIN=/opt/tivoli/tiam/bin/$MANAGE_COLLECTIONS_NAME

COLLECTIONS_FILE_DIR="/tiam_utility/config"
COLLECTIONS_FILE=$COLLECTIONS_FILE_DIR"/collections.middleware.xml"

# Timeout while waiting for manage_collections to stop (in seconds). Give it a minute
MANAGE_COLLECTIONS_STOP_TIMEOUT=60

doStart()
{

	check_file_exists $MANAGE_COLLECTIONS_BIN
	b_exists=$?
	if [ "$b_exists" != 1 ]; then
		return $ERR_FILE_DOES_NOT_EXIST
	fi
	check_file_executable $MANAGE_COLLECTIONS_BIN
	b_is_executable=$?
	if [ "$b_is_executable" != 1 ]; then
		return $ERR_FILE_NOT_EXECUTABLE
	fi

	doStatus
	running_status=$?

	if [ $running_status -ne $ERR_PROCESS_NOT_RUNNING ]; then
		return $running_status
	fi

        args=""

	if [ "${LOAD_BALANCE}" = "y" ]; then
		args="${args} --balance"
	fi	

	if [ "${DAEMONIZE}" = "y" ]; then
		args="${args} --daemonize"
	fi	

	${MANAGE_COLLECTIONS_BIN} $args > /dev/null 2>&1 &

	doStatus
	running_status=$?

	if [ $running_status -ne $ERR_PROCESS_IS_RUNNING ]; then
		return $running_status
	fi

	return 0
}

doRestart()
{
	killall $MANAGE_COLLECTIONS_NAME
	doStart
	return $?
}

doStatus()
{
	num_running=0

	running_pid_list=`/bin/ps -C \`basename $MANAGE_COLLECTIONS_BIN\` -f | tail +2 | sed 's/ -d / /g' | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`

	for manage_collections_pid in $running_pid_list
	do
		num_running=`expr $num_running + 1`
	done


	if [ $num_running -eq 0 ]; then
		return $ERR_PROCESS_NOT_RUNNING
	elif [ $num_running -eq 1 ]; then
		return $ERR_PROCESS_IS_RUNNING
	elif [ $num_running -gt 1 ]; then
		return $ERR_PROCESS_MULTIPLE_RUNNING
	fi
}

doStop()
{
	running_pid_list=`/bin/ps -C \`basename $MANAGE_COLLECTIONS_BIN\` -f | tail +2 | sed 's/ -d / /g' | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`

	for manage_collections_pid in $running_pid_list
	do
		kill_process $manage_collections_pid SIGKILL
	done

	waited_seconds=0
	timeout_seconds=$MANAGE_COLLECTIONS_STOP_TIMEOUT
	wait_interval=1
	while [ $waited_seconds -lt $timeout_seconds ]
	do
		doStatus
		running_status=$?

		if [ $running_status -eq $ERR_PROCESS_NOT_RUNNING ]; then
			echo "Collection manager stopped. Stopping collections"
			#manage_collections is now stopped. We can stop each collection now.
			collections=`/opt/tivoli/tiam/bin/wlm_enumeratecollections.sh $COLLECTIONS_FILE`
			for collection in $collections
			do
				echo -n "Stopping" $collection "... "
				stop_output=`/opt/tivoli/tiam/bin/wlm_stopcollection.sh $collection`
				stop_retval=$?
				if [ $stop_retval -ne 0 ]; then
					echo " unable to stop collection" $collection "error string:" $stop_output
					return $ERR_PROCESS_FAILED_TO_DIE
				fi
				echo "stopped"
			done

			#We can kill auditd because all collections are stopped.
			killall $AUDITD_NAME

			return 0
		fi

		sleep $wait_interval
		waited_seconds=`expr $waited_seconds + $wait_interval`
	done

	return $ERR_PROCESS_FAILED_TO_DIE
}


parse_args $@

case ${OPERATION} in
	start)
		doStart
		start_retval=$?
		if [ $start_retval -eq 0 ]; then
			exit 0
		else
			if [ $start_retval -eq $ERR_FILE_DOES_NOT_EXIST ]; then
				echo "start issued but $MANAGE_COLLECTIONS_BIN does not exist."
			elif [ $start_retval -eq $ERR_FILE_NOT_EXECUTABLE ]; then
				echo "start issued but $MANAGE_COLLECTIONS_BIN is not executable."
			elif [ $start_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
				echo "start issued but $MANAGE_COLLECTIONS_BIN is already running"
			elif [ $start_retval -eq $ERR_PROCESS_MULTIPLE_RUNNING ]; then
				echo "start issued but multiple $MANAGE_COLLECTIONS_BIN are already running"
			elif [ $start_retval -eq $ERR_PROCESS_NOT_HEALTHY ]; then
				echo "start issued but $MANAGE_COLLECTIONS_BIN (which is unhealthy) is already running"
			fi

			exit $start_retval
		fi
		;;
	restart)
		doRestart
		restart_retval=$?
		if [ $restart_retval -eq 0 ]; then
			exit 0
		else
			if [ $restart_retval -eq $ERR_FILE_DOES_NOT_EXIST ]; then
				echo "restart issued but $MANAGE_COLLECTIONS_BIN does not exist."
			elif [ $restart_retval -eq $ERR_FILE_NOT_EXECUTABLE ]; then
				echo "restart issued but $MANAGE_COLLECTIONS_BIN is not executable."
			elif [ $restart_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
				echo "restart issued but $MANAGE_COLLECTIONS_BIN is already running"
			elif [ $restart_retval -eq $ERR_PROCESS_MULTIPLE_RUNNING ]; then
				echo "restart issued but multiple $MANAGE_COLLECTIONS_BIN are already running"
			elif [ $restart_retval -eq $ERR_PROCESS_NOT_HEALTHY ]; then
				echo "start issued but $MANAGE_COLLECTIONS_BIN (which is unhealthy) is already running"
			fi

			exit $restart_retval
		fi
		;;
	status)
		doStatus
		status_retval=$?
		if [ $status_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
			exit 0
		else
			exit $status_retval
		fi
		;;
	stop)
		doStop
		stop_retval=$?
		if [ $stop_retval -eq 0 ]; then
			exit 0
		else
			if [ $stop_retval -eq $ERR_PROCESS_NOT_RUNNING ]; then
				echo "stop issued for $MANAGE_COLLECTIONS_BIN but it was not running"
			fi
			exit $stop_retval
		fi
		;;
	*)
		printHelp
		;;
esac
