#! /usr/bin/env python
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Change Activity
#
# Name           Date         Change ID     Description
# Brad Trubey    07/12/2010   TIAMC00014566 Initial Version in CCRC
# Brad Trubey    08/03/2010   TIAMC00014894 disable the age thresholds, which are preventing the collection of needed files
# Brad Trubey    08/03/2010   TIAMC00013723 Don't zip the always_collect files, since no one can find them in nested zips
# Brad Trubey    10/26/2010   TIAMC00016264 Progess indicator shows > 100%
# Brad Trubey    10/27/2010   TIAMC00015805 Provide a way to exclude undesired file extensions, move LC variables from MC to RAS
# Brad Trubey    11/04/2010   TIAMC00016265 Make it easier to find middleware logs by untarring mw_logs.tgz
################################################################################

import getopt, sys, os, time, stat, pwd, subprocess, fileinput
import md5
from datetime import datetime
from tiam_util import *
from ia_util import *
from logcollector_util import *


global_summary      = "skipped.txt"
global_filelist     = "filelist.txt"
global_progressfile = "progress.txt"
global_progressmax  = 0
global_progresspos  = 0


def usage():
    exitmsg('usage: ' + sys.argv[0] +
            ' -l logdir -h hostname [-c on/off][-g on/off]' +
            '[-a on/off][-i instructionfile][-f filepatternfile][-m on/off]', 1)


def fastcall( cmd):
    #logprint( cmd)
    return os.system( cmd)


def replace_template( list, template, replist):
    newlist = []
    fulltemplate = "%(" + template +")s"
    for cmd in list:
        pos = cmd.find( fulltemplate)
        if pos > -1:
            for repentry in replist:
                newcmd = cmd.replace( fulltemplate, repentry)
                newlist.append( newcmd)
        else:
            newlist.append( cmd)
    return newlist


def get_all_collections():
    result = []

    #get collections via mount
    cmd = 'mmlsmount all | grep -v tiam_utility'
    try:
        output, exitcode = syscallread2( cmd, 0, cmd)
        syscall_checkres( cmd, exitcode)
    except:
        return result

    for line in output:
        parts = re.split( "[ \t]+", line.strip())
        result.append( parts[ 2])

    return result


# 16264: show_progress adds an entry to the progress.txt file, so it should only be called as new commands
# are executed or new files collected.  If it is called for anything else, the count of commands/files
# executed will exceed the progressmax count, and you will end up seeing a progress indication > 100%.
def show_progress( activity):
    logprint( activity)

    global global_progressfile
    try:
        f = open( global_progressfile, "a+")
    except:
        return

    global global_progressmax
    global global_progresspos
    global_progresspos += 1
    now = time.time()
    timeint = int( now)
    f.write( "%d|%d|%d|%s\n" % (global_progresspos, global_progressmax, timeint, activity))
    f.close()


def read_ia_commands( filename):
    cmdlist = read_list( filename)
    if len( cmdlist) == 0:
        # get at least some system info
        cmdlist = ['/opt/tivoli/tiam/bin/tiamversion',
                   '/sbin/lspci -vvv',
                   '/bin/rpm -qa',
                   '/usr/sbin/dmidecode output']

    # replace template for collection
    cmdlist = replace_template( cmdlist, "collection", get_all_collections())

    # replace template for node
    cmdlist = replace_template( cmdlist, "node", get_all_nodes())

    return cmdlist


def do_ia_commands( cmdlist, tempdir, hostname, master):
    logprint('do_ia_commands running '+str(len(cmdlist))+' commands in '+tempdir+' on '+str(hostname)+' with master='+str(master))
    systeminfo = open(tempdir + '/systemconfig.txt', 'w')
    for cmd in cmdlist:
        if cmd.startswith('[SingleNodeOnly]'):
            if master:
                # strip '[SingleNodeOnly]' metadata
                cmd = cmd[16:]
            else:
                show_progress( 'skipping  "%s"' % cmd)
                continue
        show_progress( 'executing  "%s"' % cmd)
        systeminfo.write( '*** "%s" output is:\n' % cmd)
        f_in, f_out = os.popen4(cmd)
        systeminfo.write(f_out.read() + "\n\n")
        f_in.close()
        f_out.close()
    systeminfo.close()


def optimize_logfiles( desclist, hostname):
    newlist = []
    tiam_utility = "/tiam"
    tulen = len( tiam_utility)
    for filedesc in desclist:
        # collect tiam* only on node1
        if filedesc[:tulen] == tiam_utility:
            if hostname == "ianode1":
                newlist.append( filedesc)
        else:
            newlist.append( filedesc)

    return newlist


def read_ia_logfiles( filename, hostname):
    desclist = read_list( filename)
    if len( desclist) == 0:
        # get at least some usefull files
        desclist = ['/opt/tivoli/tiam/log', '/tiam_utility/config', '/tiam/*/utility']

    desclist = optimize_logfiles( desclist, hostname)
    return desclist


def dir_size( dir):
    cmd = "du -ms %s" % dir #summary for dir in MB
    size = int( re.split( "[ \t]+", syscallread( cmd)[0].strip())[0])
    return size


def write_size( what, sizeinfo):
    size = 0
    if os.path.isfile( what):
        size = file_size( what)
    elif os.path.isdir( what):
        size = dir_size( what)

    sizefile = open( sizeinfo, "w")
    sizefile.write( str( size) + "\n")
    sizefile.close()
    return size


def file_size( f):
    # get size of file in MB
    fsize = os.stat(f)[stat.ST_SIZE]
    fsize = fsize / (1024 * 1024)
    return fsize


def file_age( f):
    # get age of file in days
    age = int( time.time()) - os.path.getmtime( f)
    age = age / (3600 * 24)
    return age


def skipped_size( f):
    line = "%s %d %d" % (f, file_size( f), get_logc_config()["logc_size_cutoff"])
    addline( global_summary + "size_cutoff.txt", line)


def skipped_age( f):
    mtime = str( datetime.datetime.fromtimestamp( os.path.getmtime( f)))
    line = "%s %s" % (f, mtime)
    addline( global_summary + "age_cutoff.txt", line)


def compress_files( flname, tempdir):
    try:
        fl = open( flname, "r")
    except:
        return

    for f in fl:
        f = f.strip()
        head, tail = os.path.split(f)
        break

    fl.close()

    tgzname = head.replace( '/', '_')
    fastcall( "tar -T %s -czf %s/%s.tgz" % (flname, tempdir, tgzname))


def list_len( fl):
    sline = syscallreadhide( "wc -l %s" % fl, "")[0].strip()
    retval = int( re.split( "[ \t]+", sline)[0])
    return retval


def list_cutoff( fl, cutoff):
    if list_len( fl) > cutoff and cutoff > 0:
        fastcall( "mv %s %s.tmp" % (fl, fl))
        fastcall( "head -n %d %s.tmp > %s" % (cutoff, fl, fl))
        fastcall( "tail --lines=+%d %s.tmp >> %s" % (cutoff, fl, global_summary + "count_cutoff.txt"))
        fastcall( "rm %s.tmp" % fl)


def compress_file( f, newdir):
    head, tail = os.path.split( f)
    fastcall( "gzip -c -1 %s > %s.gz" % (f, newdir + tail))


def copy_file( f, newdir):
    # Defect 16265: People examining log collection archives are having trouble finding the middleware logs, because they are unaware
    # the logs are getting archived into ianode1/tmp/mw_logs.tgz.  So instead of capturing that file, the log collector unpacks that
    # archive into ianode1 instead.
    if f=='/tmp/mw_logs.tgz':
        # strip off the trailing "tmp/" so that the middleware logs get unzipped into ianode1/tiam_utility, not ianode1/tmp/tiam_utility
        newdir = newdir[0:len(newdir)-4]
        logprint('Unzippping middleware logs '+f+' into directory '+newdir+' instead of copying them')
        fastcall( 'tar xf ' + f + ' -C ' + newdir)
    else:
        fastcall( 'cp -aL ' + f + ' ' + newdir)


def copy_files( global_filelist, tempdir, all_files):
    try:
        fl = open( global_filelist, "r")
    except:
        return

    maxsize = get_logc_config()["logc_size_cutoff"]

    for f in fl:
        f = f.strip()
        head, tail = os.path.split(f)

        newdir = tempdir + head + '/'
        if not os.path.isdir(newdir):
            fastcall( 'mkdir -p ' + newdir)

        # limit size and age of files to copy, because the size and count of some files like e.g. raw_log is insane
        fsize = file_size( f)
        if not(all_files):

            if fsize >= maxsize and maxsize > 0:
                msgname = newdir + tail + ".txt"
                msgtext = "%s not copied. size of %d MB is bigger than allowed maximum %d MB!"
                msgtext = msgtext % (f, fsize, maxsize)
                addline( msgname, msgtext)
                # add error indication to skipped.txt
                skipped_size( f)
                continue

            # defect 14894: remove the age thresholds, which are preventing the collection of needed files.

        # don't copy if there is no free space
        if 1.1 * fsize > free_space( head):
            logprint( "disk full on '%s'" % head)
            continue;

        if fsize >= get_logc_config()["logc_size_compress"]:
            compress_file( f, newdir)
        else:
            copy_file( f, newdir)

    fl.close()


def check_always_collect( global_filelist):
    for always_pattern in get_logc_config()["logc_always_collect"]:
        cmd = 'grep "^%s" %s | wc -l' % (always_pattern, global_filelist)
        matched_count = int( syscallreadhide( cmd, "")[0].strip())
        if matched_count > 0:
            logprint( "  - check_always_collect found" + str(matched_count) + " matches; returning True")
            return True
    return False


def do_ia_logfiles( desclist, tempdir, all_files):
    logprint('do_ia_logfiles collecting '+str(len(desclist))+' files in '+tempdir+' with all_files='+str(all_files))

    global global_filelist

    for filedesc in desclist:
        try:
            show_progress( 'collecting "%s"' % filedesc)

            excludeNames=[]
            excludedFiles=[]
            # parse out the exclude_list for this directory, if specified
            leftBracketIdx  = filedesc.rfind('[')
            rightBracketIdx = filedesc.rfind(']')
            if leftBracketIdx != -1:
                excludeString = filedesc[leftBracketIdx+1:rightBracketIdx]
                excludeNames = splitCommasIntoList(excludeString)
                filedesc = filedesc[0:leftBracketIdx]
                logprint('  - found filename suffixes to exclude from '+filedesc+': '+str(excludeNames))

            cmd = 'find ' + filedesc + ' -type f >' + global_filelist
            fastcall( cmd)

            # 15805: If this line had a [] entry with a list of suffixes to exclude, then iterate over the
            # global_filelist and remove any matching entries before the files get collected.
            # Note that logprint CANNOT be called within the for loop processing global_filelist, because the
            # logprint output will get redirected into the global_filelist output, which will break the colletor.
            if len(excludeNames) > 0:
                for line in fileinput.input(global_filelist, inplace=1):
                    includeLine = True
                    for suffix in excludeNames:
                        if line.strip().endswith(suffix):
                            includeLine = False
                            excludedFiles.append(line.strip())

                    if includeLine == True:
                        print line.strip()

            if len(excludedFiles) > 0:
                logprint('  - excluded these files: '+str(excludedFiles))

        except:
            try:
                fastcall( "rm -f %s" % global_filelist)
            except:
                pass
            fastcall( "touch %s" % global_filelist)

        always_collect = check_always_collect( global_filelist)

        if not( all_files) and not( always_collect):
            logc_count_cutoff = get_logc_config()["logc_count_cutoff"]
            list_cutoff( global_filelist, logc_count_cutoff)

        logc_count_compress = get_logc_config()["logc_count_compress"]
        # defect 13723: do not zip the always_collect files
        if (list_len( global_filelist) > logc_count_compress):
            compress_files( global_filelist, tempdir)
        else:
            copy_files( global_filelist, tempdir, all_files)


def fakemarker():

    #info is always in /tmp
    tmp = "/tmp"
    infodir = tmp + "/logcollection_info"
    startmarker = infodir + "/begin.txt"
    endmarker   = infodir + "/end.txt"
    runninginfo = infodir + "/info.txt"

    #write a fake ia_logcollector marker
    #to make instance_running work on all nodes
    if not( instance_running( startmarker, endmarker)):
        syscall_try( "mkdir -p " + infodir)
        timestamp( startmarker)


def main():
    #read the command line
    try:
        opts, args = getopt.getopt(sys.argv[1:],
                                   "d:l:c:g:h:i:f:a:m:")
    except getopt.GetoptError:
        usage()

    #option flags
    create_archive = False
    use_gpfs = False
    all_files = False
    master = False
    hostname = syscallread("hostname")[0].strip()
    logdir = "/tmp/logcollection_node"
    datestamp = time.strftime('%Y%m%d%H%M', time.gmtime())
    cmdlistname = get_logc_config()["ia_commands"]
    filelistname = get_logc_config()["ia_logfiles"]
    for opt, arg in opts:
        if opt in ("-l"):
            logdir = arg
        elif opt in ("-h"):
            hostname = arg
        elif opt in ("-i"):
            cmdlistname = arg
        elif opt in ("-f"):
            filelistname = arg
        elif opt in ("-d"):
            datestamp = arg
        elif opt in ("-c"):
            if arg == "on":
                create_archive = True
        elif opt in ("-g"):
            if arg == "on":
                use_gpfs = True
        elif opt in ("-a"):
            if arg == "on":
                all_files = True
        elif opt in ("-m"):
            if arg == "on":
                master = True
        else:
            exitmsg("unhandled option", 1)

    logdir = logdir + "/" + hostname
    infodir = logdir + "_info"
    syscall_try( "rm -rf %s" % logdir)
    fastcall( "mkdir -p %s" % logdir)
    fastcall( "mkdir -p %s" % infodir)
    startmarker = infodir + "/begin.txt"
    endmarker   = infodir + "/end.txt"
    sizeinfo    = infodir + "/sizeinfo.txt"
    global global_progressfile
    global_progressfile = infodir + "/progress.txt"
    global global_filelist
    global_filelist = infodir + "/filelist.txt"
    global global_summary
    global_summary = logdir + "/../skipped_" + hostname + "_"
    try:
        os.remove( global_progressfile)
        os.remove( global_filelist)
        fastcall( "rm -f %s*" % global_summary)
    except:
        pass

    # parallelize it
    daemonize()

    #write start marker
    starttime = timestamp( startmarker)
    fakemarker()

    cmdlist = read_ia_commands( cmdlistname)
    filelist = read_ia_logfiles( filelistname, hostname)
    global global_progressmax
    # + 1 for "ready."
    # + 1 for md5
    # + 1 for cleanup
    global_progressmax = len( cmdlist) + len(filelist) + 3
    if create_archive:
        global_progressmax += 1
    global_progresspos = 0
    do_ia_commands( cmdlist, logdir, hostname, master)
    do_ia_logfiles( filelist, logdir, all_files)
    show_progress( 'cleaning up temporary data ...')
    do_ia_cleanup()

    # create md5 list for collected files
    show_progress( 'creating md5 hashfiles ...')
    cmd = 'find ' + logdir + ' -type f >' + global_filelist
    output = syscallreadhide( cmd, "")

    md5hashfile = open( logdir + '/md5hash.txt', 'w')
    collected_files = open( global_filelist, "r")
    for filename in collected_files:
        filename = filename.strip()
        f = open( filename, 'r')
        digest = md5.new( f.read()).hexdigest()
        md5hashfile.write( filename + ' : ' + digest + '\n')
        f.close()

    md5hashfile.close()

    #create new tar file
    if create_archive:
        logtemplate = logdir + '/../%s.tgz'
        logname = logtemplate % hostname
        show_progress( "creating '%s' ..." % logname)
        if 1.1 * write_size( logdir, sizeinfo) >= free_space( logdir):
            msg = "no space for '%s'" % logname
            logprint( msg)
            f = open( logname, "w")
            f.write( msg + "\n")
            f.close()
        else:
            syscall( 'cd %s/..; tar czf %s %s' % (logdir, logname, hostname))
        write_size( logname, sizeinfo)
        #delete source for tar
        syscall( "rm -rf %s" % logdir)

    #write end marker
    endtime = timestamp( endmarker)
    duration = endtime - starttime

    show_progress( "ready.")


if __name__ == "__main__":
    do_main( main)
