#!/bin/bash
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Filename: setCIFSTrace.sh
#
# Component: RAS
#
# Description:
#    This file contains a bash script for setting the trace level of the CIFS
#    component. It maps the CIFS trace capabilities into min/mid/max/disable,
#    to match the IA Trace Config utility design.
#
#  Usage:
#      setCIFSTrace.sh min|mid|max|disable
#      min: basic tracing only
#      mid: same as min
#      max: full tracing
#      disable: no trace
#
# Change history
#   Modifier                     Date                     CQ Change
#   ---------------------------------------------------------------------------
#  Nambi                         07/ 13 /2010
#  Nambi                         07/ 19 /2010            Added error handlling
# *************************************************************************************

USAGE="Usage: $0 trace_level\n   trace_level values: min|mid|max|disable"
LEVEL="$1"
# getting node name for the current node
NODENAME=`mw_print_local_gpfs_nodehostname`

# include ras_functions
. /opt/tivoli/tiam/bin/ras_functions
rc=0

# log the actions taken by this script
create_new_log_file setCompTrace

# debug level for CTDB
# disable set to level WARNING
# min set to level NOTICE
# mid set to level INFO
# max set to level DEBUG
CTDB_LOG_LEVEL=WARNING

# Log level for SAMBA config
# disable set to level 2
# min set to level 3
# mid set to level 4
# max set to level 5
SAMBA_LOG_LEVEL=2


# Validate the trace levels as one of the following [min,mid,max,disable]
case "${LEVEL}" in
   "min" | "mid" | "max" | "disable" )
     ras_log "Valid trace level found : ${LEVEL}";;
   "" ) ras_log -v "Error, trace level not provided $USAGE"
      exit 1;;
   *) ras_log -v "Error, invalid trace level ${LEVEL} $USAGE"
      exit 1;;
esac

ras_log "Setting trace level for $NODENAME to ${LEVEL}"

if [ "${LEVEL}" == "min" ]; then
    ras_log -v "Setting trace level for $NODENAME to ${LEVEL}"

    # CTDB trace level set to notice
      CTDB_LOG_LEVEL=NOTICE
   SAMBA_LOG_LEVEL=3
    rc=$?
elif [ "${LEVEL}" == "mid" ]; then
    ras_log -v "Setting trace level for $NODENAME to ${LEVEL}"

    # CTDB trace level set to info
        CTDB_LOG_LEVEL=INFO
   SAMBA_LOG_LEVEL=4

    rc=$?
elif [ "${LEVEL}" == "max" ]; then
    ras_log -v "Setting trace level for $NODENAME to ${LEVEL}"

    # CTDB trace level set to debug
        CTDB_LOG_LEVEL=DEBUG
   SAMBA_LOG_LEVEL=5
    rc=$?

elif [ "${LEVEL}" == "disable" ]; then
    ras_log -v "Setting trace level for $NODENAME to ${LEVEL}"

    # CTDB trace level set to warning
    rc=$?
fi

#setting up the CTDB config file trace level

CTDBCONF="/etc/sysconfig/ctdb"

if [ -f $CTDBCONF ]
then
cp $CTDBCONF ${CTDBCONF}.tmp

#checking for ctdb for debug level option entry

if grep -q "CTDB_DEBUGLEVEL" "${CTDBCONF}.tmp"; then
       ras_log -v "CTDB_DEBUGLEVEL option found in  ctdb conf file adding CTDB_DEBUGLEVEL for $NODENAME to ${LEVEL}"
    sed -i "/CTDB_DEBUGLEVEL/ c\ CTDB_DEBUGLEVEL=$CTDB_LOG_LEVEL"  ${CTDBCONF}.tmp
else
       ras_log -v "CTDB_DEBUGLEVEL option not found in ctdb conf file adding CTDB_DEBUGLEVEL for $NODENAME to ${LEVEL}"

   sed -i '$a\ CTDB_DEBUGLEVEL='${CTDB_LOG_LEVEL} ${CTDBCONF}.tmp


fi

mv ${CTDBCONF}.tmp $CTDBCONF
else
       ras_log -v "$CTDB file is missing in $NODENAME , trace level ${LEVEL} is not set for CTDB"
#warning returning non zero
rc=-1

fi

ctdb setdebug $CTDB_LOG_LEVEL

       CTDBLOG=`ctdb getdebug`
       if [[ "${CTDBLOG}" == *${CTDB_LOG_LEVEL}* ]];  then
   ras_log -v "${CTDBLOG}: CTDB command executed succesfully"
       else
   ras_log -v "${CTDBLOG}: CTDB command failed to set the trace value to $$CTDB_LOG_LEVEL try command :\"ctdb setdebug ${CTDB_LOG_LEVEL}\" manually"
         rc=-1
       fi

SMBCONF="/etc/samba/smb.conf"
:


if [ -f $SMBCONF ]
then
cp $SMBCONF ${SMBCONF}.tmp

#checking for smb.conf for log level option entry

if grep -q "log level" "${SMBCONF}.tmp"; then
     ras_log -v "log level option found in smb.conf adding smb.conf log level for $NODENAME to $SAMBA_LOG_LEVEL"
    sed -i "/log level/ c\ log level = $SAMBA_LOG_LEVEL"  ${SMBCONF}.tmp

else
        ras_log -v "log level option not found in smb.conf adding log level for $NODENAME to $SAMBA_LOG_LEVEL"
        sed -i '/\[global\]/a   log level='${SAMBA_LOG_LEVEL} ${SMBCONF}.tmp

fi

mv ${SMBCONF}.tmp $SMBCONF
else
   ras_log -v "$SMBCONF file is missing in $NODENAME , trace level ${LEVEL} is not set for SAMBA"
#warning returning non zero
rc=-1

fi

# Check return code
if [[ $rc -ne 0 ]]; then
  ras_log -v "Unable to set trace level to ${LEVEL} for $NODENAME , please see log file for more information."
else
  ras_log -v "CTDB and SAMBA Trace levels are set to ${LEVEL} for $NODENAME"
fi

exit $rc
