#!/bin/bash
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Filename: setDSTrace.sh
#
# Component: RAS
#
# Description:
#    This file contains a bash script for setting the trace level of the IBM Director Server
#    components. It maps the Director Server trace capabilities into min/mid/max/disable,
#    to match the IA Trace Config utility design. In addition to the Director Server, this
#    script governs the trace behavior of the ESA and ISS Director Extensions.
#
#  Usage:
#      setDSTrace.sh min|mid|max|disable
#      min: basic tracing only, without the derby.log
#      mid: same as min
#      max: full tracing, including the derby.log
#      disable: no trace
#
# Change history
#   Modifier                     Date                     CQ Change
#   ---------------------------------------------------------------------------
#   Del Ence                     9/02/2010                   TIAMC00015056
#      Updated to use 'lwilog.sh -refresh' whenever possible instead of stopping and restarting director
#
# *************************************************************************************

ISS_DEBUG_FILE="/opt/ibm/director/ISS/ISS_DEBUGON.dat"
USAGE="Usage: $0 trace_level\n   trace_level values: min|mid|max|disable"
LEVEL="$1"
rcDir=0
rcISS=0
doRestart=0

# include ras_functions
. /opt/tivoli/tiam/bin/dir_functions
. /opt/tivoli/tiam/bin/ras_functions
rc_reset

# disableDirectorLogging copies disable logging configuration files
function disableDirectorLogging() {
    ras_log "Disabling Director Server logging"
    execute_and_log cp /opt/tivoli/tiam/etc/USMIKernel.disable /opt/ibm/director/lwi/conf/overrides/USMIKernel.properties
    execute_and_log cp /opt/tivoli/tiam/etc/directorLogging.disable /opt/ibm/director/lwi/conf/overrides/logging.properties
    rcDir=$?
    if [ $rcDir -eq 0 ]; then
      ras_log "Successfully disabled Director Server logging"
    else
      ras_log "Error: unable to disabled Director Server logging"
    fi

}
# minDirectorLogging copies min logging configuration files
function minDirectorLogging() {
    ras_log "Enabling minimal Director Server logging"
    execute_and_log cp /opt/tivoli/tiam/etc/USMIKernel.disable /opt/ibm/director/lwi/conf/overrides/USMIKernel.properties
    execute_and_log cp /opt/tivoli/tiam/etc/directorLogging.min /opt/ibm/director/lwi/conf/overrides/logging.properties
    rcDir=$?
    if [ $rcDir -eq 0 ]; then
      ras_log "Successfully enabled minimal Director Server logging"
    else
      ras_log "Error: unable to enabled minimal Director Server logging"
    fi
}
# midDirectorLogging copies mid logging configuration files
function midDirectorLogging() {
    ras_log "Enabling mid Director Server logging"
    execute_and_log cp /opt/tivoli/tiam/etc/USMIKernel.disable /opt/ibm/director/lwi/conf/overrides/USMIKernel.properties
    execute_and_log cp /opt/tivoli/tiam/etc/directorLogging.mid /opt/ibm/director/lwi/conf/overrides/logging.properties
    rcDir=$?
    if [ $rcDir -eq 0 ]; then
      ras_log "Successfully enabled mid Director Server logging"
    else
      ras_log "Error: unable to enabled mid Director Server logging"
    fi
}
# maxDirectorLogging copies maximum logging configuration files
function maxDirectorLogging() {
    ras_log "Enabling maximum Director Server logging"
    execute_and_log cp /opt/tivoli/tiam/etc/USMIKernel.enable /opt/ibm/director/lwi/conf/overrides/USMIKernel.properties
    execute_and_log cp /opt/tivoli/tiam/etc/directorLogging.max /opt/ibm/director/lwi/conf/overrides/logging.properties
    rcDir=$?
    if [ $rcDir -eq 0 ]; then
      ras_log "Successfully enabled maximum Director Server logging"
    else
      ras_log "Error: unable to enabled maximum Director Server logging"
    fi

}

# enableISSLogging will create the  /opt/ibm/director/ISS/ISS_DEBUGON.dat
function enableISSLogging() {
   ras_log "Enabling ISS logging"
   if [ -e ${ISS_DEBUG_FILE} ]; then
      rcISS=0
   else
      execute_and_log touch ${ISS_DEBUG_FILE}
      rcISS=$?
      doRestart=1
   fi

   if [ $rcISS -eq 0 ]; then
      ras_log "Successfully enabled ISS logging"
   else
      ras_log "Error: unable to enabled ISS logging"
   fi
}

# disableISSLogging will remove the  /opt/ibm/director/ISS/ISS_DEBUGON.dat
function disableISSLogging() {
   ras_log "Disabling ISS logging"
   if [ -e ${ISS_DEBUG_FILE} ]; then
      execute_and_log rm ${ISS_DEBUG_FILE}
      rcISS=$?
      doRestart=1
   else
      rcISS=0
   fi
   if [ $rcISS -eq 0 ]; then
      ras_log "Successfully disabled ISS logging"
   else
      ras_log "Error: unable to disabled ISS logging"
   fi
}

# log the actions taken by this script
create_new_log_file setDSTrace

# Validate the trace levels as one of the following [min,mid,max,disable]
case "${LEVEL}" in
   "min" | "mid" | "max" | "disable" )
     ras_log "Valid trace level found : ${LEVEL}";;
   "" ) ras_log -v "Error, trace level not provided $USAGE"
      exit 1;;
   *) ras_log -v "Error, invalid trace level ${LEVEL} $USAGE"
      exit 1;;
esac

if [ "${LEVEL}" == "min" ]; then
    ras_log -v "Setting trace level to ${LEVEL}"
    minDirectorLogging
    enableISSLogging
elif [ "${LEVEL}" == "mid" ]; then
    ras_log -v "Setting trace level to ${LEVEL}"
    midDirectorLogging
    enableISSLogging
elif [ "${LEVEL}" == "max" ]; then
    ras_log -v "Setting trace level to ${LEVEL}"
    maxDirectorLogging
    enableISSLogging
elif [ "${LEVEL}" == "disable" ]; then
    ras_log -v "Setting trace level to ${LEVEL}"
    disableDirectorLogging
    disableISSLogging
fi

# Check if we need to restart Director to pick up the changes or if we can just refresh logging settings
if [ $doRestart -eq 1 ]; then
    # Stop director
    stop_director
    # Re-start director to complete update
    start_director
    rc=$?
else
    # Refresh the logging settings
    execute_and_log /opt/ibm/director/lwi/bin/lwilog.sh -refresh
    rc=$?
fi

# Check return code
if [ $rc -eq 0 -a $rcISS -eq 0 -a $rcDir -eq 0 ]; then
  ras_log -v "Trace level set to ${LEVEL}."
else
  ras_log -v "Error: setting trace level to ${LEVEL}, please see log file for more information."
fi

exit $rc
