#!/bin/bash
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Filename: setGPFSTrace.sh
#
# Component: RAS
# Author   : Neeta Garimella
#
# Description:
#    This file contains a bash script for setting the trace level of the GPFS
#    It maps the GPFS trace capabilities into min/mid/max/disable, to match the
#    IA Trace Config utility design.
#
# IMPORTANT
#
# GPFS tracing is contolled from a single node for the entire cluster. This
# script must be run only on one cluster node to manage GPFS tracing at the
# cluster level. This is because -N <nodename> option on mmtracectl commands
#  does not work as documented. This seems to be a bug in the GPFS code which
# is being pursued with the team.
#
#  Usage:
#      setGPFSTrace.sh min|mid|max|disable
#      min: basic tracing only
#      mid: same as min
#      max: full tracing
#      disable: no trace
#
#******************************************************************************
# Change history
#   Modifier      Date         CQ Change       Description
#   -----------------------------------------------------------------------------------
#   Brad Trubey   07/29/2009   TIAMC00007817   Change GPFS trace to set "vnode 0" for all
#                                              trace levels to address GPFS memory overruns.
#   Brad Trubey   09/08/2009   TIAMC00006538   Add cNFS debugging support
#   Brad Trubey   09/08/2009   TIAMC00008901   vnode changes requested by Neeta in support of GPFS defect 718560
#   Brad Trubey   09/29/2009   TIAMC00009595   Explicitly clear values when disable is called
#   Brad Trubey   11/02/2009   TIAMC00010460   Move assertOnStructureError from Trace Config to create_gpfs
#   Brad Trubey   01/20/2010   TIAMC00011539   Set mmtracectl to off as part of disable call
#   Brad Trubey   05/27/2010   TIAMC00013851   Change default settings for GPFS trace based on 5/26/2010 note
#******************************************************************************

USAGE="Usage: $0 trace_level\n   trace_level values: min|mid|max|disable"
LEVEL="$1"
rc=0

# include ras_functions
. /opt/tivoli/tiam/bin/ras_functions
rc_reset

# log the actions taken by this script
create_new_log_file setGPFSTrace

# Validate the trace levels as one of the following [min,mid,max,disable]
case "${LEVEL}" in
   "min" | "mid" | "max" | "disable" )
     ras_log "Valid trace level found : ${LEVEL}";;
   "" ) ras_log -v "Error, trace level not provided $USAGE"
      exit 1;;
   *) ras_log -v "Error, invalid trace level ${LEVEL} $USAGE"
      exit 1;;
esac

ras_log "Setting trace level to ${LEVEL}"


# First clear any prior trace settigns to start fresh
# 1. Stop the current trace
# 2. Reset all settings from previous trace levels
#
ras_log -v "Stopping GPFS trace."
mmtracectl --stop
rc=$?
if [[ $rc -ne 0 ]]; then
   ras_log -v "Unable to stop gpfs tracing, mmtracectl --stop command failed"
   exit 1
fi
ras_log -v "Successfully stopped GPFS tracing. "

ras_log -v "Clearing GPFS tracing from any prior setting"
mmchconfig trace="all 0"  -i
mmchconfig cnfsdebug=0 -i
rc=$?
if [[ $rc -ne 0 ]]; then
     ras_log -v "Unable to configure trace setting, mmchconfig command failed"
     exit 1
fi
ras_log -v "Successfully cleared GPFS trace settings."


#-----------------------------------------------------------------------------
#
# -trace-recycle. This parameter controls how GPFS recycle traces.
# Setting it to 'globaly' will force trace to be cut at time of GPFS internal
# errors.
# MIN/MID: 'global'
# MAX: 'globalOnShutdown' does not require traces to restarted on reboot.
#      For simplicity call components tracing is set to pre-reboot levels.
#
# trace-file-size: This is a matter of disk spaces.
# MIN: 256M
# MID: 500M
# MAX: 800M
#
# trace-gen-dump:  Rationale from Chen Luo for enabling this setting:
# When traceGenDump is set to true, GPFS will take internaldump when it cut traces.
# This gives me both traces and dumps at the same time. In some IA defects, I have
# only the trace without corresponding internaldump which make debugging difficult.
#
# trace class: This controls trace level. Increasing trace level will only
# decrease the time coverage of the trace file.
# MIN/MID: use the default levels:
#      "all 4 tm 2 thread 1 mutex 1 vnode 2 ksvfs 3 klockl 2 io 3 pgalloc 1 mb 1 lock 2 fsck 3"
# MAX: "all 4 mutex 1"
#-----------------------------------------------------------------------------


TRACE_FILE_SIZE="0"
TRACE_RECYCLE="global"
TRACE_GENDUMP="no"
TRACE_CNFSDEBUG="2"
TRACE_CLASS1=""

DS_DUMPONPANIC=""
DS_DUMP="yes"


case "${LEVEL}" in

   "disable")
    ras_log -v "Setting GPFS trace level to disable..."
    TRACE_FILE_SIZE="0M"
    TRACE_RECYCLE=""
    TRACE_GENDUMP=""
    TRACE_CNFSDEBUG=""
    TRACE_CLASS1="all 0"
    ;;

   "min")
    ras_log -v "Setting GPFS trace level to minimum level..."
    TRACE_FILE_SIZE="999M"
    TRACE_RECYCLE="global"
    TRACE_GENDUMP="yes"
    TRACE_CNFSDEBUG="2"
    TRACE_CLASS1="all 4 tm 2 thread 1 mutex 1 vnode 2 ksvfs 3 klockl 2 io 3 pgalloc 1 mb 1 lock 2 fsck 3"
    ;;

   "mid")
    ras_log -v "Setting GPFS trace level to medium level..."
    TRACE_FILE_SIZE="999M"
    TRACE_RECYCLE="global"
    TRACE_GENDUMP="yes"
    TRACE_CNFSDEBUG="2"
    TRACE_CLASS1="all 4 tm 2 thread 1 mutex 1 vnode 2 ksvfs 3 klockl 2 io 3 pgalloc 1 mb 1 lock 2 fsck 3"
    ;;

   "max")
    ras_log -v "Setting GPFS trace level to maximum level..."
    TRACE_FILE_SIZE="999M"
    TRACE_RECYCLE="global"
    TRACE_GENDUMP="yes"
    TRACE_CNFSDEBUG="2"
    TRACE_CLASS1="all 4 mutex 1"
    ;;

   *)
    # we checked this before.. should never come here.
     ras_log -v "Error, invalid trace level ${LEVEL} $USAGE"
     exit 1;;
esac

ras_log -v "Trace settings: traceRecycle=$TRACE_RECYCLE  traceFileSize=$TRACE_FILE_SIZE  traceGenDump=$TRACE_GENDUMP  trace=\"$TRACE_CLASS1\" "


#
# Use GPFS commands now to configure above settings.
#
echo Running: mmchconfig dataStructureDumpOnSGPanic=$DS_DUMPONPANIC,dataStructureDump=$DS_DUMP,traceRecycle=$TRACE_RECYCLE,traceFileSize=$TRACE_FILE_SIZE,traceGenDump=$TRACE_GENDUMP,cnfsdebug=$TRACE_CNFSDEBUG -i
mmchconfig dataStructureDumpOnSGPanic=$DS_DUMPONPANIC,dataStructureDump=$DS_DUMP,traceRecycle=$TRACE_RECYCLE,traceFileSize=$TRACE_FILE_SIZE,traceGenDump=$TRACE_GENDUMP,cnfsdebug=$TRACE_CNFSDEBUG -i
rc=$?
if [[ $rc -ne 0 ]]; then
  ras_log -v "Unable to update GPFS trace setting, mmchconfig command failed"
  exit 1
fi
ras_log -v "Successfully updated GPFS trace configuration."


#Set trace class
echo Running: mmchconfig trace=\"${TRACE_CLASS1}\"  -i
mmchconfig trace="${TRACE_CLASS1}"  -i
rc=$?
if [[ $rc -ne 0 ]]; then
  ras_log -v "Unable to set trace classes, mmchconfig command failed"
  exit 1
fi
ras_log -v "Successfully updated GPFS trace classes."

#
# If "disable" is the new level, then exit without starting mmtracectl
#
if [ "${LEVEL}" == "disable" ]
then
   ras_log -v "Setting GPFS trace to disable level..."
   mmtracectl --off
   ras_log -v "GPFS tracing has now been disabled."
   exit 0
fi

ras_log -v "Restarting GPFS trace."
mmtracectl  --start
rc=$?
if [[ $rc -ne 0 ]]; then
  ras_log -v "Unable to start the tracing, mmtracectl --start command failed"
  exit 1
fi
ras_log -v "Successfully started GPFS trace."

exit $rc
