#!/bin/bash
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Filename: setHSMTrace.sh
#
# Component: RAS
#
# Description:
#    This file contains a bash script for setting the trace level of the HSM client.
#    It maps the HSM client trace capabilities into min/mid/max/disable, to match the
#    IA Trace Config utility design.
#
#  Usage:
#      setHSMTrace.sh min|mid|max|disable
#      min: basic tracing only
#      mid: same as min
#      max: full tracing
#      disable: no trace
#
# *************************************************************************************

DSM_OPT_FILE="/opt/tivoli/tsm/client/ba/bin/dsm.opt"
USAGE="Usage: $0 trace_level\n   trace_level values: min|mid|max|disable"
LEVEL="$1"
rc=0

# include ras_functions
. /opt/tivoli/tiam/bin/ras_functions
rc_reset

# log the actions taken by this script
create_new_log_file setHSMTrace

# Validate the trace levels as one of the following [min,mid,max,disable]
case "${LEVEL}" in
   "min" | "mid" | "max" | "disable" )
     ras_log "Valid trace level found : ${LEVEL}";;
   "" ) ras_log -v "Error, trace level not provided $USAGE"
      exit 1;;
   *) ras_log -v "Error, invalid trace level ${LEVEL} $USAGE"
      exit 1;;
esac

ras_log "Setting trace level to ${LEVEL}"


# Copy appropriate dsm.opt.<level> template file to dsm.opt, and
# Enable logrotate to manage size/versions of trace file to keep
if [ "${LEVEL}" == "min" ]; then
    ras_log "Setting trace level to ${LEVEL}"

    ras_log -v "Copying dsm.opt.${LEVEL} to dsm.opt..."
    execute_and_log cp /opt/tivoli/tiam/etc/dsm.opt.min ${DSM_OPT_FILE}

    ras_log -v "Enabling logrotate for HSM trace_tsm.out..."
    execute_and_log /opt/tivoli/tiam/bin/ia_hsm_logrotate.py -c enable

elif [ "${LEVEL}" == "mid" ]; then
    ras_log "Setting trace level to ${LEVEL}"

    ras_log -v "Copying dsm.opt.${LEVEL} to dsm.opt..."
    execute_and_log cp /opt/tivoli/tiam/etc/dsm.opt.mid ${DSM_OPT_FILE}

    ras_log -v "Enabling logrotate for HSM trace_tsm.out..."
    execute_and_log /opt/tivoli/tiam/bin/ia_hsm_logrotate.py -c enable

elif [ "${LEVEL}" == "max" ]; then
    ras_log "Setting trace level to ${LEVEL}"

    ras_log -v "Copying dsm.opt.${LEVEL} to dsm.opt..."
    execute_and_log cp /opt/tivoli/tiam/etc/dsm.opt.max ${DSM_OPT_FILE}

    ras_log -v "Enabling logrotate for HSM trace_tsm.out..."
    execute_and_log /opt/tivoli/tiam/bin/ia_hsm_logrotate.py -c enable

elif [ "${LEVEL}" == "disable" ]; then
    ras_log "Setting trace level to ${LEVEL}"

    ras_log -v "Copying dsm.opt.${LEVEL} to dsm.opt..."
    execute_and_log cp /opt/tivoli/tiam/etc/dsm.opt.disable ${DSM_OPT_FILE}

    ras_log -v "Disabling logrotate for HSM trace_tsm.out..."
    execute_and_log /opt/tivoli/tiam/bin/ia_hsm_logrotate.py -c disable
fi

ras_log -v "Stopping the HSM Daemons."

# No longer calling dsmmigfs disablefailover since dsmmigfs stop handles it
# execute_and_log /opt/tivoli/tsm/client/hsm/bin/dsmmigfs disablefailover
execute_and_log /opt/tivoli/tsm/client/hsm/bin/dsmmigfs stop

# killall -w will wait for all processes to stop before returning control here
# this removes the need for the sleep
ras_log -v "killing the dsmwatchd process"
execute_and_log killall -s 15 -w dsmwatchd

# ...dsmwatchd will be respawned...
ras_log -v "Restarting the HSM Daemons."

execute_and_log /opt/tivoli/tsm/client/hsm/bin/dsmmigfs start

# No longer calling sleep 10 or dsmmigfs enablefailover since dsmmigfs start handles it
# sleep 10
# execute_and_log /opt/tivoli/tsm/client/hsm/bin/dsmmigfs enablefailover


# Check return code
if [[ $rc -ne 0 ]]; then
  ras_log -v "Unable to set trace level to ${LEVEL}, please see log file for more information."
else
  ras_log -v "Trace level set to ${LEVEL}."
fi

exit $rc
