#!/bin/bash
# *************************************************************************************
# * Licensed Materials - Property of IBM
# * IBM Information Archive 2.1.0.0.efix_1
# *
# * (C) Copyright IBM Corp. 2008, 2009, 2010. All Rights Reserved
# *
# * US Government Users Restricted Rights - Use, duplication or
# * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# *
# * All files within this archive are protected by IBM's copyright.
# *************************************************************************************
#
# Filename: setRSMTrace.sh
#
# Component: RAS
#
# Description:
#    This file contains a bash script for setting the trace level of the RSM application.
#    It maps the RSM application trace capabilities into min/mid/max/disable, to match the IA Trace
#    Config utility design.
#
#  Usage:
#      setRSMTrace.sh min|mid|max|disable
#      min: basic tracing only
#      mid: same as min
#      max: full tracing
#      disable: no trace
#
# Change history
#   Modifier        Date         CQ Change    Description
#   ---------------------------------------------------------------------------
#   Brad Trubey     06/02/2009                Initial creation
#   Rob Jose        06/24/2009   6552         Added copy of config files and restart of rsm server
#   Brad Trubey     09/02/2009   8346         Add message to inform user that Remote Access to RSM
#                                             is disabled by setting RSM trace level.
# *************************************************************************************

USAGE="\nUsage: $0 trace_level\n   trace_level values: min|mid|max|disable"
LEVEL="$1"
rc=0
CONFIG_TO_DIR="/etc/rsm/ibm"
CONFIG_FROM_DIR="/opt/tivoli/tiam/etc"
CONFIG_FILE="rsm-server_swat.conf"

# include ras_functions
. /opt/tivoli/tiam/bin/ras_functions
rc_reset

# log the actions taken by this script
create_new_log_file setRSMTrace

# Validate the trace levels as one of the following [min,mid,max,disable]
case "${LEVEL}" in
   "min" | "mid" | "max" | "disable" )
     ras_log "Valid trace level found : ${LEVEL}";;
   "" ) ras_log -v "Error, trace level not provided $USAGE"
      exit 1;;
   *) ras_log -v "Error, invalid trace level ${LEVEL} $USAGE"
      exit 1;;
esac

ras_log "Setting trace level to ${LEVEL}"

if [ "${LEVEL}" == "disable" ]; then
   # to disable tracing, we just need to remove the configuration file.  Use the -f option in case the file has already been removed
    rm -f ${CONFIG_TO_DIR}/${CONFIG_FILE}
else
   # copy the correct configuration file
    cp ${CONFIG_FROM_DIR}/${CONFIG_FILE}.${LEVEL} ${CONFIG_TO_DIR}/${CONFIG_FILE}
fi

rc=$?
# Check return code on copy or remove command
if [[ $rc -ne 0 ]]; then
  ras_log -v "Unable to set trace level to ${LEVEL}, unable to copy or remove trace config file ${CONFIG_TO_DIR}/${CONFIG_FILE}."
  exit 1
else
  ras_log -v "Trace level set to ${LEVEL}."
fi

# restart the rsm server
ras_log -v "Restarting RSM server so changes take affect."
/rsm/rsm-start

rc=$?
# Check return code on rsm restart
if [[ $rc -ne 0 ]]; then
  ras_log -v "Unable to (re)start rsm server, please see log files under /var/log/rsm for more information."
else
  ras_log -v "RSM started successfully."
  ras_log -v "Warning: Restarting the RSM Server disables Remote Access.  If you need remote access to the RSM"
  ras_log -v "Server, then you will need to reenable it through the RSM web interface."
fi

exit $rc
