#! /usr/bin/env python

########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Solr Server 
# *
# * FILE NAME:       ssm_manageConfig.py
# *
# * DESCRIPTION:     Manage configuration of the solr server for a collection. 
# *                  Options/Functions: -doCreate, -doStart, -doUpdate
# *                  doCreate - called from GUI during collection creation
# *                  doStart  - called from wlm before startup of solr
# *                  doUpdate - called from wlm after update of collection.xml
# *                             handles dynamic configuration
# *
# * AUTHOR:          K.W.Brannon
# *                  IBM Almaden Research Center
# *                  kbrannon@us.ibm.com
# *
# *
# ****************************************************************************
# */

import sys, os, subprocess
import getopt
from ssm_defines   import *
from ssm_common    import *
from ssm_config_common import *
from time import *
OP_START = "start"
OP_CREATE = "create"
OP_UPDATE = "update"
global logger, ia_tracelvl, ia_loglvl

def usage(logger, errmsg):
    umsg =       "\nusages: "  +sys.argv[0] +" -m collectionMountPoint --doStart\n"
    umsg = umsg +"        "  +sys.argv[0] +" -m collectionMountPoint --doUpdate\n"
    umsg = umsg +"        "  +sys.argv[0] +" -m collectionMountPoint -l indexlevel --doCreate\n"
    if (logger != None):
        logger.LogMsg(SSM_LOGMSG_USAGE, (errmsg,umsg))
        logger.TraceMsg("Usage Error msg: %s" %(errmsg))
        logger.TraceMsg("Usage: %s" %(umsg))
    else:
        sys.stdout.write("Error: " +errmsg)
        sys.stdout.write(umsg)
    sys.exit(1)

def genLoggingPropertiesFile():
    #Use this during start of the Solr server
    #For update call jsp to update logging level
    rc, guid = ssm_run_cmdStdout(logger, SSM_GUID_CMD)
    guid = guid.strip()
    propdir = str("%s%s/logging/%s" %(mountpt, SSM_SOLR_INDEX_DIR_RMNT, guid))
    cmd = str("mkdir -p %s" %(propdir))
    rc,stdout = ssm_run_cmdStdout(logger, cmd)
    if rc == 0:
        trace_level = SSM_SOLR_LOG_LEVELS[int(ia_tracelvl)]
        log_level = SSM_SOLR_LOG_LEVELS[int(ia_loglvl)]
        trace_file = str("%s/CTJID/%s%s/%s/ffdc/solrserver_trace.txt" %(SSM_LOG_PATH,SSM_LOG_IN,mountpt,guid))
        log_file = str("%s/CTJID/%s%s/%s/logs/solrserver_log.txt" %(SSM_LOG_PATH,SSM_LOG_IN,mountpt,guid))
        propfile = str("%s/%s" %(propdir,SSM_SOLR_LOG_PROPERTIES_FILE))
        c = str("#DO NOT MODIFY THIS FILE\n")
        c = str("%s.level = INFO\n" %c)
        c = str("%shandlers = java.util.logging.ConsoleHandler, java.util.logging.FileHandler\n" %c)
        c = str("%sjava.util.logging.FileHandler.formatter = java.util.logging.SimpleFormatter\n" %c)
        c = str("%sjava.util.logging.FileHandler.pattern = %s\n" %(c,trace_file))
        c = str("%sjava.util.logging.FileHandler.append = true\n" %c)
        c = str("%sjava.util.logging.FileHandler.count = 3\n" %c)
        c = str("%sjava.util.logging.FileHandler.limit = 30000000\n" %c)
        c = str("%sjava.util.logging.FileHandler.level = %s\n" %(c,trace_level))
        c = str("%sjava.util.logging.ConsoleHandler.formatter = java.util.logging.SimpleFormatter\n" %c)
        c = str("%sjava.util.logging.ConsoleHandler.pattern = %s\n" %(c,log_file))
        c = str("%sjava.util.logging.ConsoleHandler.append = true\n" %c)
        c = str("%sjava.util.logging.ConsoleHandler.count = 2\n" %c)
        c = str("%sjava.util.logging.ConsoleHandler.limit = 30000000\n" %c)
        c = str("%sjava.util.logging.ConsoleHandler.level = %s\n" %(c,log_level))
    else:
        pmsg = str("Could not create %s because could not mkdir %s" 
                   %(propfile,propdir))
        logger.TraceMsg(pmsg)
        logger.LogMsg(SSM_LOGMSG_SYS_ERR, (pmsg))
        return 1
    try:
        pf = open(propfile, "w")
        pf.write(c)
        pf.flush()
        pf.close()
        return 0
    except:
        pmsg = str("Could not write %s"  %propfile)
        logger.TraceMsg(pmsg)
        logger.LogMsg(SSM_LOGMSG_SYS_ERR, (pmsg))
        return 1
       
mountpt = None
indexlvl = None
operation = None
logger = None
try:
    opts, args = getopt.getopt(sys.argv[1:],"m:l:h",['doStart','doUpdate','doCreate'])
except getopt.GetoptError, err:
    pmsg = str(err)
    usage(logger, pmsg)
for opt, arg in opts:
    if opt in ("-m"):
        mountpt = arg.rstrip('/')
    elif opt in ("-l"):
        indexlvl = arg
    elif opt in ("--doStart"):
        operation = OP_START
    elif opt in ("--doUpdate"):
        operation = OP_UPDATE
    elif opt in ("--doCreate"):
        operation = OP_CREATE
    elif opt in ("-h"):
        usage(logger, "")
    else:
        pmsg = "unhandled option"
        usage(logger, pmsg)

#check that mountpt is a directory
if mountpt == None or not os.path.isdir( mountpt):
    pmsg = str("collectionMountPoint, %s must be a fully qualified directory"
               %(str(mountpt)))
    usage(logger, pmsg)

cdotxml = str("%s%s" %(mountpt, SSM_CDOTXML_RMNT))
if os.path.isfile(cdotxml):
    parsed_cdotxml = ssm_nl_parse_xmlfile(cdotxml)
    if parsed_cdotxml == None:
        msg = str("Error in %s, parsing %s" %(sys.argv[0],cdotxml))
        sys.stdout.write(msg)
        sys.exit(1)
    loglevel = ssm_nl_get_loglevel(parsed_cdotxml,cdotxml)
    if loglevel == None:
        loglevel = SSM_LOG_DEFAULT_LOGLEVEL
    tracelevel = ssm_nl_get_tracelevel(parsed_cdotxml,cdotxml)
    if tracelevel == None:
        tracelevel = SSM_LOG_DEFAULT_TRACELEVEL
else:
    parsed_cdotxml = None
    loglevel = SSM_LOG_DEFAULT_LOGLEVEL
    tracelevel = SSM_LOG_DEFAULT_TRACELEVEL

logger = ssm_nl_get_logger(SSM_LOG_SSM, mountpt, loglevel, tracelevel)
if (logger != None):
    logger.TraceMsg("Got logger in %s" %(sys.argv[0]))
else:
    msg = str("Error in %s, cannot get logger" %(sys.argv[0]))
    sys.stdout.write(msg)
    sys.exit(1)
logger.TraceMsg("The cdotxml used for parsing was: %s" %(cdotxml))
logger.TraceMsg("os.path.isfile(cdotxml) returned: %s" %(str(os.path.isfile(cdotxml))))

euid = os.geteuid()
if euid != SSM_ROOTUID:
    pmsg = "This script must be run as root"
    usage(logger, pmsg)

if operation == None:
    pmsg = "Must specify one of doStart, doUpdate, doCreate"
    usage(logger, pmsg)
#For start or update, indexlevel must be in collection.xml
if operation == OP_START or operation == OP_UPDATE:
    if indexlvl == None:
        indexlvl = ssm_get_valueFromXmlfile(logger, SSM_C_EL_INDEXLEVEL, 
                                            parsed_cdotxml, cdotxml)
        if not indexlvl in SSM_INDEX_LEVEL:
            pmsg = str("bad indexlevel %s specified in %s" %(str(indexlvl),cdotxml))
            logger.LogMsg(SSM_LOGMSG_SYS_ERR, (pmsg))
            sys.exit(1)
    else:
        pmsg = "indexlevel should not be specified for doStart or doUpdate"
        usage(logger, pmsg)
else:        
#For create, indexlevel is not yet available in collection.xml
    if indexlvl == None:
        pmsg = "indexlevel must be specified for doCreate"
        usage(logger, pmsg) 
    else:
        if SSM_INDEX_LEVEL[indexlvl] == 'no_index':
            pmsg = "bad indexlevel specified for doCreate: " +indexlvl
            usage(logger, pmsg)
        if not indexlvl in SSM_INDEX_LEVEL:
            pmsg = "bad indexlevel specified for doCreate: " +indexlvl
            usage(logger, pmsg)
            
msg = str("%s called with operation %s indexlevel %s for collection %s"
          %(sys.argv[0], str(operation), str(indexlvl), mountpt))
logger.TraceMsg(msg)

if operation == OP_START or operation == OP_UPDATE:
    logger.TraceMsg("getting jettyip and jettyport from %s" %(cdotxml))
    jettyname = ssm_get_solrValueFromCdotxml(logger, 
                                             SSM_SOLR_EL_VIRTUAL_JETTYIP,
                                             parsed_cdotxml, cdotxml)
    if (jettyname != None):
        jettyip = ssm_get_ipnumberFromName(logger, jettyname)
        if (jettyip != None):
            jettyport = ssm_get_solrValueFromCdotxml(logger, 
                                                     SSM_SOLR_EL_JETTYPORT,
                                                     parsed_cdotxml, cdotxml)
            if (jettyport == None):
                logger.LogMsg(SSM_LOGMSG_PARSE_EL, (cdotxml, 
                                                    SSM_SOLR_EL_JETTYPORT))
                sys.exit(1)
        else:
            msg = str("Cannot get ipnumber for ipname: %s" %(str(jettyname)))
            logger.LogMsg(SSM_LOGMSG_SYS_ERR, (msg))
            sys.exit(1)
    else:
        if SSM_INDEX_LEVEL[indexlvl] == 'no_index':
            #Never having solr element is OK for indexlevel 3
            msg = str("%s doUpdate ssm_init_indexing_config nothing to do for mountpt:%s"
                      %(sys.argv[0], mountpt))
            logger.TraceMsg(msg)
            sys.exit(0)
        else:
            logger.LogMsg(SSM_LOGMSG_PARSE_EL, (cdotxml,
                                                SSM_SOLR_EL_VIRTUAL_JETTYIP))
            sys.exit(1)
    logger.TraceMsg("got jettyip:%s jettyport:%s for collection %s"
                    %(str(jettyip), str(jettyport), str(mountpt)))
    ia_tracelvl = ssm_get_valueFromXmlfile(logger, SSM_C_EL_TRACELEVEL, 
                                            parsed_cdotxml, cdotxml)
    ia_loglvl = ssm_get_valueFromXmlfile(logger, SSM_C_EL_LOGLEVEL, 
                                            parsed_cdotxml, cdotxml)

if operation == OP_CREATE:
    ret = ssm_create_solrConfig(logger, mountpt, indexlvl,
                                parsed_cdotxml, cdotxml)
    if ret == SSM_CREATE_SUCCESS:
        sys.exit(0)
    elif ret == SSM_CREATE_LOCKED:
        msg = str("%s: mountpt:%s indexlevel:%s doCreate, system error create of solr config already in progress"
                  %(sys.argv[0],mountpt,str(indexlvl)))
        logger.TraceMsg(msg)
        logger.LogMsg(SSM_LOGMSG_SYS_ERR, (msg))
        sys.exit(1)
    elif ret == SSM_CREATE_ERROR:
        msg = str("%s: mountpt:%s indexlevel:%s doCreate, ssm_create_solrConfig failure error creating solr config"
                  %(sys.argv[0],mountpt,str(indexlvl)))
        logger.TraceMsg(msg)
        sys.exit(1)
    else:
        msg = str("%s: mountpt:%s indexlevel:%s doCreate, ssm_create_solrConfig failure"
                  %(sys.argv[0],mountpt,str(indexlvl)))
        logger.TraceMsg(msg)
        logger.LogMsg(SSM_LOGMSG_SYS_ERR, (msg))
        sys.exit(1)

elif operation == OP_START:
    if os.path.isdir(mountpt +SSM_SOLR_INDEX_DIR_RMNT):
        # index/solr directory present - initial solr config OK
        # create the logging properties file
        cmd = str("mkdir -p %s%s" %(mountpt, SSM_SOLR_INDEX_DIR_RMNT))
        rc, stdout = ssm_run_cmdStdout(logger, cmd)
        if rc != 0:
            pmsg("Could not create %s%s" %(mountpt, SSM_SOLR_INDEX_DIR_RMNT))
            logger.TraceMsg(pmsg)
            logger.LogMsg(SSM_LOGMSG_SYS_ERR, (pmsg))
            sys.exit(1)
        rc = genLoggingPropertiesFile()
        if rc != 0:
            pmsg("Could not create logging properties file for %s" %mountpt)
            logger.TraceMsg(pmsg)
            logger.LogMsg(SSM_LOGMSG_SYS_ERR, (pmsg))
            sys.exit(1)
        #remove any existing lucene lock files
        #Let solr handle this via config parameter in solrconfig.xml
        #cmd = str("rm -f  %s%s/core*/dataDir/index/lucene*write.lock" %(mountpt, SSM_SOLR_CORES_DIR_RMNT))
        #rc, stdout = ssm_run_cmdStdout(logger, cmd)
        # cleanup any solrNotHealthy signal since are starting solr
        rc = ssm_cleanup_solrNotHealthy_signal(logger, mountpt)
         
        func = "ssm_update_solrConfig"
        createnewcore = False
        starting = True
        ret = ssm_update_solrConfig(logger, mountpt, indexlvl, jettyip,
                                    jettyport, createnewcore, starting)
        if ret == SSM_UPDATE_SUCCESS:
            sys.exit(0)
        elif ret == SSM_UPDATE_LOCKED:
            #system error
            msg = str("Should never get SSM_UPDATE_LOCKED on start for collection %s"
                      %(mountpt))
            logger.TraceMsg(msg)
            logger.LogMsg(SSM_LOGMSG_SYS_ERR, (msg))
            sys.exit(1)
        elif ret == SSM_UPDATE_ERROR:
            #system error
            msg = str("Unknown error getting update lock for collection %s"
                      %(mountpt))
            logger.TraceMsg(msg)
            logger.LogMsg(SSM_LOGMSG_SYS_ERR, (msg))
            sys.exit(1)
        else:
            msg = str("Unknown error getting update lock for collection %s"
                      %(mountpt))
            logger.TraceMsg(msg)
            logger.LogMsg(SSM_LOGMSG_SYS_ERR, (msg))
            sys.exit(1)
    else:
        #No index/solr directory means initial solr config is missing
        if SSM_INDEX_LEVEL[indexlvl] != 'no_index':
            func = ssm_create_solr_config
            ret = ssm_create_solr_config(logger, mountpt, indexlvl)
            if ret == SSM_CREATE_SUCCESS:
                msg = str("%s, operation:%s mountpt:%s indexlevel:%s call to %s \
                          rc:%s success"
                          %(sys.argv[0],operation,mountpt,str(indexlvl),func,ret))
                logger.TraceMsg(msg)
                sys.exit(0)
            elif ret == SSM_CREATE_LOCKED:
                msg = str("%s, operation:%s mountpt:%s indexlevel:%s call to %s \
                          rc:%s failure, create already in progress"
                          %(sys.argv[0],operation,mountpt,str(indexlvl),func,ret))
                logger.TraceMsg(msg)
                sys.exit(1)
            elif ret == SSM_CREATE_ERROR:
                msg = str("%s, operation:%s mountpt:%s indexlevel:%s call to %s \
                          rc:%s failure, cannot get lock on create"
                          %(sys.argv[0],operation,mountpt,str(indexlvl),func,ret))
                logger.TraceMsg(msg)
                sys.exit(1)   
            else:
                msg = str("%s, operation:%s mountpt:%s indexlevel:%s call to %s \
                          rc:%s failure"
                          %(sys.argv[0],operation,mountpt,str(indexlvl),func,str(ret)))
                logger.TraceMsg(msg)
                sys.exit(ret)

elif operation == OP_UPDATE:
    #update of indexlevel, metafile may have happened
    #but nothing to do if no index fileset
    starting = False
    if SSM_INDEX_LEVEL[indexlvl] != 'no_index':
        if os.path.isdir(mountpt +SSM_SOLR_CONFIG_RMNT):
            #update the config since the index fileset exists
            createnewcore = False
            ret = ssm_update_solrConfig(logger, mountpt, indexlvl, jettyip,
                                        jettyport, createnewcore, starting)
            if ret == SSM_UPDATE_SUCCESS:
                sys.exit(0)
            elif ret == SSM_UPDATE_LOCKED:
                #need to wait for lock
                waited = 0
                n = 0
                while n < SSM_LOCK_WAIT_MAX_NUM:
                    logger.TraceMsg("waiting %s sec to get lock" %SSM_LOCK_WAIT_TIME)
                    sleep(SSM_LOCK_WAIT_TIME)
                    ret = ssm_update_solrConfig(logger, mountpt, indexlvl,
                                                jettyip, jettyport,
                                                createnewcore, starting)
                    if ret == SSM_UPDATE_SUCCESS:
                        sys.exit(0)
                    else:
                        n = n + 1
                        waited = waited + SSM_LOCK_WAIT_TIME
                msg = str("Waited %s sec for the update lock on collection %s, terminating"
                          %(str(waited), mountpt))
                logger.TraceMsg(msg)
                logger.LogMsg(SSM_LOGMSG_LOGWAIT, (mountpt,operation))
                sys.exit(1)
        else:
            #need to create the index config directories
            ret = ssm_create_solrConfig(logger, mountpt, indexlvl,
                                        parsed_cdotxml, cdotxml)
            if ret == SSM_CREATE_SUCCESS:
                sys.exit(0)
            else:
                #no one else should be going after this lock
                msg = str("%s doUpdate create index config failed for mountpt:%s"
                          %(sys.argv[0], mountpt))
                logger.TraceMsg(msg)
                logger.LogMsg(SSM_LOGMSG_SYS_ERR, (msg))
                sys.exit(1)
    else:
        if os.path.isdir(mountpt +SSM_SOLR_CONFIG_RMNT):
            #could have deleted fields e.g. even tho indexlevel is 3
            createnewcore = False
            starting = False
            ret = ssm_update_solrConfig(logger, mountpt, indexlvl, jettyip,
                                        jettyport, createnewcore, starting)
            if ret == SSM_UPDATE_SUCCESS:
                sys.exit(0)
            elif ret == SSM_UPDATE_LOCKED:
                #need to wait for lock
                waited = 0
                n = 0
                while n < SSM_LOCK_WAIT_MAX_NUM:
                    logger.TraceMsg("waiting %s sec to get lock" %LOCK_WAIT_TIME)
                    sleep(SSM_LOCK_WAIT_TIME)
                    ret = ssm_update_solrConfig(logger, mountpt, indexlvl,
                                                jettyip, jettyport,
                                                createnewcore, starting)
                    if ret == SSM_UPDATE_SUCCESS:
                        sys.exit(0)
                    else:
                        n = n + 1
                        waited = waited + SSM_LOCK_WAIT_TIME
                msg = str("Waited %s sec for the update lock on collection %s, terminating"
                          %(str(waited), mountpt))
                logger.TraceMsg(msg)
                logger.LogMsg(SSM_LOGMSG_LOGWAIT, (mountpt,operation))
                sys.exit(1)
            elif ret == SSM_UPDATE_ERROR:
                #system error
                msg = str("Unknown error getting update lock for collection %s"
                          %(mountpt))
                logger.TraceMsg(msg)
                logger.LogMsg(SSM_LOGMSG_SYS_ERR, (msg))
                sys.exit(1)
        else:
            #nothing to do, no indexfileset
            msg = str("%s doUpdate ssm_init_indexing_config nothing to do for mountpt:%s"
                      %(sys.argv[0], mountpt))
            logger.TraceMsg(msg)
            sys.exit(0)
