#!/bin/sh
########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Workload Manager
# *
# * FILE NAME:       wlm_controlaudit.sh
# *
# * DESCRIPTION:     Controls the audit component of a FRAnK collection
# *
# * AUTHOR:          Mark A. Smith
# *                  IBM Almaden Research Center
# *                  mark1smi@us.ibm.com
# *
# *                  Michael Richmond
# *                  IBM Almaden Research Center
# *                  mar@almaden.ibm.com
# *
# ****************************************************************************
# */

# Configure shell variables
. `dirname $BASH_SOURCE`/mw_environment.sh

# Read the common error values
. `dirname $BASH_SOURCE`/wlm_controlerrors.sh

# Read the common control functions
. `dirname $BASH_SOURCE`/wlm_controlcommon.sh

AUDIT_NAME=auditd
AUDIT_DIR=/sbin
AUDIT_BIN=/sbin/$AUDIT_NAME

AUDITCTL_NAME=auditctl
AUDITCTL_BIN=/sbin/$AUDITCTL_NAME

BLACKLIST_APPS=(
    "/opt/tivoli/tiam/bin/iam_scan_statistics"
    "/opt/tivoli/tiam/bin/iam_document_manager"
    "/opt/tivoli/tsm/client/hsm/bin/dsmmigrate"
    "/opt/tivoli/tsm/client/ba/bin/dsmadmc"
    "/opt/tivoli/tiam/bin/iam_log_manager"  
    "/opt/tivoli/tsm/db2/bin/db2fm"
    "/sbin/auditd"
)

# where temporary Linux Audit files will be saved
RAWLOGDIR=raw_logs

DEV_MAJOR=0
DEV_MINOR=0

function findMajorMinor()
{
    local mountpoint=$1

    mount_device=`/bin/mount | grep $mountpoint | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 1`

    if [ "$mount_device" == "" ]; then
        return $ERR_COLLECTION_NOT_MOUNTED
    fi

    major_minor=`stat -c '%d' $mountpoint`
    DEV_MAJOR=$[$major_minor & 0xff00]
    DEV_MINOR=$[$major_minor & 0x00ff]

    if [ "$DEV_MAJOR" == "" ]; then
        return $ERR_COLLECTION_NOT_MOUNTED
    fi

    if [ "$DEV_MINOR" == "" ]; then
        return $ERR_COLLECTION_NOT_MOUNTED
    fi

    return 0
}

function setUniqueMachineString()
{
    guid=`/opt/tivoli/tiam/bin/mw_print_unique_nodeid`
    UNIQUEMACHINESTRING=$guid
}

function MonitorNonFsAsync()
{
    mkdir -p $COLLECTIONMNTPOINT/utility/$RAWLOGDIR
    log_filename=$COLLECTIONMNTPOINT/utility/$RAWLOGDIR/$UNIQUEMACHINESTRING.rawlog

    cmd="$AUDITCTL_BIN $@ -F "uid!=$TSMUSER_ID" -L $log_filename"
    $cmd && return 0
    echo Error evaluating: $cmd
    return -1
}

function get_log_filename()
{
    mkdir -p $COLLECTIONMNTPOINT/utility/$RAWLOGDIR 2> /dev/null
    log_filename=$COLLECTIONMNTPOINT/utility/$RAWLOGDIR/$UNIQUEMACHINESTRING.rawlog
    echo $log_filename
}

function MonitorAsync()
{
    log_filename=`get_log_filename`
    cmd="$AUDITCTL_BIN $@ -F devmajor=$DEV_MAJOR -F devminor=$DEV_MINOR \
                 -F "uid!=$TSMUSER_ID" \
                 -L $log_filename"
    $cmd && return 0
    echo Error evaluating: $cmd
    return -1
}

function MonitorSync()
{
    log_filename=`get_log_filename`
    cmd="$AUDITCTL_BIN $@ -F devmajor=$DEV_MAJOR -F devminor=$DEV_MINOR \
                 -F synchronous=1 -F "uid!=$TSMUSER_ID" \
                 -L $log_filename"
    $cmd && return 0
    echo Error evaluating: $cmd
    return -1
}

set_events()
{
	local add_remove_switch=$1
	local prepend_remove_switch=$add_remove_switch

	if [ "$prepend_remove_switch" = "-a" ]
	then prepend_remove_switch="-A"
	fi

	setUniqueMachineString

### Blacklist ###
	for exe in ${BLACKLIST_APPS[@]}
	do
		cmd="$AUDITCTL_BIN $prepend_remove_switch vfs,never -F exe=$exe \
			-F devmajor=$DEV_MAJOR -F devminor=$DEV_MINOR"
		$cmd || { echo "Error evaluating: $cmd"; return -1; }

		cmd="$AUDITCTL_BIN $prepend_remove_switch exit,never -F exe=$exe \
			-F devmajor=$DEV_MAJOR -F devminor=$DEV_MINOR"
		$cmd || { echo "Error evaluating: $cmd"; return -1; }
	done

### Synchronous events ###

	# unlink(87,263) 
	MonitorSync $add_remove_switch vfs,always -S unlink -S unlinkat -x "archive.INdocid"
	MonitorSync $add_remove_switch exit,always -S unlink -S unlinkat -x "archive.INdocid"

	# rename(82,264)
	MonitorSync $add_remove_switch vfs,always -S rename -S renameat
	MonitorSync $add_remove_switch exit,always -S rename -S renameat
	
#### Asynchronous events ###

	# creat(85,2,257) - 64 is O_CREAT in base 10
	MonitorAsync $add_remove_switch vfs,always -S creat 
	MonitorAsync $add_remove_switch vfs,always -S open -S openat -F "a1&64" 
	MonitorAsync $add_remove_switch exit,always -S creat 
	MonitorAsync $add_remove_switch exit,always -S open -S openat -F "a1&64" 

	# write(1,20,18) - ok
#	MonitorAsync $add_remove_switch vfs,always -S write -S writev -S pwrite

	# truncate(76,77,2,257) - 512 is O_TRUNC in base 10
#	MonitorAsync $add_remove_switch vfs,always -S truncate -S ftruncate
#	MonitorAsync $add_remove_switch vfs,always -S open -S openat -F "a1&512" 

	# chown(92,93,94,260)
#	MonitorAsync $add_remove_switch vfs,always -S chown -S fchown -S lchown -S fchownat

	# chmod(90,91,268) - ok
	# We need to audit these to detect snaplock commit
	# It would be great if we could
	# audit only ones where we're doing a -w 
	MonitorAsync $add_remove_switch vfs,always -S chmod -S fchmod -S fchmodat
	MonitorAsync $add_remove_switch exit,always -S chmod -S fchmod -S fchmodat

	# fcntl(72) - ok
	# Auditing of fcntl was removed as part of defect 17121
	# smbd (CIFS) uses locking so extensively that it was overflowing the
	# raw logs. We do nothing with the auited record of this system call anyway today,
	# so removal is fine.
	# MonitorAsync $add_remove_switch vfs,always -S fcntl 
	# MonitorAsync $add_remove_switch exit,always -S fcntl 

	# utime(132,235,261,280)
#	MonitorAsync $add_remove_switch vfs,always -S utime -S utimes -S futimesat -S utimensat

	# setxattr(188,189,190)
	MonitorAsync $add_remove_switch vfs,always -S setxattr -S lsetxattr -S fsetxattr
	MonitorAsync $add_remove_switch exit,always -S setxattr -S lsetxattr -S fsetxattr

	# system clock adjustments. Make sure we audit system clock
	# adjustments because these can be abused in an attempt to
	# artificially expire a retention on a file.
	# settimeofday()
	# clock_settime()
	MonitorNonFsAsync $add_remove_switch exit,always -S settimeofday -S clock_settime -S adjtimex
}


start_auditing()
{
	local mountpoint=$1

	findMajorMinor $mountpoint
	found_majorminor=$?
	if [ $found_majorminor -ne 0 ]; then
		return $found_majorminor
	fi

	$AUDITCTL_BIN -b 1048576
	auditpid=`ps -C auditd | tail -n 1 | awk {'print $1'}`
	renice -20 $auditpid

	# Request auditd to resume operation
	kill -USR2 $auditpid

	set_events -a
}

stop_auditing()
{
	local mountpoint=$1
	# Take the dev major, minor and TSM user ID information from existing rules
	DEV_MAJOR=$($AUDITCTL_BIN -l | grep "logfile=$mountpoint" | grep devmajor= | head -n 1 | sed 's,.* devmajor=\(.[^ ]*\) .*,\1,g')
	DEV_MINOR=$($AUDITCTL_BIN -l | grep "logfile=$mountpoint" | grep devminor= | head -n 1 | sed 's,.* devminor=\(.[^ ]*\) .*,\1,g')
	COL_UID=$($AUDITCTL_BIN -l | grep "logfile=$mountpoint" | grep uid\!= | head -n 1 | sed 's,.* uid\!=\(.[^ ]*\) .*,\1,g')
	TSMUSER_ID="$COL_UID"
	if [ -z "$DEV_MAJOR" -o -z "$DEV_MINOR" ]
	then
		return $ERR_COLLECTION_NOT_BEING_AUDITED
	fi
	set_events -d
	log_filename=`get_log_filename`
	$AUDITCTL_BIN -C "$log_filename"
}

doStart()
{
	check_file_exists $AUDIT_BIN
	b_exists=$?
	if [ "$b_exists" != 1 ]; then
	    return $ERR_FILE_DOES_NOT_EXIST
	fi

	check_file_executable $AUDIT_BIN
	b_is_executable=$?
	if [ "$b_is_executable" != 1 ]; then
	    return $ERR_FILE_NOT_EXECUTABLE
	fi

	check_file_exists $AUDITCTL_BIN
	b_exists=$?
	if [ "$b_exists" != 1 ]; then
	    return $ERR_FILE_DOES_NOT_EXIST
	fi

	check_file_executable $AUDITCTL_BIN
	b_is_executable=$?
	if [ "$b_is_executable" != 1 ]; then
	    return $ERR_FILE_NOT_EXECUTABLE
	fi

	num_running=0
	running_collection_pid_list=`/bin/ps -C \`basename $AUDIT_BIN\` -f | tail +2 | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`
	for audit_pid in $running_collection_pid_list
	do
		num_running=`expr $num_running + 1`
	done

	# if there are no audits running, start one.
	if [ $num_running -eq 0 ]; then
		$AUDIT_BIN 2>&1
		sleep 10
	fi

	start_auditing $COLLECTIONMNTPOINT

	doStatus
	running_status=$?

	if [ $running_status -ne $ERR_PROCESS_IS_RUNNING ]; then
		return $running_status
	fi

	return 0
	
}

doStatus()
{
	num_running=0

	running_collection_pid_list=`/bin/ps -C \`basename $AUDIT_BIN\` -f | tail +2 | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`
	for audit_pid in $running_collection_pid_list
	do
		num_running=`expr $num_running + 1`
	done

	if [ $num_running -eq 0 ]; then
		return $ERR_PROCESS_NOT_RUNNING
	elif [ $num_running -gt 1 ]; then
		#multiple auditds running. this should never happen. Let's try to clean up.
		skipped_one_running=0
		for audit_pid in $running_collection_pid_list
		do
			if [ $skipped_one_running -eq 1 ]; then
				kill_process $audit_pid SIGTERM
			fi
			skipped_one_running=1
		done
	fi


	#ok, only one auditd should be running now. Let's see if its running for our collection


	#find out the major, minor pair for our mountpoint because this is how audit filters
	findMajorMinor $COLLECTIONMNTPOINT
	found_majorminor=$?

	if [ $found_majorminor -ne 0 ]; then
		return $found_majorminor
	fi

	audit_rules=`$AUDITCTL_BIN -l | grep "devmajor=$DEV_MAJOR devminor=$DEV_MINOR" `

	if [ "$audit_rules" == "" ]; then
		#we have no rules for this mount point
		return $ERR_PROCESS_NOT_RUNNING
	fi

	return $ERR_PROCESS_IS_RUNNING
}

doStop()
{
	doStatus
	running_status=$?

	if [ $running_status -eq $ERR_PROCESS_NOT_RUNNING ]; then
		return $running_status
	fi

	stop_auditing $COLLECTIONMNTPOINT

	doStatus
	running_status=$?

	if [ $running_status -ne $ERR_PROCESS_NOT_RUNNING ]; then
		return $ERR_PROCESS_FAILED_TO_DIE
	fi

	return 0
}

doUpdate()
{
	#Nothing to do for audit.
	return 0
}


doGetRunningState()
{
	num_running=0

	running_collection_pid_list=`/bin/ps -C \`basename $AUDIT_BIN\` -f | tail +2 | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`
	for audit_pid in $running_collection_pid_list
	do
		num_running=`expr $num_running + 1`
	done

	if [ $num_running -eq 0 ]; then
		return 0
	fi


	#ok, auditd is running. Let's see if its running for our collection.


	#find out the major, minor pair for our mountpoint because this is how audit filters
	findMajorMinor $COLLECTIONMNTPOINT
	found_majorminor=$?

	if [ $found_majorminor -ne 0 ]; then
		return 0
	fi

	audit_rules=`$AUDITCTL_BIN -l | grep "devmajor=$DEV_MAJOR devminor=$DEV_MINOR" `

	if [ "$audit_rules" == "" ]; then
		#we have no rules for this mount point
		return 0
	fi

	return 1
}

doGetEnabledState()
{
	return 1
}

doGetOperationalState()
{
	#For the audit system, if it's healthy, it's operational
	doGetHealthyState
	return $?
}

doGetHealthyState()
{
	setUniqueMachineString

	#make sure that auditd is auditing events for our collection.
	#find out the major, minor pair for our mountpoint because this is how audit filters
	findMajorMinor $COLLECTIONMNTPOINT
	found_majorminor=$?

	if [ $found_majorminor -ne 0 ]; then
		echo "could not determine the major and minor for filesystem $COLLECTIONMNTPOINT"
		return 0
	fi

	audit_rules=`$AUDITCTL_BIN -l | grep "devmajor=$DEV_MAJOR devminor=$DEV_MINOR" `

	if [ "$audit_rules" == "" ]; then
		#we have no rules for this mount point
		echo "No rules for filesystem $COLLECTIONMNTPOINT"
		return 0
	fi

	#make sure that auditd can audit events.
	log_dirname=$COLLECTIONMNTPOINT/utility/$RAWLOGDIR
	log_filename=$log_dirname/$UNIQUEMACHINESTRING.rawlog
	logdir_status=`stat $log_dirname -c "%Z"`
	last_rawlog_update=`stat $log_filename -c "%Z"`
	now=`date +%s`

	if [ -z "$last_rawlog_update" -a -z "$logdir_status" ]; then
		#Stale NFS directory?
		#Unhealthy
		echo "Cannot stat filesystem $COLLECTIONMNTPOINT"
		return 0
	elif [ -z "$last_rawlog_update" ]; then
		#File no longer exists but mount point is ok
		#Unhealthy
		echo "Cannot stat file $log_filename"
		return 0
	fi

	if [ $now -ge $last_rawlog_update ]; then
		difference=$((now-last_rawlog_update))
		if [ $difference -le 60 ]; then
			#the raw log has been written to recently.
			return 1
		fi
	fi

	unique_extension=`uuidgen`
	audit_health_dummy_file=$COLLECTIONMNTPOINT/data/audit_health_dummy_file.$unique_extension

	#create a dummy synchronous event that we will be able to find immediately.
	start_offset=`stat -c "%s" "$log_filename"`
	rm -f $audit_health_dummy_file
	end_offset=`stat -c "%s" "$log_filename"`
		
	if [ $end_offset -gt $start_offset ]
	then
		#The event is expected to be found in $log_filename. We use the efficient cat to search for it.
		ecat ${log_filename} $start_offset | grep -m1 $audit_health_dummy_file >/dev/null 2>&1
		find_error=$?
	else
		#The event can be at the beginning of $log_filename or at the end of the rotated log.
		ecat ${log_filename}.1 $start_offset | grep -m1 $audit_health_dummy_file >/dev/null 2>&1
		find_error=$?
		if [ $find_error != 0 ]
		then
			#Try to find it in the new log file (starting from the back, wherefore the use of tac)
			tac ${log_filename} | grep -m1 $audit_health_dummy_file >/dev/null 2>&1
			find_error=$?
		fi
	fi

	if [ $find_error -eq 0 ]; then
		#Found it in the raw log, we're healthy
		return 1
	else
		#didn't find it in the raw log or the rotated raw log. Unhealthy
		echo "After issuing synchronous unlink of $audit_health_dummy_file we are unable to find event in the raw audit log"
		return 0
	fi
}

parse_args $@

if [ ! -z "$COLLECTIONMNTPOINT" ]
then
	findCollectionID $COLLECTIONMNTPOINT
	TSMUSER_ID="u${COLLECTION_ID}"
fi

case ${OPERATION} in
	start)
		doStart
		start_retval=$?
		if [ $start_retval -eq 0 ]; then
			exit 0
		else
			if [ $start_retval -eq $ERR_FILE_DOES_NOT_EXIST ]; then
				echo "start issued but $AUDIT_BIN does not exist."
			elif [ $start_retval -eq $ERR_FILE_NOT_EXECUTABLE ]; then
				echo "start issued but $AUDIT_BIN is not executable."
			elif [ $start_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
				echo "start issued but $AUDIT_BIN is already running"
			elif [ $start_retval -eq $ERR_PROCESS_MULTIPLE_RUNNING ]; then
				echo "start issued but multiple $AUDIT_BIN are already running"
			elif [ $start_retval -eq $ERR_PROCESS_NOT_HEALTHY ]; then
				echo "start issued but $AUDIT_BIN (which is unhealthy) is already running"
			fi

			exit $start_retval
		fi
		;;
	status)
		doStatus
		status_retval=$?
		if [ $status_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
			exit 0
		else
			exit $status_retval
		fi
		;;
	stop)
		doStop
		stop_retval=$?
		if [ $stop_retval -eq 0 ]; then
			exit 0
		else
			if [ $stop_retval -eq $ERR_PROCESS_NOT_RUNNING ]; then
				echo "stop issued for $AUDIT_BIN but it was not running"
			fi
			exit $stop_retval
		fi
		;;
	update)
		doUpdate
		update_retval=$?
		exit $update_retval
		;;
	getrunningstate)
		doGetRunningState
		getrunningstate_retval=$?
		exit $getrunningstate_retval
		;;
	gethealthystate)
		doGetHealthyState
		gethealthystate_retval=$?
		exit $gethealthystate_retval
		;;
	getenabledstate)
		doGetEnabledState
		getenabledstate_retval=$?
		exit $getenabledstate_retval
		;;
	getoperationalstate)
		doGetOperationalState
		getoperationalstate_retval=$?
		exit $getoperationalstate_retval
		;;
	*)
		printHelp
		exit -1
		;;
esac
