#!/bin/sh
########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Workload Manager
# *
# * FILE NAME:       wlm_controltsm.sh
# *                  (hard linked from wlm_controlfranktsm.sh)
# *                  (hard linked from wlm_controlssamtsm.sh)
# *
# * DESCRIPTION:     Controls the TSM component in IIA collections
# *
# * AUTHOR:          Mark A. Smith
# *                  IBM Almaden Research Center
# *                  mark1smi@us.ibm.com
# *
# *                  Michael A. Richmond
# *                  IBM Almaden Research Center
# *                  mar@almaden.ibm.com
# *
# ****************************************************************************
# */

# Configure shell variables
. `dirname $BASH_SOURCE`/mw_environment.sh

# Read the common error values
. `dirname $BASH_SOURCE`/wlm_controlerrors.sh

# Read the common control functions
. `dirname $BASH_SOURCE`/wlm_controlcommon.sh

# Locations of files and executables required for TSM
DSMSERV_NAME=dsmserv
DSMSERV_DIR=/opt/tivoli/tsm/server/bin
DSMSERV_BIN=/opt/tivoli/tsm/server/bin/$DSMSERV_NAME
DB2FM_NAME=db2fm
DB2FM_DIR=/opt/tivoli/tsm/db2/bin
DB2FM_BIN=$DB2FM_DIR/$DB2FM_NAME
DSMXLOG_BIN=/opt/tivoli/tiam/bin/dsmxlog

IIA_ROOT_DIR=/opt/tivoli/tiam
PRINT_XML_FIELD_BIN="${IIA_ROOT_DIR}/bin/mw_print_xml_field"
CREATE_SERVICE_IP_BIN="${IIA_ROOT_DIR}/bin/create_service_ip.py"
DELETE_SERVICE_IP_BIN="${IIA_ROOT_DIR}/bin/delete_service_ip.py"

# in seconds. Give it 30 minutes.
# Careful with this value because the watchdog is watching
# for control script hangs that last longer than 30 minutes
# if we increase either of these values to be greater than
# 30 minutes, we also need to adjust the watchdog to allow
# these scripts to be running for more than 30 minutes.
DSMSERV_STOP_TIMEOUT=1800
DSMSERV_START_TIMEOUT=1800
DSMSERV_PERCENT_TICK_TIMEOUT=1600

#give the process 2 minutes to appear in ps output.
PROCESS_TABLE_APPEARANCE_TIMEOUT=120


# Determine which collection type this TSM will support. The collection
# type influences ancillary startup/shutdown operations including the
# number of virtual IP addresses to establish for this TSM server.
FRANK_COLLECTION="FRAnK"
SSAM_COLLECTION="SSAM"

command_name=`basename $0`

case ${command_name} in
    wlm_controlfranktsm.sh)
	COLLECTION_TYPE="${FRANK_COLLECTION}"
	;;
    wlm_controlssamtsm.sh)
	COLLECTION_TYPE="${SSAM_COLLECTION}"
	;;
    *)
	echo "error: Unable to identify collection type to control TSM for" 2>&1 
	echo "       control script '${command_name}'" 2>&1
	exit ${ERR_BAD_PROBLEM}
	;;
esac

case ${COLLECTION_TYPE} in
    $FRANK_COLLECTION)
	#F for FRAnK
	#Must not conflict with any other prefixes for interfaces in the system
	TSM_INTERFACE_ALIAS_PREFIX=F
	;;
    $SSAM_COLLECTION)
	#S for SSAM
	#Must not conflict with any other prefixes for interfaces in the system
	TSM_INTERFACE_ALIAS_PREFIX=S
	;;
    *)
	echo "error: unknown COLLECTION_TYPE: '${COLLECTION_TYPE}'" 2>&1 
	exit ${ERR_BAD_PROBLEM}
	;;
esac

findInternalVirtualIP()
{
	VIRTUAL_INTERNAL_HOSTNAME=`${PRINT_XML_FIELD_BIN} collection/tsmserver/tsmserverhostname $COLLECTIONMNTPOINT/utility/config/collection.xml`
	VIRTUAL_INTERNAL_IP=`awk -v tsmhostname=$VIRTUAL_INTERNAL_HOSTNAME {'if($2==tsmhostname) { print $1 } '} /etc/hosts`
}

findExternalVirtualIP()
{
	VIRTUAL_EXTERNAL_IP=`${PRINT_XML_FIELD_BIN} collection/tsmserver/externalserverip $COLLECTIONMNTPOINT/utility/config/collection.xml`
}

registerInternalVirtualIP()
{
	findInternalVirtualIP
	findCollectionID $COLLECTIONMNTPOINT

	case $COLLECTION_TYPE in
	    $FRANK_COLLECTION)
		interface_xml_path="WorkloadManagerConfig/ControlScripts/frank/TSM/InternalInterface"
		;;
	    $SSAM_COLLECTION)
		interface_xml_path="WorkloadManagerConfig/ControlScripts/ssam/TSM/InternalInterface"
		;;
	    *)
		# case should be caught by collection type case that determines TSM_INTERFACE_ALIAS_PREFIX
		# script should abort during the earlier case statement.
		;;
	esac
	internalInterface=`${PRINT_XML_FIELD_BIN} $interface_xml_path /tiam_utility/config/workloadmanager_config.xml`
	cmd="${CREATE_SERVICE_IP_BIN} -i $internalInterface -e $TSM_INTERFACE_ALIAS_PREFIX$COLLECTION_ID -s $VIRTUAL_INTERNAL_IP"
	echo "$command_name running:" $cmd
	$cmd
	create_service_ip_rc=$?
	if [ $create_service_ip_rc -eq 0 ]; then
		return 0
	elif [ $create_service_ip_rc -eq 2 ]; then
		#Already existed. Success
		echo "Internal service IP already existed. Success."
		return 0
	fi

	#Error
	return 1
}

registerExternalVirtualIP()
{
	findExternalVirtualIP
	findCollectionID $COLLECTIONMNTPOINT

	interface_xml_path="WorkloadManagerConfig/ControlScripts/ssam/TSM/ExternalInterface"

	externalInterface=`${PRINT_XML_FIELD_BIN} $interface_xml_path /tiam_utility/config/workloadmanager_config.xml`
	cmd="${CREATE_SERVICE_IP_BIN} -i $externalInterface -e $TSM_INTERFACE_ALIAS_PREFIX$COLLECTION_ID -s $VIRTUAL_EXTERNAL_IP"
	echo "$command_name running:" $cmd
	$cmd
	create_service_ip_rc=$?
	if [ $create_service_ip_rc -eq 0 ]; then

		#best effort gratuitous arp
		echo "Created service IP. Sending gratuitous arp."
		arping_cmd="/sbin/arping -c 3 -A -I $externalInterface $VIRTUAL_EXTERNAL_IP"
		echo "$command_name running:" $arping_cmd
		arping_output=`$arping_cmd`
		echo "$arping_cmd returned: '$arping_output'"

		return 0
	elif [ $create_service_ip_rc -eq 2 ]; then
		#Already existed. Success
		echo "External service IP already existed. Success."
		return 0
	fi

	#Error
	return 1
}

unregisterInternalVirtualIP()
{
	findCollectionID $COLLECTIONMNTPOINT

	case $COLLECTION_TYPE in
	    $FRANK_COLLECTION)
		interface_xml_path="WorkloadManagerConfig/ControlScripts/frank/TSM/InternalInterface"
		;;
	    $SSAM_COLLECTION)
		interface_xml_path="WorkloadManagerConfig/ControlScripts/ssam/TSM/InternalInterface"
		;;
	    *)
		# case should be caught by collection type case that determines TSM_INTERFACE_ALIAS_PREFIX
		# script should abort during the earlier case statement.
		;;
	esac

	internalInterface=`${PRINT_XML_FIELD_BIN} $interface_xml_path /tiam_utility/config/workloadmanager_config.xml`
	cmd="${DELETE_SERVICE_IP_BIN} -i $internalInterface -e $TSM_INTERFACE_ALIAS_PREFIX$COLLECTION_ID"
	echo "$command_name running:" $cmd
	$cmd
}

unregisterExternalVirtualIP()
{
    findCollectionID $COLLECTIONMNTPOINT

    interface_xml_path="WorkloadManagerConfig/ControlScripts/ssam/TSM/ExternalInterface"

    externalInterface=`${PRINT_XML_FIELD_BIN} $interface_xml_path /tiam_utility/config/workloadmanager_config.xml`
    cmd="${DELETE_SERVICE_IP_BIN} -i $externalInterface -e $TSM_INTERFACE_ALIAS_PREFIX$COLLECTION_ID"
    echo "$command_name running:" $cmd
    $cmd
}

findExecutableName()
{
	# determine where the server is installed
	server_version=`rpm -q TIVsm-server`

	# if string server_version is not NULL, it means we got something
	if [[ -n "$server_version" ]]; then
		prefix=`rpm -q --queryformat %'{INSTALLPREFIX}' $server_version`
		DSMSERV_BIN=$prefix/tivoli/tsm/server/bin/$DSMSERV_NAME
		DSMSERV_DIR=$prefix/tivoli/tsm/server/bin
		return 0
	else
		#IBM Tivoli Storage Manager Server not found on this system
		return -1
	fi

}

doStart()
{

	findExecutableName
	b_foundname=$?
	if [ "$b_foundname" != 0 ]; then
		return $ERR_FILE_DOES_NOT_EXIST
	fi

	check_file_exists $DSMSERV_BIN
	b_exists=$?
	if [ "$b_exists" != 1 ]; then
	    return $ERR_FILE_DOES_NOT_EXIST
	fi
	check_file_executable $DSMSERV_BIN
	b_is_executable=$?
	if [ "$b_is_executable" != 1 ]; then
	    return $ERR_FILE_NOT_EXECUTABLE
	fi

	doStatus
	running_status=$?

	if [ $running_status -ne $ERR_PROCESS_NOT_RUNNING ]; then
		return $running_status
	fi

	#Make room for this new TSM and DB2 in memory
	echo 3 > /proc/sys/vm/drop_caches

	# set TSM up and start it
	ulimit -c unlimited
	PATH=$PATH:$DSMSERV_DIR
	export DSMSERV_DIR
	DSMSERV_CONFIG=./dsmserv.opt
	export DSMSERV_CONFIG
	export COLLECTIONMNTPOINT
	tsminstancedir=$COLLECTIONMNTPOINT/tsm
	cd $tsminstancedir

	# The TSM userID is based on the management id for this collection
	findCollectionID $COLLECTIONMNTPOINT
	tsmuser="u${COLLECTION_ID}"
	tsmuserhome=`awk -F':' -v tsmusername=$tsmuser {'if($1==tsmusername) { print $6 } '} /etc/passwd`

	registerVirtualIPs

	. ${tsmuserhome}/sqllib/db2profile
	$DB2FM_BIN -i $tsmuser -a off
	$DB2FM_BIN -i $tsmuser -f off
	nohup $DSMSERV_BIN -u ${tsmuser} -i ${tsminstancedir} 2>&1 | $DSMXLOG_BIN ${tsminstancedir}/log/dsmserv.log >/dev/null 2>&1 &

	b_appeared_in_process_table=0
	waited_seconds=0
	timeout_seconds=$PROCESS_TABLE_APPEARANCE_TIMEOUT
	wait_interval=10
	start_time=`date +%s`
	while [ $waited_seconds -lt $timeout_seconds ]
	do
		this_start_time=`date +%s`

		doStatus
		running_status=$?

		if [ $running_status -eq $ERR_PROCESS_IS_RUNNING ]; then
			b_appeared_in_process_table=1
			echo "$DSMSERV_NAME appeared in the process table."
			break
		fi

		echo "$DSMSERV_NAME has not yet appeared in the process table. Sleeping $wait_interval seconds and looking again."
		now=`date +%s`
		waited_this_time=$((now-this_start_time))
		if [ $waited_this_time -lt $wait_interval ]; then
			more_sleep_needed=$((wait_interval-waited_this_time))
			sleep $more_sleep_needed
		fi
		now=`date +%s`
		waited_seconds=$((now-start_time))
	done

	if [ $b_appeared_in_process_table -eq 0 ]; then
		echo "$DSMSERV_NAME never appeared in the process table. Verifying and completing stoppage of any TSM components (DB2)"
		doStop 0
		return $running_status
	fi

#Ok, we're running, now we need to wait a bit to get healthy.
#They just asked us to start, so we've done our job, and can
#return success, but it makes sense to wait at least some
#time to verify we come up healthy.

	waited_seconds=0
	timeout_seconds=$DSMSERV_START_TIMEOUT
	wait_interval=60
	start_time=`date +%s`
	while [ $waited_seconds -lt $timeout_seconds ]
	do
		this_start_time=`date +%s`

		#verify that we're still running. Otherwise we'll be in
		#this loop for a long time!
		doStatus
		running_status=$?

		if [ $running_status -ne $ERR_PROCESS_IS_RUNNING ]; then
			echo "$DSMSERV_NAME died during startup shortly after being started, but before becoming healthy. Verifying and completing stoppage of any TSM components (DB2)"
			doStop 0
			return $running_status
		fi

		doGetHealthyState 0
		healthy_state=$?

		if [ $healthy_state -eq 1 ]; then
			#Ok, we're healthy
			echo "$DSMSERV_NAME started up and became healthy after $waited_seconds seconds"
			return 0
		fi

		now=`date +%s`
		waited_this_time=$((now-this_start_time))
		if [ $waited_this_time -lt $wait_interval ]; then
			more_sleep_needed=$((wait_interval-waited_this_time))
			sleep $more_sleep_needed
		fi
		now=`date +%s`
		waited_seconds=$((now-start_time))
	done

	echo "$DSMSERV_NAME has not become healthy after $waited_seconds seconds"

	return 0
}

doStatus()
{
	num_running=0

	running_collection_pid_list=`/bin/ps -C \`basename $DSMSERV_BIN\` -f | tail +2 | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`
	for dsmserv_pid in $running_collection_pid_list
	do
		dsmserv_environment=`/bin/ps -p $dsmserv_pid -f e | tail +2 | tr -s ' ' | sed 's/^[ ]//g' `
		for env_var in $dsmserv_environment
		do
			mount_name=${env_var##COLLECTIONMNTPOINT=}
			if [ x$mount_name != x$env_var ]; then
				if [ "$mount_name" = "$COLLECTIONMNTPOINT" ]; then
					#For debugging, let's print out the pids of the multiple dsmservs for this collection
					echo "Found TSM with pid $dsmserv_pid running for this collection"
					num_running=`expr $num_running + 1`
				fi
			fi
		done
	done

	if [ $num_running -eq 0 ]; then
		return $ERR_PROCESS_NOT_RUNNING
	fi

	#Multiple dsmserv running is ok for TSM, it can fork and exec, and there will be multiple processes
	#under the dsmserv name for a short period of time. TSM protects itself against multiple
	#of the same instance running simultaneously.
	return $ERR_PROCESS_IS_RUNNING
}

doUpdate()
{
	case $COLLECTION_TYPE in
	    $FRANK_COLLECTION)
		# Nothing to do for TSM in FRAnK collections
		return 0
		;;
	    $SSAM_COLLECTION)
		# Update the external service IP if we're running on this node.
		doStatus
		status_retval=$?

		if [ $status_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
			registerExternalVirtualIP 
		fi

		return 0

		;;
	esac
}

#returns 0-101 for percent complete
#returns 101 for index recreate
#returns 255 for error
#returns 254 for not in recovery mode
getPercentRecoveryComplete()
{
	db2_checker_executable="/opt/tivoli/tiam/bin/ia_tsm_query_db2crashrecovery"
	check_file_executable $db2_checker_executable
	b_is_executable=$?
	if [ "$b_is_executable" != 1 ]; then
		echo "$db2_checker_executable is not executable"
		return 255
	fi

	query_db2_cmd="$db2_checker_executable $COLLECTIONMNTPOINT"

	percent_recovery_complete=0
	max_tries=10
	num_tries=0
	sleep_interval=10
	while [ $num_tries -lt $max_tries ]
	do
		echo "running:" $query_db2_cmd
		query_output=`$query_db2_cmd`
		query_db2_rc=$?
		if [ $query_db2_rc -eq 255 ]; then
			echo "Error verifying crash recovery:"
			echo $query_output
			return 255
		elif [ $query_db2_rc -eq 254 ]; then
			num_tries=`expr $num_tries + 1`
			if [ $num_tries -eq $max_tries ]; then
				echo "After $max_tries verifications, TSM/DB2 reports not in crash recovery mode."
				echo $query_output
				return 254
			fi
			echo "TSM/DB2 not in crash recovery mode. Sleeping $sleep_interval and verifying through retry"
			echo $query_output
			sleep $sleep_interval
		elif [ $query_db2_rc -eq 253 ]; then
			num_tries=`expr $num_tries + 1`
			if [ $num_tries -eq $max_tries ]; then
				echo "After $max_tries attempts, unable to detect whether TSM/DB2 in crash recovery mode. Unhealthy:"
				echo $query_output
				return 255
			fi
			echo "Unable to detect whether TSM/DB2 is in crash recovery mode. Sleeping $sleep_interval and trying again:"
			echo $query_output
			sleep $sleep_interval
		elif [ $query_db2_rc -le 101 -a $query_db2_rc -ge 0 ]; then
			percent_recovery_complete=$query_db2_rc
			return $percent_recovery_complete
		else
			echo "Unknown return code from $db2_checker_executable : $query_db2_rc"
			return 255
		fi
	done

}

#Return values:
#Making progress: 0
#Not in recovery mode: 1
#No progress made: 2
#Error: 255
verifyRecoveryMode()
{
	echo "Verifying that TSM/DB2 is in recovery mode and that progress is being made."

	getPercentRecoveryComplete
	start_percent=$?
	if [ $start_percent -eq 255 ]; then
		#Reason already echoed to stdout
		return 255
	elif [ $start_percent -eq 254 ]; then
		#Not in recovery mode.
		#Reason already echoed to stdout
		return 1
	fi

	echo "TSM/DB2 in crash recovery mode. Percent Complete: $start_percent"
	echo "Verifying that progress is being made."

	waited_seconds=0
	timeout_seconds=$DSMSERV_PERCENT_TICK_TIMEOUT
	wait_interval=60
	start_time=`date +%s`
	sleep $wait_interval
	while [ $waited_seconds -lt $timeout_seconds ]
	do
		this_start_time=`date +%s`
		now=`date +%s`

		getPercentRecoveryComplete
		now_percent=$?
		if [ $now_percent -eq 255 ]; then
			#Reason already echoed to stdout
			return 255
		elif [ $now_percent -eq 254 ]; then
			#No longer in recovery mode. Finished?
			#Give it the benefit of the doubt.
			#If it's not finished then we'll catch this on our next health check.
			echo "DB2 no longer in recovery mode. Assuming finished."
			return 0
		fi

		if [ $now_percent -gt $start_percent ]; then
			echo "DB2 recovery making progress. Start percent: $start_percent. Now percent: $now_percent. Healthy!"
			return 0
		else
			echo "DB2 no progress yet. Start percent: $start_percent. Now percent: $now_percent"
		fi

		waited_this_time=$((now-this_start_time))
		if [ $waited_this_time -lt $wait_interval ]; then
			more_sleep_needed=$((wait_interval-waited_this_time))
			sleep $more_sleep_needed
		fi
		now=`date +%s`
		waited_seconds=$((now-start_time))
	done

	echo "$DSMSERV_NAME has not made process with DB2 recovery for $waited_seconds seconds. Unhealthy"

	return 2
}

doGetEnabledState()
{
	return 1
}

#We want to know if it's operational regardless of what node it's running on.
doGetOperationalState()
{
	#For TSM, just because it's healthy, doesn't mean it can actually be talked to.
	#Sometimes, TSM can be coming up healthily for hours, but is not actually operational
	#whereby clients can communicate with it, etc
	doGetHealthyState 2
	operational_rc=$?
	if [ $operational_rc -eq 1 ]; then
		echo "TSM for collection $COLLECTIONMNTPOINT is operational"
	else
		echo "TSM for collection $COLLECTIONMNTPOINT is not operational"
	fi
	return $operational_rc
}

#we want to know if it's healthy _on_this_node
#first we have to verify that it's running on this node,
#then we can try to talk to it.
doGetHealthyState()
{
	flag=$1
	recursive_attempt=0
	operational_check=0
	if [ $flag -eq 1 ]; then
		recursive_attempt=1
	elif [ $flag -eq 2 ]; then
		operational_check=1
	fi

	#If we just care if it's operational, we don't care where it's running
	if [ $operational_check -eq 0 ]; then
		running_pid=""

		running_collection_pid_list=`/bin/ps -C \`basename $DSMSERV_BIN\` -f | tail +2 | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`
		for dsmserv_pid in $running_collection_pid_list
		do
			dsmserv_environment=`/bin/ps -p $dsmserv_pid -f e | tail +2 | tr -s ' ' | sed 's/^[ ]//g' `
			for env_var in $dsmserv_environment
			do
				mount_name=${env_var##COLLECTIONMNTPOINT=}
				if [ x$mount_name != x$env_var ]; then
					if [ "$mount_name" = "$COLLECTIONMNTPOINT" ]; then
						running_pid=$dsmserv_pid
					fi
				fi
			done
		done

		if [ "$running_pid" = "" ]; then
			echo "TSM no longer running"
			return 0
		fi

		#ok, it's running on this node. Let's talk to it.
	fi

	findCollectionName $COLLECTIONMNTPOINT

	tsm_output=`dsmc Query SEssion -se=$COLLECTION_NAME -testflag=POLLTIMEOUT:120 2>&1`
	tsm_status=$?

	echo "TSM dsmc returned status $tsm_status"

	if [ $tsm_status -eq 0 ]; then
		#Healthy, and we know that the internal service IP is healthy too, since
		#that's what we just spoke to TSM on. Re-register external virtual IP
		#just in case.
		if [ $COLLECTION_TYPE == $SSAM_COLLECTION ]; then
			#This virtual IP may already be registered, but this can't hurt, and can only help.
			registerExternalVirtualIP 
		fi
		echo "Healthy."
		return 1
	else
		max_tries=3
		num_tries=0
		while [ $num_tries -lt $max_tries ]
		do
			#Please contact Erick Kissel for questions relating to the TSM health check.
			#This is to address IA defect TIAMC00004968.
			#This health check section is nasty. Typically you would expect that
			#if our status was 0, we're healthy, if non-zero, unhealthy... but TSM
			#doesn't actually have a clean way of checking health, so we have
			#to do this really uncomfortable thing where we check the error
			#code from dsmadmc now. Also, we have to account for timeouts.
			#
			#The "Healthy" return codes would be
			#  0 - success
			#  53 - unknown id
			#  55 - server sessions disabled
			#  137 - authentication failure (wrong password)
			#
			#A special return code is
			#  206 - timeout (we will try three times before we consider this a TSM hang)
	
			echo "TSM dsmc returned output '$tsm_output': attempting health check using dsmadmc"
	
			tsm_output=`dsmadmc -se=$COLLECTION_NAME -testflag=POLLTIMEOUT:120 -id=healthcheck -pass=healthcheck q status 2>&1`
			tsm_status=$?
	
			echo "TSM dsmadmc returned status $tsm_status"
	
			if [ $tsm_status -eq 0 -o $tsm_status -eq 53 -o $tsm_status -eq 55 -o $tsm_status -eq 137 ]; then
				#Healthy, and we know that the internal service IP is healthy too, since
				#that's what we just spoke to TSM on. Re-register external virtual IP
				#just in case.
				if [ $COLLECTION_TYPE == $SSAM_COLLECTION ]; then
					#This virtual IP may already be registered, but this can't hurt, and can only help.
					registerExternalVirtualIP 
				fi
				echo "Healthy."
				return 1
			elif [ $tsm_status -eq 206 ]; then
				echo "TSM dsmadmc timeout returned output '$tsm_output'"
				num_tries=`expr $num_tries + 1`
				if [ $operational_check -eq 1 ]; then
					#If we're just checking operational state, we don't want to muck with things
					#like re-registering service IPs, etc. Just sleep and try again. The health checker
					#will be doing all of that stuff, we just want to know if TSM is currently
					#working.
					echo "Timeout occurred talking with TSM. Trying again."
					sleep 1
					continue
				elif [ $num_tries -eq $max_tries ]; then
					#Unhealthy? Or recovery mode?
					echo "Too many TSM dsmadmc timeouts. Checking whether TSM is going through db2 recovery"
					verifyRecoveryMode
					recovery_rc=$?
					if [ $recovery_rc -eq 0 ]; then
						echo "TSM in recovery mode and making progress. Healthy."
						return 1
					elif [ $recovery_rc -eq 1 ]; then
						echo "TSM not in recovery mode."
						if [ $recursive_attempt -eq 1 ]; then
							#This is already our recursive attempt
							echo "Second attempt to verify healthy failed. Unhealthy."
							return 0
						else
							echo "This could mean that recovery mode just finished, but TSM is not accepting connections yet. Retrying."
							doGetHealthyState 1
							healthy_rc=$?
							return $healthy_rc
						fi
					elif [ $recovery_rc -eq 2 ]; then
						echo "TSM in recovery mode but not making progress. Unhealthy."
						return 0
					elif [ $recovery_rc -eq 255 ]; then
						echo "Error determining if TSM is in recovery mode. Unhealthy."
						return 0
					else
						echo "Unknown error determine whether TSM is in recovery mode: $recovery_rc. Unhealthy."
						return 0
					fi
				fi
				#Maybe we just couldn't talk to TSM.
				#Try re-registering the service IPs and trying again.
				echo "Re-registering service IPs and trying again."
				registerVirtualIPs
			else
				#Unhealthy
				echo "TSM dsmadmc returned unknown error. output '$tsm_output': Unhealthy"
				return 0
			fi
		done

		echo "Did not get a healthy return code from dsmc or dsmadmc after $num_tries tries. Unhealthy"
		return 0
	fi
}

doGetRunningState()
{
	running_collection_pid_list=`/bin/ps -C \`basename $DSMSERV_BIN\` -f | tail +2 | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`
	for dsmserv_pid in $running_collection_pid_list
	do
		dsmserv_environment=`/bin/ps -p $dsmserv_pid -f e | tail +2 | tr -s ' ' | sed 's/^[ ]//g' `
		for env_var in $dsmserv_environment
		do
			mount_name=${env_var##COLLECTIONMNTPOINT=}
			if [ x$mount_name != x$env_var ]; then
				if [ "$mount_name" = "$COLLECTIONMNTPOINT" ]; then
					return 1
				fi
			fi
		done
	done

	return 0;
}

TSM_stopDB2()
{
	tsmuser=$1

	sudo -u $tsmuser -i -- -c "db2stop force"
	sudo -u $tsmuser -i -- -c "db2_kill"
}

unregisterVirtualIPs()
{
	echo "Unregistering service IP. It may already be unregistered. This is not necessarily an error"

	case $COLLECTION_TYPE in
	    $FRANK_COLLECTION)
		unregisterInternalVirtualIP
		;;
	    $SSAM_COLLECTION)
		unregisterInternalVirtualIP
		unregisterExternalVirtualIP
		;;
	    *)
		# case should be caught by collection type case that determines TSM_INTERFACE_ALIAS_PREFIX
		# script should abort during the earlier case statement.
		;;
	esac
}

registerVirtualIPs()
{
	echo "Registering service IP. It may already be registered. This is not necessarily an error"

	case $COLLECTION_TYPE in
	    $FRANK_COLLECTION)
		registerInternalVirtualIP
		register_rc=$?
		if [ $register_rc -ne 0 ]; then
			return 1
		fi
		;;
	    $SSAM_COLLECTION)
		registerInternalVirtualIP
		register_rc=$?
		if [ $register_rc -ne 0 ]; then
			return 1
		fi
		registerExternalVirtualIP
		register_rc=$?
		if [ $register_rc -ne 0 ]; then
			return 1
		fi
		;;
	    *)
		# case should be caught by collection type case that determines TSM_INTERFACE_ALIAS_PREFIX
		# script should abort during the earlier case statement.
		;;
	esac

	return 0
}

doStop()
{
	forcibly=$1

	b_found_tsm=0

	findCollectionID $COLLECTIONMNTPOINT
	tsmuser="u${COLLECTION_ID}"
	tsmuserhome=`awk -F':' -v tsmusername=$tsmuser {'if($1==tsmusername) { print $6 } '} /etc/passwd`

	running_collection_pid_list=`/bin/ps -C \`basename $DSMSERV_BIN\` -f | tail +2 | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`

	for dsmserv_pid in $running_collection_pid_list
	do
		pid_to_kill=""

		dsmserv_environment=`/bin/ps -p $dsmserv_pid -f e | tail +2 | tr -s ' ' | sed 's/^[ ]//g' `
		for env_var in $dsmserv_environment
		do
			if [ "$env_var" == "<defunct>" ]; then
				echo "Found a defunct dsmserv process $dsmserv_pid. Attempting to forcibly kill."
				kill_process $dsmserv_pid SIGKILL
			fi

			mount_name=${env_var##COLLECTIONMNTPOINT=}
			if [ x$mount_name != x$env_var ]; then
				#We found an environment variable with COLLECTIONMNTPOINT=something
				if [ "$mount_name" = "$COLLECTIONMNTPOINT" ]; then
					#We found the dsmserv for our COLLECTIONMNTPOINT
					pid_to_kill=$dsmserv_pid
				fi
			fi
		done

		if [ "$pid_to_kill" == "" ]; then
			#This dsmserv was not for this collection. Look for the next one.
			echo "This dsmserv $dsmserv_pid was not for this collection. Looking for the next one."
			continue
		fi

		if [ "$pid_to_kill" != "" ]; then
			b_found_tsm=1
			echo "Killing dsmserv process $pid_to_kill (and stopping DB2 in case TSM fails to stop it)"
			if [ $forcibly -eq 1 ]; then
				kill_process $pid_to_kill SIGABRT
				#TSM may fail to kill DB2 when killed like this.
				#We will need to stop it ourselves later. Best if we can
				#Wait until after we see this process die, but if the
				#process won't die, we will stop DB2 anyway.

				#Unregister the virtual IP here because
				#forcible stoppage of DB2 sometimes hangs and we don't
				#want the service IP hanging around during all that time.
				unregisterVirtualIPs
			else
				kill_process $pid_to_kill SIGTERM
				#Just in case TSM forgot to stop DB2,
				#we will need to stop it after we see this process die.
			fi
		fi
	done


	if [ $b_found_tsm -eq 0 ]; then
		unregisterVirtualIPs
		echo "No $DSMSERV_NAME was found to be running for this collection. Asking TSM's DB2 to stop just in case TSM failed to stop it."
		#Just in case, clean up DB2 for this TSM for this collection on this node.
		TSM_stopDB2 $tsmuser
		return 0
	fi

	echo "Waiting for $DSMSERV_NAME to stop completely."

	waited_seconds=0
	timeout_seconds=$DSMSERV_STOP_TIMEOUT
	wait_interval=10
	start_time=`date +%s`
	while [ $waited_seconds -lt $timeout_seconds ]
	do
		this_start_time=`date +%s`

		doStatus
		running_status=$?

		if [ $running_status -eq $ERR_PROCESS_NOT_RUNNING ]; then
			unregisterVirtualIPs
			echo "$DSMSERV_NAME has stopped. Asking TSM's DB2 to stop just in case TSM failed to stop it"
			#just in case TSM forgot to stop DB2
			TSM_stopDB2 $tsmuser
			return 0
		fi

		now=`date +%s`
		waited_this_time=$((now-this_start_time))
		if [ $waited_this_time -lt $wait_interval ]; then
			more_sleep_needed=$((wait_interval-waited_this_time))
			sleep $more_sleep_needed
		fi
		now=`date +%s`
		waited_seconds=$((now-start_time))
	done

#At this point, we know that tsm refused to die.

	if [ $forcibly -eq 0 ]; then
		#It's time to get medieval.
		echo "$DSMSERV_NAME refused to stop. Attempting to stop forcibly."
		doStop 1
		return $?
	fi

	#Make sure IPs are removed
	unregisterVirtualIPs

	#We forcibly attempted to kill TSM.
	#TSM may fail to kill DB2 when killed like this.
	#Do it ourselves.
	TSM_stopDB2 $tsmuser

	return $ERR_PROCESS_FAILED_TO_DIE
}

parse_args $@

case ${OPERATION} in
	start)
		doStart
		start_retval=$?
		if [ $start_retval -eq 0 ]; then
			exit 0
		else
			if [ $start_retval -eq $ERR_FILE_DOES_NOT_EXIST ]; then
				echo "start issued but $DSMSERV_BIN does not exist."
			elif [ $start_retval -eq $ERR_FILE_NOT_EXECUTABLE ]; then
				echo "start issued but $DSMSERV_BIN is not executable."
			elif [ $start_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
				echo "start issued but $DSMSERV_BIN is already running"
			elif [ $start_retval -eq $ERR_PROCESS_MULTIPLE_RUNNING ]; then
				echo "start issued but multiple $DSMSERV_BIN are already running"
			elif [ $start_retval -eq $ERR_PROCESS_NOT_HEALTHY ]; then
				echo "start issued but $DSMSERV_BIN (which is unhealthy) is already running"
			fi

			exit $start_retval
		fi
		;;
	status)
		doStatus
		status_retval=$?
		if [ $status_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
			exit 0
		else
			exit $status_retval
		fi
		;;
	stop)
		doStop 0
		stop_retval=$?
		if [ $stop_retval -eq 0 ]; then
			exit 0
		else
			if [ $stop_retval -eq $ERR_PROCESS_NOT_RUNNING ]; then
				echo "stop issued for $DSMSERV_NAME but it was not running"
			fi
			exit $stop_retval
		fi
		;;
	update)
		doUpdate
		update_retval=$?
		exit $update_retval
		;;
	getrunningstate)
		doGetRunningState
		getrunningstate_retval=$?
		exit $getrunningstate_retval
		;;
	gethealthystate)
		doGetHealthyState 0
		gethealthystate_retval=$?
		exit $gethealthystate_retval
		;;
	getenabledstate)
		doGetEnabledState
		getenabledstate_retval=$?
		exit $getenabledstate_retval
		;;
	getoperationalstate)
		doGetOperationalState
		getoperationalstate_retval=$?
		exit $getoperationalstate_retval
		;;
	*)
		printHelp
		exit -1
		;;
esac
