#!/bin/sh
########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Workload Manager
# *
# * FILE NAME:       wlm_controlindexbroker.sh
# *
# * DESCRIPTION:     Controls the Index Broker component 
# *                  of a FRAnK collection
# *
# * AUTHOR:          K.W.Brannon
# *                  IBM Almaden Research Center
# *                  kbrannon@us.ibm.com
# *
# *
# ****************************************************************************
# */

# Configure shell variables
. `dirname $BASH_SOURCE`/mw_environment.sh
# Read the common error values
. `dirname $BASH_SOURCE`/wlm_controlerrors.sh
#echo "finished wlm_controlerrors.sh"
# Read the common control functions
. `dirname $BASH_SOURCE`/wlm_controlcommon.sh
#echo "finished wlm_controlcommon.sh"

IIA_ROOT_DIR="/opt/tivoli/tiam"
PRINT_XML_FIELD_BIN="${IIA_ROOT_DIR}/bin/mw_print_xml_field"
SSM_JAVA_BIN=/opt/ibm/java2-x86_64-50/jre/bin/java
SSM_INDEXBROKER_JAR="/opt/tivoli/tiam/lib/index/indexbroker/lib/indexbroker.jar"
JAVA_INITHEAP=1024m
JAVA_MAXHEAP=1024m

#in seconds. Give it a minute.
IB_STOP_TIMEOUT=60

doStart()
{

	check_file_exists $SSM_JAVA_BIN
	b_exists=$?
	if [ "$b_exists" != 1 ]; then
	    return $ERR_FILE_DOES_NOT_EXIST
	fi
        
        check_file_exists $SSM_INDEXBROKER_JAR
	b_exists=$?
	if [ "$b_exists" != 1 ]; then
	    return $ERR_FILE_DOES_NOT_EXIST
	fi
        
        #Check that jetty port is numeric
        if [ -n "`echo \"$SSM_JETTY_PORT\" | /usr/bin/tr -d \"[:digit:]\"`" ]; then
            return $ERR_BAD_PROBLEM
        fi

        #Check that indexbroker port is numeric
        if [ -n "`echo \"$SSM_INDEXBROKER_PORT\" | /usr/bin/tr -d \"[:digit:]\"`" ]; then
            return $ERR_BAD_PROBLEM
        fi

        #Check that java is executable
        check_file_executable $SSM_JAVA_BIN
	b_is_executable=$?
	if [ "$b_is_executable" != 1 ]; then
	    return $ERR_FILE_NOT_EXECUTABLE
	fi

	#Create the log and trace dirs if they do not exist
	UNIQUE_NODEID=`/opt/tivoli/tiam/bin/mw_print_unique_nodeid`
	LOGDIR=$COLLECTIONMNTPOINT/$UNIQUE_NODEID
	mkdir -p /tiam_utility/log/CTJID/in/$LOGDIR
	mkdir -p /tiam_utility/log/CTJID/in/$LOGDIR/ffdc
	mkdir -p /tiam_utility/log/CTJID/in/$LOGDIR/logs

	# Retrieve the logging and tracing levels for the collection
	LOG_LEVEL=`/opt/tivoli/tiam/bin/mw_print_xml_field collection/logginglevel $COLLECTIONMNTPOINT/utility/config/collection.xml`
	TRACE_LEVEL=`/opt/tivoli/tiam/bin/mw_print_xml_field collection/tracinglevel $COLLECTIONMNTPOINT/utility/config/collection.xml`
	
	doStatus
	running_status=$?

	if [ $running_status -ne $ERR_PROCESS_NOT_RUNNING ]; then
		return $running_status
	fi

        IB_COLLECTIONMNTPOINT=$COLLECTIONMNTPOINT
        export IB_COLLECTIONMNTPOINT
        nohup $SSM_JAVA_BIN -Xms$JAVA_INITHEAP -Xmx$JAVA_MAXHEAP -Dindexbrokerport=$SSM_INDEXBROKER_PORT -Djettyport=$SSM_JETTY_PORT -Djettyip=$SSM_JETTY_IP -Dcollectionandnode=$LOGDIR -Dlogginglevel=$LOG_LEVEL -Dtracinglevel=$TRACE_LEVEL -jar $SSM_INDEXBROKER_JAR >/tmp/indexbroker_$SSM_INDEXBROKER_PORT.log 2>&1 &
        echo "sleeping 10 sec"
        sleep 10
	doStatus
	running_status=$?

	if [ $running_status -ne $ERR_PROCESS_IS_RUNNING ]; then
		return $running_status
	fi

	return 0
	
}

doStatus()
{
	num_running=0
	running_collection_pid_list=`/bin/ps -C \`basename $SSM_JAVA_BIN\` -f | tail -n +2 | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`
	for indexbroker_pid in $running_collection_pid_list
	    do
		indexbroker_env=`/bin/ps -p $indexbroker_pid -f e | tail -n +2 | tr -s ' ' | sed 's/^[ ]//g' `
		for env_var in $indexbroker_env
		do
			mount_name=${env_var##IB_COLLECTIONMNTPOINT=}
			if [ x$mount_name != x$env_var ]; then
				if [ "$mount_name" = "$COLLECTIONMNTPOINT" ]; then
					if [ $num_running -eq 0 ]; then
						num_running=`expr $num_running + 1`
					else
						return $ERR_PROCESS_MULTIPLE_RUNNING
					fi
				fi
			fi
		done
	done


	if [ $num_running -eq 0 ]; then
		return $ERR_PROCESS_NOT_RUNNING
	elif [ $num_running -eq 1 ]; then
		return $ERR_PROCESS_IS_RUNNING
	fi
}

doStop()
{
	doStatus
	running_status=$?

	if [ $running_status -eq $ERR_PROCESS_NOT_RUNNING ]; then
		return $running_status
	fi

	running_collection_pid_list=`/bin/ps -C \`basename $SSM_JAVA_BIN\` -f | tail +2 | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`

	for indexbroker_pid in $running_collection_pid_list
	do
		indexbroker_env=`/bin/ps -p $indexbroker_pid -f e | tail +2 | tr -s ' ' | sed 's/^[ ]//g' `
		for env_var in $indexbroker_env
		do
			mount_name=${env_var##IB_COLLECTIONMNTPOINT=}
			if [ x$mount_name != x$env_var ]; then
				if [ "$mount_name" = "$COLLECTIONMNTPOINT" ]; then
					kill_process $indexbroker_pid SIGTERM
				fi
			fi
		done
	done

	waited_seconds=0
	timeout_seconds=$IB_STOP_TIMEOUT
	wait_interval=1
	while [ $waited_seconds -lt $timeout_seconds ]
	do

		doStatus
		running_status=$?

		if [ $running_status -eq $ERR_PROCESS_NOT_RUNNING ]; then
                        #nothing to do
			return 0
		fi

		sleep $wait_interval
		waited_seconds=`expr $waited_seconds + $wait_interval`
	done

	return $ERR_PROCESS_FAILED_TO_DIE
}

doUpdate()
{
	#Nothing to do for solr server
	return 0
}

doGetRunningState()
{
	running_collection_pid_list=`/bin/ps -C \`basename $SSM_JAVA_BIN\` -f | tail +2 | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`

	for indexbroker_pid in $running_collection_pid_list
	do
		indexbroker_env=`/bin/ps -p $indexbroker_pid -f e | tail +2 | tr -s ' ' | sed 's/^[ ]//g' `
		for env_var in $indexbroker_env
		do
			mount_name=${env_var##IB_COLLECTIONMNTPOINT=}
			if [ x$mount_name != x$env_var ]; then
				if [ "$mount_name" = "$COLLECTIONMNTPOINT" ]; then
					return 1
				fi
			fi
		done
	done


	return 0
}

doGetEnabledState()
{
	fs_device=`awk -v colmount=$COLLECTIONMNTPOINT '{ if ($2==colmount) { print $1 } }' /etc/fstab`
	getdev_rc=$?
	if [ $getdev_rc -ne 0 ]; then
		echo "Error $getdev_rc getting device for $COLLECTIONMNTPOINT from /etc/fstab"
		return -1
	fi
	if [ "$fs_device" == "" ]; then
		echo "Error getting device for $COLLECTIONMNTPOINT from /etc/fstab. device string empty"
		return -1
	fi
	fs_status=`/usr/lpp/mmfs/bin/mmlsfileset $fs_device | awk '{ if ($1=="index") { print $2 } }'`
	fsstatus_rc=$?
	if [ $fsstatus_rc -ne 0 ]; then
		echo "Error $fsstatus_rc getting mmlsfileset information for '$fs_device'"
		return -1
	fi
	if [ "$fs_status" == "" ]; then
		echo "index fileset does not exist for '$fs_device'"
		return 0
	fi
	echo "index fileset exists for '$fs_device' with status $fs_status"

	#Now we check whether we are in searchonly mode.
	INDEX_MODE=`/opt/tivoli/tiam/bin/mw_print_xml_field collection/indexmode $COLLECTIONMNTPOINT/utility/config/collection.xml`

	if [ "$INDEX_MODE" == "" ]; then
		#Disabled - probably an old collection?
		echo "indexmode not defined in collection.xml. Disabled"
		return 0
	elif [ "$INDEX_MODE" == "searchonly" ]; then
		#Search only mode still needs the index broker running for deletes
		echo "In search only mode. Enabled for deletes"
		return 1
	elif [ "$INDEX_MODE" == "indexandsearch" ]; then
		#Enabled
		echo "In indexandsearch mode. Enabled" 
		return 1
	elif [ "$INDEX_MODE" == "disabled" ]; then
		echo "In disabled mode."
		return 0
	else
		echo "indexmode '$INDEX_MODE' corrupt in collection.xml or unable to read collection.xml"
		return -1
	fi
}

doGetOperationalState()
{
	#For the index broker, if it's healthy, it's operational
	doGetHealthyState
	return $?
}

doGetHealthyState()
{
	running_collection_pid_list=`/bin/ps -C \`basename $SSM_JAVA_BIN\` -f | tail +2 | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`

	for indexbroker_pid in $running_collection_pid_list
	do
		indexbroker_env=`/bin/ps -p $indexbroker_pid -f e | tail +2 | tr -s ' ' | sed 's/^[ ]//g' `
		for env_var in $indexbroker_env
		do
			mount_name=${env_var##IB_COLLECTIONMNTPOINT=}
			if [ x$mount_name != x$env_var ]; then
				if [ "$mount_name" = "$COLLECTIONMNTPOINT" ]; then
					return 1
				fi
			fi
		done
	done

	return 0
}

parse_args $@
echo "operation=$OPERATION"
echo "collectionmntpoint=$COLLECTIONMNTPOINT"
if [ ! -z "$COLLECTIONMNTPOINT" ]
then
     SSM_JETTY_PORT=`${PRINT_XML_FIELD_BIN} collection/solr/jettyport $COLLECTIONMNTPOINT/utility/config/collection.xml`
     SSM_JETTY_IPNAME=`${PRINT_XML_FIELD_BIN} collection/solr/jettyip $COLLECTIONMNTPOINT/utility/config/collection.xml`
     SSM_JETTY_IP=`awk -v jettyipname=$SSM_JETTY_IPNAME {'if($2==jettyipname){print $1}'} /etc/hosts`
     SSM_INDEXBROKER_PORT=`${PRINT_XML_FIELD_BIN} collection/solr/indexbrokerport $COLLECTIONMNTPOINT/utility/config/collection.xml`
fi
case ${OPERATION} in
	start)
		doStart
		start_retval=$?
		if [ $start_retval -eq 0 ]; then
			exit 0
		else
			if [ $start_retval -eq $ERR_FILE_DOES_NOT_EXIST ]; then
				echo "start on Solr Server issued but $SSM_JAVA_BIN or "
                                echo "$SSM_INDEXBROKER_JAR does not exist. "
			elif [ $start_retval -eq $ERR_FILE_NOT_EXECUTABLE ]; then
				echo "start on Index Broker issued but $SSM_JAVA_BIN is not executable."
                        elif [ $start_retval -eq $ERR_BAD_PROBLEM ]; then
                                echo "start on Index Broker issued but problem with jetty port: $SSM_JETTY_PORT or indexbrokerport: #SSM_INDEXBROKER_PORT"
			elif [ $start_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
				echo "start on Index Broker issued but $SSM_JAVA_BIN is already running"
			elif [ $start_retval -eq $ERR_PROCESS_MULTIPLE_RUNNING ]; then
				echo "start on Index Broker issued but multiple $SSM_JAVA_BIN are already running"
			elif [ $start_retval -eq $ERR_PROCESS_NOT_HEALTHY ]; then
				echo "start on Index Broker issued but $SSM_JAVA_BIN (which is unhealthy) is already running"
			fi

			exit $start_retval
		fi
		;;
	status)
		doStatus
		status_retval=$?
		if [ $status_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
			exit 0
		else
			exit $status_retval
		fi
		;;
	stop)
		doStop
		stop_retval=$?
		if [ $stop_retval -eq 0 ]; then
			exit 0
		else
			if [ $stop_retval -eq $ERR_PROCESS_NOT_RUNNING ]; then
				echo "stop on Index Broker issued for $SSM_JAVA_BIN process but it was not running"
			fi
			exit $stop_retval
		fi
		;;
	update)
		doUpdate
		update_retval=$?
		exit $update_retval
		;;
	getrunningstate)
		doGetRunningState
		getrunningstate_retval=$?
		exit $getrunningstate_retval
		;;
	gethealthystate)
		doGetHealthyState
		gethealthystate_retval=$?
		exit $gethealthystate_retval
		;;
	getenabledstate)
		doGetEnabledState
		getenabledstate_retval=$?
		exit $getenabledstate_retval
		;;
	getoperationalstate)
		doGetOperationalState
		getoperationalstate_retval=$?
		exit $getoperationalstate_retval
		;;
	*)
		printHelp
		;;
esac
