#!/bin/sh
########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Workload Manager
# *
# * FILE NAME:       wlm_controlingestcoordinator.sh
# *
# * DESCRIPTION:     Controls the ingest coordinator component
# *                  of a FRAnK collection
# *
# * AUTHOR:          Mark A. Smith
# *                  IBM Almaden Research Center
# *                  mark1smi@us.ibm.com
# *
# *                  Michael Richmond
# *                  IBM Almaden Research Center
# *                  mar@almaden.ibm.com
# *
# ****************************************************************************
# */

# Configure shell variables
. `dirname $BASH_SOURCE`/mw_environment.sh

# Read the common error values
. `dirname $BASH_SOURCE`/wlm_controlerrors.sh

# Read the common control functions
. `dirname $BASH_SOURCE`/wlm_controlcommon.sh

##############################################################################
## Control debug level for ingest_coordinator.
##   disable debugging for ingest_coordinator 
DEBUG="n"
#   enable debugging for ingest_coordinator 
#DEBUG="y"

# Locations of files and executables required for ingest_coordinator
INGESTCOORDINATOR_NAME=ingest_coordinator
INGESTCOORDINATOR_BIN=/opt/tivoli/tiam/bin/$INGESTCOORDINATOR_NAME

#in seconds. Give it a minute
INGESTCOORDINATOR_STOP_TIMEOUT=60

#give the process 2 minutes to appear in ps output.
PROCESS_TABLE_APPEARANCE_TIMEOUT=120


doStart()
{

	check_file_exists $INGESTCOORDINATOR_BIN
	b_exists=$?
	if [ "$b_exists" != 1 ]; then
		return $ERR_FILE_DOES_NOT_EXIST
	fi
	check_file_executable $INGESTCOORDINATOR_BIN
	b_is_executable=$?
	if [ "$b_is_executable" != 1 ]; then
		return $ERR_FILE_NOT_EXECUTABLE
	fi

	doStatus
	running_status=$?

	if [ $running_status -ne $ERR_PROCESS_NOT_RUNNING ]; then
		return $running_status
	fi

	if [ "${DEBUG}" = "y" ]; then
		DEBUG_FLAG="-d"
	else
		DEBUG_FLAG=""
	fi	

	export COLLECTIONMNTPOINT
	${INGESTCOORDINATOR_BIN} ${DEBUG_FLAG} ${COLLECTIONMNTPOINT} > /dev/null 2>&1 &

	b_appeared_in_process_table=0
	waited_seconds=0
	timeout_seconds=$PROCESS_TABLE_APPEARANCE_TIMEOUT
	wait_interval=10
	start_time=`date +%s`
	while [ $waited_seconds -lt $timeout_seconds ]
	do
		this_start_time=`date +%s`

		doStatus
		running_status=$?

		if [ $running_status -eq $ERR_PROCESS_IS_RUNNING ]; then
			b_appeared_in_process_table=1
			echo "ingest_coordinator appeared in the process table."
			break
		fi

		echo "ingest_coordator has not yet appeared in the process table. Sleeping $wait_interval seconds and looking again. Error Code: $running_status"
		now=`date +%s`
		waited_this_time=$((now-this_start_time))
		if [ $waited_this_time -lt $wait_interval ]; then
			more_sleep_needed=$((wait_interval-waited_this_time))
			sleep $more_sleep_needed
		fi
		now=`date +%s`
		waited_seconds=$((now-start_time))
	done

	if [ $b_appeared_in_process_table -eq 0 ]; then
		echo "ingest_coordinator never appeared in the process table."
		doStop
		return $running_status
	fi

	return 0
}

doStatus()
{
	num_running=0

	running_collection_pid_list=`/bin/ps -C \`basename $INGESTCOORDINATOR_BIN\` -f | tail +2 | sed 's/ -d / /g' | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`

	for ingest_coordinator_pid in $running_collection_pid_list
	do
		ic_environment=`/bin/ps -p $ingest_coordinator_pid -f e | tail +2 | tr -s ' ' | sed 's/^[ ]//g' `
		for env_var in $ic_environment
		do
			mount_name=${env_var##COLLECTIONMNTPOINT=}
			if [ x$mount_name != x$env_var ]; then
				if [ "$mount_name" = "$COLLECTIONMNTPOINT" ]; then
					echo "found pid $ingest_coordinator_pid running"
					if [ $num_running -eq 0 ]; then
						num_running=`expr $num_running + 1`
					else
						return $ERR_PROCESS_MULTIPLE_RUNNING
					fi
				fi
			fi
		done
	done


	if [ $num_running -eq 0 ]; then
		return $ERR_PROCESS_NOT_RUNNING
	elif [ $num_running -eq 1 ]; then
		return $ERR_PROCESS_IS_RUNNING
	fi
}

doStop()
{
	doStatus
	running_status=$?

	if [ $running_status -eq $ERR_PROCESS_NOT_RUNNING ]; then
		/opt/tivoli/tiam/bin/mw_abortmmapplypolicy.sh --mountpoint=$COLLECTIONMNTPOINT
		#The following line was added for 13740
		/opt/tivoli/tiam/bin/mw_abortdocumentexpirer.sh --mountpoint=$COLLECTIONMNTPOINT
		return $running_status
	fi

	running_collection_pid_list=`/bin/ps -C \`basename $INGESTCOORDINATOR_BIN\` -f | tail +2 | sed 's/ -d / /g' | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`

	for ingest_coordinator_pid in $running_collection_pid_list
	do
		ic_environment=`/bin/ps -p $ingest_coordinator_pid -f e | tail +2 | tr -s ' ' | sed 's/^[ ]//g' `
		for env_var in $ic_environment
		do
			mount_name=${env_var##COLLECTIONMNTPOINT=}
			if [ x$mount_name != x$env_var ]; then
				if [ "$mount_name" = "$COLLECTIONMNTPOINT" ]; then
					kill_process $ingest_coordinator_pid SIGKILL
				fi
			fi
		done
	done

	waited_seconds=0
	timeout_seconds=$INGESTCOORDINATOR_STOP_TIMEOUT
	wait_interval=1
	start_time=`date +%s`
	while [ $waited_seconds -lt $timeout_seconds ]
	do
		this_start_time=`date +%s`

		doStatus
		running_status=$?

		if [ $running_status -eq $ERR_PROCESS_NOT_RUNNING ]; then
			/opt/tivoli/tiam/bin/mw_abortmmapplypolicy.sh --mountpoint=$COLLECTIONMNTPOINT
			#The following line was added for 13740
			/opt/tivoli/tiam/bin/mw_abortdocumentexpirer.sh --mountpoint=$COLLECTIONMNTPOINT
			return 0
		fi

		now=`date +%s`
		waited_this_time=$((now-this_start_time))
		if [ $waited_this_time -lt $wait_interval ]; then
			more_sleep_needed=$((wait_interval-waited_this_time))
			sleep $more_sleep_needed
		fi
		now=`date +%s`
		waited_seconds=$((now-start_time))
	done

	return $ERR_PROCESS_FAILED_TO_DIE
}

doUpdate()
{
	#Nothing to do for ingest coordinator
	return 0
}

doGetRunningState()
{
	running_collection_pid_list=`/bin/ps -C \`basename $INGESTCOORDINATOR_BIN\` -f | tail +2 | sed 's/ -d / /g' | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`

	for ingest_coordinator_pid in $running_collection_pid_list
	do
		ic_environment=`/bin/ps -p $ingest_coordinator_pid -f e | tail +2 | tr -s ' ' | sed 's/^[ ]//g' `
		for env_var in $ic_environment
		do
			mount_name=${env_var##COLLECTIONMNTPOINT=}
			if [ x$mount_name != x$env_var ]; then
				if [ "$mount_name" = "$COLLECTIONMNTPOINT" ]; then
					return 1
				fi
			fi
		done
	done

	return 0
}

doGetEnabledState()
{
	return 1
}

doGetOperationalState()
{
	#For the ingest coordinator, if it's healthy, it's operational
	doGetHealthyState
	return $?
}

doGetHealthyState()
{
	running_collection_pid_list=`/bin/ps -C \`basename $INGESTCOORDINATOR_BIN\` -f | tail +2 | sed 's/ -d / /g' | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`

	for ingest_coordinator_pid in $running_collection_pid_list
	do
		ic_environment=`/bin/ps -p $ingest_coordinator_pid -f e | tail +2 | tr -s ' ' | sed 's/^[ ]//g' `
		for env_var in $ic_environment
		do
			mount_name=${env_var##COLLECTIONMNTPOINT=}
			if [ x$mount_name != x$env_var ]; then
				if [ "$mount_name" = "$COLLECTIONMNTPOINT" ]; then
					echo "found pid $ingest_coordinator_pid running"
					return 1
				fi
			fi
		done
	done

	return 0
}

parse_args $@

case ${OPERATION} in
	start)
		doStart
		start_retval=$?
		if [ $start_retval -eq 0 ]; then
			exit 0
		else
			if [ $start_retval -eq $ERR_FILE_DOES_NOT_EXIST ]; then
				echo "start issued but $INGESTCOORDINATOR_BIN does not exist."
			elif [ $start_retval -eq $ERR_FILE_NOT_EXECUTABLE ]; then
				echo "start issued but $INGESTCOORDINATOR_BIN is not executable."
			elif [ $start_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
				echo "start issued but $INGESTCOORDINATOR_BIN is already running"
			elif [ $start_retval -eq $ERR_PROCESS_MULTIPLE_RUNNING ]; then
				echo "start issued but multiple $INGESTCOORDINATOR_BIN are already running"
			elif [ $start_retval -eq $ERR_PROCESS_NOT_HEALTHY ]; then
				echo "start issued but $INGESTCOORDINATOR_BIN (which is unhealthy) is already running"
			fi

			exit $start_retval
		fi
		;;
	status)
		doStatus
		status_retval=$?
		if [ $status_retval -eq $ERR_PROCESS_IS_RUNNING ]; then
			exit 0
		else
			exit $status_retval
		fi
		;;
	stop)
		doStop
		stop_retval=$?
		if [ $stop_retval -eq 0 ]; then
			exit 0
		else
			if [ $stop_retval -eq $ERR_PROCESS_NOT_RUNNING ]; then
				echo "stop issued for $INGESTCOORDINATOR_BIN but it was not running"
			fi
			exit $stop_retval
		fi
		;;
	update)
		doUpdate
		update_retval=$?
		exit $update_retval
		;;
	getrunningstate)
		doGetRunningState
		getrunningstate_retval=$?
		exit $getrunningstate_retval
		;;
	gethealthystate)
		doGetHealthyState
		gethealthystate_retval=$?
		exit $gethealthystate_retval
		;;
	getenabledstate)
		doGetEnabledState
		getenabledstate_retval=$?
		exit $getenabledstate_retval
		;;
	getoperationalstate)
		doGetOperationalState
		getoperationalstate_retval=$?
		exit $getoperationalstate_retval
		;;
	*)
		printHelp
		exit -1
		;;
esac
