#!/bin/sh
########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Workload Manager
# *
# * FILE NAME:       wlm_startcollectiom.sh
# *
# * DESCRIPTION:     Starts managing a collection
# *
# * AUTHOR:          Mark A. Smith
# *                  IBM Almaden Research Center
# *                  mark1smi@us.ibm.com
# *
# ****************************************************************************
# */

# Configure shell variables
. `dirname $BASH_SOURCE`/mw_environment.sh

# Requires one argument.
# The first argument is the mount point of the collection to add

[ $1 ] || exit -1
COLLECTIONMNTPOINT=$1

startCollection()
{
	local collection_mount=$1
	local collection_type=$2

	rc=0

	surrogate_lock_dir=/tiam_utility/surrogate_locks/dm/$collection_mount
	mkdir -p $surrogate_lock_dir
	chmod 777 $surrogate_lock_dir
	chmod_rc=$?
	if [ $chmod_rc -ne 0 ]; then
		echo "Unable to create surrogate_lock dir" $surrogate_lock_dir "for" $collection_mount "error" $chmod_rc
		rc=-1
		return $rc
	fi

	component_occurrences_cmd="/opt/tivoli/tiam/bin/mw_print_xml_children DEFINITIONS/$collection_type /tiam_utility/config/collection_definitions.xml"
	component_occurrences=`$component_occurrences_cmd 2>&1`

	component_occurrences_rc=$?
	if [ $component_occurrences_rc -ne 0 ]; then
		echo "Unable to determine number of components in collection using command '" $component_occurrences_cmd "' command output = '" $component_occurrences "'"
		rc=-1
		return $rc
	fi

	num_components=0
	for compstring in $component_occurrences;
	do
		num_components=`expr $num_components + 1`
	done

	echo "The command '$component_occurrences_cmd' yielded $num_components components"

	for comp_num in `seq 1 $num_components`;
	do
		component_name_cmd="/opt/tivoli/tiam/bin/mw_print_xml_field DEFINITIONS/$collection_type/COMPONENT[$comp_num]/NAME /tiam_utility/config/collection_definitions.xml"
		component_name=`$component_name_cmd 2>&1`
		component_name_rc=$?
		if [ $component_name_rc -ne 0 ]; then
			echo "Unable to determine component name using '$component_name_cmd': output '$component_name'"
			rc=-1
			return $rc
		fi

		component_description_cmd="/opt/tivoli/tiam/bin/mw_print_xml_field DEFINITIONS/$collection_type/COMPONENT[$comp_num]/DESCRIPTION /tiam_utility/config/collection_definitions.xml"
		component_description=`$component_description_cmd 2>&1`
		component_description_rc=$?
		if [ $component_description_rc -ne 0 ]; then
			echo "Unable to determine component description using '$component_description_cmd': output '$component_description"
			rc=-1
			return $rc
		fi
		

		component_control_script_cmd="/opt/tivoli/tiam/bin/mw_print_xml_field DEFINITIONS/$collection_type/COMPONENT[$comp_num]/CONTROL /tiam_utility/config/collection_definitions.xml"
		component_control_script=`$component_control_script_cmd 2>&1`
		component_control_rc=$?
		if [ $component_control_rc -ne 0 ]; then
			echo "Unable to determine component control script using '$component_control_script_cmd': output '$component_control_script'"
			rc=-1
			return $rc
		fi

		component_type_cmd="/opt/tivoli/tiam/bin/mw_print_xml_field DEFINITIONS/$collection_type/COMPONENT[$comp_num]/TYPE /tiam_utility/config/collection_definitions.xml"
		component_type=`$component_type_cmd 2>&1`
		component_type_rc=$?
		if [ $component_type_rc -ne 0 ]; then
			echo "Unable to determine component type using '$component_type_cmd': output '$component_type'"
			rc=-1
			return $rc
		fi


		if [ "$component_type" = "symmetric" ]; then
			echo "Starting" $component_name "( symmetric ) for" $collection_mount
			export COLLECTIONMNTPOINT
			/opt/tivoli/tiam/bin/manage_process --mountpoint=$collection_mount --controlscript=$component_control_script --updatefilename=$collection_mount/freshness.guarantee --processname=$component_name -d
			startup_rc=$?
			if [ $startup_rc -ne 0 ]; then
				echo "Unable to start" $component_name "for" $collection_mount
				rc=-1
			fi
			
		elif [ "$component_type" = "singleton" ]; then
			component_group=`/opt/tivoli/tiam/bin/mw_print_xml_field DEFINITIONS/$collection_type/COMPONENT[$comp_num]/BALANCING/GROUP /tiam_utility/config/collection_definitions.xml`
			lockfile=$collection_mount/locks/$component_name
			mkdir -p $collection_mount/locks
			if [ ! -f $lockfile ]; then
				touch $lockfile
			fi
			echo "Starting" $component_name "( singleton in group" $component_group") for" $collection_mount
			export COLLECTIONMNTPOINT
			/opt/tivoli/tiam/bin/manage_process --mountpoint=$collection_mount --controlscript=$component_control_script --updatefilename=$collection_mount/freshness.guarantee --processname=$component_name --lockfile=$lockfile --load_calculator=/opt/tivoli/tiam/bin/wm_calc_load --group=$component_group -d
			startup_rc=$?
			if [ $startup_rc -ne 0 ]; then
				echo "Unable to start" $component_name "for" $collection_mount
				rc=-1
			fi
		else
			echo "Unknown component type: '$component_type'"
			rc=-1
		fi
	done

	return $rc
}

COLLECTION_TYPE=`/opt/tivoli/tiam/bin/mw_print_xml_field collection/collectiontype $COLLECTIONMNTPOINT/utility/config/collection.xml`

if [ "$COLLECTION_TYPE" = "frank" ]; then
	startCollection $COLLECTIONMNTPOINT $COLLECTION_TYPE
	frank_retval=$?
	if [ $frank_retval != 0 ]; then
		echo "Error starting FRAnK collection " $COLLECTIONMNTPOINT " Error: " $frank_retval
	else
		echo "Successfully started FRAnK collection " $COLLECTIONMNTPOINT
	fi
elif [ "$COLLECTION_TYPE" = "ssam" ]; then
	startCollection $COLLECTIONMNTPOINT $COLLECTION_TYPE
	ssam_retval=$?
	if [ $ssam_retval != 0 ]; then
		echo "Error starting SSAM collection " $COLLECTIONMNTPOINT " Error: " $ssam_retval
	else
		echo "Successfully started SSAM collection " $COLLECTIONMNTPOINT
	fi
else
	echo "Unknown collection type: " $COLLECTION_TYPE
	startCollection $COLLECTIONMNTPOINT $COLLECTION_TYPE
	unknown_retval=$?
	if [ $unknown_retval != 0 ]; then
		echo "Error starting $COLLECTION_TYPE collection " $COLLECTIONMNTPOINT " Error: " $unknown_retval
	else
		echo "Successfully started $COLLECTION_TYPE collection " $COLLECTIONMNTPOINT
	fi
fi
