#!/bin/sh
########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Workload Manager
# *
# * FILE NAME:       wlm_stopcollectiom.sh
# *
# * DESCRIPTION:     Stops a collection and its management
# *
# * AUTHOR:          Mark A. Smith
# *                  IBM Almaden Research Center
# *                  mark1smi@us.ibm.com
# *
# ****************************************************************************
# */

# Configure shell variables
. `dirname $BASH_SOURCE`/mw_environment.sh

# Read the common error values
. `dirname $BASH_SOURCE`/wlm_controlerrors.sh

# Requires one argument.
# The first argument is the mount point of the collection to stop

[ $1 ] || exit -1
COLLECTIONMNTPOINT=$1

stopManagersForCollection()
{
	echo "Stopping process managers for collection" $COLLECTIONMNTPOINT
	running_managers_pid_list=`/bin/ps -C manage_process -f | tail +2 | tr -s ' ' | sed 's/^[ ]//g' | cut -d' ' -f 2`
	for manager_pid in $running_managers_pid_list
	do
		b_is_correct_collection=0
		manager_environment=`/bin/ps -p $manager_pid -f e | tail +2 | tr -s ' ' | sed 's/^[ ]//g' `
		for env_var in $manager_environment
		do
			mount_name=${env_var##COLLECTIONMNTPOINT=}
			if [ x$mount_name != x$env_var ]; then
				if [ "$mount_name" = "$COLLECTIONMNTPOINT" ]; then
					b_is_correct_collection=1
				fi
			fi
		done
		if [ $b_is_correct_collection -eq 1 ]; then
			echo "Killing process manager with pid" $manager_pid
			kill $manager_pid
		fi
	done
}

stopCollection()
{
	local collection_mount=$1
	local collection_type=$2

	stopManagersForCollection

	rc=0

	component_occurrences_cmd="/opt/tivoli/tiam/bin/mw_print_xml_children DEFINITIONS/$collection_type /tiam_utility/config/collection_definitions.xml"
	component_occurrences=`$component_occurrences_cmd 2>&1`

	component_occurrences_rc=$?
	if [ $component_occurrences_rc -ne 0 ]; then
		echo "Unable to determine number of components in collection using command '" $component_occurrences_cmd "' command output = '" $component_occurrences "'"
		rc=-1
		return $rc
	fi

	num_components=0
	for compstring in $component_occurrences;
	do
		num_components=`expr $num_components + 1`
	done

	echo "The command '$component_occurrences_cmd' yielded $num_components components"

	for comp_num in `seq $num_components -1 1`;
	do
		component_name_cmd="/opt/tivoli/tiam/bin/mw_print_xml_field DEFINITIONS/$collection_type/COMPONENT[$comp_num]/NAME /tiam_utility/config/collection_definitions.xml"
		component_name=`$component_name_cmd 2>&1`
		component_name_rc=$?
		if [ $component_name_rc -ne 0 ]; then
			echo "Unable to determine component name using '$component_name_cmd': output '$component_name'"
		fi

		component_control_script_cmd="/opt/tivoli/tiam/bin/mw_print_xml_field DEFINITIONS/$collection_type/COMPONENT[$comp_num]/CONTROL /tiam_utility/config/collection_definitions.xml"
		component_control_script=`$component_control_script_cmd 2>&1`
		component_control_rc=$?
		if [ $component_control_rc -ne 0 ]; then
			echo "Unable to determine component control script using '$component_control_script_cmd': output '$component_control_script'"
			rc=-1
			return $rc
		fi

		component_type_cmd="/opt/tivoli/tiam/bin/mw_print_xml_field DEFINITIONS/$collection_type/COMPONENT[$comp_num]/TYPE /tiam_utility/config/collection_definitions.xml"
		component_type=`$component_type_cmd 2>&1`
		component_type_rc=$?
		if [ $component_type_rc -ne 0 ]; then
			echo "Unable to determine component type using '$component_type_cmd': output '$component_type'"
		fi

		stop_cmd="$component_control_script -stop --mountpoint=$collection_mount"

		echo "Stopping $component_name ($component_type) for $collection_mount using command '$stop_cmd'"

		stop_output=`$stop_cmd 2>&1`
		stop_rc=$?
		if [ $stop_rc -ne 0 ]; then
			#No problem if it wasn't running anyway.
			if [ $stop_rc -ne $ERR_PROCESS_NOT_RUNNING ]; then
				echo "Unable to stop $component_name ($component_type) for $collection_mount return code=$stop_rc, output='$stop_output'"
				rc=-1
			fi
		fi
	done

	return $rc

}

COLLECTION_TYPE=`/opt/tivoli/tiam/bin/mw_print_xml_field collection/collectiontype $COLLECTIONMNTPOINT/utility/config/collection.xml`

if [ "$COLLECTION_TYPE" = "frank" ]; then
	stopCollection $COLLECTIONMNTPOINT $COLLECTION_TYPE
	frank_retval=$?
	if [ $frank_retval != 0 ]; then
		echo "Error removing FRAnK collection " $COLLECTIONMNTPOINT " Error: " $frank_retval
	else
		echo "Successfully stopped FRAnK collection " $COLLECTIONMNTPOINT
	fi
elif [ "$COLLECTION_TYPE" = "ssam" ]; then
	stopCollection $COLLECTIONMNTPOINT $COLLECTION_TYPE
	ssam_retval=$?
	if [ $ssam_retval != 0 ]; then
		echo "Error removing SSAM collection " $COLLECTIONMNTPOINT " Error: " $ssam_retval
	else
		echo "Successfully stopped SSAM collection " $COLLECTIONMNTPOINT
	fi
else
	echo "Unknown collection type: " $COLLECTION_TYPE
	stopCollection $COLLECTIONMNTPOINT $COLLECTION_TYPE
	unknown_retval=$?
	if [ $unknown_retval != 0 ]; then
		echo "Error removing $COLLECTION_TYPE collection " $COLLECTIONMNTPOINT " Error: " $unknown_retval
	else
		echo "Successfully stopped $COLLECTION_TYPE collection " $COLLECTIONMNTPOINT
	fi
fi
