#!/bin/sh
########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Workload Manager
# *
# * FILE NAME:       workloadmanager.sh
# *
# * DESCRIPTION:     The workload manager interface
# *
# * AUTHOR:          Mark A. Smith
# *                  IBM Almaden Research Center
# *                  mark1smi@us.ibm.com
# *
# ****************************************************************************
# */

# Configure shell variables
. `dirname $BASH_SOURCE`/mw_environment.sh

# Read the common interface functions
. `dirname $BASH_SOURCE`/wlm_interfacecommon.sh

WLM_LOAD_INFO_DIR="/tiam_utility/load"
COLLECTIONS_FILE_DIR="/tiam_utility/config"
COLLECTIONS_FILE=$COLLECTIONS_FILE_DIR"/collections.middleware.xml"
TMP_COLLECTIONS_FILE=$COLLECTIONS_FILE_DIR"/collections.middleware.temp.xml"
ROOT_EUID=0

createCollectionsFile()
{
	mkdir -p $COLLECTIONS_FILE_DIR
	echo "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" > $TMP_COLLECTIONS_FILE
	echo "<collections>" >> $TMP_COLLECTIONS_FILE
	echo "</collections>" >> $TMP_COLLECTIONS_FILE
	mv $TMP_COLLECTIONS_FILE $COLLECTIONS_FILE
}

#return 0 for success
#return -1 for failure
#return 1 if collection already defined
doAddCollection()
{
	check_file_exists $COLLECTIONS_FILE
	b_exists=$?
	if [ $b_exists -ne 1 ]; then
		createCollectionsFile
	fi

	collections=`/opt/tivoli/tiam/bin/wlm_enumeratecollections.sh $COLLECTIONS_FILE`
	for collection in $collections
	do
		if [ "$COLLECTIONMNTPOINT" == "$collection" ]; then
			echo "Collection" $COLLECTIONMNTPOINT "already defined"
			return 1
		fi
	done

	while read line
	do
		if [ "$line" == "</collections>" ]; then
			echo "<mount>"$COLLECTIONMNTPOINT"</mount>" >> $TMP_COLLECTIONS_FILE
			if [ $? -ne 0 ]; then
				echo "Unable to write temporary file" $TMP_COLLECTIONS_FILE "to add collection " $COLLECTIONMNTPOINT
				return -1
			fi
			echo $line >> $TMP_COLLECTIONS_FILE
			if [ $? -ne 0 ]; then
				echo "Unable to write temporary file" $TMP_COLLECTIONS_FILE "to add collection " $COLLECTIONMNTPOINT
				return -1
			fi
		else
			echo $line >> $TMP_COLLECTIONS_FILE
			if [ $? -ne 0 ]; then
				echo "Unable to write temporary file" $TMP_COLLECTIONS_FILE "to add collection " $COLLECTIONMNTPOINT
				return -1
			fi
		fi
	done < "$COLLECTIONS_FILE"

	mv $TMP_COLLECTIONS_FILE $COLLECTIONS_FILE
	if [ $? -ne 0 ]; then
		echo "Unable to mv" $TMP_COLLECTIONS_FILE "to" $COLLECTIONS_FILE "to add collection"
		return -1
	fi
	return 0
}

#return 0 for success
#return -1 for failure
#return 1 if collection is not defined
#return 2 if collection is enabled
doRemoveCollection()
{
	check_file_exists $COLLECTIONS_FILE
	b_exists=$?
	if [ $b_exists -ne 1 ]; then
		return -1
	fi

	COL_ENABLED=`/opt/tivoli/tiam/bin/mw_print_xml_field collection/enabled $COLLECTIONMNTPOINT/utility/config/collection.xml`

	if [ "$COL_ENABLED" == "true" ]; then
		echo "Collection" $COLLECTIONMNTPOINT "is enabled. Cannot remove enabled collections."
		return 2
	fi

	origmd5=`md5sum $COLLECTIONS_FILE | awk '{print $1}'`

	while read line
	do
		if [ "$line" != "<mount>"$COLLECTIONMNTPOINT"</mount>" ]; then
			echo $line >> $TMP_COLLECTIONS_FILE
			if [ $? -ne 0 ]; then
				echo "Unable to write temporary file" $TMP_COLLECTIONS_FILE "to remove collection " $COLLECTIONMNTPOINT
				return -1
			fi
		fi
	done < "$COLLECTIONS_FILE"

	mv $TMP_COLLECTIONS_FILE $COLLECTIONS_FILE
	if [ $? -ne 0 ]; then
		echo "Unable to mv" $TMP_COLLECTIONS_FILE "to" $COLLECTIONS_FILE "to add collection"
		return -1
	fi

	newmd5=`md5sum $COLLECTIONS_FILE | awk '{print $1}'`

	if [ "$newmd5" == "$origmd5" ]; then
		echo "Collection" $COLLECTIONMNTPOINT "not defined"
		return 1
	fi

	return 0
}

doCleanupCollection()
{
	retval=0

	#/tiam_utility/load/
	rm -rf $WLM_LOAD_INFO_DIR/collections/$COLLECTIONMNTPOINT
	rm -rf /tiam_utility/$COLLECTIONMNTPOINT

	return $retval
}

doUpdateCollection()
{
	# manage_process watches /collection_mount/freshness.guarantee
	echo `date +%Y%m%d%H%M%S` > /$COLLECTIONMNTPOINT/freshness.guarantee
	return $?
}

doNodeStatus()
{
	local node=$1
	node_load_dir="/tiam_utility/load/nodes"

	echo -n "    <NODE ID=\"$node\" "

	node_ip=`/opt/tivoli/tiam/bin/mw_print_xml_field node/ip $node_load_dir/$node`
	echo -n "IP=\"$node_ip\" "

	node_hostname=`/opt/tivoli/tiam/bin/mw_print_xml_field node/hostname $node_load_dir/$node`
	echo -n "HOSTNAME=\"$node_hostname\" "

	/opt/tivoli/tiam/bin/ia_administrativeleave -o $node_hostname 2>&1 >>/dev/null
	on_leave=$?
	if [ $on_leave -eq 0 ]; then
		echo -n "ON_LEAVE=\"true\" "
		/opt/tivoli/tiam/bin/ia_administrativeleave -a $node_hostname 2>&1 >>/dev/null
		automatic_leave=$?
		if [ $automatic_leave -eq 0 ]; then
			echo -n "AUTOMATIC_LEAVE=\"true\" "
		else
			echo -n "AUTOMATIC_LEAVE=\"false\" "
		fi
	else
		echo -n "ON_LEAVE=\"false\" "
	fi

	/opt/tivoli/tiam/bin/mw_test_lock $node_load_dir/$node
	collection_manager_healthy=$?
	if [ $collection_manager_healthy -eq 1 ]; then
		echo -n "MANAGING_COLLECTIONS=\"true\" "
	else
		echo -n "MANAGING_COLLECTIONS=\"false\" "
	fi
	echo "/>"
}

doNodesStatus()
{
	node_load_dir="/tiam_utility/load/nodes"
	nodes=`ls $node_load_dir | grep -v "refreshcount"`

	echo "  <NODES>"

	for node in $nodes
	do
		doNodeStatus $node
	done

	echo "  </NODES>"
}

doCollectionStatus()
{
	local mountpoint=$1
	collection_load_dir="/tiam_utility/load/collections/"$mountpoint
	node_load_dir="/tiam_utility/load/nodes"

	if [ "$mountpoint" == "" ]; then
		return
	fi

	COL_TYPE=`/opt/tivoli/tiam/bin/mw_print_xml_field collection/collectiontype $mountpoint/utility/config/collection.xml`
	COL_NAME=`/opt/tivoli/tiam/bin/mw_print_xml_field collection/name $mountpoint/utility/config/collection.xml`
	COL_ENABLED=`/opt/tivoli/tiam/bin/mw_print_xml_field collection/enabled $mountpoint/utility/config/collection.xml`
	INDEX_MODE=`/opt/tivoli/tiam/bin/mw_print_xml_field collection/indexmode $mountpoint/utility/config/collection.xml`

	if [ "$INDEX_MODE" == "" ]; then
		INDEX_MODE_TEXT="DISABLED"
	elif [ "$INDEX_MODE" == "disabled" ]; then
		INDEX_MODE_TEXT="DISABLED"
	elif [ "$INDEX_MODE" == "searchonly" ]; then
		INDEX_MODE_TEXT="SEARCH_ONLY"
	elif [ "$INDEX_MODE" == "indexandsearch" ]; then
		INDEX_MODE_TEXT="INDEX_AND_SEARCH"
	else
		INDEX_MODE_TEXT="UNKNOWN"
	fi


	if [ "$COL_TYPE" == "" -o "$COL_NAME" == "" -o "$COL_ENABLED" == "" ]; then
		echo "    <COLLECTION MOUNT=\"$mountpoint\" NAME=\"$COL_NAME\" TYPE=\"$COL_TYPE\" ENABLED=\"$COL_ENABLED\" INDEX_MODE=\"$INDEX_MODE_TEXT\" ERROR=\"true\">"
		echo "    </COLLECTION>"
		return
	else
		echo "    <COLLECTION MOUNT=\"$mountpoint\" NAME=\"$COL_NAME\" TYPE=\"$COL_TYPE\" ENABLED=\"$COL_ENABLED\" INDEX_MODE=\"$INDEX_MODE_TEXT\">"
	fi

	nodes=`ls $collection_load_dir`
	for node in $nodes
	do

		echo -n "      <NODE ID=\"$node\" "
		echo ">"

		num_components=`echo "ls DEFINITIONS/$COL_TYPE" | xmllint --shell /tiam_utility/config/collection_definitions.xml | grep COMPONENT | wc -l`
		for comp_num in `seq 1 $num_components`;
		do
			component_name=`/opt/tivoli/tiam/bin/mw_print_xml_field DEFINITIONS/$COL_TYPE/COMPONENT[$comp_num]/NAME /tiam_utility/config/collection_definitions.xml`
			component_description=`/opt/tivoli/tiam/bin/mw_print_xml_field DEFINITIONS/$COL_TYPE/COMPONENT[$comp_num]/DESCRIPTION /tiam_utility/config/collection_definitions.xml`
			component_type=`/opt/tivoli/tiam/bin/mw_print_xml_field DEFINITIONS/$COL_TYPE/COMPONENT[$comp_num]/TYPE /tiam_utility/config/collection_definitions.xml`
			check_file_exists $collection_load_dir/$node/$component_name
			b_exists=$?
			if [ $b_exists -ne 1 ]; then
				#If the load file doesn't exist, then the component's manage_process may never have run.
				#Treat as disabled until the process manager tells us otherwise
				enabled_status="false"
			else
				enabled_status=`/opt/tivoli/tiam/bin/mw_print_xml_field management/managee_is_enabled $collection_load_dir/$node/$component_name`
			fi

			echo -n "          <COMPONENT DESCRIPTION=\"$component_description\" "
			echo -n "NAME=\"$component_name\" "
			echo -n "TYPE=\"$component_type\" "
			if [ "$enabled_status" == "true" ]; then
				echo -n "ENABLED=\"true\" "
			else
				echo -n "ENABLED=\"false\" "
			fi
			/opt/tivoli/tiam/bin/mw_test_lock $collection_load_dir/$node/$component_name
			manager_healthy=$?
			if [ $manager_healthy -eq 1 ]; then
				echo -n "MANAGER_RUNNING=\"true\" "
				running_status=`/opt/tivoli/tiam/bin/mw_print_xml_field management/managee_is_running $collection_load_dir/$node/$component_name`
				if [ "$running_status" == "true" ]; then
					#A running component must be healthy or it is
					#brought down by the manager
					echo -n "RUNNING=\"true\" "
				else
					echo -n "RUNNING=\"false\" "
					if [ "$enabled_status" == "true" ]; then
						unsuccessful_start_attempts=`/opt/tivoli/tiam/bin/mw_print_xml_field management/managee_unsuccessful_starts $collection_load_dir/$node/$component_name`
						echo -n "UNSUCCESSFUL_STARTS=\"$unsuccessful_start_attempts\" "

						if [ "$unsuccessful_start_attempts" != "0" ]; then
							unsuccessful_start_attempt_time=`/opt/tivoli/tiam/bin/mw_print_xml_field management/managee_unsuccessful_start_attempt_time $collection_load_dir/$node/$component_name`
							echo -n "UNSUCCESSFUL_START_TIME=\"$unsuccessful_start_attempt_time\" "
						fi
					fi
					
				fi
			else
				echo -n "MANAGER_RUNNING=\"false\" "
			fi
			echo "/>"
		done

		echo "      </NODE>"
	done
	echo "    </COLLECTION>"
}

doCollectionsStatus()
{
	echo "  <COLLECTIONS>"
	collections=`/opt/tivoli/tiam/bin/wlm_enumeratecollections.sh $COLLECTIONS_FILE`
	for collection in $collections
	do
		doCollectionStatus $collection
	done
	echo "  </COLLECTIONS>"
}
doStatus()
{

	echo "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"

	echo "<STATUS>"

	doNodesStatus
	if [ "$COLLECTIONMNTPOINT" == "" ]; then
		doCollectionsStatus
	else
		echo "  <COLLECTIONS>"
		doCollectionStatus $COLLECTIONMNTPOINT
		echo "  </COLLECTIONS>"
	fi

	echo "</STATUS>"


}

if [ $EUID -ne $ROOT_EUID ]; then
        echo "This program must be run as root. Your effective uid is $EUID."
        exit -1
fi

parse_args $@

case ${OPERATION} in
	addcollection)
		doAddCollection
		addcollection_retval=$?
		if [ $addcollection_retval -eq 0 ]; then
			exit 0
		else
			exit $addcollection_retval
		fi
		;;
	updatecollection)
		doUpdateCollection
		updatecollection_retval=$?
		if [ $updatecollection_retval -eq 0 ]; then
			exit 0
		else
			exit $updatecollection_retval
		fi
		;;
	removecollection)
		doRemoveCollection
		removecollection_retval=$?
		if [ $removecollection_retval -eq 0 ]; then
			exit 0
		else
			exit $removecollection_retval
		fi
		;;
	cleanupcollection)
		doCleanupCollection
		cleanupcollection_retval=$?
		if [ $cleanupcollection_retval -eq 0 ]; then
			exit 0
		else
			exit $cleanupcollection_retval
		fi
		;;
	status)
		doStatus
		status_retval=$?
		if [ $status_retval -eq 0 ]; then
			exit 0
		else
			exit $status_retval
		fi
		;;
	*)
		printHelp
		;;
esac
