########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

# ----------------------------------------------------------------------------
# $Revision: $ $Date: $ $Author: $
# ----------------------------------------------------------------------------
#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Workload Manager
# *
# * FILE NAME:       iamiddleware/administrativeleave.py
# *
# * DESCRIPTION:     Library routines used by the IA Middleware to
# *                  manage and query the administrative_leave.xml file.
# *
# * AUTHOR:          Michael A. Richmond
# *                  IBM Almaden Research Center
# *                  mar@almaden.ibm.com
# *
# ****************************************************************************
# */
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
#              Change Log
#              ----------
# 09 Oct 2008  Initial version.
#
# 04 Dec 2008  Code moved into iamiddleware package.
#
# 13 Jan 2009  Updated to refer to nodeId's rather than nodeNames.
#              administrativeleave.xml file should only contain
#              middleware unique node ids (internal ip addresses).
#

import  xml
import  xml.dom.ext
import  xml.dom.minidom
import  xml.xpath


class AdministrativeLeaveController:
    """Handler class to abstract details of maintaining
       the IA Middleware administrative_leave.xml file.
       Class provides operations to place nodes on leave,
       take nodes off leave, and check the leave status
       for nodes. (All operations are based on the name
       of the node in the cluster where the name of the
       node is the string reported by `hostname` for
       the node on the internal (private) cluster network.
    """

    __XML_FILE_ENCODING = 'UTF-8'
    
    __TIAM_UTILITY_CONFIG_PATH = '/tiam_utility/config'
    __ADMINISTRATIVE_LEAVE_FILENAME = 'administrative_leave.xml'
    
    __NODES_ON_ADMINISTRATIVE_LEAVE_ELEMENTNAME = 'nodesonadministrativeleave'
    __NODE_ELEMENTNAME   = 'node'
    __NODE_ELEMENT_PATH  = 'node'
    __AUTOMATIC_LEAVE_ATTRIBUTENAME = 'automaticleave'
    __ON_AUTOMATIC_LEAVE_ATTRIBUTE_VALUE = 'true'
    
    def __init__(self):                
        try:
            document = self.__parse_administrativeleave_file()
        except:
            print 'Cluster does not have an administrative_leave.xml file. Creating a new file.'
            document = self.__create_administrativeleave_document()
            self.__write_administrativeleave_file(document)
            document.unlink()
        document.unlink()
    
    
    def __get_administrativeleave_file_path(self):

        return self.__TIAM_UTILITY_CONFIG_PATH + '/' + self.__ADMINISTRATIVE_LEAVE_FILENAME
    
    
    def __parse_administrativeleave_file(self):
        return xml.dom.minidom.parse(self.__get_administrativeleave_file_path())
    
    
    def __write_administrativeleave_file(self, document):
        """Write administrative leave DOM document
           to a file.
           (Private method to allow reuse.)
        """

        documentFileName = self.__get_administrativeleave_file_path()
                           
        try:
            fileObject = open(documentFileName, "w")
            xml.dom.ext.PrettyPrint(document, fileObject, self.__XML_FILE_ENCODING)
            fileObject.close()
        except:
            print "TODO: better log and check here Error encountered while creating adminsistrativeleave document"


    def __create_administrativeleave_document(self):
        """Create an empty administrative leave
           DOM document.
        """
                
        document = xml.dom.minidom.Document()
        
        nodesOnAdministrativeLeaveElement = document.createElement(self.__NODES_ON_ADMINISTRATIVE_LEAVE_ELEMENTNAME)
        
        document.appendChild(nodesOnAdministrativeLeaveElement)
        
        return document


    def __find_node_elements_for_node_id(self, document, nodeId):
        """Return the list of node elements in the
           administrative leave file that match
           nodeId.
           (Private method to allow reuse of 
           functionality without re-parsing the 
           administrativeleave.xml file.)
        """

        nodesOnLeave = xml.xpath.Evaluate(self.__NODE_ELEMENT_PATH, document.documentElement)

        nodesMatchingNodeId = [ node for node in nodesOnLeave if node.childNodes[0].nodeValue.strip() == nodeId.strip() ]

        return nodesMatchingNodeId
    

    def __find_node_element_for_node_name(self, document, nodeId):
        """Return the first node element in the
           administrative leave file that matches
           nodeId.
           (Private method to allow reuse of 
           functionality without re-parsing the 
           administrativeleave.xml file.)
        """

        nodesMatchingNodeId = self.__find_node_elements_for_node_id(document, nodeId)

        if len(nodesMatchingNodeId) == 0:
            return None
        else:
            return nodesMatchingNodeId[0]            


    def __is_node_on_leave(self, document, nodeId):
        """Return True only if nodeId is on
           administrative leave.
           (Private method to allow invocation
           without re-parsing the administrativeleave.xml
           file.)
        """

        
        nodeElement = self.__find_node_element_for_node_name(document, nodeId)

        onLeave = (nodeElement != None)

        return onLeave
    

    def is_node_on_leave(self, nodeId):
        """Return True only if nodeId is on
           administrative leave.
        """

        
        document = self.__parse_administrativeleave_file()
        
        onLeave = self.__is_node_on_leave(document, nodeId)

        document.unlink()
        return onLeave
    
    
    def is_node_on_automatic_leave(self, nodeId):
        """Return True only if nodeId was placed on
           administrative leave automatically.
        """
  
        document = self.__parse_administrativeleave_file()
        
        nodeElement = self.__find_node_element_for_node_name(document, nodeId)

        onLeave = ( nodeElement != None ) and                                             \
                  ( nodeElement.getAttribute(self.__AUTOMATIC_LEAVE_ATTRIBUTENAME) ==     \
                    self.__ON_AUTOMATIC_LEAVE_ATTRIBUTE_VALUE )

        document.unlink()
        return onLeave
    
    
    def list_nodes_on_leave(self):
        """Get a list containing the node name for all nodes
           currently on leave.
        """
        
        document = self.__parse_administrativeleave_file()

        nodesOnLeaveElements = xml.xpath.Evaluate(self.__NODE_ELEMENT_PATH, document.documentElement)
    
        nodesOnLeave = [ nodeElement.childNodes[0].nodeValue.strip() for nodeElement in nodesOnLeaveElements ]
        
        return nodesOnLeave
    
    
    def put_node_on_leave(self, nodeId, automaticLeave=False):
        """Put nodeId on administrative leave.
           Mark nodeId as being on automatic administrative
           leave if administrativeLeave == True.
           Return True only if nodeId was not on leave
           before invocation and the node was placed on
           leave during invocation.           
        """

        document = self.__parse_administrativeleave_file()
        
        if self.__is_node_on_leave(document, nodeId):
            document.unlink()
            return False
        
        newNodeElement = document.createElement(self.__NODE_ELEMENTNAME)
        newNodeElementText = document.createTextNode(nodeId)
        newNodeElement.appendChild(newNodeElementText)
        
        if automaticLeave:
            newNodeElement.setAttribute(self.__AUTOMATIC_LEAVE_ATTRIBUTENAME, self.__ON_AUTOMATIC_LEAVE_ATTRIBUTE_VALUE)
        
        document.documentElement.appendChild(newNodeElement)
        
        self.__write_administrativeleave_file(document)
        
        document.unlink()
        return True
    
    
    def take_node_off_leave(self, nodeId):
        """Take nodeId off of administrative leave.
           Return True only if nodeId was on leave
           before invocation and the node was taken
           off leave during invocation.
        """
    
        
        document = self.__parse_administrativeleave_file()
        
        nodesMatchingNodeId = self.__find_node_elements_for_node_id(document, nodeId)

        # If the node is not on leave then do nothing.
        # The self.__is_node_on_leave(document, nodeId) may seem to be a better
        # choice for this test. However, __is_node_on_leave constructs the 
        # nodesMatchingnNodeId list internally. If nodeId is on leave then we
        # will need that list to take the node off leave.
        # Therefore, by using the result of __find_node_elements_for_node_id(document, nodeId)
        #  in this test we avoid a second iteration through the DOM tree.
        if (nodesMatchingNodeId == None) or ( len(nodesMatchingNodeId) == 0 ):
            return False

        for node in nodesMatchingNodeId:
            document.documentElement.removeChild(node)
        
        self.__write_administrativeleave_file(document)
        
        document.unlink()
        return True