
########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

# ----------------------------------------------------------------------------
# $Revision: $ $Date: $ $Author: $
# ----------------------------------------------------------------------------
#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Workload Manager
# *
# * FILE NAME:       iamiddleware/logging.py
# *
# * DESCRIPTION:     Logging library routines used by IA Middleware 
# *                  components.
# *
# * AUTHOR:          Michael A. Richmond
# *                  IBM Almaden Research Center
# *                  mar@almaden.ibm.com
# *
# ****************************************************************************
# */
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
#              Change Log
#              ----------
# 07 Oct 2008  Initial version.
#
# 04 Dec 2008  Code moved into iamiddleware package.
#
# 16 Jan 2009  Updated __LOGFILE_PATH and __TRACEFILE_PATH to be based on
#              COMPONENT_IDENTIFIER string.
#              New implemenation of LogMsg method to obtain message format
#              and type from an array of tuples.
#
# 01 Apr 2009  Fixed bug in log/trace level change code.
#              Provided option to omit source code filename from trace
#              messages.
#
# 08 May 2009  Provided option to specify logfile prefix independently
#              of setting component name.
#
# 10 Jun 2009  Modified error print statements to output to stderr.
#
# 27 Jul 2009  Added shutdown method to allow clean flush and shutdown of
#              all logger instances.
#
# 17 Aug 2009  Changed update_logging_levels() method so that it re-opens
#              the logfile as a side-effect of invoking the update method.
#              This allows the program to continue logging even if the
#              original logfile is deleted or moved.
#
# 14 Sep 2009  Updated log handler configuration so that log and trace 
#              messages shown on standard out have timestamps.
#
# 25 Sep 2009  Added explicit close for log/trace files when refreshing
#              file handle. (To avoid accumulating an ever-increasing 
#              number of open file handles.
#
# 08 Feb 2010  Created a list object for the log and trace handlers
#
# 08 Apr 2010  Changed log and trace formatter to include the timezone
#              offset.
#

import  commands
import  errno
import  logging               # logging package and ...
import  logging.handlers      # extra subpackage
import  os
import  sys
import  traceback

INFO    = """I"""
WARNING = """W"""
ERROR   = """E"""

logging.raiseExceptions = 0

class IALogger:
    TRACING_OFF = logging.DEBUG
    TRACING_ON = logging.DEBUG

    CRITICAL = logging.CRITICAL
    ERROR    = logging.ERROR
    WARNING  = logging.WARNING
    INFO     = logging.INFO
    DEBUG    = logging.DEBUG
    
    msgTypes  = { "W":WARNING, "E":ERROR, "I":INFO }
    msgLevels = { "0":CRITICAL, "1":ERROR, 
                  "2":WARNING,  "3":DEBUG }

    ID        = 0
    SEVERITY  = 1
    FORMAT    = 2

    DEFAULT_LOG_SIZE                  = (50 * (1024**2))
    DEFAULT_LOG_BACKUPS               = 3
    DEFAULT_TRACE_INCLUDE_SOURCE_NAME = True

    COMPONENT_IDENTIFIER = "CTJID"

    __LOGFILE_PATH   = '%s/' + COMPONENT_IDENTIFIER + '/%s/%s/logs/' # where to write log 
    __TRACEFILE_PATH = '%s/' + COMPONENT_IDENTIFIER + '/%s/%s/ffdc/' #   and trace files

    # Define output logging and tracing file names
    __LOGFILE_NAME   = '%s_log.txt'      # name of log file
    __TRACEFILE_NAME = '%s_trace.txt'    #  and trace file

    __LOGGING_LEVEL_CHANGED_MSG = """Logging level changed to %s""" 
    __TRACING_LEVEL_CHANGED_MSG = """Tracing level changed to %s""" 

    SOURCE_FILENAME     = 0
    SOURCE_LINENUMBER   = 1


    def __init__(self, basePath, component, collection, nodeid, logfilePrefix,
                 loggerName, loggerFormat, loglevel,
                 traceName,  traceFormat,  tracelevel,
                 logSize=DEFAULT_LOG_SIZE, logBackups=DEFAULT_LOG_BACKUPS,
                 traceIncludeSourceName=DEFAULT_TRACE_INCLUDE_SOURCE_NAME, useStdout=True):
        self.__log = None
        self.__trace = None

        # Normalize base path
        if basePath[-1:] == '/':
            self.__basePath = basePath[ :len(basePath)-1]
        else:
            self.__basePath = basePath

        self.__component = component
        self.__collection = collection
        self.__nodeid = nodeid
        self.__logfilePrefix = logfilePrefix
        self.__logSize = logSize
        self.__logBackups = logBackups
        self.__useStdout = useStdout
        
        self.__traceIncludeSourceName = traceIncludeSourceName
        
        self.__loggerFormat = loggerFormat
        self.__traceFormat = traceFormat
        
        self.__configure_logger(loggerName, self.__loggerFormat, loglevel, traceName, self.__traceFormat, tracelevel, self.__useStdout)
    
    
    def shutdown(self):
        logging.shutdown()
        
    def __configure_logger(self, loggerName, loggerFormat, loglevel,
                                 traceName, traceFormat, tracelevel, useStdout):

        if self.__collection is None:
            expandedComponent = self.__component
        else:
            expandedComponent = self.__component + self.__collection
        
        logfilePath =   self.__LOGFILE_PATH % ( self.__basePath, expandedComponent, self.__nodeid )
        tracefilePath = self.__TRACEFILE_PATH % ( self.__basePath, expandedComponent, self.__nodeid )
    
        logfileName   = self.__LOGFILE_NAME % self.__logfilePrefix
        tracefileName = self.__TRACEFILE_NAME % self.__logfilePrefix
        
        # Make sure the log and trace output file directories exist
        try:
            self.__mkdirs( logfilePath, 0771 )
            self.__mkdirs( tracefilePath, 0771 )
        except OSError, err:
            print >>sys.stderr, err
            print >>sys.stderr, "Unable to create directories for logging and/or tracing files."
            exit( 99 )

        # Create rotating log managers for logging and tracing files
        logging.basicConfig()

        self.__log = logging.getLogger( loggerName )
        if useStdout:
		logStreamHandler = logging.StreamHandler( sys.stdout )
        logFileHandler = logging.handlers.RotatingFileHandler( logfilePath + logfileName, 'a', 
                                                               self.__logSize, self.__logBackups )
        
        if len(self.__log.handlers) > 0:
            # Remove any old handlers before adding a new one.
            for oldHandler in list(self.__log.handlers):
                self.__log.removeHandler(oldHandler)
                # Explicitly close the handler to ensure that we don't accumulate an
                # ever-increasing number of open file handles.
                oldHandler.close()
                
	if useStdout:
        	self.__log.addHandler( logStreamHandler )
        self.__log.addHandler( logFileHandler )
        
        self.__log.setLevel( self.msgLevels[ loglevel ] )

        logFormatter = logging.Formatter( loggerFormat, "%Y-%m-%d %H:%M:%S %Z" )

	if useStdout:
        	logStreamHandler.setFormatter( logFormatter )
        logFileHandler.setFormatter( logFormatter )

        self.__log.propagate = False

        self.__trace = logging.getLogger( traceName )
	if useStdout:
        	traceStreamHandler = logging.StreamHandler( sys.stdout )
        traceFileHandler = logging.handlers.RotatingFileHandler( tracefilePath + tracefileName, 'a', 
                                                                 self.__logSize, self.__logBackups )

        if len(self.__trace.handlers) > 0:
            # Remove any old handlers before adding a new one.
            for oldHandler in list(self.__trace.handlers):
                self.__trace.removeHandler(oldHandler)
                # Explicitly close the handler to ensure that we don't accumulate an
                # ever-increasing number of open file handles.
                oldHandler.close()
                
	if useStdout:
        	self.__trace.addHandler( traceStreamHandler )
        self.__trace.addHandler( traceFileHandler )
        
        self.__trace.setLevel( self.msgLevels[ tracelevel ] )

        traceFormatter = logging.Formatter( traceFormat, "%Y-%m-%d %H:%M:%S %Z" )
	if useStdout:
        	traceStreamHandler.setFormatter( traceFormatter )
        traceFileHandler.setFormatter( traceFormatter )
        
        self.__trace.propagate = False

    
    def __mkdirs(self, newdir, mode=0777):
        try:
            os.makedirs(newdir, mode)
        except OSError, err:
            # Reraise the error unless it's about an already existing directory
            if err.errno != errno.EEXIST or not os.path.isdir(newdir):
                raise


    def get_logging_level(self):
        return self.__log.level
    
    
    def __get_logger_name(self):
        return self.__log.name
    
    
    def __get_logger_format(self):
        return self.__loggerFormat
    
        
    def get_tracing_level(self):
        return self.__trace.level
    
    
    def __get_tracing_name(self):
        return self.__trace.name
    
        
    def __get_tracing_format(self):
        return self.__traceFormat
    
        
    def update_logging_levels(self, loglevel='1', tracelevel='0'):
    
        newLogLevel = self.msgLevels[ loglevel ]
        newTraceLevel = self.msgLevels[ tracelevel ]
    
        logLevelChanged = False
        traceLevelChanged = False
    
        if newLogLevel != self.get_logging_level():
            self.__log.setLevel( newLogLevel )
            logLevelSymbol = [key for key in self.msgLevels.keys() if self.msgLevels[key] == newLogLevel][0]
            print self.__LOGGING_LEVEL_CHANGED_MSG % logLevelSymbol
            logLevelChanged = True

        if newTraceLevel != self.get_tracing_level():
            self.__trace.setLevel( newTraceLevel )
            traceLevelSymbol = [key for key in self.msgLevels.keys() if self.msgLevels[key] == newTraceLevel][0]
            print self.__TRACING_LEVEL_CHANGED_MSG % traceLevelSymbol
            traceLevelChanged = True
        
        loggingLevel = [key for key in self.msgLevels.keys() if self.msgLevels[key] == self.get_logging_level()][0]
        tracingLevel = [key for key in self.msgLevels.keys() if self.msgLevels[key] == self.get_tracing_level()][0]
        self.__configure_logger( self.__get_logger_name(), self.__get_logger_format(), loggingLevel,
                                 self.__get_tracing_name(), self.__get_tracing_format(), tracingLevel, self.__useStdout )
            
        return logLevelChanged, traceLevelChanged
    

    def __writeLogMessage(self, messageType, messageNumber, messageString):
        """Function to write a message to the log file based on current setting"""
    
        logString = "%s%d%s %s" % ( self.COMPONENT_IDENTIFIER, messageNumber, messageType, messageString )

        self.__log.log(self.msgTypes[messageType], logString + "\n")


    def LogMsg(self, messageTuple, substitution=None):
        """Function to write a message to the log file based on the current 
           logger settings.
           messageArray is an array containing log message tuples. Each tuple
           contains a severity string and a format string.
           substitution is a sequence type to be substituted into the message
           format string contained in the messageTuple stored in messageArray at 
           messageIndex."""
           
        messageType = messageTuple[self.SEVERITY]
        messageID = messageTuple[self.ID]
        messageString = messageTuple[self.FORMAT]

        if substitution is not None:
            messageString = messageString % substitution
        
        self.__writeLogMessage(messageType, messageID, messageString)


    def TraceMsg(self, msg):
        """Function to write a message to the trace file based on current setting."""
    
        stackFrameIndex = 3
        if self.__traceIncludeSourceName:
            # Output "filename(line number) message"
            self.__trace.debug( str(self.__caller(stackFrameIndex)[self.SOURCE_FILENAME]) +      \
                                "(" + str(self.__caller(stackFrameIndex)[self.SOURCE_LINENUMBER]) + ") " + msg + "\n")
        else:
            # Output "(line number) message"
            self.__trace.debug( "(" + str(self.__caller(stackFrameIndex)[self.SOURCE_LINENUMBER]) + ") " + msg + "\n")

    
    def __caller(self, up=0):
        '''Get file name, line number, function name and
           source text of the caller's caller as 4-tuple:
           (file, line, func, text).

           The optional argument 'up' allows retrieval of
           a caller further back up into the call stack.

           Note, the source text may be None and function
           name may be '?' in the returned result.  In
           Python 2.3+ the file name may be an absolute
           path.
        '''
    
        try:
            stack = traceback.extract_stack()
            callerFrame = stack[ len( stack ) - up ]
        except:
            callerFrame = ( '', 0, '', None )
      
        return callerFrame    


    __FUNCTION_TRACE_DIRECTION = { "ENTER":"enter", "EXIT":"exit" }
    __TRACE_FUNCTIONPOINT_FORMAT_STRING = '%s function %s::(%s)::%s'

    __FILE_NAME = 0
    __LINE_NUMBER = 1
    __FUNCTION_NAME = 2
    
    def __TraceFunctionBoundary(self, direction):
        """Produce a trace message for the boundary of function calls"""

        stackFrameIndex = 4
    
        self.__trace.debug( self.__TRACE_FUNCTIONPOINT_FORMAT_STRING %                               \
                            ( direction,                                                             \
                              str(self.__caller(stackFrameIndex)[self.__FILE_NAME]),                 \
                              str(self.__caller(stackFrameIndex)[self.__LINE_NUMBER]),               \
                              str(self.__caller(stackFrameIndex)[self.__FUNCTION_NAME]) )  + '\n')


    def TraceFunctionEntry(self):
        """"Produce a trace message for the entry of a function"""
    
        self.__TraceFunctionBoundary(self.__FUNCTION_TRACE_DIRECTION['ENTER'])

    
    def TraceFunctionExit(self):
        """"Produce a trace message for the exit of a function"""
    
        self.__TraceFunctionBoundary(self.__FUNCTION_TRACE_DIRECTION['EXIT'])
