########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008-2010
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

# ----------------------------------------------------------------------------
# $Revision: $ $Date: $ $Author: $
# ----------------------------------------------------------------------------
#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Workload Manager
# *
# * FILE NAME:       iamiddleware.powercontrol.py
# *
# * DESCRIPTION:     Library routines used by IA Middleware 
# *                  components to issue commands to remote
# *                  hardware monitor adapters. For example,
# *                  Remote Service Adapter (RSA) hardware
# *                  and integrated Baseboard Management 
# *                  Controller (iBMC) hardware.
# *
# * AUTHOR:          Michael A. Richmond
# *                  IBM Almaden Research Center
# *                  mar@almaden.ibm.com
# *
# ****************************************************************************
# */
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
#              Change Log
#              ----------
# 07 Oct 2008  Initial version.
#              (Michael Richmond)
#
# 11 Oct 2008  Fixed issue caused by unicode strings as ip addresses.
#              (Michael Richmond)
#
# 13 Oct 2008  Fixed "104 connection reset by peer" issue by introducing
#              a minimum allowed time before establishing a new connection
#              to a host.
#              (Michael Richmond)
#
# 07 Nov 2008  Fixed typo in methods is_booting() and is_os_running() that
#              resulted in abnormal termination of code.
#              (Michael Richmond)
#
# 10 Nov 2008  Updated __STATE_LINE_OS_RUNNING_MSG to match string used
#              by RSA card with Linux driver installed.
#              (Michael Richmond)
#
# 12 Nov 2008  Added methods is_perform_shutdown() and 
#              set_perform_shutdown(boolean) to support hard power resets in
#              ia_powercontrol script.
#              (Michael Richmond)
#
# 04 Dec 2008  Code moved into iamiddleware package.
#              (Michael Richmond)
#
# 09 Feb 2009  Introduced connection error reporting as "None" result for 
#              operations. Added optional constructor argument to control
#              error reporting. (To minimize impact of error reporting on
#              main watchdog code base.)
#              (Michael Richmond)
#
#  2 Jun 2009  Added varying level of debug messages. (Controlled by
#              verboseLevel in class constructor.)
#              Changes to support new IA release hardware and allow easier 
#              expansion to support future power control hardware.
#              Changes to raise UnknownHardwareType exception if the provided
#              hardware type is not known in this implementation.
#              Error print statements output to stderr.
#              (Michael Richmond)
#
# 15 Jul 2009  Added strings for "x3x50" and "x3x50-M2" hardware id."
#              (Michael Richmond)
#
# 29 Aug 2009  Added loop to handle longer delay between password being entered
#              and command prompt being available from power control hardware.
#              (Michael Richmond)
#
# 10 Sep 2009  In __issue_commands(): added loop to handle longer delay when
#              reading the telnet session until the username and password prompt 
#              and closed the telnet session before we return None
#              (Liming Michael Yu)
#
# 06 Oct 2009  Added setter and getters for retryAfterConnectionErrorCount,
#              and retryAfterConnectionErrorSleep values.
#              Added __issue_commands_with_retry() and updated all calls to
#              __issue_commands() to use __isue_commands_with_retry().
#              (Michael Richmond)
#
# 06 Oct 2009  At the end of __issue_commands(), made sure we're closing the
#              telnetSession in all places.
#
# 23 Oct 2009  Accept optional "Logger" object during init. If logger object
#              is defined log command and output of command received from IMM.
#              Check "State" line from power state command result. If "State"
#              line contains an unrecognized state, retry the power state
#              command.
#              (Michael Richmond)
#
# 28 Jan 2010  Switch from datetime.now() to datetime.utcnow() for time calculation
#

import datetime
import socket
import sys
import telnetlib
import time
import traceback


class UnknownHardwareTypeException(Exception):
    def __init__(self, msg):
        self.message = msg


def get_known_hardware_ids():
    return IAPowerController.HARDWARE_TYPES.keys()


def is_valid_ip_address(address):
    """Return True if the string 'address'
       contains a valid ip address.
       Note that this function does not check
       whether the address is assigned, in use,
       or reachable.
    """

    # Ensure that 'address' is a string.
    if type(address).__name__ != "str":
        raise TypeError("is_valid_ip_address(addr) function expects addr to be a string")
    
    address = address.strip()
    
    # An IP address must be of the form 'A.B.C.D'.
    octets = address.split('.')
    if len(octets) != 4:
        return False

    # Each octet must be an integer.
    for octetString in octets:
        try:
            octet = int(octetString)
        except ValueError:
            return False

        # The value of the octet integer must be
        # between 0 and 255 inclusive.
        if (octet < 0) or (octet > 255):
            return False
        
    return True


class IAPowerController:

    DEBUG = False
    #DEBUG = True
    
    # Default verbose level to high value. This will be
    # reset in the object initializer.
    __verbose_level = 2

    __DEFAULT_VERBOSE_LEVEL = 0
    
    __DEFAULT_PAUSE_BETWEEN_COMMANDS = 0                # seconds

    __RECENT_CONNECTION_TIMEOUT = 2                     # seconds
    __DEFAULT_PAUSE_BETWEEN_CONNECTIONS = 2             # seconds
    
    __DEFAULT_REPORT_CONNECTION_ERRORS = False
    
    __DEFAULT_RETRY_AFTER_CONNECTION_ERROR = 3
    __DEFAULT_RETRY_SLEEP_AFTER_CONNECTION_ERROR = 30   # seconds
    
    __GARBAGE_STATE_FROM_IMM_RETRY_COUNT = 3
    __GARBAGE_STATE_FROM_IMM_RETRY_SLEEP = 30           # seconds
    
    __DEFAULT_OS_SHUTDOWN_FLAG = "-s"

    __MIN_IP_LENGTH = 2     # smallest valid hostname/ip address
    
    __INITIAL_CONNECT_TIMEOUT      = 10
    __LOGIN_PROMPT_TIMEOUT = 5
    __LOGIN_PROMPT_TIMEOUT_MAX_ATTEMPTS = 6
    __PROMPT_TIMEOUT               = 10
    __PROMPT_TIMEOUT_MAX_ATTEMPTS  = 5
    __COMMAND_TIMEOUT              = 5
    __COMMAND_TIMEOUT_MAX_ATTEMPTS = 100
    
    __HARDWARE_TYPE_x3x50           = "x3x50"
    __HARDWARE_TYPE_x3x50_M2        = "x3x50-M2"

    # Keys in HARDWARE_TYPES match hardware_id string value
    # from:
    #     * the "hardware" value from the iacluster_config.xml file, and/or
    #     * -H "hardware" command line argument on ia_powercontrol command.
    HARDWARE_TYPES = {
                       "x3x50":     __HARDWARE_TYPE_x3x50,
                       "x3550":     __HARDWARE_TYPE_x3x50,
                       "x3650":     __HARDWARE_TYPE_x3x50,
                       "x3x50-M2":  __HARDWARE_TYPE_x3x50_M2,
                       "x3550-M2":  __HARDWARE_TYPE_x3x50_M2,
                       "x3650-M2":  __HARDWARE_TYPE_x3x50_M2
                     }
    
    # Hardware-specific dictionary keys
    __LOGIN_PROMPT                  = "loginprompt"
    __PASSWORD_PROMPT               = "passwordprompt"
    __MAX_TELNET_SESSIONS_MESSAGE   = "maxtelnetsesions"
    __COMMAND_PROMPT                = "commandprompt"
    __COMMAND_TERMINATION_CHARACTER = "commandterminationcharacter"
    __COMMAND_CONFIRMATION_MESSAGE  = "commandconfirmation"
    __OFF_STATES                    = "offstates"
    __BOOTING_STATES                = "bootingstates"
    __RUNNING_STATES                = "runningstates"
    __ERROR_STATES                  = "errorstates"
   
   # ###################################################################### #
   # Hardware-specific strings for hardware type "x3x50"                    #
   # ###################################################################### #
    __STATE_POWER_OFF_UNKNOWN_MSG_x3650                 = 'State: System power off/State unknown'
    __STATE_BEFORE_POST_MSG_x3650                       = 'State: System power on/Prior to POST'
    __STATE_ON_IN_POST_MSG_x3650                        = 'State: System in POST'
    __STATE_BOOTED_FLASH_OR_SYSTEM_PARTITION_MSG_x3650  = 'State: Booted Flash or System partition'
    __STATE_BOOTING_OS_MSG_x3650                        = 'State: Booting OS or in unsupported OS'
    __STATE_OS_RUNNING_MSG_x3650                        = 'State: OS booted'

    __OFF_STATES_x3x50     = [ __STATE_POWER_OFF_UNKNOWN_MSG_x3650
                             ]
    __BOOTING_STATES_x3x50 = [ __STATE_BEFORE_POST_MSG_x3650,
                               __STATE_ON_IN_POST_MSG_x3650,
                               __STATE_BOOTED_FLASH_OR_SYSTEM_PARTITION_MSG_x3650,
                               __STATE_BOOTING_OS_MSG_x3650
                             ]
    __RUNNING_STATES_x3x50 = [ __STATE_OS_RUNNING_MSG_x3650
                             ]
    __ERROR_STATES_x3x50   = []
    
# TODO: CPUs held in reset
   
    __x3x50_HARDWARE_SPECIFIC_STRINGS =    { __LOGIN_PROMPT:                   "username:",
                                             __PASSWORD_PROMPT:                "password:",
                                             __COMMAND_PROMPT:                 "> ",
                                             __COMMAND_TERMINATION_CHARACTER:  "\r\n",
                                             __COMMAND_CONFIRMATION_MESSAGE:   "ok",
                                             __OFF_STATES:                     __OFF_STATES_x3x50,
                                             __BOOTING_STATES:                 __BOOTING_STATES_x3x50,
                                             __RUNNING_STATES:                 __RUNNING_STATES_x3x50,
                                             __ERROR_STATES:                   __ERROR_STATES_x3x50
                                           }


   # ###################################################################### #
   # Hardware-specific strings for hardware type "x3x50-M2"                 #
   # ###################################################################### #
    __STATE_POWER_OFF_UNKNOWN_MSG_x3x50_M2              = 'State: System power off/State unknown'
    __STATE_BEFORE_UEFI_MSG_x3x50_M2                    = 'State: Unknown'
    __STATE_STARTING_UEFI_MSG_x3x50_M2                  = 'State: System on/starting UEFI'
    __STATE_RUNNING_UEFI_MSG_x3x50_M2                   = 'State: System running in UEFI'
    __STATE_BOOTING_OS_MSG_x3x50_M2                     = 'State: Booting OS'
    __STATE_UNSUPPORTED_OS_MSG_x3x50_M2                 = 'State: unsupported OS'
    __STATE_OS_RUNNING_MSG_x3x50_M2                     = 'State: OS booted'
    __STATE_STOPPED_IN_UEFI_MSG_x3x50_M2                = 'State: System stopped in UEFI (Error detected)'
    __STATE_UNKNOWN_MSG_x3x50_M2                        = 'State: Unknown'

    __OFF_STATES_x3x50_M2     =  [ __STATE_POWER_OFF_UNKNOWN_MSG_x3x50_M2
                                 ]
    __BOOTING_STATES_x3x50_M2 =  [ __STATE_BEFORE_UEFI_MSG_x3x50_M2,
                                   __STATE_STARTING_UEFI_MSG_x3x50_M2,
                                   __STATE_RUNNING_UEFI_MSG_x3x50_M2,
                                   __STATE_BOOTING_OS_MSG_x3x50_M2,
                                   __STATE_UNSUPPORTED_OS_MSG_x3x50_M2
                                 ]
    __RUNNING_STATES_x3x50_M2  = [ __STATE_OS_RUNNING_MSG_x3x50_M2
                                 ]
    __ERROR_STATES_x3x50_M2   =  [ __STATE_STOPPED_IN_UEFI_MSG_x3x50_M2,
                                   __STATE_UNKNOWN_MSG_x3x50_M2
                                 ]
    
    __x3x50_M2_HARDWARE_SPECIFIC_STRINGS = { __LOGIN_PROMPT:                   "login : ",
                                             __PASSWORD_PROMPT:                "Password: ",
                                             __MAX_TELNET_SESSIONS_MESSAGE:    "Telnet Maximum Connection reached",
                                             __COMMAND_PROMPT:                 "system>",
                                             __COMMAND_TERMINATION_CHARACTER:  "\r\n",
                                             __COMMAND_CONFIRMATION_MESSAGE:   "ok",
                                             __OFF_STATES:                     __OFF_STATES_x3x50_M2,
                                             __BOOTING_STATES:                 __BOOTING_STATES_x3x50_M2,
                                             __RUNNING_STATES:                 __RUNNING_STATES_x3x50_M2,
                                             __ERROR_STATES:                   __ERROR_STATES_x3x50_M2
                                           }

    # Dictionary of hardware-specific string sets    
    __HARDWARE_SPECIFIC_STRING_SETS = { __HARDWARE_TYPE_x3x50:        __x3x50_HARDWARE_SPECIFIC_STRINGS,
                                        __HARDWARE_TYPE_x3x50_M2:     __x3x50_M2_HARDWARE_SPECIFIC_STRINGS
                                       }

    # Commands understood by all hardware types
    __POWERSTATE_COMMAND   = "power state"
    __POWERCYCLE_COMMAND   = "power cycle"
    __POWEROFF_COMMAND     = "power off"
    __POWERON_COMMAND      = "power on"
    __POWER_COMMAND_ATTEMPT_SHUTDOWN_FLAG = __DEFAULT_OS_SHUTDOWN_FLAG
    __RESET_COMMAND        = "reset"
    __EXIT_COMMAND         = "exit"

    # TODO: Move power messages into hardware-specific structures    
    __POWER_OFF_MSG            = 'Power: Off'
    __POWER_ON_MSG             = 'Power: On'
    
    __POWER_MESSAGES = { 'OFF': __POWER_OFF_MSG, 'ON': __POWER_ON_MSG }
    
    __STATE_LINE = { 'POWER': 0, 'STATE': 1 }
    
    # class field to globally control whether the controller
    #   should actually reboot the node.
    allowReboots = True


    def __init__( self, user, password, hardwareId,
                  delayBetweenCommands=__DEFAULT_PAUSE_BETWEEN_COMMANDS,
                  reportConnectionErrors=__DEFAULT_REPORT_CONNECTION_ERRORS,
                  verboseLevel=__DEFAULT_VERBOSE_LEVEL,
                  retryAfterConnectionErrorCount=__DEFAULT_RETRY_AFTER_CONNECTION_ERROR,
                  retryAfterConnectionErrorSleep=__DEFAULT_RETRY_SLEEP_AFTER_CONNECTION_ERROR,
                  iaLogger=None ):
        """Initialize the IAPowerController object.
           user is the username to use when connecting to the
           power control adapter.
           password is used when connecting to the 
           power control adapter to authenticate user.
           delay specifies a time in seconds to wait between
           issuing successive commands to the power control
           adapter from list of commands. 
        """
        
        # Set the verbosity level first to allow other setters
        # access to the verbosity level. 
        self.set_verbose_level(verboseLevel)
        
        self.set_user(user)
        self.set_password(password)
        self.set_hardware_id(hardwareId)
        self.set_delay_between_commands(delayBetweenCommands)
        self.set_report_connection_errors(reportConnectionErrors)
        self.set_retry_after_connection_error_count(retryAfterConnectionErrorCount)
        self.set_retry_after_connection_error_sleep(retryAfterConnectionErrorSleep)
        self.__lastConnectionTable = dict()
        self.__iaLogger = iaLogger
        if iaLogger is not None:
            iaLoggerClass = iaLogger.__class__.__name__
            if iaLoggerClass != "IALogger":
                print "error: Initializing IAPowerControl with an unexpected logger class '%s'." % iaLoggerClass
                print "Setting ialogger to 'None' for safety."
                self.__iaLogger = None


    def __str__(self):
         """Return IAPowerController values"""
         
         return  "IAPowerController("                  +                                                              \
                 "user="                               + str(self.get_user() ) + ", " +                               \
                 "password="                           + str(self.get_password()) + ", " +                            \
                 "hardware_id="                        + str(self.get_hardware_id()) + ", " +                         \
                 "delay_between_commands="             + str(self.get_delay_between_commands()) + ", " +              \
                 "report_connection_errors="           + str(self.get_report_connection_errors()) + ", " +            \
                 "verbose_level="                      + str(self.get_verbose_level()) +  ", " +                      \
                 "retry_after_connection_error_count=" + str(self.get_retry_after_connection_error_count()) + ", " +  \
                 "retry_after_connection_error_sleep=" + str(self.get_retry_after_connection_error_sleep()) + ", " +  \
                 "iaLogger="                           + str(self.__iaLogger) +                                       \
                 ")"


    def __verbose_message(self, level, message):
        """Print message depending on level of verbosity."""
        
        verboselevel = self.get_verbose_level()

        if verboselevel >= level:
                print "verbose%d: %s" % (level, message)


    def set_user(self, user):
        self.__user = user


    def get_user(self):
        return self.__user


    def set_password(self, password):
        self.__password = password


    def get_password(self):
        return self.__password


    def set_hardware_id(self, hardware):
        """
           Set the __hardware_id field and ensure that
           __hardware_specific_strings points to the
           string set for __hardware_id.
        """
        
        self.__hardware_id = hardware

        if self.HARDWARE_TYPES.has_key(self.__hardware_id):
            hardware_type = self.HARDWARE_TYPES[self.__hardware_id]
            self.__verbose_message( 1, "Matched hardware id '%s' to hardware type '%s'" % (self.__hardware_id, hardware_type) )

            self.__hardware_specific_strings = self.__HARDWARE_SPECIFIC_STRING_SETS[ hardware_type ]
            self.__verbose_message( 1, "Setting power control library to use hardware specific string set for type '%s'" % hardware_type )
        else:
            print >>sys.stderr, "error: unknown hardware id '%s'" % self.__hardware_id
            raise UnknownHardwareTypeException( "Hardware id '%s' is not supported by the iapowercontrol.powercontrol library." %   \
                                                self.__hardware_id )


    def get_hardware_id(self):
        return self.__hardware_id


    def set_delay_between_commands(self, delay):
        self.__delay_between_commands = delay


    def get_delay_between_commands(self):
        return self.__delay_between_commands


    def set_report_connection_errors(self, reportConnectionErrors):
        self.__report_connection_errors = reportConnectionErrors


    def get_report_connection_errors(self):
        return self.__report_connection_errors


    def set_verbose_level(self, verboseLevel):
        self.__verbose_level = verboseLevel


    def get_verbose_level(self):
        return self.__verbose_level


    def set_retry_after_connection_error_count(self, retryAfterConnectionErrorCount):
        self.__retry_after_connection_error_count = retryAfterConnectionErrorCount


    def get_retry_after_connection_error_count(self):
        return self.__retry_after_connection_error_count


    def set_retry_after_connection_error_sleep(self, retryAfterConnectionErrorSleep):
        self.__retry_after_connection_error_sleep = retryAfterConnectionErrorSleep


    def get_retry_after_connection_error_sleep(self):
        return self.__retry_after_connection_error_sleep


#    def get_known_hardware_ids(self):
#        return self.HARDWARE_TYPES.keys()


    def __get_power_control_disallowed_msg(self, pwrctlIP, powerOperation):
        msg =   \
"""#####################################################################
    IAPowerController is not allowed to control node
    power supply. Otherwise, the node connected to 
    power control hardware '%s' would be %s now.
#####################################################################"""

        return msg % (pwrctlIP, powerOperation)


    def __connected_recently(self, pwrctlIP):
        """Check whether we have connected to pwrctlIP recently."""
        
        connectedRecently = False
        
        if self.__lastConnectionTable.has_key(pwrctlIP):
            lastConnectionTime = self.__lastConnectionTable[pwrctlIP]
            
            deltaSinceLastConnection = datetime.datetime.utcnow() - lastConnectionTime

            lastConnectionWindow = datetime.timedelta(seconds = self.__RECENT_CONNECTION_TIMEOUT)
            
            connectedRecently = ( deltaSinceLastConnection < lastConnectionWindow )
        
        if connectedRecently:
            self.__verbose_message( 1, "This machine has recently connected to power control hardware at '%s'" % pwrctlIP )

        return connectedRecently
        
    
    def __update_last_connection_time(self, pwrctlIP):
        """Update the table of ip::connectiontime with the
           current time for pwrctlIP.
        """
        now = datetime.datetime.utcnow()
        self.__lastConnectionTable[pwrctlIP] = now
        
        self.__verbose_message( 2, "Last update time for ip '%s' updated to %s" % (pwrctlIP, now) )


    def __issue_commands_with_retry(self, pwrctlIP, commandList=[]):
        """Connect to the power control adapter at pwrctlIP and
           issue the commands listed in commandList.
           If a connection error occurs sleep for a period of
           time and retry for a limited number of retries.
        """

        self.__verbose_message( 1, "entering __issue_commands_with_retry( '%s', [ '%s' ] )" % (pwrctlIP, commandList) )

        commandResult = None
        retryCount = 0
        while retryCount < self.get_retry_after_connection_error_count():
            retryCount += 1
            commandResult = self.__issue_commands(pwrctlIP, commandList)

            if commandResult is not None:
                break
            else:
                self.__verbose_message( 1, "Encountered power control connection error. Sleeping for %s seconds before retry. Retry count is %s." % (self.get_retry_after_connection_error_sleep(), retryCount) )
                time.sleep( self.get_retry_after_connection_error_sleep() )

        self.__verbose_message( 1, "exiting __issue_commands_with_retry( '%s', [ '%s' ] )" % (pwrctlIP, commandList) )
        return commandResult


    def __issue_commands(self, pwrctlIP, commandList=[]):
        """Connect to the power control adapter at pwrctlIP
           and issue the commands listed in commandList to 
           the power control adapter card.
           Returns the output of each command in a list
           of strings. Each output string corresponds to
           to command at the same index in the commandList.
        """
           
        self.__verbose_message( 1, "entering __issue_commands( '%s', [ '%s' ] )" % (pwrctlIP, commandList) )
                
        if (pwrctlIP == None) or (len(pwrctlIP) < self.__MIN_IP_LENGTH):
            self.__verbose_message( 1, "pwrctlIP is None or (len(pwrctlIP) < %s) so we are aborting" % self.__MIN_IP_LENGTH )
            return None
        
        # The power control adapter telnet server will reset the connection
        # if a client establishes a new connection to the server too quickly.
        # Therefore ia_powercontol maintains a table of ip::lastConnectTime
        # values. If we have connected to the ip recently then back off
        # and sleep for a short time before continuing.
        if self.__connected_recently(pwrctlIP):
            self.__verbose_message( 0, "Connected to server recently. Sleeping for %s seconds before reconnecting." %    \
                                       ( self.__DEFAULT_PAUSE_BETWEEN_CONNECTIONS ) )
            time.sleep( self.__DEFAULT_PAUSE_BETWEEN_CONNECTIONS )
        
        self.__update_last_connection_time(pwrctlIP)
        
        telnetSession = telnetlib.Telnet()
        if self.DEBUG:
            telnetSession.set_debuglevel(10)
        
        if self.get_verbose_level() >= 2:
            telnetSession.set_debuglevel(90)
        
        commandResults = []

        try:
            self.__verbose_message( 2, "Calling telnetSession.open( '%s' )" % pwrctlIP )
            telnetSession.open( str(pwrctlIP) )
            self.__verbose_message( 2, "       telnetSession.open( '%s' ) returned" % pwrctlIP )
        except Exception, e:
            print >>sys.stderr, "error: Encountered exception while opening telnet session."
            print >>sys.stderr, "error: Exception is %s" % e
            telnetSession.close()
            return None

        # Note that all writes to the telnet session are wrapped in str() calls
        # to force conversion from unicode strings to acsii strings. This is to
        # work around a bug in string conversion that occurred when this library
        # was used in the wlm_watchdog command, but did not occur when this
        # library is invoked from ia_testpowercontrol.
        self.__verbose_message( 1, "Logging in to power control hardware...")
        try:
            read_until = ""
            count = 0
            while count < self.__LOGIN_PROMPT_TIMEOUT_MAX_ATTEMPTS: 
                count += 1
                read_until += telnetSession.read_until(self.__hardware_specific_strings[self.__LOGIN_PROMPT], 
                                                       self.__LOGIN_PROMPT_TIMEOUT )
                self.__verbose_message( 2, "telnet bytes from read>>%s<<" % read_until )
              
                if read_until.endswith( self.__hardware_specific_strings[self.__LOGIN_PROMPT] ):
                    self.__verbose_message( 1, "Found login prompt" )
                    break
              
                if count >= self.__LOGIN_PROMPT_TIMEOUT_MAX_ATTEMPTS:
                    self.__verbose_message( 0, "Attempt to read login prompt from power control telnet server took over %d seconds." %       \
                                                   (count*self.__LOGIN_PROMPT_TIMEOUT) )
                    telnetSession.close()
                    return None

            read_until = ""
            count = 0
            telnetSession.write( str(self.get_user() +          \
                                 self.__hardware_specific_strings[self.__COMMAND_TERMINATION_CHARACTER]) )

            while count < self.__LOGIN_PROMPT_TIMEOUT_MAX_ATTEMPTS: 
                count += 1
                read_until = telnetSession.read_until( self.__hardware_specific_strings[self.__PASSWORD_PROMPT],
                                                       self.__LOGIN_PROMPT_TIMEOUT )
                self.__verbose_message( 2, "telnet bytes from read>>%s<<" % read_until )

                if read_until.endswith( self.__hardware_specific_strings[self.__PASSWORD_PROMPT] ):
                    self.__verbose_message( 1, "Found password prompt" )
                    break
              
                if count >= self.__LOGIN_PROMPT_TIMEOUT_MAX_ATTEMPTS:
                    self.__verbose_message( 0, "Attempt to read password prompt from power control telnet server took over %d seconds." %       \
                                                   (count*self.__LOGIN_PROMPT_TIMEOUT) )
                    telnetSession.close()
                    return None

            read_until = ""
            telnetSession.write( str(self.get_password() +      \
                                 self.__hardware_specific_strings[self.__COMMAND_TERMINATION_CHARACTER]) )
        except Exception, e:
            print >>sys.stderr, "error: Encountered exception while logging in to telnet server."
            print >>sys.stderr, "error: Exception is %s" % e
            print >>sys.stderr, traceback.extract_stack()

            telnetSession.close()
            return None

        self.__verbose_message( 1, "Login to power control hardware succeeded." )

        self.__verbose_message( 1, "Reading command prompt..." )

        try:
            prompt = ""
            counter = 0
            while counter < self.__PROMPT_TIMEOUT_MAX_ATTEMPTS:
                counter += 1
                prompt += telnetSession.read_until( self.__hardware_specific_strings[self.__COMMAND_PROMPT], self.__PROMPT_TIMEOUT )
                self.__verbose_message( 2, "telnet bytes from read>>%s<<" % prompt ) 
                
                if prompt.endswith( self.__hardware_specific_strings[self.__COMMAND_PROMPT] ):
                    self.__verbose_message( 1, "Found power control prompt" )
                    break
            
                if counter >= self.__PROMPT_TIMEOUT_MAX_ATTEMPTS:
                    self.__verbose_message( 0, "Attempt to read prompt from power control telnet server took over %d seconds." %       \
                                                   (counter*self.__PROMPT_TIMEOUT) )
                    telnetSession.close()
                    return None
               
            # If the hardware type is x3x50-M2 then check for a max telnet connection message.
            if self.HARDWARE_TYPES[self.get_hardware_id()] == self.__HARDWARE_TYPE_x3x50_M2:
                try:
                    prompt.index( self.__hardware_specific_strings[self.__MAX_TELNET_SESSIONS_MESSAGE] )
                except ValueError:
                    # ValueError is raised if the index call does not find the argument as a substring
                    # of prompt. If the string is not a substring of prompt do nothing.
                    pass
                else:
                    # "Telnet Maximum Connection reached" string was received, so abort the telnet session.
                    self.__verbose_message( 1, "Maximum telnet connections limit has been reached. Aborting connection." )
                    telnetSession.close()
                    return None

            tokenizedPrompt = prompt.split( self.__hardware_specific_strings[self.__COMMAND_TERMINATION_CHARACTER] )
            prompt = tokenizedPrompt[-1]

            self.__verbose_message( 2, "Power control command prompt is '%s'" % prompt )

            # If we haven't found a command prompt we must abort.
            if len(prompt) < 1:
                self.__verbose_message( 1, "Cannot find the command prompt. Aborting connection." )
                telnetSession.close()
                return None

            for command in commandList:
                telnetSession.write( str(command + self.__hardware_specific_strings[self.__COMMAND_TERMINATION_CHARACTER]) )

                if self.__iaLogger is not None:
                    self.__iaLogger.TraceMsg( "Issued command '%s' to IMM..." % str(command) )

                self.__verbose_message( 1, "issuing command '%s' to power control adapter" % command )

                result = ""
                counter = 0
                while counter < self.__COMMAND_TIMEOUT_MAX_ATTEMPTS:
                    counter += 1
                    result += telnetSession.read_until( prompt, self.__COMMAND_TIMEOUT)
                    
                    self.__verbose_message( 2, "telnetSession.read_until returned. cumulative result>>%s<<" % result )
                    
                    if result.endswith(prompt):
                        self.__verbose_message( 1, "read power control prompt back" )
                        break
                    
                    if counter >= self.__COMMAND_TIMEOUT_MAX_ATTEMPTS:
                        self.__verbose_message( 0, "Attempt to issue command \"%s\" to the telnet server took over %d seconds." %       \
                                                   (command, counter*self.__COMMAND_TIMEOUT) )
                        result = None
                        break
                    
                if result is not None:
                    # Clean up the result string by removing the command issued...
                    result = result.replace(command + self.__hardware_specific_strings[self.__COMMAND_TERMINATION_CHARACTER], '')
                    # ...and the prompt received after the command returned
                    result = result.rstrip(prompt)
                    #result = result.replace(prompt, '')
                    # Note: result.strip('\r') does not remove the LF characters so we must use result.replace('\r','')
                    result = result.replace('\r','')

                # The IMM hardware has been returning invalid and incorrect data. So we
                # need to log the strings returned from IMM to capture the invalid and
                # erroneous data.
                if self.__iaLogger is not None:
                    self.__iaLogger.TraceMsg( "IMM responded with string >>%s<<" % str(result) )

                commandResults.append(result)
                
                if result is None:
                    # Result will be None if we didn't get the command prompt back after the last command.
                    # Since we didn't get the prompt back we cannot continue issuing any further commands
                    # so we need to break out of the for loop.
                    self.__verbose_message( 1, "Last command to power control hardware did not return to a command prompt." % command )
                    self.__verbose_message( 1, "Aborting any subsequent commands." % command )
                    break
                
                if self.get_delay_between_commands() > 0:
                    sleep(self.get_delay_between_commands())
        except Exception, e:
            print >>sys.stderr, "error: Encountered exception while issuing commands to power control adapter."
            print >>sys.stderr, "error: Exception is %s" % e
            telnetSession.close()
            return None

        try:
            self.__verbose_message( 2, "Sending exit command to power control adapter '%s'." % pwrctlIP )
            telnetSession.write( str(self.__EXIT_COMMAND + self.__hardware_specific_strings[self.__COMMAND_TERMINATION_CHARACTER]) )

        except Execption, e:
            print >>sys.stderr, "error: Encountered exception while closing telnet session."
            print >>sys.stderr, "error: Exception is %s" % e
            telnetSession.close()
            self.__verbose_message( 2, "Calling telnetSession.close( '%s' )" % pwrctlIP )
            return None

        self.__verbose_message( 2, "Calling telnetSession.close( '%s' )" % pwrctlIP )
        telnetSession.close()
        self.__verbose_message( 1, "        __issue_commands( '%s', [ '%s' ] ) returned '%s'" % (pwrctlIP, commandList, commandResults) )
        return commandResults


    def __state_lines_contain_known_state(self, stateLines):
        """Validate that the stateLines returned from
           an IMM 'power state' do not contain garbage.
           stateLines is expected to be a list of strings
           with each string containing the result of spliting
           the raw result string (commandResult.split()).
        """

        stateGroups = [ self.__OFF_STATES, self.__BOOTING_STATES, self.__RUNNING_STATES, self.__ERROR_STATES ]

        for group in stateGroups:
            if stateLines[ self.__STATE_LINE['STATE'] ] in self.__hardware_specific_strings[group]:
                return True
        
        if self.__iaLogger is not None:
            self.__iaLogger.TraceMsg( "error: IMM hardware provided an unknown 'State:' value. Unknown IMM output is >>%s<<" % stateLines )
        return False


    def __issue_powerstate_command(self, pwrctlIP):
        """Issue a command to check the pwrctlIP state of the node
           connected to the power control adapter at pwrctlIP
           address.
           Return the output lines from the power state command.
               line[0] is the power status message.
               line[1] is the node status message.
        """

        retryCount = 0
        while True:
            retryCount += 1
            commandResults = self.__issue_commands_with_retry( pwrctlIP, [ self.__POWERSTATE_COMMAND ] )
        
            if commandResults is None:
                return None
        
            if self.__state_lines_contain_known_state(commandResults[0].splitlines()):
                break
            elif retryCount > self.__GARBAGE_STATE_FROM_IMM_RETRY_COUNT:
                if self.__iaLogger is not None:
                    self.__iaLogger.TraceMsg( "error: IMM produced garbage state value after %d retry attempts." % self.__GARBAGE_STATE_FROM_IMM_RETRY_COUNT)
                break
            else:
                time.sleep( self.__GARBAGE_STATE_FROM_IMM_RETRY_SLEEP )

        return commandResults[0].splitlines()


    def issue_reset_command(self, pwrctlIP, shutdown=True):
        """Issue a command to reset the node attached to
           the power control adapter at pwrctlIP.
           If shutdown is True (default) then perform an OS
           shutdown before resetting the node.
           Note: The reset command will only be issued if
           IAPowerController.allowReboots is set to TRUE
        """
        
        if not self.allowReboots:
            print self.__get_power_control_disallowed_msg(pwrctlIP, "resetting")
            return False
        
        if shutdown:
            command = self.__RESET_COMMAND + ' ' + self.__POWER_COMMAND_ATTEMPT_SHUTDOWN_FLAG
        else:
            command = self.__RESET_COMMAND

        commandResults = self.__issue_commands_with_retry(pwrctlIP, [command] )

        if commandResults == None:
            if self.get_report_connection_errors():
                return None
            else:
                return False

        resultMessage = commandResults[0].strip('\n')

        return resultMessage == self.__hardware_specific_strings[self.__COMMAND_CONFIRMATION_MESSAGE]


    def issue_power_cycle_command(self, pwrctlIP, shutdown=True):
        """Issue a command to power cycle the node attached to
           the power control adapter at pwrctlIP.
           If shutdown is True (default) then perform an OS
           shutdown before cycling power.
           Note: The power cycle command will only be issued if
           IAPowerController.allowReboots is set to TRUE
        """
        
        if not self.allowReboots:
            print self.__get_power_control_disallowed_msg(pwrctlIP, "power cycling")
            return False
        
        if shutdown:
            command = self.__POWERCYCLE_COMMAND + ' ' + self.__POWER_COMMAND_ATTEMPT_SHUTDOWN_FLAG
        else:
            command = self.__POWERCYCLE_COMMAND

        commandResults = self.__issue_commands_with_retry(pwrctlIP, [command] )

        if commandResults == None:
            if self.get_report_connection_errors():
                return None
            else:
                return False

        resultMessage = commandResults[0].strip('\n')

        return resultMessage == self.__hardware_specific_strings[self.__COMMAND_CONFIRMATION_MESSAGE]


    def issue_power_off_command(self, pwrctlIP, shutdown=True):
        """Issue a command to power down the node attached to
           the power control adapter at pwrctlIP.
           If shutdown is True (default) then perform an OS
           shutdown before powering off.
           Note: The power down command will only be issued if
           IAPowerController.allowReboots is set to TRUE
        """
        
        if not self.allowReboots:
            print self.__get_power_control_disallowed_msg(pwrctlIP, "powering off")
            return False
        
        if shutdown:
            command = self.__POWEROFF_COMMAND + ' ' + self.__POWER_COMMAND_ATTEMPT_SHUTDOWN_FLAG
        else:
            command = self.__POWEROFF_COMMAND

        commandResults = self.__issue_commands_with_retry(pwrctlIP, [command] )

        if commandResults == None:
            if self.get_report_connection_errors():
                return None
            else:
                return False

        resultMessage = commandResults[0].strip('\n')

        return resultMessage == self.__hardware_specific_strings[self.__COMMAND_CONFIRMATION_MESSAGE]


    def issue_power_on_command(self, pwrctlIP, shutdown=True):
        """Issue a command to power on the node attached to
           the power control adapter at pwrctlIP.
           Note: The power on command will only be issued if
           RSAPowerController.allowReboots is set to TRUE
        """
        
        if not self.allowReboots:
            print self.__get_power_control_disallowed_msg(pwrctlIP, "powering on")
            return True
        
        commandResults = self.__issue_commands_with_retry(pwrctlIP, [self.__POWERON_COMMAND] )

        if commandResults == None:
            if self.__report_connection_errors:
                return None
            else:
                return False

        resultMessage = commandResults[0].strip('\n')

        return resultMessage == self.__hardware_specific_strings[self.__COMMAND_CONFIRMATION_MESSAGE]


    def issue_commands(self, pwrctlIP, commandList):
        """Make issue_commands a public method. Probably only 
           needed for testing.
        """
        
        return self.__issue_commands_with_retry(pwrctlIP, commandList)


    def is_power_on(self, pwrctlIP):
        """Check whether power is on for the node connected to
           the power control adapter at pwrctlIP address.
        """
        
        stateLines = self.__issue_powerstate_command(pwrctlIP)

        if stateLines == None:
            return None 

        return stateLines[ self.__STATE_LINE['POWER'] ] == self.__POWER_MESSAGES['ON']

        
    def is_booting(self, pwrctlIP):
        """Check whether the node connected to the power
           control adapter at pwrctlIP address is 
           currently booting.
        """

        stateLines = self.__issue_powerstate_command(pwrctlIP)
        
        if stateLines is None:
            return None
            
        bootingStates = self.__hardware_specific_strings[self.__BOOTING_STATES]
        
        return ( stateLines[ self.__STATE_LINE['POWER'] ] == self.__POWER_MESSAGES['ON'] ) and   \
               ( stateLines[ self.__STATE_LINE['STATE'] ] in bootingStates )

    
    def is_os_running(self, pwrctlIP):
        """Check whether the node is up and running the OS."""
        
        stateLines = self.__issue_powerstate_command(pwrctlIP)
        
        if stateLines is None:
            return None
        
        runningStates = self.__hardware_specific_strings[self.__RUNNING_STATES]
        
        return ( stateLines[ self.__STATE_LINE['POWER'] ] == self.__POWER_MESSAGES['ON'] ) and   \
               ( stateLines[ self.__STATE_LINE['STATE'] ] in runningStates )


    def is_in_hardware_error(self, pwrctlIP):
        """Check whether the node is in a hardware error state."""
        
        stateLines = self.__issue_powerstate_command(pwrctlIP)
        
        if stateLines is None:
            return None
                
        errorStates = self.__hardware_specific_strings[self.__ERROR_STATES]
        
        return ( stateLines[ self.__STATE_LINE['POWER'] ] == self.__POWER_MESSAGES['ON'] ) and   \
               ( stateLines[ self.__STATE_LINE['STATE'] ] in errorStates )
    

    def is_perform_shutdown(self):
        """Return whether power down/cycle commands 
           will perform an OS shutdown.
        """
        
        return self.__POWER_COMMAND_ATTEMPT_SHUTDOWN_FLAG == self.__DEFAULT_OS_SHUTDOWN_FLAG


    def set_perform_shutdown(self, doOSShutdown):
        """Set flag to force an OS shutdown if doOSShutdown
           is true. Otherwise, set the flag to not perform
           a shutdown.
        """

        if doOSShutdown:
            self.__POWER_COMMAND_ATTEMPT_SHUTDOWN_FLAG = self.__DEFAULT_OS_SHUTDOWN_FLAG
        else:
            self.__POWER_COMMAND_ATTEMPT_SHUTDOWN_FLAG = ""
