########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

# ----------------------------------------------------------------------------
# $Revision: $ $Date: $ $Author: $
# ----------------------------------------------------------------------------
#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Watchdog
# *
# * FILE NAME:       watchdog/healthcheckfactory.py
# *
# * DESCRIPTION:     Library routines used by the IA Middleware 
# *                  Watchdog component.
# *
# * AUTHOR:          Michael A. Richmond
# *                  IBM Almaden Research Center
# *                  mar@almaden.ibm.com
# *
# ****************************************************************************
# */
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
#              Change Log
#              ----------
# 05 Dec 2008  Initial version. (Moved from ia_watchdog file)
#
# 16 Jan 2009  Updated all logger.LogMsg(...) calls to match new LogMsg(...)
#              implementation and logmessages.py file.
#

iamInstallBase        = '/opt/tivoli/tiam'
iamiddlewarePythonLib = '%s/lib' % iamInstallBase

import  sys
import  xml
import  xml.dom.ext
import  xml.dom.minidom
import  xml.xpath

if iamiddlewarePythonLib not in sys.path:
    sys.path.append(iamiddlewarePythonLib)

import  iamiddleware.ialogging
import  watchdog.settings

INFO      = iamiddleware.ialogging.INFO
WARNING   = iamiddleware.ialogging.WARNING
ERROR     = iamiddleware.ialogging.ERROR

FORMAT    = 2

class HealthCheck:
    """Generic health check"""
    
    def __init__(self, name, description, logger):
        self.__name = name
        self.__description = description
        self.logger = logger
    
    
    def __str__(self):
         """Return HealthCheck values"""
         
         return "(name=" + str(self.__name ) + ", " +                        \
                 "description=" + str(self.__description) + ")"

    @staticmethod
    def get_remote_shell_command(node_ip):
        return ( "%s %s -l %s %s " %
                (watchdog.settings.values['RemoteShellCommand'], watchdog.settings.values['RemoteShellArguments'],
                 watchdog.settings.values['RemoteShellUser'], node_ip) )
    
    def get_name(self):
        return self.__name

    def get_description(self):
        return self.__description
    
    def init_settings(self, watchdog_settings):
        """
           Do not store references to the watchdog_settings 
           dictionary inside your healthcheck class. Rather,
           copy the settings values from the watchdog_settings
           dictionary into your private __settings dictionary.
        """
        raise NotImplementedError

    def get_error_msg_tuple(self):
        raise NotImplementedError
    
    def is_healthy(self, node):
        raise NotImplementedError
    
    def has_health_improvement(self):
        return False  

    def improve_health(self, node):
        raise NotImplementedError
        
    
class HealthCheckFactory:
    """Handler class to abstract details of loading and
       instantiating the IA Watchdog health check
       implementation objects.
    """

    __XML_FILE_ENCODING = 'UTF-8'
    
    __TIAM_UTILITY_CONFIG_PATH = '/tiam_utility/config'
    __WATCHDOG_HEALTHCHECKS_FILENAME = 'watchdog_healthchecks.xml'
    
    __HEALTHCHECK_ELEMENT_PATH = 'healthcheck'
    
    __NAME_ELEMENT_NAME = 'name'
    __DESCRIPTION_ELEMENT_NAME ='description'
    __MESSAGEID_ELEMENT_NAME = 'messageid'
    __MODULENAME_ELEMENT_NAME = 'modulename'
    __CLASSNAME_ELEMENT_NAME = 'class'

    def __init__(self, logger):
        self.__logger = logger               
        try:
            document = self.__parse_watchdoghealthcheck_file()
        except Exception, error:
            print error
            if self.__logger is not None:
                # messagetuples[6140] = "Cluster does not have an '%s' file. Watchdog startup will abort."
                self.__logger.LogMsg( watchdog.logmessages.messagetuples[6140], self.__WATCHDOG_HEALTHCHECKS_FILENAME )
            print watchdog.logmessages.messagetuples[6140][FORMAT] % self.__WATCHDOG_HEALTHCHECKS_FILENAME
            sys.exit(1)
            
        document.unlink()
            

    def __get_watchdoghealthcheck_file_path(self):

        return self.__TIAM_UTILITY_CONFIG_PATH + '/' + self.__WATCHDOG_HEALTHCHECKS_FILENAME
    
    
    def __parse_watchdoghealthcheck_file(self):

        return xml.dom.minidom.parse(self.__get_watchdoghealthcheck_file_path())
    

    def __import_module(self, structured_name):
        """Dynamically import the class named in 
           structured_name.
        """
        
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()

        component_names = structured_name.split('.')
        
        try:
            module = __import__( '.'.join(component_names[:-1]) )
            for component_name in component_names[1:]:
                module = getattr(module, component_name)
        except Exception,error:
            if self.__logger is not None:
                # messagetuples[6144] = "Unable to load module for health check '%s' error is '%s'"
                self.__logger.LogMsg( watchdog.logmessages.messagetuples[6144], (structured_name, error) )
            else:
                print watchdog.logmessages.messagetuples[6144][FORMAT] % structured_name
            module = None


        if self.__logger is not None:
            self.__logger.TraceFunctionExit()

        return module


    def create_healthcheck_instances(self):
        """Parse the health check configuration file and
           return a list populated with health check
           implementation instances.
        """

        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()

        healthCheckInstances = []

        if self.__logger is not None:
            # messagetuples[6141] = "Parsing watchdog healthcheck definition file '%s'"
            self.__logger.LogMsg( watchdog.logmessages.messagetuples[6141], self.__WATCHDOG_HEALTHCHECKS_FILENAME )
        else:
            print watchdog.logmessages.messagetuples[6141][FORMAT] % self.__WATCHDOG_HEALTHCHECKS_FILENAME
        
        document = self.__parse_watchdoghealthcheck_file()
        
        healthCheckElements = xml.xpath.Evaluate(self.__HEALTHCHECK_ELEMENT_PATH, document.documentElement)

        for healthCheckElement in healthCheckElements:            
            name = xml.xpath.Evaluate(self.__NAME_ELEMENT_NAME, healthCheckElement)[0].childNodes[0].nodeValue
            description = xml.xpath.Evaluate(self.__DESCRIPTION_ELEMENT_NAME, healthCheckElement)[0].childNodes[0].nodeValue
#            messageId = xml.xpath.Evaluate(self.__MESSAGEID_ELEMENT_NAME, healthCheckElement)[0].childNodes[0].nodeValue
            moduleName = xml.xpath.Evaluate(self.__MODULENAME_ELEMENT_NAME, healthCheckElement)[0].childNodes[0].nodeValue
            className = xml.xpath.Evaluate(self.__CLASSNAME_ELEMENT_NAME, healthCheckElement)[0].childNodes[0].nodeValue

            if self.__logger is not None:
                # messagetuples[6142] = "Loading healthcheck: [name: '%s', module: '%s', class: '%s']"
                self.__logger.LogMsg( watchdog.logmessages.messagetuples[6142], (name, moduleName, className) )
            else:
                print watchdog.logmessages.messagetuples[6142][FORMAT] (name, moduleName, className)

            qualifiedClassName = moduleName + '.' + className
            
            healthCheckClass = self.__import_module(qualifiedClassName)
            
            logger = self.__logger
            
            if healthCheckClass is None:
                if self.__logger is not None:
                    # messagetuples[6145] = "Unable to load healthcheck '%s'."
                    self.__logger.LogMsg( watchdog.logmessages.messagetuples[6145], name )
                else:
                    print watchdog.logmessages.messagetuples[6145][FORMAT] % (name, moduleName, className)
                
                continue
            
            instantiateHealthCheckString = "%s(name, description, logger)" % qualifiedClassName
            instance = eval( instantiateHealthCheckString )
            
            if instance is None:
                if self.__logger is not None:
                    # messagetuples[6146] = "Unable to create healthcheck instance '%s'."
                    self.__logger.LogMsg( watchdog.logmessages.messagetuples[6146], name )
                else:
                    print watchdog.logmessages.messagetuples[6146][FORMAT] % (name, moduleName, className)
                
                continue            
            
            # TODO: If init calls start producing exceptions, wrap init_settings() call in a try/except block.
            initSuccessful = instance.init_settings(watchdog.settings.values)
            if initSuccessful:
                healthCheckInstances.append(instance)
            else:
                if self.__logger is not None:
                    # messagetuples[6151] = "Encountered error while initializing settings for healthcheck '%s'. Continuing without this healthcheck."
                    self.__logger.LogMsg( watchdog.logmessages.messagetuples[6150], name )
                else:
                    print watchdog.logmessages.messagetuples[6150][FORMAT] % (name, moduleName, className)
        
        document.unlink()

        healthCheckNames = [ str(healthCheck.get_name()) for healthCheck in healthCheckInstances  ]
        if self.__logger is not None:
            # messagetuples[6143] = "Watchdog running with %s healthchecks defined. %s"
            self.__logger.LogMsg( watchdog.logmessages.messagetuples[6143], (str( len(healthCheckNames) ), healthCheckNames) )
        else:
            print watchdog.logmessages.messagetuples[6143][FORMAT] % (str( len(healthCheckNames) ), str(healthCheckNames))

        if self.__logger is not None:
            self.__logger.TraceFunctionExit()

        return healthCheckInstances

