########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

# ----------------------------------------------------------------------------
# $Revision: $ $Date: $ $Author: $
# ----------------------------------------------------------------------------
#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Watchdog
# *
# * FILE NAME:       watchdog/healthchecks/healthcheck_controlscripts.py
# *
# * DESCRIPTION:     The controlscripts healthcheck routines used by the 
# *                  IA Middleware Watchdog. 
# *
# * AUTHOR:          Liming (Michael) Yu
# *                  IBM Tivoli
# *                  yul@us.ibm.com
# *
# ****************************************************************************
# */
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
#              Change Log
#              ----------
# 07 May 2009  Initial version.
#
# 22 May 2009  TIAMC00006112: changed architecture to handle running control
#              scripts on multiple collections.
#
#  1 Jul 2009  Updated access to IANode fields to use getter methods.
#
# 24 Aug 2010  TIAMC00015092: output pstree for hanging controlscript
#

iamInstallBase        = '/opt/tivoli/tiam'
iamiddlewarePythonLib = '%s/lib' % iamInstallBase

import  sys

if iamiddlewarePythonLib not in sys.path:
    sys.path.append(iamiddlewarePythonLib)

import  iamiddleware.externalcommand

import  watchdog.healthcheck
import  watchdog.healthchecks.healthcheck_controlscripts_messages

class ControlScriptsHealthCheck(watchdog.healthcheck.HealthCheck):
    """"SSH to the node in question and run ia_watchdog_controlscripts_elapsedtime"""

    # Explicitly create a super init reference so that the
    # superclass name is only explicitly stated in one place.
    __super_init = watchdog.healthcheck.HealthCheck.__init__

    __tiamUtilityPath = '/tiam_utility'
    __workloadmanagerConfigFile = __tiamUtilityPath + '/config/workloadmanager_config.xml'

    __WATCHDOG_ELEMENT_PATH = 'Watchdog'

    __settings = dict()


    def __init__(self, name, description, logger):
        self.__super_init(name, description, logger)


    def init_settings(self, watchdog_settings):
        
        """Load settings from workloadmanager_config.xml file."""

        self.logger.TraceMsg( 'Loading %s health-check configuration values.' % self.get_name() )

        settingsAreValid = True

        self.__settings.setdefault('RunControlScriptsCommand', 'ia_watchdog_controlscripts_elapsedtime')
        self.__settings.setdefault('ControlScriptsMaxRunTime', '480')

        #   define the filenames for the controlscripts
        self.__settings.setdefault('ControlAuditScript',             'wlm_controlaudit.sh')
        self.__settings.setdefault('ControlFrankTSMScript',          'wlm_controlfranktsm.sh')
        self.__settings.setdefault('ControlFuseMetaScript',          'wlm_controlfusemeta.sh')
        self.__settings.setdefault('ControlIngestCoordinatorScript', 'wlm_controlingestcoordinator.sh')
        self.__settings.setdefault('ControlSSAMTSMScript',           'wlm_controlssamtsm.sh')

        configuration_file_keys = [ 'IAScriptPath',
                                    'ControlScriptsMaxRunTime'
                                  ]

        for key in configuration_file_keys:
            if watchdog_settings.has_key( key ):
                self.__settings[key] = watchdog_settings[key]
            else:
                self.logger.TraceMsg( "Health check '%s' was unable to load setting '%s' from configuration file." % (self.get_name(), key) )
                settingsAreValid = False

        self.logger.TraceMsg( '%s health-check configuration values are: %s' % (self.get_name(), self.__settings) )

        return settingsAreValid


    def get_error_msg_tuple(self):
        return watchdog.healthchecks.healthcheck_controlscripts_messages.format[6320]


    def __perform_check(self, node):
        if self.logger is not None:
            self.logger.TraceFunctionEntry()
            
        node_ip = node.getInternalAddress()
            
        sshCommand = self.get_remote_shell_command(node_ip) + self.__settings['IAScriptPath'] + '/' +            \
                        self.__settings['RunControlScriptsCommand'] +                                            \
                        " " + self.__settings['ControlAuditScript'] +                                            \
                        " " + self.__settings['ControlFrankTSMScript'] +                                         \
                        " " + self.__settings['ControlFuseMetaScript'] +                                         \
                        " " + self.__settings['ControlIngestCoordinatorScript'] +                                \
                        " " + self.__settings['ControlSSAMTSMScript']

        if self.logger is not None:
            # healthcheck_controlscripts_messages.format[6321] = "Remote check for controlscripts command :: %s"
            self.logger.LogMsg( watchdog.healthchecks.healthcheck_controlscripts_messages.format[6321], sshCommand )
    
        resultCode, commandOutput = iamiddleware.externalcommand.issue_external_command(sshCommand, True, self.logger)
       
        if self.logger is not None:
            # healthcheck_controlscripts_messages.format[6324] = "Remote check for controlscripts command result output:: %s"
            self.logger.LogMsg( watchdog.healthchecks.healthcheck_controlscripts_messages.format[6324], commandOutput )

        if commandOutput == "":
            # good: all controlscripts are not running
            if self.logger is not None:
                # healthcheck_controlscripts_messages.format[6322] = "Remote check for controlscripts command result :: rc=%d, controlscriptName='%s', collectionMountpoint='%s', pid=%s, elapsedtime=%s seconds"
                self.logger.LogMsg( watchdog.healthchecks.healthcheck_controlscripts_messages.format[6322], 
                                    (resultCode, "n/a", "n/a", "n/a", "n/a") )

        else:
            # ok: at least one controlscript is running, the question is how long...
            commandOutputList = commandOutput.split("\n")
            for controlscriptResult in commandOutputList:
                controlscriptPid = controlscriptResult.split()[0]
                controlscriptElapsedtime = controlscriptResult.split()[1]
                controlscriptName = controlscriptResult.split()[2]
                controlscriptCollectionMountpoint = controlscriptResult.split()[3]

                if self.logger is not None:
                    # healthcheck_controlscripts_messages.format[6322] = "Remote check for controlscripts command result :: rc=%d, controlscriptName='%s', collectionMountpoint='%s', pid=%s, elapsedtime=%s seconds"
                    self.logger.LogMsg( watchdog.healthchecks.healthcheck_controlscripts_messages.format[6322], 
                                        (resultCode, controlscriptName, controlscriptCollectionMountpoint, controlscriptPid, controlscriptElapsedtime) )

                if int(controlscriptElapsedtime) > int(self.__settings['ControlScriptsMaxRunTime']):
                    # 15092
                    pstreeCommand = self.get_remote_shell_command(node_ip) + "/usr/bin/pstree " + controlscriptPid + " -A -a -c -l -p"
                    pstreeRc, pstreeOut = iamiddleware.externalcommand.issue_external_command(pstreeCommand, True, self.logger)
                    # healthcheck_controlscripts_messages.format[6323] = "'%s' has been running for %s seconds.  It can not run for more than %s seconds.  Pstree output: \"%s\""
                    self.logger.LogMsg( watchdog.healthchecks.healthcheck_controlscripts_messages.format[6323], 
                                        (controlscriptName + " " + controlscriptCollectionMountpoint + " " + controlscriptPid, 
                                         controlscriptElapsedtime, self.__settings['ControlScriptsMaxRunTime'],
                                         pstreeOut) ) 
                    # when one controlscript exceeds the max runtime, we report this and exit
                    resultCode = -1
                    break

        if self.logger is not None:
            self.logger.TraceFunctionExit()
            
        return resultCode == 0


    def is_healthy(self, node):
        return self.__perform_check(node);
