########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

# ----------------------------------------------------------------------------
# $Revision: $ $Date: $ $Author: $
# ----------------------------------------------------------------------------
#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Watchdog
# *
# * FILE NAME:       watchdog/healthchecks/healthcheck_managecollections.py
# *
# * DESCRIPTION:     The manage_collections health check routines used by the 
# *                  IA Middleware Watchdog.
# *
# * AUTHOR:          Michael A. Richmond
# *                  IBM Almaden Research Center
# *                  mar@almaden.ibm.com
# *
# ****************************************************************************
# */
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
#              Change Log
#              ----------
# 05 Dec 2008  Initial version. (Moved from ia_watchdog file)
#
# 16 Jan 2009  Updated all logger.LogMsg(...) calls to match new LogMsg(...)
#              implementation.
#
# 28 May 2009  Logging output from command.
#
#  1 Jul 2009  Updated access to IANode fields to use getter methods.
#

iamInstallBase        = '/opt/tivoli/tiam'
iamiddlewarePythonLib = '%s/lib' % iamInstallBase

import  sys
import  time
import  xml
import  xml.dom.ext
import  xml.xpath

if iamiddlewarePythonLib not in sys.path:
    sys.path.append(iamiddlewarePythonLib)

import  iamiddleware.externalcommand

import  watchdog.healthcheck
import  watchdog.healthchecks.healthcheck_managecollections_messages

class ManageCollectionLockHealthCheck(watchdog.healthcheck.HealthCheck):
    """"Verify that a lock is held against the file associated with
        the manage_collection process on the node being checked.
    """

    # Explicitly create a super init reference so that the
    # superclass name is only explicitly stated in one place.
    __super_init = watchdog.healthcheck.HealthCheck.__init__

    __tiamUtilityPath = '/tiam_utility'
    __workloadmanagerConfigFile = __tiamUtilityPath + '/config/workloadmanager_config.xml'

    __WATCHDOG_ELEMENT_PATH = 'Watchdog'

    __STATUS_NODES_NODE_ELEMENT_PATH = 'NODES/NODE'
    __IP_ATTRIBUTE_NAME = 'IP'
    __MANAGING_COLLECTIONS_ATTRIBUTE = 'MANAGING_COLLECTIONS'

    __settings = dict()


    def __init__(self, name, description, logger):
        self.__super_init(name, description, logger)


    def init_settings(self, watchdog_settings):
        """Load settings from workloadmanager_config.xml file."""

        self.logger.TraceMsg( 'Loading %s health-check configuration values.' % self.get_name() )

        settingsAreValid = True

        self.__settings.setdefault( 'WorkloadManagerStatusCommand', 'workloadmanager.sh -status' )
        self.__settings.setdefault( 'ManageCollectionsImproveHealthCommand', 'mw_controlwlm.sh -start')
        self.__settings.setdefault('ManageCollectionsMaxCheckCount', '3')
        self.__settings.setdefault('ManageCollectionsCheckDelay', '10')

        configuration_file_keys = [ 'IAScriptPath',
                                    'ManageCollectionsMaxCheckCount',
                                    'ManageCollectionsCheckDelay'
                                  ]

        for key in configuration_file_keys:
            if watchdog_settings.has_key( key ):
                self.__settings[key] = watchdog_settings[key]
            else:
                self.logger.TraceMsg( "Health check '%s' was unable to load setting '%s' from configuration file." % (self.get_name(), key) )
                settingsAreValid = False

        self.logger.TraceMsg( '%s health-check configuration values are: %s' % (self.get_name(), self.__settings) )

        return settingsAreValid
        

    def get_error_msg_tuple(self):
        return watchdog.healthchecks.healthcheck_managecollections_messages.format[6306]


    def __isManagingCollections(self, node_ip):
        if self.logger is not None:
            self.logger.TraceFunctionEntry()

        workloadManagerStatusCommand = self.__settings['IAScriptPath'] + '/' +      \
                                       self.__settings['WorkloadManagerStatusCommand']

        rc, status = iamiddleware.externalcommand.issue_external_command( workloadManagerStatusCommand, True, self.logger )

        if rc != 0:
            if self.logger is not None:
                # healthcheck_managecollections_messages.format[6307] = 'Workload manager status command result :: rc=%d, %s'
                self.logger.LogMsg( watchdog.healthchecks.healthcheck_managecollections_messages.format[6307], (rc, status))

            if self.logger is not None:
                self.logger.TraceFunctionExit()
            return False

        statusDocument = xml.dom.minidom.parseString(status)

        nodeElement = None
        # find "STATUS/NODES/NODE" elements in status xml
        for currNode in xml.xpath.Evaluate(self.__STATUS_NODES_NODE_ELEMENT_PATH, statusDocument.documentElement):
            if currNode.getAttribute(self.__IP_ATTRIBUTE_NAME) == node_ip:
                nodeElement = currNode
                break

        if nodeElement is None:
            if self.logger is not None:
                # healthcheck_managecollections_messages.format[6308] = "Unable to find status information for node '%s'"
                self.logger.LogMsg( watchdog.healthchecks.healthcheck_managecollections_messages.format[6308], node_ip)

            statusDocument.unlink()
            if self.logger is not None:
                self.logger.TraceFunctionExit()
            return False

        managingCollections = nodeElement.getAttribute(self.__MANAGING_COLLECTIONS_ATTRIBUTE)
        
        if self.logger is not None:
            self.logger.TraceMsg( "ManageCollectionLockHealthCheck.__isManagingCollections(%s) result=%s" %
                                    (node_ip, managingCollections) )

        statusDocument.unlink()

        if self.logger is not None:
            self.logger.TraceFunctionExit()
        return managingCollections == "true"


    def __perform_check(self, node):

        if self.logger is not None:
            self.logger.TraceFunctionEntry()

        node_ip = node.getInternalAddress()


        manageCollectionsCheckCount = 0
        while True:
            isManageCollectionsHealthy = self.__isManagingCollections(node_ip)
            manageCollectionsCheckCount += 1
            
            if isManageCollectionsHealthy:
                if self.logger is not None:
                    self.logger.TraceMsg("Node %s has healthy manage_collections process");

                if self.logger is not None:
                    self.logger.TraceFunctionExit()
                return True

            if manageCollectionsCheckCount < int(self.__settings['ManageCollectionsMaxCheckCount']):
                time.sleep( int(self.__settings['ManageCollectionsCheckDelay']) )
            else:
                break

        if self.logger is not None:
            # healthcheck_managecollections_messages.format[6309] = "The manage_collections process on node %s is not healthy (after %s attempts to check the health)"
            self.logger.LogMsg( watchdog.healthchecks.healthcheck_managecollections_messages.format[6309],      \
                                (node_ip, self.__settings['ManageCollectionsMaxCheckCount']) )
        
        if self.logger is not None:
            self.logger.TraceFunctionExit()
        return False

    def is_healthy(self, node):
        return self.__perform_check(node);


    def has_health_improvement(self):
        """This HealthCheck class provides a health improvement operation."""
        
        return True


    def improve_health(self, node):
        """Attempt to improve the health of manage_collections on
           node_ip by restarting the manage_collections process.
        """
        
        if self.logger is not None:
            self.logger.TraceFunctionEntry()
        
        node_ip = node.getInternalAddress()
        
        startManageCollectionsCommand = self.get_remote_shell_command(node_ip) +         \
                                self.__settings['IAScriptPath'] + '/' +           \
                                self.__settings['ManageCollectionsImproveHealthCommand']
        
        rc, resultMessage = iamiddleware.externalcommand.issue_external_command( startManageCollectionsCommand, True, self.logger )

        if rc != 0:
            if self.logger is not None:
                # healthcheck_managecollections_messages.format[6310] = "manage_collections health improvement command on node '%s' produced exit code '%d' and result '%s'"
                self.logger.LogMsg( watchdog.healthchecks.healthcheck_managecollections_messages.format[6310], (node_ip, rc, resultMessage) )
        
        if self.logger is not None:
            self.logger.TraceFunctionExit()
        return
