########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

# ----------------------------------------------------------------------------
# $Revision: $ $Date: $ $Author: $
# ----------------------------------------------------------------------------
#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Watchdog
# *
# * FILE NAME:       watchdog/healthchecks/healthcheck_unsuccessfullcomponentstarts.py
# *
# * DESCRIPTION:     The unsuccessful component starts health check 
# *                  routines used by the IA Middleware Watchdog.
# *
# * AUTHOR:          Michael A. Richmond
# *                  IBM Almaden Research Center
# *                  mar@almaden.ibm.com
# *
# ****************************************************************************
# */
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
#              Change Log
#              ----------
# 05 Dec 2008  Initial version. (Moved from ia_watchdog file)
#
# 16 Jan 2009  Updated all logger.LogMsg(...) calls to match new LogMsg(...)
#              implementation.
#
# 28 May 2009  Logging output from command.
#
#  1 Jul 2009  Updated access to IANode fields to use getter methods.
#

iamInstallBase        = '/opt/tivoli/tiam'
iamiddlewarePythonLib = '%s/lib' % iamInstallBase

import  sys
import  xml
import  xml.dom.ext
import  xml.xpath

if iamiddlewarePythonLib not in sys.path:
    sys.path.append(iamiddlewarePythonLib)

import    iamiddleware.externalcommand

import  watchdog.healthcheck
import  watchdog.healthchecks.healthcheck_unsuccessfulcomponentstarts_messages


class UnsuccessfulComponentStartsHealthCheck(watchdog.healthcheck.HealthCheck):
    """Verify that IAM components have not had too many
       unsuccessful starts on the node.
       Note: Test returns "True" if there have not been
       too many unsuccessful starts. "False" if there *have*
       been too many unsuccessful starts.
    """

    # Explicitly create a super init reference so that the
    # superclass name is only explicitly stated in one place.
    __super_init = watchdog.healthcheck.HealthCheck.__init__

    __tiamUtilityPath = '/tiam_utility'
    __workloadmanagerConfigFile = __tiamUtilityPath + '/config/workloadmanager_config.xml'

    __WATCHDOG_ELEMENT_PATH = 'Watchdog'

    __COLLECTIONS_COLLECTION_ELEMENT_PATH = 'COLLECTIONS/COLLECTION'
    __NODE_ELEMENT_PATH = 'NODE'
    __COMPONENT_ELEMENT_PATH = 'COMPONENT'
    __COLLECTION_NAME_ATTRIBUTE_NAME = 'NAME'
    __NODE_ID_ATTRIBUTE_NAME = 'ID'
    __UNSUCCESSFULSTARTS_ATTRIBUTE_NAME = 'UNSUCCESSFUL_STARTS'
    
    __settings = dict()


    def __init__(self, name, description, logger):
        self.__super_init(name, description, logger)
        

    def init_settings(self, watchdog_settings):
        """Load settings from workloadmanager_config.xml file."""

        self.logger.TraceMsg( 'Loading %s health-check configuration values.' % self.get_name() )

        settingsAreValid = True

        self.__settings.setdefault( 'WorkloadManagerStatusCommand', 'workloadmanager.sh -status' )

        configuration_file_keys = [ 'IAScriptPath',
                                    'MaximumUnsuccessfulComponentStarts'
                                  ]

        for key in configuration_file_keys:
            if watchdog_settings.has_key( key ):
                self.__settings[key] = watchdog_settings[key]
            else:
                self.logger.TraceMsg( "Health check '%s' was unable to load setting '%s' from configuration file." % (self.get_name(), key) )
                settingsAreValid = False

        self.logger.TraceMsg( '%s health-check configuration values are: %s' % (self.get_name(), self.__settings) )

        return settingsAreValid


    def get_error_msg_tuple(self):
        return watchdog.healthchecks.healthcheck_unsuccessfulcomponentstarts_messages.format[6314]


    def __get_status_document(self):
        if self.logger is not None:
            self.logger.TraceFunctionEntry()

        workloadManagerStatusCommand = self.__settings['IAScriptPath'] + '/' +      \
                                       self.__settings['WorkloadManagerStatusCommand']

        rc, status = iamiddleware.externalcommand.issue_external_command( workloadManagerStatusCommand, True, self.logger )

        if rc != 0:
            if self.logger is not None:
                # healthcheck_unsuccessfulcomponentstarts_messages[6315] = 'Workload manager status command result :: rc=%d, %s'
                self.logger.LogMsg( watchdog.healthchecks.healthcheck_unsuccessfulcomponentstarts_messages.format[6315], (rc, status) )
                self.logger.TraceFunctionExit()
            return None

        statusDocument = xml.dom.minidom.parseString(status)
        if self.logger is not None:
            self.logger.TraceFunctionExit()
        return statusDocument


    def __perform_check(self, node):
        if self.logger is not None:
            self.logger.TraceFunctionEntry()

        node_ip = node.getInternalAddress()

        statusDocument = self.__get_status_document()

        collectionComponentLists = dict()
        # find "STATUS/COLLECTIONS/COLLECTION" elements in status xml
        for currCollection in xml.xpath.Evaluate(self.__COLLECTIONS_COLLECTION_ELEMENT_PATH, statusDocument.documentElement):
            collectionName = currCollection.getAttribute(self.__COLLECTION_NAME_ATTRIBUTE_NAME).strip()

            componentList = []
            
            for currNode in xml.xpath.Evaluate(self.__NODE_ELEMENT_PATH, currCollection):
                if currNode.getAttribute(self.__NODE_ID_ATTRIBUTE_NAME).strip() == node_ip:
                    componentListForCollectionOnNode = xml.xpath.Evaluate(self.__COMPONENT_ELEMENT_PATH, currNode)

                    componentList.extend(componentListForCollectionOnNode)
            collectionComponentLists[collectionName] = componentList


        unsuccessfulStartsUnderThreshhold = True
        for collectionName in collectionComponentLists.keys():
            componentList = collectionComponentLists[collectionName]
            for component in componentList:
                try:
                    unsuccessfulStartCount = int( component.getAttribute(self.__UNSUCCESSFULSTARTS_ATTRIBUTE_NAME) )
                except:
                    # If we could not convert the attribute to an integer then the
                    # the attribute holds a value that we don't care about.
                    unsuccessfulStartCount = None                    
                    
                if ( unsuccessfulStartCount is not None ) and                                           \
                   ( unsuccessfulStartCount > int(self.__settings['MaximumUnsuccessfulComponentStarts']) ):
                    unsuccessfulStartsUnderThreshhold = False
                    if self.logger is not None:
                        self.logger.TraceMsg( ("Collection '%s' on node '%s' has experienced " +
                                          "too many unsuccessful component starts for %s") %    \
                                          (collectionName, node_ip, component.toxml()) )

        statusDocument.unlink()
        
#        print "unsuccessfulStarts = %s" % unsuccessfulStartsUnderThreshhold

        if self.logger is not None:
            self.logger.TraceFunctionExit()
        return unsuccessfulStartsUnderThreshhold


    def is_healthy(self, node):
        isHealthy = self.__perform_check(node);
        
        return isHealthy
        
