########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

# ----------------------------------------------------------------------------
# $Revision: $ $Date: $ $Author: $
# ----------------------------------------------------------------------------
#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Watchdog
# *
# * FILE NAME:       watchdog/noderefresh.py
# *
# * DESCRIPTION:     Library routines used by the IA Middleware 
# *                  Watchdog component.
# *
# * AUTHOR:          Michael A. Richmond
# *                  IBM Almaden Research Center
# *                  mar@almaden.ibm.com
# *
# ****************************************************************************
# */
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
#              Change Log
#              ----------
# 09 Dec 2008  Initial version. (Moved from wlm_watchdog file)
#
# 16 Jan 2009  Updated all logger.LogMsg(...) calls to match new LogMsg(...)
#              implementation and logmessages.py file.
#              Added creation of parent directories when creating new 
#              node refresh file.
#
# 13 Aug 2009  Implemented padding of file to in length of 1024 bytes.
#
# 28 Jan 2010  Switch from datetime.now() to datetime.utcnow() for time calculation
#

iamInstallBase        = '/opt/tivoli/tiam'
iamiddlewarePythonLib = '%s/lib' % iamInstallBase

import  datetime
import  errno
import  os
import  re
import  StringIO
import  sys
import  xml
import  xml.dom.ext
import  xml.xpath

if iamiddlewarePythonLib not in sys.path:
    sys.path.append(iamiddlewarePythonLib)

import  watchdog.settings


class NodeRefreshCount:

    __TIAM_UTILITY_REFRESH_PATH = '/tiam_utility/load/nodes'
    
    __REFRESH_COUNT_FILENAME_FORMAT = '%s_refreshcount.xml'

    __REFRESH_FILE_MIN_SIZE = 1024  # bytes
    
    __NODE_REFRESH_LIST_ELEMENTNAME = 'noderefreshlist'
    __NODE_IP_ATTRIBUTENAME         = 'ip'
    __NODE_REFRESH_TIME_ELEMENTNAME = 'refreshtime'
    __NODE_REFRESH_TIME_PATH        = 'refreshtime'
    
    __REFRESH_TIMESTAMP_FORMAT      = '%Y-%m-%dT%H:%M:%S'
    
    __XML_FILE_ENCODING = 'UTF-8'
    
    __XML_PADDING_COMMENT_HEADER = "PADDING_TO_MIN_FILE_SIZE"
    __XML_PADDING_COMMAND_CHARACTER = " "
    

    def __init__(self, nodeUID, logger=None):
        """Create a NodeRefreshCount instance. Constructor
           has side effect of creating a node refresh count
           file if one does not exist on the filesystem.
        """
        self.__logger = logger
                
        if self.__logger is None:
            print "Warning: 'NodeRefreshCount' created without a logger object"
        
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()

        self.__nodeUID = nodeUID
        try:
            self.__parse_refresh_file()
        except:
            if self.__logger is not None:
                self.__logger.TraceMsg('Node %s does not have a refreshcount file. Creating a new file.' % self.__nodeUID)
            document = self.__create_refreshcount_document()
            self.__write_refreshcount_file(document)
            document.unlink()

        if self.__logger is not None:
            self.__logger.TraceFunctionExit()
        
        
    def __str__(self):
         """Produce string representation of NodeRefreshCount 
            values.
        """
         
         return "(nnodeUID=" + str(self.__nodeUID ) + ")" 
        

    def __get_refresh_count_file_path_format(self):
        return self.__TIAM_UTILITY_REFRESH_PATH + '/' + self.__REFRESH_COUNT_FILENAME_FORMAT


    def __parse_refresh_file(self):
        return xml.dom.minidom.parse(self.__get_refresh_count_file_path_format() % self.__nodeUID)


    def __write_refreshcount_file(self, document):
        """Overwrite the refresh count file with the parsed
           XML structure represented by document.
        """
        
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()
        
        if (self.__nodeUID is None) or (len(self.__nodeUID) < 1):
            # TODO: FatalError not defined?
            raise FatalError( ("Error: attempting to write a refreshcount file for nodeUID='%s'.\n" + 
                               "(refreshcount filename starts with the nodeUID.)" ) % nodeUID )

        documentFileName = self.__get_refresh_count_file_path_format() % self.__nodeUID
        
        try:
            os.makedirs(self.__TIAM_UTILITY_REFRESH_PATH)
        except OSError, error:
            if not ( (error.errno == errno.EEXIST) and ( os.path.isdir(self.__TIAM_UTILITY_REFRESH_PATH) ) ):
                if self.__logger is not None:
                    # messagetuples[6149] = "Encountered error while creating %s folder. (error message '%s')"
                    self.__logger.LogMsg( watchdog.logmessages.messagetuples[6149],      \
                                          (self.__TIAM_UTILITY_REFRESH_PATH, str(error)) )
                else:
                    print watchdog.logmessages.messagetuples[6149][1] % (self.__TIAM_UTILITY_REFRESH_PATH, str(error))
        
        self.__pad_xml_to_bytes(document, self.__REFRESH_FILE_MIN_SIZE)
        
        try:
            fileObject = open(documentFileName, "w")
            xml.dom.ext.PrettyPrint(document, fileObject, self.__XML_FILE_ENCODING)
            fileObject.close()
        except Exception, error:
            if self.__logger is not None:
                # messagetuples[6131] = "Encountered error while writing node refresh count file '%s'. (error message '%s')"
                self.__logger.LogMsg( watchdog.logmessages.messagetuples[6131], (documentFileName, str(error)) )
            else:
                print watchdog.logmessages.messagetuples[6131][1] % (documentFileName, str(error) )

            print "Error encountered while creating refresh file: %s" % error

        if self.__logger is not None:
            self.__logger.TraceFunctionExit()


    def __pad_xml_to_bytes(self, document, bytes):
        """Create a comment element in the document
           dom tree to ensure that the saved document
           file is at least bytes long.
        """
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()
        
        # Strip off any existing padding comment...
        for child in document.documentElement.childNodes:
            if (child.nodeType == xml.dom.minidom.Node.COMMENT_NODE) and (child.nodeValue.find(self.__XML_PADDING_COMMENT_HEADER) != -1):
                document.documentElement.removeChild(child)
                child.unlink
        
        paddingCommentElement = document.createComment(self.__XML_PADDING_COMMENT_HEADER)
        document.documentElement.appendChild(paddingCommentElement)
        
        fileContentStream = StringIO.StringIO()
        xml.dom.ext.PrettyPrint(document, stream=fileContentStream, encoding=self.__XML_FILE_ENCODING)

        contentLength = len(fileContentStream.getvalue())
        if contentLength < bytes:
            paddingNeeded = bytes - contentLength
            # fastest way to produce a string of paddingNeeded characters in length
            paddingString = ''.join([self.__XML_PADDING_COMMAND_CHARACTER for i in xrange(paddingNeeded)])  
            

            if self.__logger is not None:
                self.__logger.TraceMsg( "Padding noderefresh.xml file. content length: %d, padding length: %d" %    \
                                        (contentLength, len(paddingString)) )
            else:
                print ( "Padding noderefresh.xml file. content length: %d, padding length: %d" %    \
                                        (contentLength, len(paddingString)) )
            paddingCommentElement.nodeValue = paddingCommentElement.nodeValue + paddingString

        # if the content length is longer than the bytes plus the mininum padding comment
        # then we should remove the padding comment...
        xmlCommentTagLength = 7
        fileContentStream = StringIO.StringIO()
        xml.dom.ext.PrettyPrint(document, stream=fileContentStream, encoding=self.__XML_FILE_ENCODING)
        if len(fileContentStream.getvalue()) > bytes + len(self.__XML_PADDING_COMMENT_HEADER) + xmlCommentTagLength:
            document.documentElement.removeChild(paddingCommentElement)
            paddingCommentElement.unlink()

        if self.__logger is not None:
            self.__logger.TraceFunctionExit()


    def __create_refreshcount_document(self):
        """Create a new refresh count file with top-level
           xml elements.
        """
        
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()
        
        document = xml.dom.minidom.Document()
        
        nodeRefreshListElement = document.createElement(self.__NODE_REFRESH_LIST_ELEMENTNAME)
        nodeRefreshListElement.setAttribute(self.__NODE_IP_ATTRIBUTENAME, self.__nodeUID)
        
        document.appendChild(nodeRefreshListElement)
        
        if self.__logger is not None:
            self.__logger.TraceFunctionExit()
        return document
    
    
    def __parse_date(self, dateString):
        """Parse an ISO date string as found in the
           node refresh count.
        """
        
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()

        # We're applying as UTC. Ignore the optional Z at the end.
        if dateString[-1] == 'Z':
            dateString = dateString[:-1];

        p = re.compile('[.:T-]')
        isobits = [int(s) for s in p.split(dateString)]

        if self.__logger is not None:
            self.__logger.TraceFunctionExit()
        return apply(datetime.datetime, isobits)
    
    
    def __trim_refresh_count(self):
        """Trim the contents of the node refresh file
           to remove timestamps that are older than the 
           interval we care about. Interval is from
           current time back n minutes. (Where n is
           defined in the watchdog configuration file.)
        """
        
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()

        document = self.__parse_refresh_file()
        self.__trim_refresh_count(document)
        document.unlink()

        if self.__logger is not None:
            self.__logger.TraceFunctionExit()

    
    def __trim_refresh_count(self, document):
        """Trim the contents of the node refresh file
           to remove timestamps that are older than the 
           interval we care about. Interval is from
           current time back n minutes. (Where n is
           defined in the watchdog configuration file.)
        """
        
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()

        refreshTimeDelta = datetime.timedelta(minutes=int( watchdog.settings.values['PeriodForMaxNumberOFNodeRefreshes'] ))
        
        refreshCountWindowStart = datetime.datetime.utcnow() - refreshTimeDelta
        
        refreshTimeList = xml.xpath.Evaluate(self.__NODE_REFRESH_TIME_PATH, document.documentElement)

        for timeElement in refreshTimeList:
            timeValue = self.__parse_date(timeElement.childNodes[0].nodeValue)
            if timeValue < refreshCountWindowStart:
                document.documentElement.removeChild(timeElement)
        
        self.__write_refreshcount_file(document)  

        if self.__logger is not None:
            self.__logger.TraceFunctionExit()

    
    def get_refresh_count(self):
        """Get the current refresh count trimmed to
           the interval that we care about.
        """
        
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()
        
        document = self.__parse_refresh_file()

        # Trim the refresh count file to remove times that are older 
        # than we care about.
        self.__trim_refresh_count(document)

        # Get the count of refreshes and clean up the xml document objects.
        count = self.__get_refresh_count(document)
            
        document.unlink()
        
        if self.__logger is not None:
            self.__logger.TraceFunctionExit()
        return count
    
    
    def __get_refresh_count(self, document):
        """Obtain the raw count of timestamps from the
           refresh count file.
        """
        
        return len( self.__get_raw_refresh_list(document) )


    def __get_raw_refresh_list(self, document):
        """Obtain the raw list of timestamps from the
           refresh count file.
        """
        
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()

        refreshTimeList = xml.xpath.Evaluate(self.__NODE_REFRESH_TIME_PATH, document.documentElement)

        if self.__logger is not None:
            self.__logger.TraceFunctionExit()

        return refreshTimeList
    

    def increment_refresh_count(self):
        """Increment the node refresh counter by adding
           the current timestamp to the refresh count file.
        """
        
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()
        
        document = xml.dom.minidom.parse(self.__get_refresh_count_file_path_format() % self.__nodeUID)

        self.__trim_refresh_count(document)

        timeStamp = datetime.datetime.utcnow().strftime(self.__REFRESH_TIMESTAMP_FORMAT)

        newRefreshElement = document.createElement(self.__NODE_REFRESH_TIME_ELEMENTNAME)
        newRefreshElementText = document.createTextNode(timeStamp)
        newRefreshElement.appendChild(newRefreshElementText)
        
        document.documentElement.appendChild(newRefreshElement)
        
        self.__write_refreshcount_file(document)
        
        count = self.__get_refresh_count(document)
        
        document.unlink()
        
        if self.__logger is not None:
            self.__logger.TraceFunctionExit()
        return count
