########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008-2010
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

# ----------------------------------------------------------------------------
# $Revision: $ $Date: $ $Author: $
# ----------------------------------------------------------------------------
#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Watchdog
# *
# * FILE NAME:       watchdog/nodepowerstate.py
# *
# * DESCRIPTION:     Library routines used by the IA Middleware 
# *                  Watchdog component.
# *
# * AUTHOR:          Michael A. Richmond
# *                  IBM Almaden Research Center
# *                  mar@almaden.ibm.com
# *
# ****************************************************************************
# */
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
#              Change Log
#              ----------
# 12 Aug 2009  Initial version.
#
# 21 Oct 2009  Defect 10031: 
#              1) changed __parse_powerstate_file() to handle exception cases
#              2) changed __write_xml_file() to write to temporary file first
#                 then renaming it to the original
#              3) changed update_node_powerstate() to handle exception cases
#
# 28 Jan 2010  Switch from datetime.now() to datetime.utcnow() for time calculation
#

iamInstallBase        = '/opt/tivoli/tiam'
iamiddlewarePythonLib = '%s/lib' % iamInstallBase

import  datetime
import  errno
import  fcntl
import  os
import  re
import  StringIO
import  sys
import  xml
import  xml.dom.ext
import  xml.xpath

if iamiddlewarePythonLib not in sys.path:
    sys.path.append(iamiddlewarePythonLib)

import  watchdog.logmessages
import  watchdog.settings


class State:
    ON           = 0
    OFF          = 1
    BOOTING      = 2
    UNKNOWN      = 3
    
    NAMES = { ON:      "on",
              OFF:     "off",
              BOOTING: "booting",
              UNKNOWN: "unknown"
            }


class PowerStates:

    DEBUG_LOCKING = True

    __TIAM_UTILITY_POWERSTATE_PATH = '/tiam_utility/runtime'
    
    __POWERSTATE_FILENAME_FORMAT = 'node_powerstates.xml'
    
    __POWERSTATE_LIST_ELEMENTNAME  = 'nodepowerstates'
    __POWERSTATE_ELEMENTNAME       = 'powerstate'
    __POWERSTATE_ELEMENT_PATH      = 'powerstate'
    __NODE_IP_ATTRIBUTE_PATH       = '//@ip'
    __NODE_IP_ATTRIBUTENAME        = 'ip'
    __NODE_TIMESTAMP_ATTRIBUTENAME = 'timestamp'
    
    __TIMESTAMP_FORMAT             = '%Y-%m-%dT%H:%M:%S'
    
    __POWERSTATES_FILE_MIN_SIZE    = 1024  # bytes
    
    __XML_FILE_ENCODING = 'UTF-8'
    
    __XML_PADDING_COMMENT_HEADER = "PADDING_TO_MIN_FILE_SIZE"
    __XML_PADDING_COMMAND_CHARACTER = " "

    
    def __init__(self, logger=None):
        """Create a PowerStates instance. Constructor
           has side effect of creating a powerstate.xml
           file if one does not exist on the filesystem.
        """
    
        self.__logger = logger
        
        if self.__logger is None:
            print "Warning: 'PowerStates' created without a logger object"
        
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()

        if self.__obtain_file_lock(self.__get_powerstate_file_path()):
            try:
                self.__parse_powerstate_file()
            except:
                if self.__logger is not None:
                    self.__logger.TraceMsg("System does not have a powerstate file. Creating a new file.")
                else:
                    print "System does not have a powerstate file. Creating a new file."
            
                document = self.__create_powerstate_document()
                self.__write_xml_file(document, self.__get_powerstate_file_path())
                document.unlink()

            self.__release_file_lock()
        else:
            if self.__logger is not None:
                self.__logger.TraceMsg( "Error: unable to obtain lock against for powerstate file '%s'" %       \
                                        self.__get_powerstate_file_path( ))

        if self.__logger is not None:
            self.__logger.TraceFunctionExit()


    def __obtain_file_lock(self, lockFilename):
        
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()
        
        pathElements = lockFilename.rsplit('/', 1)
        if (len(pathElements) == 2) and (len(pathElements[0]) > 0):
            folder = pathElements[0]
            folderCreated = self.__create_folder(folder)
            if not folderCreated:
                if self.__logger is not None:
                    self.__logger.TraceFunctionExit()
                return False

        if not os.path.exists(lockFilename):
            self.__lockfile = open( lockFilename, "w+" )
            self.__lockfile.close()
            self.__lockfile = None

        try:
            self.__lockfile = open( lockFilename, "r+" )
        except Exception, error:
            print "Exception is %s" % str(error)
            if self.__logger is not None:
                # messagetuples[6160] = "Unable to get a file handle for powerstate lock file '%s'"
                self.__logger.LogMsg( watchdog.logmessages.messagetuples[6160], (lockFilename) )
            else:
                print watchdog.logmessages.messagetuples[6160][1] % (lockFilename)
            
            if self.__logger is not None:
                self.__logger.TraceFunctionExit()
            return False

        try:
            fcntl.lockf(self.__lockfile.fileno(), fcntl.LOCK_EX)
        except IOError, error:
            print "Exception is %s" % str(error)
            if self.__logger is not None:
                # messagetuples[6161] = "Encountered I/O error while obtaining lock against powerstate file '%s'"
                self.__logger.LogMsg( watchdog.logmessages.messagetuples[6161], (lockFilename) )
            else:
                print watchdog.logmessages.messagetuples[6161] % (lockFilename)
            
            self.__lockfile.close()
                
            if self.__logger is not None:
                self.__logger.TraceFunctionExit()
            return False

        if self.__logger is not None:
            self.__logger.TraceFunctionExit()
        return True


    def __release_file_lock(self):

        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()

        try:
            fcntl.lockf(self.__lockfile.fileno(), fcntl.LOCK_UN)
        except IOError, error:
            print "Exception is %s" % str(error)
            if self.__logger is not None:
                # messagetuples[6162] = "Encountered I/O error while releasing lock against powerstate file '%s'"
                self.__logger.LogMsg( watchdog.logmessages.messagetuples[6162], (lockFilename) )
            else:
                print watchdog.logmessages.messagetuples[6162] % (lockFilename)
                self.__lockfile.close()
                self.__lockfile = None
        
                if self.__logger is not None:
                    self.__logger.TraceFunctionExit()
                self.__lockfile = None
                return False

        self.__lockfile.close()
        self.__lockfile = None

        if self.__logger is not None:
            self.__logger.TraceFunctionExit()
        return True
    

    def __create_folder(self, path):
        
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()
        
        folderCreated = True    
        
        if not os.path.exists(path):
            try:
                os.makedirs(path)
            except OSError, error:
                if not ( (error.errno == errno.EEXIST) and ( os.path.isdir(folder) ) ):
                    if self.__logger is not None:
                        # messagetuples[6149] = "Encountered error while creating %s folder. (error message '%s')"
                        self.__logger.LogMsg( watchdog.logmessages.messagetuples[6149], (folder, str(error)) )
                    else:
                        print watchdog.logmessages.messagetuples[6149][1] % (folder, str(error))
                folderCreated = False
        
        if self.__logger is not None:
            self.__logger.TraceFunctionExit()
        return folderCreated


    def __get_powerstate_file_path(self):
        return self.__TIAM_UTILITY_POWERSTATE_PATH + '/' + self.__POWERSTATE_FILENAME_FORMAT


    def __parse_powerstate_file(self):
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()

        try:
            f = open( self.__get_powerstate_file_path() )
        except:
            self.__logger.TraceMsg("Unable to open file: %s" % self.__get_powerstate_file_path())
            if self.__logger is not None:
                self.__logger.TraceFunctionExit()
            return None
        try:
            file_contents = f.read()
        except:
            self.__logger.TraceMsg("Unable to read file: %s" % self.__get_powerstate_file_path())
            f.close()
            if self.__logger is not None:
                self.__logger.TraceFunctionExit()
            return None
        try:
            myParsedDocument = xml.dom.minidom.parseString( file_contents )
        except:
            self.__logger.TraceMsg("Unable to parse contents of file %s: '%s'" % (self.__get_powerstate_flie_path(), file_contents))
            f.close()
            if self.__logger is not None:
                self.__logger.TraceFunctionExit()
            return None

        f.close()
        if self.__logger is not None:
            self.__logger.TraceFunctionExit()
        return myParsedDocument


    def __write_xml_file(self, document, filename):
        """Overwrite the powerstate file with the parsed
           XML structure represented by document.
        """
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()
        
        pathElements = filename.rsplit('/', 1)
        if (len(pathElements) == 2) and (len(pathElements[0]) > 0):
            folder = pathElements[0]
            folderCreated = self.__create_folder(folder)
            if not folderCreated:
                if self.__logger is not None:
                    self.__logger.TraceFunctionExit()
                return False

        self.__pad_xml_to_bytes(document, self.__POWERSTATES_FILE_MIN_SIZE)
                    
        try:
            fileObject = open(filename+".tmp", "w")
            xml.dom.ext.PrettyPrint(document, fileObject, self.__XML_FILE_ENCODING)
            fileObject.close()
            os.rename(filename+".tmp", filename)
        except Exception, error:
            if self.__logger is not None:
                # messagetuples[6163] = "Encountered error while writing powerstate file '%s'. (error message '%s')"
                self.__logger.LogMsg( watchdog.logmessages.messagetuples[6163], (filename, str(error)) )
            else:
                print watchdog.logmessages.messagetuples[6163][1] % (filename, str(error) )

            print "Error encountered while writing xml file: %s" % error

        if self.__logger is not None:
            self.__logger.TraceFunctionExit()


    def __pad_xml_to_bytes(self, document, bytes):
        """Create a comment element in the document
           dom tree to ensure that the saved document
           file is at least bytes long.
        """
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()
        
        # Strip off any existing padding comment...
        for child in document.documentElement.childNodes:
            if (child.nodeType == xml.dom.minidom.Node.COMMENT_NODE) and (child.nodeValue.find(self.__XML_PADDING_COMMENT_HEADER) != -1):
                document.documentElement.removeChild(child)
                child.unlink()
        
        paddingCommentElement = document.createComment(self.__XML_PADDING_COMMENT_HEADER)
        document.documentElement.appendChild(paddingCommentElement)
        
        fileContentStream = StringIO.StringIO()
        xml.dom.ext.PrettyPrint(document, stream=fileContentStream, encoding=self.__XML_FILE_ENCODING)

        contentLength = len(fileContentStream.getvalue())
        if contentLength < bytes:
            paddingNeeded = bytes - contentLength
            # fastest way to produce a string of paddingNeeded characters in length
            paddingString = ''.join([self.__XML_PADDING_COMMAND_CHARACTER for i in xrange(paddingNeeded)])  
            

            if self.__logger is not None:
                self.__logger.TraceMsg( "Padding noderefresh.xml file. content length: %d, padding length: %d" %    \
                                        (contentLength, len(paddingString)) )
            else:
                print ( "Padding noderefresh.xml file. content length: %d, padding length: %d" %    \
                                        (contentLength, len(paddingString)) )
            if self.DEBUG_LOCKING:
                import socket
                paddingCommentElement.nodeValue = paddingCommentElement.nodeValue + paddingString + " " + socket.gethostname()
            else:
                paddingCommentElement.nodeValue = paddingCommentElement.nodeValue + paddingString


        # if the content length is longer than the bytes plus the mininum padding comment
        # then we should remove the padding comment...
        xmlCommentTagLength = 7
        fileContentStream = StringIO.StringIO()
        xml.dom.ext.PrettyPrint(document, stream=fileContentStream, encoding=self.__XML_FILE_ENCODING)
        if len(fileContentStream.getvalue()) > bytes + len(self.__XML_PADDING_COMMENT_HEADER) + xmlCommentTagLength:
            document.documentElement.removeChild(paddingCommentElement)
            paddingCommentElement.unlink()

        if self.__logger is not None:
            self.__logger.TraceFunctionExit()        


    def __create_powerstate_document(self):
        """Create a new powerstate file with top-level
           xml elements.
        """
        
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()
        
        document = xml.dom.minidom.Document()
        
        powerStateListElement = document.createElement(self.__POWERSTATE_LIST_ELEMENTNAME)
        
        document.appendChild(powerStateListElement)
        
        if self.__logger is not None:
            self.__logger.TraceFunctionExit()
        return document


    def __parse_date(self, dateString):
        """Parse an ISO date string as found in the
           node refresh count.
        """
        
        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()

        # We're applying as UTC. Ignore the optional Z at the end.
        if dateString[-1] == 'Z':
            dateString = dateString[:-1];

        p = re.compile('[.:T-]')
        isobits = [int(s) for s in p.split(dateString)]

        if self.__logger is not None:
            self.__logger.TraceFunctionExit()
        return apply(datetime.datetime, isobits)


    def __update_document_with_node_powerstate(self, document, nodeID, powerstate):
        """Update the xml document with the powerstate for nodeID.
           Ensure that all powerstate elements in the file have an
           ip attribute and that there is only one entry for each
           nodeID.
        """

        powerstateElement = None
        powerstateList = xml.xpath.Evaluate(self.__POWERSTATE_ELEMENT_PATH, document.documentElement)
        foundNode = False
        for node in powerstateList:            
            if self.__NODE_IP_ATTRIBUTENAME not in node.attributes.keys():
                node.parentNode.removeChild(node)
                node.unlink()
                continue
            
            if node.attributes[self.__NODE_IP_ATTRIBUTENAME].nodeValue == nodeID:
                if not foundNode:
                    foundNode = True
                    powerstateElement = node
                else:
                    # if we have already found a node for nodeID then any other
                    # nodes for nodeID should be removed from the XML.
                    node.parentNode.removeChild(node)
                    node.unlink()

        if powerstateElement is None:
            powerstateElement = document.createElement(self.__POWERSTATE_ELEMENTNAME)
            powerstateElement.setAttribute(self.__NODE_IP_ATTRIBUTENAME, nodeID)
            powerstateElement.appendChild( document.createTextNode('') )
            document.documentElement.appendChild(powerstateElement)
        
        # Update timestamp...
        timestamp = datetime.datetime.utcnow().strftime(self.__TIMESTAMP_FORMAT)
        powerstateElement.setAttribute(self.__NODE_TIMESTAMP_ATTRIBUTENAME, timestamp)

        # Update state value...
        powerstateElement.childNodes[0].nodeValue = State.NAMES[powerstate]


    def update_node_powerstate(self, nodeID, powerstate):
        """Update the powerstate for nodeID to powerstate."""

        if self.__logger is not None:
            self.__logger.TraceFunctionEntry()
        
        updatePerformed = False
        
        if self.__obtain_file_lock(self.__get_powerstate_file_path()):     
            try: 
                document = self.__parse_powerstate_file()
            except:
                self.__logger.TraceMsg("There was an exception in parsing the powerstate file. The powerstate file may be corrupt. Creating a new powerstate file.")
                document = self.__create_powerstate_document()
                self.__write_xml_file(document, self.__get_powerstate_file_path())
                document.unlink()
                self.__release_file_lock()
                if self.__logger is not None:
                    self.__logger.TraceFunctionExit()
                return False

            if document == None:
                self.__logger.TraceMsg("There was an error in parsing the powerstate file. The powerstate file may be corrupt. Creating a new powerstate file.")
                document = self.__create_powerstate_document()
                self.__write_xml_file(document, self.__get_powerstate_file_path())
                document.unlink()
                self.__release_file_lock()
                if self.__logger is not None:
                    self.__logger.TraceFunctionExit()
                return False

            try:
                self.__update_document_with_node_powerstate(document, nodeID, powerstate)
                self.__write_xml_file(document, self.__get_powerstate_file_path())
                updatePerformed = True
            except:
                self.__logger.TraceMsg("There was an exception in modifying the powerstate file. The powerstate file may be corrupt. Creating a new powerstate file.")
                document = self.__create_powerstate_document()
                self.__write_xml_file(document, self.__get_powerstate_file_path())
                document.unlink()
                self.__release_file_lock()
                if self.__logger is not None:
                    self.__logger.TraceFunctionExit()
                return False

            if document != None:
                document.unlink()
            self.__release_file_lock() 

        if self.__logger is not None:
            self.__logger.TraceFunctionExit()
        return updatePerformed
