########################################################## {COPYRIGHT-TOP} ###
# Licensed Materials - Property of IBM
# IBM Information Archive (IIA)
#
# Copyright IBM Corporation 2008
#
# All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corporation.
########################################################## {COPYRIGHT-END} ###

# ----------------------------------------------------------------------------
# $Revision: $ $Date: $ $Author: $
# ----------------------------------------------------------------------------
#/****************************************************************************
# * COMPONENT NAME:  IA Middleware Watchdog
# *
# * FILE NAME:       watchdog/settings.py
# *
# * DESCRIPTION:     Library routines and defaults used by the IA Middleware 
# *                  Watchdog component.
# *
# * AUTHOR:          Michael A. Richmond
# *                  IBM Almaden Research Center
# *                  mar@almaden.ibm.com
# *
# ****************************************************************************
# */
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
#              Change Log
#              ----------
# 18 Sep 2008  Initial version.
#
# 06 Oct 2008  Added functions to support dynamic log/trace level updates
#
# 04 Dec 2008  Code moved into watchdog package.
#
# 29 Jan 2009  Added check for valid configuration values in
#              workloadmanager_config.xml file. Initial check validates that
#              the watchdog latency is not less than 5 minutes (300s). This
#              minimum value avoids an issue identified in test where a node
#              only just finishes booting and then is power-cycled due to
#              a failing health check.
#
# 30 Apr 2009  Added UnknownStateRebootTimeout (Michael Yu)
#
# 18 May 2009  Added CheckAllGPFSFilesystemsMountedCommand (Michael Yu)
#
# 22 May 2009  Added values for the controlscripts
#
# 05 Jun 2009  Changed load from xml file code to load *all* values under
#              <Watchdog> element.
#
# 16 Jul 2009  Removed default setting for ia_getnodeaddress command
#
# 07 Aug 2009  Added validation check to ensure that the
#              UnknownStateRebootTimeout is less than the WatchdogLatency
#              value.
#              Added missing PrintPowerControlIPCommand default.
#              (Michael Richmond)
#
# 23 Oct 2009  Added PingCommand and PingCommandArguments default values.
#              (Michael Richmond)
#

import  xml
import  xml.dom.minidom
import  xml.xpath

tiamUtilityPath      = '/tiam_utility'

wlmConfigFile             = tiamUtilityPath + '/config/workloadmanager_config.xml'
watchdogLoggingConfigFile = tiamUtilityPath + '/config/watchdog_logging.xml'


# *************************************************************************
# * Minimum settings values
# *************************************************************************
minimumValues = dict()
minimumValues['WatchdogLatency'] = 300        # seconds


# *************************************************************************
# * Structure to hold settings
# *************************************************************************
values = dict()


# *************************************************************************
# * Default Settings
# *
# * Note that this file should only define defaults that apply to the
# * core watchdog implementation. Default settings for specific 
# * health checks are defined in the health-check implementation class.
# * Violating this guideline makes the watchdog core dependent on a
# * specific health-check plug-in.
# *************************************************************************

# ------------------------------------------------------------------------
# ------------------------------------------------------------------------
# Internal core watchdog settings
# (These are not exposed in configuration file.)
# ------------------------------------------------------------------------
# ------------------------------------------------------------------------

#   command to obtain the hostname of the current node
values.setdefault('NodeHostnameCommand', '/opt/tivoli/tiam/bin/mw_print_local_gpfs_nodehostname')

#   command to obtain the unique node id
values.setdefault('NodeIDCommand', '/opt/tivoli/tiam/bin/mw_print_unique_nodeid')

#   command to check that /tiam_utility is mounted
values.setdefault('CheckTIAMUtilityMountedCommand', 'mw_check_fs_mounted.sh /tiam_utility')

#   command to map node name to hardware type
values.setdefault('PrintHardwareTypeCommand', 'ia_printhardwaretype')

#   command to map node name to internal ip address
values.setdefault('PrintInternalIPCommand', 'ia_printinternalip')

#   command to map node name to ip address for power control hardware
values.setdefault('PrintPowerControlIPCommand', 'ia_printpowercontrolip')

#   command to check that ia_watchdog is healthy on node
values.setdefault('CheckWatchdogHealthyCommand', 'mw_controlwatchdog.sh -status')

#   command to improve the health of ia_watchdog
values.setdefault('WatchdogImproveHealthCommand', 'mw_controlwatchdog.sh -start')

#   command to obtain the list of nodes statically defined to be part of the gpfs cluster
values.setdefault('ListNodesCommand', 'wd_printstaticnodelist.sh')

#   command to obtain the dynamic state of nodes in gpfs cluster
values.setdefault('ListNodeStatesCommand', 'wd_printnodestates.sh')

#   define ssh command arguments
values.setdefault('RemoteShellArguments', '')

#   define ping command
values.setdefault('PingCommand', '/bin/ping')

#   define ping command arguments
values.setdefault('PingCommandArguments', '-c 3')

# ------------------------------------------------------------------------
# ------------------------------------------------------------------------
# Externalized core watchdog settings...
# (These are exposed in the configuration file.)
# ------------------------------------------------------------------------
# ------------------------------------------------------------------------
values.setdefault('IAScriptPath', '/opt/tivoli/tiam/bin')

values.setdefault('MaxNumberOfNodeRefreshes', '4')
values.setdefault('PeriodForMaxNumberOFNodeRefreshes', '60')      # minutes

values.setdefault('MaximumHealthImprovementAttempts', '3')
values.setdefault('HealthImprovementCheckDelay', '10')            # minutes

values.setdefault('MaximumUnsuccessfulComponentStarts', '100')

values.setdefault('WatchdogLatency', '600')
values.setdefault('ClusterStabilizationPeriod', '120')           # seconds

values.setdefault('RemoteShellCommand', '/usr/bin/ssh')
values.setdefault('RemoteShellUser', 'root')

values.setdefault('PowerControlShellUser', 'USERID')
values.setdefault('PowerControlShellPassword', 'PASSW0RD')

values.setdefault('LogfileBasePath', '/tiam_utility/log')

values.setdefault('UnknownStateRebootTimeout', '480')             # seconds


# *************************************************************************
# * Settings parsing routines
# *************************************************************************
def get_xml_document(name):
    return xml.dom.minidom.parse(name)


def loadChildrenOfXMLElementIntoDictionary(xmlElement, settingsDict):
    """
        For each child immediately below xmlElement, insert the
        value of the element into the settingsDict.
        
        For example:
            <Settings>
                <Path>/tmp</Path>
                <UserID>root</UserID>
            </Settings>
            
        If the <Settings> element is passed in as the xmlElement
        then settingsDict should be updated with two key/value
        pairs:
            settingsDict["Path"]   = "/tmp"
            settingsDict["UserID"] = "root"
    """
    
    for node in xmlElement.childNodes:
        if node.nodeType == node.ELEMENT_NODE:
            name = node.localName
            value = node.childNodes[0].nodeValue
            if (name is not None) and (value is not None):
                settingsDict[str(name)] = str(value)
    

def read_watchdog_configuration_file():
    """Load IA watchdog configuration values from configuration file"""

    wlmConfigDocument = get_xml_document(wlmConfigFile)

    watchdogConfigElement = xml.xpath.Evaluate("Watchdog", wlmConfigDocument.documentElement)[0]

    loadChildrenOfXMLElementIntoDictionary(watchdogConfigElement, values)
            
    wlmConfigDocument.unlink()

 
def settings_valid():
    """Ensure that IA watchdog configuration values are valid.
       This involves enforcing minimum values for the watchdog
       latency.
    """
    
    valid = True
    
    watchdogLatencyKey = 'WatchdogLatency'
    if values.has_key(watchdogLatencyKey):
        if int(values[watchdogLatencyKey]) < minimumValues[watchdogLatencyKey]:
            print "error: watchdog latency value '%s' is less than minimum value '%s'" %    \
                    ( values[watchdogLatencyKey], str(minimumValues[watchdogLatencyKey]) )
            valid = valid and False
    else:
        print "error: watchdog settings do not include a value for required setting '%s'" % watchdogLatencyKey
        valid = valid and False
    
    # This test also relies on the watchdogLatencyKey existing. That check is performed above.
    unknownStateRebootTimeoutKey = 'UnknownStateRebootTimeout'
    if not values.has_key(unknownStateRebootTimeoutKey):
        print "error: watchdog settings do not include a value for required setting '%s'" % unknownStateRebootTimeoutKey
        valid = valid and False
    else:
        if int(values[unknownStateRebootTimeoutKey]) > int(values[watchdogLatencyKey]):
            print "error: watchdog unknown state timeout value '%s' must be shorter than (or equal to) watchdog latency value '%s'" %  \
                    ( values[unknownStateRebootTimeoutKey], values[watchdogLatencyKey] )
            valid = valid and False
            
    return valid 


# *************************************************************************
# * Log and Trace storage structure and default levels
# *************************************************************************
logValues = dict()

loggingLevelKey = 'logginglevel'
tracingLevelKey = 'tracinglevel'

logValues.setdefault(loggingLevelKey, '1')      # default to logging only errors
logValues.setdefault(tracingLevelKey, '0')      # default to tracing off ('3' is tracing on)


# *************************************************************************
# * Logging level routines
# *************************************************************************
def read_watchdog_logging_levels():
    """Load watchdog logging values from watchdog_logging.xml config file"""

    watchdogLoggingDocument = get_xml_document(watchdogLoggingConfigFile)

    watchdogLoggingElement = watchdogLoggingDocument.documentElement

    loadChildrenOfXMLElementIntoDictionary(watchdogLoggingElement, logValues)

    watchdogLoggingDocument.unlink()
